@@ -45,8 +45,9 @@ def fill_combo(combo, data_list):
     """
     store = gtk.ListStore(gobject.TYPE_STRING)
 
-    for data in [ item for item in data_list if item ]:
-        store.append(row=[data])
+    for data in data_list:
+        if data:
+            store.append(row=[data])
     
     combo.set_model(store)
     combo.set_text_column(0)
@@ -60,11 +61,13 @@ def fill_entry(entry, data_list):
     """
     Fill a entry with completion data
     """
+
     store = gtk.ListStore(gobject.TYPE_STRING)
 
-    for data in [ item for item in data_list if item ]:
-        store.append(row=[data])
-        
+    for data in data_list:
+        if data:
+            store.append(row=[data])
+
     completion = gtk.EntryCompletion()
     completion.set_model(store)
     completion.set_minimum_key_length(1)
@@ -111,14 +114,14 @@ class StandardCustomSelector(object):
         """
         Constructor for the StandardCustomSelector class.
 
-        @param cbe: Existing ComboBoxEntry widget to use.
-        @type cbe: gtk.ComboBoxEntry
-        @param mapping: The mapping between integer and string constants.
-        @type mapping:  dict
-        @param custom_key: The key corresponding to the custom string entry
-        @type custom_key:  int
-        @param active_key: The key for the entry to make active upon creation
-        @type active_key:  int
+        :param cbe: Existing ComboBoxEntry widget to use.
+        :type cbe: gtk.ComboBoxEntry
+        :param mapping: The mapping between integer and string constants.
+        :type mapping:  dict
+        :param custom_key: The key corresponding to the custom string entry
+        :type custom_key:  int
+        :param active_key: The key for the entry to make active upon creation
+        :type active_key:  int
         """
 
         # set variables
@@ -155,15 +158,14 @@ class StandardCustomSelector(object):
         """
         Fill with data
         """
-        keys = self.mapping.keys()
-        keys.sort(self.by_value)
+        keys = sorted(self.mapping, self.by_value)
         index = 0
         for key in keys:
             if key != self.custom_key:
                 self.store.append(row=[key, self.mapping[key]])
                 if key == self.active_key:
                     self.active_index = index
-                index = index + 1
+                index += 1
 
         if self.additional:
             for event_type in self.additional:
@@ -177,7 +179,7 @@ class StandardCustomSelector(object):
                     self.store.append(row=[int(event_type), str(event_type)])
                 if key == self.active_key:
                     self.active_index = index
-                index = index + 1
+                index += 1
 
     def by_value(self, first, second):
         """
@@ -191,8 +193,8 @@ class StandardCustomSelector(object):
         """
         Get selected values.
 
-        @return: Returns (int,str) tuple corresponding to the selection.
-        @rtype: tuple
+        :returns: Returns (int,str) tuple corresponding to the selection.
+        :rtype: tuple
         """
         active_iter = self.selector.get_active_iter()
         if active_iter:
@@ -203,8 +205,8 @@ class StandardCustomSelector(object):
         else:
             int_val = self.custom_key
             str_val = self.selector.child.get_text().strip()
-        if str_val in self.mapping.values():
-            for key in self.mapping.keys():
+        if str_val in self.mapping.itervalues():
+            for key in self.mapping:
                 if str_val == self.mapping[key]:
                     int_val = key
                     break
@@ -216,11 +218,11 @@ class StandardCustomSelector(object):
         """
         Set values according to given tuple.
 
-        @param val: (int,str) tuple with the values to set.
-        @type val: tuple
+        :param val: (int,str) tuple with the values to set.
+        :type val: tuple
         """
         key, text = val
-        if key in self.mapping.keys() and key != self.custom_key:
+        if key in self.mapping and key != self.custom_key:
             self.store.foreach(self.set_int_value, key)
         elif self.custom_key is not None:
             self.selector.child.set_text(text)
