@@ -18,7 +18,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: Bookmarks.py 12127 2009-02-26 09:23:40Z romjerome $
+# $Id: Bookmarks.py 15918 2010-09-20 23:58:16Z nick-h $
 
 "Handle bookmarks for the gramps interface."
 
@@ -51,11 +51,10 @@ import gtk
 #
 #-------------------------------------------------------------------------
 import GrampsDisplay
-from BasicUtils import name_displayer
 import ListModel
 import Utils
 import const
-from TransUtils import sgettext as _
+from gen.ggettext import sgettext as _
 
 #-------------------------------------------------------------------------
 #
@@ -105,7 +104,7 @@ class Bookmarks :
         """
         Connect the person-delete signal
         """
-        self.dbstate.db.connect('person-delete', self.remove_handles)
+        raise NotImplementedError
 
     def update_bookmarks(self, bookmarks):
         """
@@ -127,6 +126,7 @@ class Bookmarks :
             self.uistate.uimanager.remove_ui(self.active)
             self.uistate.uimanager.remove_action_group(self.action_group)
             self.action_group = gtk.ActionGroup('Bookmarks')
+            self.uistate.uimanager.ensure_update()
             self.active = DISABLED
 
     def redraw_and_report_change(self):
@@ -140,13 +140,12 @@ class Bookmarks :
         text.write(TOP)
 
         self.undisplay()
-        
+
         actions = []
         count = 0
 
         if len(self.bookmarks.get()) > 0:
             text.write('<placeholder name="GoToBook">')
-
             for item in self.bookmarks.get():
                 try:
                     label, obj = self.make_label(item)
@@ -167,12 +166,10 @@ class Bookmarks :
         text.close()
 
     def make_label(self, handle):
-        person = self.dbstate.db.get_person_from_handle(handle)
-        name = name_displayer.display(person)
-        return ("%s [%s]" % (name, person.gramps_id), person)
+        raise NotImplementedError
 
     def callback(self, handle):
-        return make_callback(handle, self.dbstate.change_active_handle)
+        raise NotImplementedError
 
     def add(self, person_handle):
         """Append the person to the bottom of the bookmarks."""
@@ -198,7 +195,7 @@ class Bookmarks :
 
     def draw_window(self):
         """Draw the bookmark dialog box."""
-        title = _("%(title)s - GRAMPS") % {'title': _("Organize Bookmarks")}
+        title = _("%(title)s - Gramps") % {'title': _("Organize Bookmarks")}
         self.top = gtk.Dialog(title)
         self.top.set_default_size(400, 350)
         self.top.set_modal(True)
@@ -281,34 +278,24 @@ class Bookmarks :
         self.namemodel.remove(the_iter)
         self.modified = True
         if row > 0:
-            row = row - 1
+            row -= 1
         self.namemodel.select_row(row)
 
     def up_clicked(self, obj):
         """Move the current selection up one row."""
         row = self.namemodel.get_selected_row()
-        if not row or row == -1:
-            return
-        store, the_iter = self.namemodel.get_selected()
-        data = self.namemodel.get_data(the_iter, range(self.namemodel_cols))
-        self.namemodel.remove(the_iter)
-        self.namemodel.insert(row-1, data, None, 1)
-        handle = self.bookmarks.pop(row)
-        self.bookmarks.insert(row-1, handle)
-        self.modified = True
+        if self.namemodel.move_up(row):
+            handle = self.bookmarks.pop(row)
+            self.bookmarks.insert(row-1, handle)
+            self.modified = True
 
     def down_clicked(self, obj):
         """Move the current selection down one row."""
         row = self.namemodel.get_selected_row()
-        if row + 1 >= self.namemodel.count or row == -1:
-            return
-        store, the_iter = self.namemodel.get_selected()
-        data = self.namemodel.get_data(the_iter, range(self.namemodel_cols))
-        self.namemodel.remove(the_iter)
-        self.namemodel.insert(row+1, data, None, 1)
-        handle = self.bookmarks.pop(row)
-        self.bookmarks.insert(row+1, handle)
-        self.modified = True
+        if self.namemodel.move_down(row):
+            handle = self.bookmarks.pop(row)
+            self.bookmarks.insert(row+1, handle)
+            self.modified = True
 
     def help_clicked(self):
         """Display the relevant portion of GRAMPS manual."""
@@ -326,6 +313,19 @@ class ListBookmarks(Bookmarks):
     def do_callback(self, handle):
         self.goto_handle(handle)
 
+class PersonBookmarks(ListBookmarks) :
+    "Handle the bookmarks interface for Gramps."
+    
+    def __init__(self, dbstate, uistate, bookmarks, goto_handle):
+        ListBookmarks.__init__(self, dbstate, uistate, bookmarks,
+                               goto_handle)
+        
+    def make_label(self, handle):
+        return Utils.navigation_label(self.dbstate.db, 'Person', handle)
+
+    def connect_signals(self):
+        self.dbstate.db.connect('person-delete', self.remove_handles)
+        
 class FamilyBookmarks(ListBookmarks) :
     "Handle the bookmarks interface for Gramps."
     
@@ -334,9 +334,7 @@ class FamilyBookmarks(ListBookmarks) :
                                goto_handle)
         
     def make_label(self, handle):
-        obj = self.dbstate.db.get_family_from_handle(handle)
-        name = Utils.family_name(obj, self.dbstate.db)
-        return ("%s [%s]" % (name, obj.gramps_id), obj)
+        return Utils.navigation_label(self.dbstate.db, 'Family', handle)
 
     def connect_signals(self):
         self.dbstate.db.connect('family-delete', self.remove_handles)
@@ -349,12 +347,7 @@ class EventBookmarks(ListBookmarks) :
                                goto_handle)
         
     def make_label(self, handle):
-        obj = self.dbstate.db.get_event_from_handle(handle)
-        if obj.get_description() == "":
-            name = str(obj.get_type())
-        else:
-            name = obj.get_description()
-        return ("%s [%s]" % (name, obj.gramps_id), obj)
+        return Utils.navigation_label(self.dbstate.db, 'Event', handle)
 
     def connect_signals(self):
         self.dbstate.db.connect('event-delete', self.remove_handles)
@@ -366,9 +359,7 @@ class SourceBookmarks(ListBookmarks) :
                                goto_handle)
         
     def make_label(self, handle):
-        obj = self.dbstate.db.get_source_from_handle(handle)
-        name = obj.get_title()
-        return ("%s [%s]" % (name, obj.gramps_id), obj)
+        return Utils.navigation_label(self.dbstate.db, 'Source', handle)
 
     def connect_signals(self):
         self.dbstate.db.connect('source-delete', self.remove_handles)
@@ -381,9 +372,7 @@ class MediaBookmarks(ListBookmarks) :
                                goto_handle)
         
     def make_label(self, handle):
-        obj = self.dbstate.db.get_object_from_handle(handle)
-        name = obj.get_description()
-        return ("%s [%s]" % (name, obj.gramps_id), obj)
+        return Utils.navigation_label(self.dbstate.db, 'Media', handle)
 
     def connect_signals(self):
         self.dbstate.db.connect('media-delete', self.remove_handles)
@@ -396,9 +385,7 @@ class RepoBookmarks(ListBookmarks) :
                                goto_handle)
         
     def make_label(self, handle):
-        obj = self.dbstate.db.get_repository_from_handle(handle)
-        name = obj.get_name()
-        return ("%s [%s]" % (name, obj.gramps_id), obj)
+        return Utils.navigation_label(self.dbstate.db, 'Repository', handle)
 
     def connect_signals(self):
         self.dbstate.db.connect('repository-delete', self.remove_handles)
@@ -411,9 +398,7 @@ class PlaceBookmarks(ListBookmarks) :
                                goto_handle)
         
     def make_label(self, handle):
-        obj = self.dbstate.db.get_place_from_handle(handle)
-        name = obj.get_title()
-        return ("%s [%s]" % (name, obj.gramps_id), obj)
+        return Utils.navigation_label(self.dbstate.db, 'Place', handle)
 
     def connect_signals(self):
         self.dbstate.db.connect('place-delete', self.remove_handles)
@@ -426,11 +411,7 @@ class NoteBookmarks(ListBookmarks) :
                                goto_handle)
         
     def make_label(self, handle):
-        obj = self.dbstate.db.get_note_from_handle(handle)
-        name = obj.get().replace('\n', ' ')
-        if len(name) > 40:
-            name = name[:40]+"..."
-        return ("%s [%s]" % (name, obj.gramps_id), obj)
+        return Utils.navigation_label(self.dbstate.db, 'Note', handle)
 
     def connect_signals(self):
         self.dbstate.db.connect('note-delete', self.remove_handles)
