@@ -19,7 +19,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: DateEdit.py 14091 2010-01-18 04:42:17Z pez4brian $
+# $Id: DateEdit.py 17544 2011-05-21 08:36:07Z romjerome $
 
 """
 Date editing module for GRAMPS. 
@@ -62,7 +62,7 @@ import gtk
 #
 #-------------------------------------------------------------------------
 from gen.ggettext import sgettext as _
-from gen.lib import Date
+from gen.lib.date import Date, NextYear
 import DateHandler
 import const
 import GrampsDisplay
@@ -150,6 +150,9 @@ class DateEdit(object):
         # if text could not be parsed it is assumed invalid
         if self.date_obj.get_modifier() == Date.MOD_TEXTONLY:
             return ValidationError(_('Bad Date'))
+        elif (self.date_obj.to_calendar(calendar_name=Date.CAL_GREGORIAN) >>
+              NextYear()):
+            return ValidationError(_('Date more than one year in the future'))
 
     def invoke_date_editor(self, obj):
         """
@@ -186,7 +189,6 @@ class DateEditorDialog(ManagedWindow.ManagedWindow):
         """
         Initiate and display the dialog.
         """
-
         ManagedWindow.ManagedWindow.__init__(self, uistate, track, self)
         
         # Create self.date as a copy of the given Date object.
@@ -240,7 +242,7 @@ class DateEditorDialog(ManagedWindow.ManagedWindow):
         
         self.dual_dated = self.top.get_object('dualdated')
         self.new_year = self.top.get_object('newyear')
-        self.new_year.set_active(self.date.get_new_year())
+        self.new_year.set_text(self.date.newyear_to_str())
 
         # Disable second date controls if not compound date
         if not self.date.is_compound():
@@ -341,7 +343,7 @@ class DateEditorDialog(ManagedWindow.ManagedWindow):
                 self.start_year.get_value_as_int(),
                 self.dual_dated.get_active())
         calendar = self.calendar_box.get_active()
-        newyear = self.new_year.get_active()
+        newyear = Date.newyear_to_code(self.new_year.get_text())
         return (quality, modifier, calendar, value, text, newyear)
 
     def switch_type(self, obj):
