@@ -18,7 +18,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
-# $Id: _DateDisplay.py 13758 2009-12-11 05:55:30Z pez4brian $
+# $Id: _DateDisplay.py 15010 2010-04-03 23:33:17Z dsblank $
 
 """
 U.S English date display class. Should serve as the base class for all
@@ -139,12 +139,17 @@ class DateDisplay(object):
         Formats the extra items (calendar, newyear) for a date.
         """
         scal = self.calendar[cal]
-        snewyear = self.newyear[newyear]
+        if isinstance(newyear, int) and newyear <= len(self.newyear):
+            snewyear = self.newyear[newyear]
+        elif isinstance(newyear, (list, tuple)):
+            snewyear = "%s-%s" % (newyear[0], newyear[1])
+        else:
+            snewyear = "Err"
         retval = ""
         for item in [scal, snewyear]:
             if item:
                 if retval:
-                    retval += ","
+                    retval += ", "
                 retval += item
         if retval:
             return " (%s)" % retval
