@@ -18,7 +18,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: _DateHandler.py 13436 2009-10-28 03:25:57Z pez4brian $
+# $Id: _DateHandler.py 17284 2011-04-26 20:19:54Z m_d_n $
 
 """
 Class handling language-specific selection for date parser and displayer.
@@ -47,13 +47,23 @@ log = logging.getLogger(".DateHandler")
 #-------------------------------------------------------------------------
 from _DateParser import DateParser
 from _DateDisplay import DateDisplay, DateDisplayEn
+import constfunc
 
 #-------------------------------------------------------------------------
 #
 # Constants 
 #
 #-------------------------------------------------------------------------
-LANG = locale.getlocale(locale.LC_TIME)[0]
+if not constfunc.win():
+    LANG = locale.getlocale(locale.LC_TIME)[0]
+else:
+    LANG = locale.getdefaultlocale(locale.LC_TIME)[0]
+
+# If LANG contains ".UTF-8" use only the part to the left of "."
+# Otherwise some date handler will not load. 
+if LANG and ".UTF-8" in LANG.upper():
+    LANG = LANG.split(".")[0]
+    
 if not LANG:
     if "LANG" in os.environ:
         LANG = os.environ["LANG"]
