@@ -19,7 +19,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: _DateParser.py 14921 2010-03-24 16:05:14Z bmcage $
+# $Id: _DateParser.py 16642 2011-02-17 08:34:48Z ldnp $
 
 """
 Date parsing class. Serves as the base class for any localized
@@ -96,18 +96,20 @@ def julian_valid(date_tuple):
 
 def swedish_valid(date_tuple):
     """ Checks if date_tuple is a valid date in Swedish Calendar """
-    valid = gregorian_valid(date_tuple)
-    # not sure how <= and >= works with tuples???
+    valid_J = julian_valid(date_tuple)
     date_tuple = (date_tuple[2], date_tuple[1], date_tuple[0])
-    if date_tuple <= (1700, 2, 28):
-        valid = False 
-    if date_tuple == (1700, 2, 29):  # leapday 1700 was skipped
-        valid = False 
-    if date_tuple == (1712, 2, 30):  # extra day was inserted 1712
-        valid = True 
-    if date_tuple >= (1712, 3, 1):   # back to julian
-        valid = False
-    return valid
+    # Swedish calendar starts as Julian 1700-03-01 and ends 1712-03-01 as Julian
+    if date_tuple >= (1700, 2, 29) and  date_tuple < (1712, 3, 1):
+        if date_tuple == (1712, 2, 30):  # extra day was inserted 1712, not valid Julian
+            return True
+        if valid_J:
+            if date_tuple == (1700, 2, 29):  # leapday 1700 was skipped
+                return False
+            return True
+        else:
+            return False
+    else:
+        return False
 
 def french_valid(date_tuple):
     """ Checks if date_tuple is a valid date in French Calendar """
@@ -216,7 +218,7 @@ class DateParser(object):
         "januari"    :  1, "februari"   :  2,
         "mars"       :  3, "april"      :  4,
         "maj"        :  5, "juni"       :  6,
-        "juli"       :  7, "augisti"    :  8,
+        "juli"       :  7, "augusti"    :  8,
         "september"  :  9, "oktober"    : 10,
         "november"   : 11, "december"   : 12,
         }
@@ -317,20 +319,25 @@ class DateParser(object):
         self._imon_str = self.re_longest_first(self.islamic_to_int.keys())
         self._smon_str = self.re_longest_first(self.swedish_to_int.keys())
         self._cal_str  = self.re_longest_first(self.calendar_to_int.keys())
-        self._ny_str = self.re_longest_first(self.newyear_to_int.keys())
+        self._ny_str   = self.re_longest_first(self.newyear_to_int.keys())
 
         # bce, calendar type and quality may be either at the end or at
         # the beginning of the given date string, therefore they will
         # be parsed from the middle and will be in match.group(2).
-        self._bce_re   = re.compile("(.*)\s+%s( ?.*)" % self._bce_str)
+        self._bce_re    = re.compile("(.*)\s+%s( ?.*)" % self._bce_str)
+
+        self._cal       = re.compile("(.*)\s+\(%s\)( ?.*)" % self._cal_str,
+                                     re.IGNORECASE)
+        self._calny     = re.compile("(.*)\s+\(%s,\s*%s\)( ?.*)" % (self._cal_str,
+                                                                    self._ny_str),
+                                     re.IGNORECASE)
+        self._calny_iso = re.compile("(.*)\s+\(%s,\s*(\d{1,2}-\d{1,2})\)( ?.*)" % self._cal_str,
+                                     re.IGNORECASE)
+
+        self._ny        = re.compile("(.*)\s+\(%s\)( ?.*)" % self._ny_str,
+                                     re.IGNORECASE)
+        self._ny_iso    = re.compile("(.*)\s+\((\d{1,2}-\d{1,2})\)( ?.*)")
 
-        self._cal      = re.compile("(.*)\s+\(%s\)( ?.*)" % self._cal_str,
-                                    re.IGNORECASE)
-        self._calny    = re.compile("(.*)\s+\(%s,%s\)( ?.*)" % (self._cal_str,
-                                                                self._ny_str),
-                                   re.IGNORECASE)
-        self._ny    = re.compile("(.*)\s+\(%s\)( ?.*)" % self._ny_str,
-                                 re.IGNORECASE)
         self._qual     = re.compile("(.* ?)%s\s+(.+)" % self._qual_str,
                                     re.IGNORECASE)
         
@@ -369,6 +376,7 @@ class DateParser(object):
                                     re.IGNORECASE)
         self._numeric  = re.compile("((\d+)[/\.]\s*)?((\d+)[/\.]\s*)?(\d+)\s*$")
         self._iso      = re.compile("(\d+)(/(\d+))?-(\d+)-(\d+)\s*$")
+        self._isotimestamp = re.compile("^\s*?(\d{4})([01]\d)([0123]\d)(?:(?:[012]\d[0-5]\d[0-5]\d)|(?:\s+[012]\d:[0-5]\d(?::[0-5]\d)?))?\s*?$")
         self._rfc      = re.compile("(%s,)?\s+(\d|\d\d)\s+%s\s+(\d+)\s+\d\d:\d\d(:\d\d)?\s+(\+|-)\d\d\d\d" 
                                     % (self._rfc_day_str, self._rfc_mon_str))
 
@@ -500,6 +508,19 @@ class DateParser(object):
             else:
                 return (d, m, y, False)
 
+        # Database datetime format, used in ex. MSSQL
+        # YYYYMMDD HH:MM:SS or YYYYMMDD or YYYYMMDDHHMMSS
+        match = self._isotimestamp.match(text)
+        if match:
+            groups = match.groups()
+            y = self._get_int(groups[0])
+            m = self._get_int(groups[1])
+            d = self._get_int(groups[2])
+            value = (d, m, y, False)
+            if not check((d, m, y)):
+                value = Date.Empty
+            return value
+
         match = self._rfc.match(text)
         if match:
             groups = match.groups()
@@ -562,6 +583,12 @@ class DateParser(object):
             cal = self.calendar_to_int[match.group(2).lower()]
             newyear = self.newyear_to_int[match.group(3).lower()]
             text = match.group(1) + match.group(4)
+        else:
+            match = self._calny_iso.match(text)
+            if match:
+                cal = self.calendar_to_int[match.group(2).lower()]
+                newyear = tuple(map(int, match.group(3).split("-")))
+                text = match.group(1) + match.group(4)
         return (text, cal, newyear)
 
     def match_newyear(self, text, newyear):
@@ -574,6 +601,11 @@ class DateParser(object):
         if match:
             newyear = self.newyear_to_int[match.group(2).lower()]
             text = match.group(1) + match.group(3)
+        else:
+            match = self._ny_iso.match(text)
+            if match:
+                newyear = tuple(map(int, match.group(2).split("-")))
+                text = match.group(1) + match.group(3)
         return (text, newyear)
 
     def match_quality(self, text, qual):
