@@ -101,63 +101,114 @@ class DateParserRU(DateParser):
 
     hebrew_to_int = {
         u"тишрей":1, 
+        u"тишрея":1, 
         u"хешван":2, 
+        u"хешвана":2, 
         u"кислев":3, 
+        u"кислева":3, 
         u"тевет":4, 
+        u"тевета":4, 
         u"шеват":5, 
+        u"шевата":5, 
         u"адар":6, 
-        u"адар бет":7, 
+        u"адара":6, 
+        u"адара бет":7, 
         u"нисан":8, 
+        u"нисана":8, 
+        u"ниссан":8, 
+        u"ниссана":8, 
         u"ияр":9, 
+        u"ияра":9, 
         u"сиван":10, 
+        u"сивана":10, 
+        u"тамуз":11, 
+        u"тамуза":11, 
         u"таммуз":11, 
+        u"таммуза":11, 
         u"ав":12, 
+        u"ава":12, 
         u"элул":13, 
+        u"элула":13, 
+        u"элуль":13, 
+        u"элуля":13, 
     }
 
     islamic_to_int = {
         u"мухаррам":1, 
+        u"мухаррама":1, 
         u"сафар":2, 
+        u"сафара":2, 
         u"раби-аль-авваль":3, 
+        u"раби-аль-авваля":3, 
         u"раби-ассани":4, 
         u"джумада-аль-уля":5, 
         u"джумада-аль-ахира":6, 
         u"раджаб":7, 
+        u"раджаба":7, 
         u"шаабан":8, 
+        u"шаабана":8, 
         u"рамадан":9, 
+        u"рамадана":9, 
         u"шавваль":10, 
+        u"шавваля":10, 
         u"зуль-каада":11, 
         u"зуль-хиджжа":12, 
         }
 
     persian_to_int = {
         u"фарвардин":1, 
+        u"фарвардина":1, 
         u"урдбихишт":2, 
+        u"урдбихишта":2, 
         u"хурдад":3, 
+        u"хурдада":3, 
         u"тир":4, 
+        u"тира":4, 
         u"мурдад":5, 
+        u"мурдада":5, 
         u"шахривар":6, 
+        u"шахривара":6, 
         u"михр":7, 
+        u"михра":7, 
         u"абан":8, 
+        u"абана":8, 
         u"азар":9, 
+        u"азара":9, 
         u"дай":10, 
+        u"дая":10, 
         u"бахман":11, 
+        u"бахмана":11, 
         u"исфаидармуз":12, 
+        u"исфаидармуза":12, 
         }
 
     french_to_int = {
         u"вандемьер":1, 
+        u"вандемьера":1, 
         u"брюмер":2, 
+        u"брюмера":2, 
         u"фример":3, 
+        u"фримера":3, 
         u"нивоз":4, 
+        u"нивоза":4, 
         u"плювиоз":5, 
+        u"плювиоза":5, 
         u"вантоз":6, 
+        u"вантоза":6, 
         u"жерминаль":7, 
+        u"жерминаля":7, 
         u"флореаль":8, 
+        u"флореаля":8, 
         u"прериаль":9, 
+        u"прериаля":9, 
         u"мессидор":10, 
+        u"мессидора":10, 
         u"термидор":11, 
+        u"термидора":11, 
         u"фрюктидор":12, 
+        u"фрюктидора":12, 
+        u"доп.":13, 
+        u"дополн.":13, 
         u"дополнит.":13, 
         }
 
@@ -187,13 +238,12 @@ class DateDisplayRU(DateDisplay):
     """
     Russian language date display class. 
     """
-    # TODO: Translate these month strings:
-    long_months = ( u"", u"January", u"February", u"March", u"April", u"May", 
-                    u"June", u"July", u"August", u"September", u"October", 
-                    u"November", u"December" )
+    long_months = ( u"", u"января", u"февраля", u"марта", u"апреля", u"мая", 
+                    u"июня", u"июля", u"августа", u"сентября", u"октября", 
+                    u"ноября", u"декабря" )
     
-    short_months = ( u"", u"Jan", u"Feb", u"Mar", u"Apr", u"May", u"Jun", 
-                     u"Jul", u"Aug", u"Sep", u"Oct", u"Nov", u"Dec" )
+    short_months = ( u"", u"янв", u"фев", u"мар", u"апр", u"мая", u"июн", 
+                     u"июл", u"авг", u"сен", u"окт", u"ноя", u"дек" )
     
     calendar = (
         u"", 
@@ -202,7 +252,7 @@ class DateDisplayRU(DateDisplay):
         u"республиканский", 
         u"персидский", 
         u"исламский", 
-        u"Swedish" 
+        u"шведский" 
         )
 
     _mod_str = (
@@ -218,69 +268,69 @@ class DateDisplayRU(DateDisplay):
 
     formats = (
         "ГГГГ-ММ-ДД (ISO)", "Численный", "Месяц День, Год", 
-        "МЕС ДД, ГГГГГ", "День Месяц, Год", "ДД МЕС, ГГГГГ"
+        "МЕС ДД, ГГГГ", "День Месяц, Год", "ДД МЕС, ГГГГ"
         )
 
     hebrew = ( u"", 
-        u"Тишрей", 
-        u"Хешван", 
-        u"Кислев", 
-        u"Тевет", 
-        u"Шеват", 
-        u"Адар", 
-        u"Адар бет", 
-        u"Нисан", 
-        u"Ияр", 
-        u"Сиван", 
-        u"Таммуз", 
-        u"Ав", 
-        u"Элул", 
+        u"тишрея", 
+        u"хешвана", 
+        u"кислева", 
+        u"тевета", 
+        u"шевата", 
+        u"адара", 
+        u"адара бет", 
+        u"нисана", 
+        u"ияра", 
+        u"сивана", 
+        u"таммуза", 
+        u"ава", 
+        u"элула", 
         )
 
     islamic = ( u"", 
-        u"Мухаррам", 
-        u"Сафар", 
-        u"Раби-аль-авваль", 
-        u"Раби-ассани", 
-        u"Джумада-аль-уля", 
-        u"Джумада-аль-ахира", 
-        u"Раджаб", 
-        u"Шаабан", 
-        u"Рамадан", 
-        u"Шавваль", 
-        u"Зуль-каада", 
-        u"Зуль-хиджжа", 
+        u"мухаррама", 
+        u"сафара", 
+        u"раби-аль-авваля", 
+        u"раби-ассани", 
+        u"джумада-аль-уля", 
+        u"джумада-аль-ахира", 
+        u"раджаба", 
+        u"шаабана", 
+        u"рамадана", 
+        u"шавваля", 
+        u"зуль-каада", 
+        u"зуль-хиджжа", 
         )
 
     persian = ( u"", 
-        u"Фарвардин", 
-        u"Урдбихишт", 
-        u"Хурдад", 
-        u"Тир", 
-        u"Мурдад", 
-        u"Шахривар", 
-        u"Михр", 
-        u"Абан", 
-        u"Азар", 
-        u"Дай", 
-        u"Бахман", 
-        u"Исфаидармуз", 
+        u"фарвардина", 
+        u"урдбихишта", 
+        u"хурдада", 
+        u"тира", 
+        u"мурдада", 
+        u"шахривара", 
+        u"михра", 
+        u"абана", 
+        u"азара", 
+        u"дая", 
+        u"бахмана", 
+        u"исфаидармуза", 
         )
 
     french = ( u"", 
-        u"Вандемьер", 
-        u"Брюмер", 
-        u"Фример", 
-        u"Нивоз", 
-        u"Плювиоз", 
-        u"Вантоз", 
-        u"Жерминаль", 
-        u"Флореаль", 
-        u"Прериаль", 
-        u"Мессидор", 
-        u"Термидор", 
-        u"Фрюктидор", 
-        u"Дополнит."
+        u"вандемьера", 
+        u"брюмера", 
+        u"фримера", 
+        u"нивоза", 
+        u"плювиоза", 
+        u"вантоза", 
+        u"жерминаля", 
+        u"флореаля", 
+        u"прериаля", 
+        u"мессидора", 
+        u"термидора", 
+        u"фрюктидора", 
+        u"дополнит."
         )
 
     def display(self, date):
