@@ -20,7 +20,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: DdTargets.py 12977 2009-08-12 10:01:28Z acraphae $
+# $Id: DdTargets.py 16035 2010-10-24 14:43:47Z bmcage $
 
 """
    This module provides management of all the drag and drop target information
@@ -125,11 +125,15 @@ class _DdTargets(object):
         self.REPO_LINK          = _DdType(self, 'repo-link')
         self.REPOREF            = _DdType(self, 'reporef')
         self.PERSON_LINK        = _DdType(self, 'person-link')
-        self.PERSON_LINK_LIST   = _DdType(self, 'person-link-list')
+        self.FAMILY_LINK        = _DdType(self, 'family-link')
+        self.LINK_LIST          = _DdType(self, 'link-list')
+        self.RAW_LIST           = _DdType(self, 'raw-list')
+        self.HANDLE_LIST        = _DdType(self, 'handle-list')
         self.PERSONREF          = _DdType(self, 'personref')
         self.SOURCEREF          = _DdType(self, 'srcref')
         self.SOURCE_LINK        = _DdType(self, 'source-link')
         self.URL                = _DdType(self, 'url')
+        self.SURNAME            = _DdType(self, 'surname')
 
         # List of all types that are used between
         # gramps widgets but should not be exported
@@ -146,13 +150,17 @@ class _DdTargets(object):
             self.NOTE_LINK, 
             self.PLACE_LINK, 
             self.PERSON_LINK, 
-            self.PERSON_LINK_LIST, 
+            self.FAMILY_LINK, 
+            self.LINK_LIST, 
+            self.RAW_LIST, 
+            self.HANDLE_LIST, 
             self.PERSONREF, 
             self.REPO_LINK, 
             self.REPOREF, 
             self.SOURCEREF, 
             self.SOURCE_LINK, 
             self.URL, 
+            self.SURNAME
         ]
         
         self.CHILD         = _DdType(self, 'child')
