@@ -20,7 +20,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: DisplayState.py 15301 2010-05-02 13:48:49Z dsblank $
+# $Id: DisplayState.py 16160 2010-11-08 15:50:51Z nick-h $
 
 #-------------------------------------------------------------------------
 #
@@ -82,6 +82,8 @@ class History(gen.utils.Callback):
 
     def __init__(self, dbstate, nav_type):
         gen.utils.Callback.__init__(self)
+        self.dbstate = dbstate
+        self.nav_type = nav_type
         self.clear()
 
         dbstate.connect('database-changed', self.connect_signals)
@@ -105,6 +107,11 @@ class History(gen.utils.Callback):
         self.index = -1
         self.lock = False
 
+        if self.dbstate.open and self.nav_type == 'Person':
+            initial_person = self.dbstate.db.find_initial_person()
+            if initial_person:
+                self.push(initial_person.get_handle())
+
     def remove(self, handle, old_id=None):
         """
         Remove a handle from the history list
@@ -307,19 +314,30 @@ class WarnHandler(RotateHandler):
         self.button = button
         button.on_clicked(self.display)
         self.timer = None
+        self.last_line = '-1'
 
     def emit(self, record):
-        if self.timer:
-            gobject.source_remove(self.timer)
-        gobject.timeout_add(180*1000, self._clear)
+        if self.timer is None:
+            #check every 3 minutes if warn button can disappear
+            self.timer = gobject.timeout_add(3*60*1000, self._check_clear)
         RotateHandler.emit(self, record)
         self.button.show()
 
+    def _check_clear(self):
+        new_last_line = self.get_buffer()[-1]
+        if self.last_line == new_last_line:
+            #buffer has not changed for 3 minutes, let's clear it:
+            self._clear()
+            return False
+        else:
+            self.last_line = new_last_line
+            return True
+
     def _clear(self):
         self.button.hide()
         self.set_capacity(self._capacity)
+        self.last_line = '-1'
         self.timer = None
-        return False
 
     def display(self, obj):
         obj.hide()
@@ -329,7 +347,6 @@ class WarnHandler(RotateHandler):
         buf = msg.get_buffer()
         for i in self.get_formatted_log():
             buf.insert_at_cursor(i + '\n')
-        self.set_capacity(self._capacity)
         top.run()
         top.destroy()
 
@@ -499,7 +516,7 @@ class DisplayState(gen.utils.Callback):
     def show_filter_results(self, dbstate, matched, total):
         #nav_type = self.viewmanager.active_page.navigation_type()
         #text = ((_("%(nav_type)s View") % {"nav_type": _(nav_type)}) + 
-        text = (self.viewmanager.active_page.title +
+        text = (self.viewmanager.active_page.get_title() +
                 (": %d/%d" % (matched, total)))
         self.status.pop(1, self.last_bar)
         self.status.push(1, text, self.last_bar)
