@@ -2,7 +2,10 @@
 # Gramps - a GTK+/GNOME based genealogy program
 #
 # Copyright (C) 2004-2007 Donald N. Allingham
-# Copyright (C) 2008-2009 Brian G. Matherly
+# Copyright (C) 2008      Brian G. Matherly
+# Contribution  2009 by   Brad Crittenden <brad [AT] bradcrittenden.net>
+# Copyright (C) 2008      Benny Malengier
+# Copyright (C) 2010      Jakim Friant
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -28,8 +31,7 @@
 #-------------------------------------------------------------------------
 import os
 import sys
-from gettext import gettext as _
-from xml.sax.saxutils import escape
+from gen.ggettext import gettext as _
 
 #-------------------------------------------------------------------------
 #
@@ -52,10 +54,11 @@ import gtk
 #-------------------------------------------------------------------------
 
 import const
-import Config
-from gen.plug import PluginManager
+import config
+from gui.pluginmanager import GuiPluginManager
 import Utils
 import ManagedWindow
+from QuestionDialog import ErrorDialog
 
 #-------------------------------------------------------------------------
 #
@@ -92,6 +95,7 @@ class ExportAssistant(gtk.Assistant, ManagedWindow.ManagedWindow) :
     
     """
 
+    #override predefined do_xxx signal handlers
     __gsignals__ = {"apply": "override", "cancel": "override",
                     "close": "override", "prepare": "override"}
                     
@@ -108,7 +112,7 @@ class ExportAssistant(gtk.Assistant, ManagedWindow.ManagedWindow) :
         
         self.writestarted = False
         
-        #set up Assisant
+        #set up Assistant
         gtk.Assistant.__init__(self)
         ##workaround around bug http://bugzilla.gnome.org/show_bug.cgi?id=56070
         self.forward_button = None
@@ -119,21 +123,24 @@ class ExportAssistant(gtk.Assistant, ManagedWindow.ManagedWindow) :
         self.top_title = _("Export Assistant")
         ManagedWindow.ManagedWindow.__init__(self,uistate,[],
                                                  self.__class__)
-        self.set_window(self, None, self.top_title, isWindow=True)
-        
+        #set_window is present in both parent classes
+        ManagedWindow.ManagedWindow.set_window(self, self, None,
+            self.top_title, isWindow=True)        
+
         #set up callback method for the export plugins
         self.callback = self.pulse_progressbar
             
-        if self.dbstate.active:
-            self.person = self.dbstate.get_active_person()
-        else:
+        person_handle = self.uistate.get_active('Person')
+        self.person = self.dbstate.db.get_person_from_handle(person_handle)
+        if not self.person:
             self.person = self.dbstate.db.find_initial_person()
             
         self.logo      = gtk.gdk.pixbuf_new_from_file(_gramps_png)
         self.splash    = gtk.gdk.pixbuf_new_from_file(_splash_jpg)
             
-        pmgr = PluginManager.get_instance()
+        pmgr = GuiPluginManager.get_instance()
         self.__exporters = pmgr.get_export_plugins()
+        self.map_exporters = {}
         
         self.__previous_page = -1
 
@@ -146,6 +153,7 @@ class ExportAssistant(gtk.Assistant, ManagedWindow.ManagedWindow) :
         #no progress page, looks ugly, and user needs to hit forward at end!
         self.create_page_summary()
         
+        self.option_box_instance = None
         #we need our own forward function as options page must not always be shown
         self.set_forward_page_func(self.forward_func, None)
         
@@ -160,6 +168,22 @@ class ExportAssistant(gtk.Assistant, ManagedWindow.ManagedWindow) :
         if isinstance(arg, gtk.Button) and arg.get_label() == 'gtk-go-forward':
             self.forward_button = arg
 
+    def get_cancel_button(self, arg):
+        if isinstance(arg, gtk.HBox):
+            arg.forall(self._cancel_btn)
+
+    def _cancel_btn(self, arg):
+        if isinstance(arg, gtk.Button) and arg.get_label() == 'gtk-cancel':
+            self.cancel_button = arg
+
+    def get_close_button(self, arg):
+        if isinstance(arg, gtk.HBox):
+            arg.forall(self._close_btn)
+
+    def _close_btn(self, arg):
+        if isinstance(arg, gtk.Button) and arg.get_label() == 'gtk-close':
+            self.close_button = arg
+
     def build_menu_names(self, obj):
         """Override ManagedWindow method."""
         return (self.top_title, None)
@@ -197,23 +221,25 @@ class ExportAssistant(gtk.Assistant, ManagedWindow.ManagedWindow) :
         table.set_row_spacings(6)
         table.set_col_spacings(6)
         
-        tip = gtk.Tooltips()
-        
         group = None
-        recent_type = Config.get(Config.RECENT_EXPORT_TYPE)
+        recent_type = config.get('behavior.recent-export-type')
         
-        for ix in range(len(self.__exporters)):
-            title = self.__exporters[ix].get_name()
-            description= self.__exporters[ix].get_description()
-
+        exporters = [(x.get_name().replace("_", ""), x) for x in self.__exporters]
+        exporters.sort()
+        ix = 0
+        for sort_title, exporter in exporters:
+            title = exporter.get_name()
+            description= exporter.get_description()
+            self.map_exporters[ix] = exporter
             button = gtk.RadioButton(group,title)
+            button.set_tooltip_text(description)
             if not group:
                 group = button
             self.format_buttons.append(button)
-            table.attach(button,0,2,2*ix,2*ix+1)
-            tip.set_tip(button,description)
+            table.attach(button, 0, 2, 2*ix, 2*ix+1)
             if ix == recent_type :
                 button.set_active(True)
+            ix += 1
         
         box.add(table)
         
@@ -249,30 +275,34 @@ class ExportAssistant(gtk.Assistant, ManagedWindow.ManagedWindow) :
         """
         if pagenumber == _ExportAssistant_pages['exporttypes'] :
             #decide if options need to be shown:
+            self.option_box_instance = None
             ix = self.get_selected_format_index()
-            if self.__exporters[ix].get_config(): 
-                return pagenumber + 1
-            else :
+            if not self.map_exporters[ix].get_config(): 
                 # no options needed
                 return pagenumber + 2
-        else :
-            return pagenumber + 1
+        elif pagenumber == _ExportAssistant_pages['options']:
+            # need to check to see if we should show file selection
+            if (self.option_box_instance and 
+                hasattr(self.option_box_instance, "no_fileselect")):
+                # don't show fileselect, but mark it ok
+                return pagenumber + 2
+        return pagenumber + 1
         
     def create_options(self):
         """This method gets the option page, and fills it with the options."""
         option = self.get_selected_format_index()
         vbox = self.get_nth_page(_ExportAssistant_pages['options'])
-        (config_title, config_box_class) = self.__exporters[option].get_config()
+        (config_title, config_box_class) = self.map_exporters[option].get_config()
         self.set_page_title(vbox, config_title)
         # remove present content of the vbox
         vbox.foreach(vbox.remove)
         # add new content
         if config_box_class:
-            self.option_box_instance = config_box_class(self.person)
+            self.option_box_instance = config_box_class(self.person, self.dbstate, self.uistate)
+            box = self.option_box_instance.get_option_box()
+            vbox.add(box)
         else: 
             self.option_box_instance = None
-        box = self.option_box_instance.get_option_box()
-        vbox.add(box)
         vbox.show_all()
         
         # We silently assume all options lead to accepted behavior
@@ -372,28 +402,24 @@ class ExportAssistant(gtk.Assistant, ManagedWindow.ManagedWindow) :
         self.set_page_complete(page, False)
         self.set_page_type(page, gtk.ASSISTANT_PAGE_SUMMARY)
 
-        
     def do_apply(self):
         pass
 
-    def do_cancel(self):
-        if self.writestarted :
-            return True
-        else : 
-            self.close()
-
     def do_close(self):
         if self.writestarted :
-            return True
-        else : 
+            pass
+        else :
             self.close()
 
+    def do_cancel(self):
+        self.do_close()
+
     def do_prepare(self, page):
         """
         The "prepare" signal is emitted when a new page is set as the 
         assistant's current page, but before making the new page visible.
         
-        @param page:	the new page to prepare for display.
+        :param page:   the new page to prepare for display.
         
         """
         #determine if we go backward or forward
@@ -435,29 +461,48 @@ class ExportAssistant(gtk.Assistant, ManagedWindow.ManagedWindow) :
             # The confirm page with apply button
             # Present user with what will happen
             ix = self.get_selected_format_index()
-            format = self.__exporters[ix].get_name()
-
-            #Allow for exotic error: file is still not correct
-            self.check_fileselect(self.chooser, show=False)
-            if self.get_page_complete(self.chooser) :
-                filename = Utils.get_unicode_path(self.chooser.get_filename())
-                name = os.path.split(filename)[1]
-                folder = os.path.split(filename)[0]
+            format = self.map_exporters[ix].get_name()
+            page_complete = False
+            # If no file select:
+            if (self.option_box_instance and 
+                hasattr(self.option_box_instance, "no_fileselect")):
+                # No file selection
+                filename = ''
                 confirm_text = _(
-                'The data will be saved as follows:\n\n'
-                'Format:\t%s\nName:\t%s\nFolder:\t%s\n\n'
-                'Press Apply to proceed, Back to revisit '
-                'your options, or Cancel to abort') % (format.replace("_",""), name, folder)
-                self.set_page_complete(page, True)
-            else :
-                confirm_text = _(
-                'The selected file and folder to save to '
-                'cannot be created or found.\n\n'
-                'Press Back to return and select a valid filename.'
-                ) 
-                self.set_page_complete(page, False)
-            
-            page.set_label(escape(confirm_text))
+                    'The data will be exported as follows:\n\n'
+                    'Format:\t%s\n\n'
+                    'Press Apply to proceed, Back to revisit '
+                    'your options, or Cancel to abort') % (format.replace("_",""), )
+                page_complete = True
+            else:
+                #Allow for exotic error: file is still not correct
+                self.check_fileselect(self.chooser, show=False)
+                if self.get_page_complete(self.chooser) :
+                    filename = Utils.get_unicode_path_from_file_chooser(self.chooser.get_filename())
+                    name = os.path.split(filename)[1]
+                    folder = os.path.split(filename)[0]
+                    confirm_text = _(
+                    'The data will be saved as follows:\n\n'
+                    'Format:\t%s\nName:\t%s\nFolder:\t%s\n\n'
+                    'Press Apply to proceed, Back to revisit '
+                    'your options, or Cancel to abort') % (format.replace("_",""), name, folder)
+                    page_complete = True
+                else :
+                    confirm_text = _(
+                        'The selected file and folder to save to '
+                        'cannot be created or found.\n\n'
+                        'Press Back to return and select a valid filename.'
+                        ) 
+                    page_complete = False
+            # Set the page_complete status
+            self.set_page_complete(page, page_complete)
+            # If it is ok, then look for alternate confirm_text
+            if (page_complete and
+                self.option_box_instance and 
+                hasattr(self.option_box_instance, "confirm_text")):
+                # Override message
+                confirm_text = self.option_box_instance.confirm_text
+            page.set_label(confirm_text)
                 
         elif self.get_page_type(page) ==  gtk.ASSISTANT_PAGE_SUMMARY :
             # The summary page
@@ -475,7 +520,7 @@ class ExportAssistant(gtk.Assistant, ManagedWindow.ManagedWindow) :
                 'The copy of your data has been '
                 'successfully saved. You may press Close button '
                 'now to continue.\n\n'
-                'Note: the database currently opened in your GRAMPS '
+                'Note: the database currently opened in your Gramps '
                 'window is NOT the file you have just saved. '
                 'Future editing of the currently opened database will '
                 'not alter the copy you have just made. ')
@@ -489,7 +534,7 @@ class ExportAssistant(gtk.Assistant, ManagedWindow.ManagedWindow) :
                 'Note: your currently opened database is safe. '
                 'It was only '
                 'a copy of your data that failed to save.')
-            self.labelsum.set_label(escape(conclusion_text))
+            self.labelsum.set_label(conclusion_text)
             self.set_page_title(page, conclusion_title)
             self.set_page_complete(page, True)
         else :
@@ -507,15 +552,15 @@ class ExportAssistant(gtk.Assistant, ManagedWindow.ManagedWindow) :
         
     def close(self, *obj) :
         #clean up ManagedWindow menu, then destroy window, bring forward parent
-        
+        gtk.Assistant.destroy(self)
         ManagedWindow.ManagedWindow.close(self,*obj)
 
     def get_intro_text(self):
-        return _('Under normal circumstances, GRAMPS does not require you '
+        return _('Under normal circumstances, Gramps does not require you '
                  'to directly save your changes. All changes you make are '
                  'immediately saved to the database.\n\n'
                  'This process will help you save a copy of your data '
-                 'in any of the several formats supported by GRAMPS. '
+                 'in any of the several formats supported by Gramps. '
                  'This can be used to make a copy of your data, backup '
                  'your data, or convert it to a format that will allow '
                  'you to transfer it to a different program.\n\n'
@@ -539,12 +584,12 @@ class ExportAssistant(gtk.Assistant, ManagedWindow.ManagedWindow) :
     def suggest_filename(self):
         """Prepare suggested filename and set it in the file chooser."""
         ix = self.get_selected_format_index()
-        ext = self.__exporters[ix].get_extension()
+        ext = self.map_exporters[ix].get_extension()
         
         # Suggested folder: try last export, then last import, then home.
-        default_dir = Config.get(Config.RECENT_EXPORT_DIR)
+        default_dir = config.get('paths.recent-export-dir')
         if len(default_dir)<=1:
-            default_dir = Config.get(Config.RECENT_IMPORT_DIR)
+            default_dir = config.get('paths.recent-import-dir')
         if len(default_dir)<=1:
             default_dir = const.USER_HOME
 
@@ -563,19 +608,24 @@ class ExportAssistant(gtk.Assistant, ManagedWindow.ManagedWindow) :
         Depending on the success status, set the text for the final page.
         
         """
-        filename = Utils.get_unicode_path(self.chooser.get_filename())
-        Config.set(Config.RECENT_EXPORT_DIR, os.path.split(filename)[0])
+        if (self.option_box_instance and 
+            hasattr(self.option_box_instance, "no_fileselect")):
+            filename = ""
+        else:
+            filename = Utils.get_unicode_path_from_file_chooser(self.chooser.get_filename())
+            config.set('paths.recent-export-dir', os.path.split(filename)[0])
         ix = self.get_selected_format_index()
-        Config.set(Config.RECENT_EXPORT_TYPE, ix)
-        export_function = self.__exporters[ix].get_export_function()
+        config.set('behavior.recent-export-type', ix)
+        export_function = self.map_exporters[ix].get_export_function()
         success = export_function(self.dbstate.db,
                                   filename,
+                                  ErrorDialog,
                                   self.option_box_instance,
                                   self.callback)
         return success
     
     def pre_save(self,page):
-        #as all is locked, show the page, which assistent normally only does
+        #as all is locked, show the page, which assistant normally only does
         # after prepare signal!
         self.writestarted = True
         page.set_child_visible(True)
