@@ -21,7 +21,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: _HasSource.py 14091 2010-01-18 04:42:17Z pez4brian $
+# $Id: _HasSource.py 14906 2010-03-22 17:17:24Z romjerome $
 
 #-------------------------------------------------------------------------
 #
@@ -43,5 +43,5 @@ from Filters.Rules._HasSourceBase import HasSourceBase
 class HasSource(HasSourceBase):
     """Families with sources"""
 
-    name        = _('Families with <count> source')
-    description = _("Matches families with a certain number of items in the source")
+    name        = _('Families with <count> sources')
+    description = _("Matches families with a certain number of sources connected to it")
