@@ -1,7 +1,8 @@
 #
 # Gramps - a GTK+/GNOME based genealogy program
 #
-# Copyright (C) 2002-2006  Donald N. Allingham
+# Copyright (C) 2002-2007  Donald N. Allingham
+# Copyright (C) 2007-2008  Brian G. Matherly
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,14 +19,12 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: __init__.py 6521 2006-05-03 01:02:54Z rshura $
+# $Id: __init__.py 16490 2011-01-28 09:03:32Z romjerome $
 
 """
 Package providing filter rules for GRAMPS.
 """
 
-__author__ = "Don Allingham"
-
 from _SearchFatherName import SearchFatherName
 from _SearchMotherName import SearchMotherName
 from _SearchChildName import SearchChildName
@@ -35,38 +34,54 @@ from _RegExpChildName import RegExpChildName
 
 from _HasRelType import HasRelType
 from _AllFamilies import AllFamilies
+from _HasGallery import HasGallery
 from _HasIdOf import HasIdOf
+from _HasLDS import HasLDS
 from _RegExpIdOf import RegExpIdOf
+from _HasNote import HasNote
 from _HasNoteRegexp import HasNoteRegexp
 from _HasNoteMatchingSubstringOf import HasNoteMatchingSubstringOf
+from _HasSource import HasSource
+from _HasReferenceCountOf import HasReferenceCountOf
 from _FamilyPrivate import FamilyPrivate
 from _HasAttribute import HasAttribute
 from _HasEvent import HasEvent
-from _HasMarkerOf import HasMarkerOf
+from _IsBookmarked import IsBookmarked
 from _MatchesFilter import MatchesFilter
+from _MatchesSourceConfidence import MatchesSourceConfidence
 from _FatherHasNameOf import FatherHasNameOf
 from _FatherHasIdOf import FatherHasIdOf
 from _MotherHasNameOf import MotherHasNameOf
 from _MotherHasIdOf import MotherHasIdOf
 from _ChildHasNameOf import ChildHasNameOf
 from _ChildHasIdOf import ChildHasIdOf
+from _ChangedSince import ChangedSince
+from _HasTag import HasTag
 
 editor_rule_list = [
     AllFamilies,
     HasRelType,
+    HasGallery,
     HasIdOf,
+    HasLDS,
+    HasNote,
     RegExpIdOf,
     HasNoteRegexp,
     HasNoteMatchingSubstringOf,
+    HasReferenceCountOf,
+    HasSource,
     FamilyPrivate,
     HasEvent,
     HasAttribute,
-    HasMarkerOf,
+    IsBookmarked,
     MatchesFilter,
+    MatchesSourceConfidence,
     FatherHasNameOf,
     FatherHasIdOf,
     MotherHasNameOf,
     MotherHasIdOf,
     ChildHasNameOf,
     ChildHasIdOf,
+    ChangedSince,
+    HasTag,
 ]
