@@ -19,7 +19,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: _MatchesRegexpOf.py 15596 2010-06-28 07:28:16Z ldnp $
+# $Id: _MatchesRegexpOf.py 17659 2011-06-02 08:27:36Z romjerome $
 
 #-------------------------------------------------------------------------
 #
@@ -43,8 +43,8 @@ class MatchesRegexpOf(Rule):
 
     labels      = [ _('Regular expression:')]
     name        = _('Notes containing <regular expression>')
-    description = _("Matches notes who contain text "
-                    "matching a regular expression")
+    description = _("Matches notes that contain text "
+                    "which matches a regular expression")
     category    = _('General filters')
 
     def __init__(self, list):
