@@ -19,7 +19,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: _MatchesSubstringOf.py 14091 2010-01-18 04:42:17Z pez4brian $
+# $Id: _MatchesSubstringOf.py 17659 2011-06-02 08:27:36Z romjerome $
 
 #-------------------------------------------------------------------------
 #
@@ -39,17 +39,17 @@ from Filters.Rules import Rule
 # "Events having notes that contain a substring"
 #-------------------------------------------------------------------------
 class MatchesSubstringOf(Rule):
-    """Notes having notes containing <subtring>"""
+    """Notes having notes containing <substring>"""
 
     labels      = [ _('Substring:')]
     name        = _('Notes containing <substring>')
-    description = _("Matches notes who contain text "
-                    "matching a substring")
+    description = _("Matches notes that contain text "
+                    "which matches a substring")
     category    = _('General filters')
 
     def apply(self, db, note):
         """ Apply the filter """
-        text = unicode(note.get())
+        text = note.get()
         if text.upper().find(self.list[0].upper()) != -1:
             return True
         return False
