@@ -41,7 +41,7 @@ from _MatchesFilter import MatchesFilter
 # HasCommonAncestorWithFilterMatch
 #
 #-------------------------------------------------------------------------
-class HasCommonAncestorWithFilterMatch(HasCommonAncestorWith,MatchesFilter):
+class HasCommonAncestorWithFilterMatch(HasCommonAncestorWith):
     """Rule that checks for a person that has a common ancestor with
     someone matching a filter"""
 
@@ -51,8 +51,8 @@ class HasCommonAncestorWithFilterMatch(HasCommonAncestorWith,MatchesFilter):
                     "with anybody matched by a filter")
     category    = _("Ancestral filters")
 
-    def __init__(self,list):
-        HasCommonAncestorWith.__init__(self,list)
+    def __init__(self, list):
+        HasCommonAncestorWith.__init__(self, list)
         self.ancestor_cache = {}
 
     def prepare(self, db):
@@ -64,13 +64,13 @@ class HasCommonAncestorWithFilterMatch(HasCommonAncestorWith,MatchesFilter):
         self.ancestor_cache = {}
         self.with_people = []
         filt = MatchesFilter(self.list)
-        filt.prepare(db)
+        filt.requestprepare(db)
         for handle in db.iter_person_handles():
             person = db.get_person_from_handle(handle)
-            if filt.apply (db, person):
+            if person and filt.apply(db, person):
                 #store all people in the filter so as to compare later
                 self.with_people.append(person.handle)
                 #fill list of ancestor of person if not present yet
                 if handle not in self.ancestor_cache:
                     self.add_ancs(db, person)
-        filt.reset()
+        filt.requestreset()
