@@ -18,14 +18,14 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: _IsLessThanNthGenerationDescendantOf.py 9912 2008-01-22 09:17:46Z acraphae $
+# $Id: _IsLessThanNthGenerationDescendantOf.py 15369 2010-05-09 13:43:50Z dsblank $
 
 #-------------------------------------------------------------------------
 #
 # Standard Python modules
 #
 #-------------------------------------------------------------------------
-from gettext import gettext as _
+from gen.ggettext import gettext as _
 
 #-------------------------------------------------------------------------
 #
@@ -52,29 +52,30 @@ class IsLessThanNthGenerationDescendantOf(Rule):
 
     def prepare(self,db):
         self.db = db
-        self.map = {}
+        self.map = set()
         try:
             root_person = db.get_person_from_gramps_id(self.list[0])
-            self.init_list(root_person,0)
+            self.init_list(root_person, 0)
         except:
             pass
 
     def reset(self):
-        self.map = {}
+        self.map.clear()
 
-    def apply(self,db,person):
-        return self.map.has_key(person.handle)
+    def apply(self, db, person):
+        return person.handle in self.map
 
     def init_list(self,person,gen):
         if not person:
             return
         if gen:
-            self.map[person.handle] = 1
+            self.map.add(person.handle)
             if gen >= int(self.list[1]):
                 return
 
         for fam_id in person.get_family_handle_list():
             fam = self.db.get_family_from_handle(fam_id)
-            for child_ref in fam.get_child_ref_list():
-                self.init_list(
-                    self.db.get_person_from_handle(child_ref.ref),gen+1)
+            if fam:
+                for child_ref in fam.get_child_ref_list():
+                    self.init_list(
+                        self.db.get_person_from_handle(child_ref.ref), gen+1)
