@@ -18,14 +18,14 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: _IsWitness.py 9912 2008-01-22 09:17:46Z acraphae $
+# $Id: _IsWitness.py 15369 2010-05-09 13:43:50Z dsblank $
 
 #-------------------------------------------------------------------------
 #
 # Standard Python modules
 #
 #-------------------------------------------------------------------------
-from gettext import gettext as _
+from gen.ggettext import gettext as _
 
 #-------------------------------------------------------------------------
 #
@@ -48,7 +48,7 @@ class IsWitness(Rule):
 
     def apply(self,db,person):
         for event_ref in person.event_ref_list:
-            if event_ref.role == EventRoleType.WITNESS:
+            if event_ref and event_ref.role == EventRoleType.WITNESS:
                 # This is the witness.
                 # If event type was given, then check it.
                 if self.list[0]:
