@@ -29,12 +29,12 @@ from _Disconnected import Disconnected
 from _Everyone import Everyone
 from _FamilyWithIncompleteEvent import FamilyWithIncompleteEvent
 from _HasAddress import HasAddress
+from _HasAlternateName import HasAlternateName
 from _HasAssociation import HasAssociation
 from _HasAttribute import HasAttribute
 from _HasBirth import HasBirth
 from _HasCommonAncestorWith import HasCommonAncestorWith
 from _HasCommonAncestorWithFilterMatch import HasCommonAncestorWithFilterMatch
-from _HasCompleteRecord import HasCompleteRecord
 from _HasDeath import HasDeath
 from _HasEvent import HasEvent
 from _HasFamilyAttribute import HasFamilyAttribute
@@ -43,16 +43,19 @@ from _HasGallery import HavePhotos
 from _HasIdOf import HasIdOf
 from _HasLDS import HasLDS
 from _HasNameOf import HasNameOf
+from _HasNameOriginType import HasNameOriginType
+from _HasNameType import HasNameType
+from _HasNickname import HasNickname
 from _HasNote import HasNote
 from _HasNoteMatchingSubstringOf import HasNoteMatchingSubstringOf
 from _HasNoteRegexp import HasNoteRegexp
 from _HasRelationship import HasRelationship
 from _HasSource import HasSource
 from _HasSourceOf import HasSourceOf
+from _HasTag import HasTag
 from _HasTextMatchingRegexpOf import HasTextMatchingRegexpOf
 from _HasTextMatchingSubstringOf import HasTextMatchingSubstringOf
 from _HasUnknownGender import HasUnknownGender
-from _HasMarkerOf import HasMarkerOf
 from _HaveAltFamilies import HaveAltFamilies
 from _HaveChildren import HaveChildren
 from _IncompleteNames import IncompleteNames
@@ -64,6 +67,7 @@ from _IsDefaultPerson import IsDefaultPerson
 from _IsDescendantFamilyOf import IsDescendantFamilyOf
 from _IsDescendantOf import IsDescendantOf
 from _IsDescendantOfFilterMatch import IsDescendantOfFilterMatch
+from _IsDuplicatedAncestorOf import IsDuplicatedAncestorOf
 from _IsFemale import IsFemale
 from _IsLessThanNthGenerationAncestorOf import \
      IsLessThanNthGenerationAncestorOf
@@ -84,10 +88,12 @@ from _IsSpouseOfFilterMatch import IsSpouseOfFilterMatch
 from _IsWitness import IsWitness
 from _MatchesFilter import MatchesFilter
 from _MatchesEventFilter import MatchesEventFilter
+from _MatchesSourceConfidence import MatchesSourceConfidence
 from _MissingParent import MissingParent
 from _MultipleMarriages import MultipleMarriages
 from _NeverMarried import NeverMarried
 from _NoBirthdate import NoBirthdate
+from _NoDeathdate import NoDeathdate
 from _PeoplePrivate import PeoplePrivate
 from _PersonWithIncompleteEvent import PersonWithIncompleteEvent
 from _ProbablyAlive import ProbablyAlive
@@ -113,22 +119,25 @@ editor_rule_list = [
     IsMale,
     IsDefaultPerson,
     IsBookmarked,
+    HasAlternateName,
     HasAddress,
     HasAssociation,
     HasIdOf,
     HasLDS,
     HasNameOf,
+    HasNameOriginType,
+    HasNameType,
+    HasNickname,
     HasRelationship,
     HasDeath,
     HasBirth,
-    HasCompleteRecord,
     HasEvent,
     HasFamilyEvent,
     HasAttribute,
     HasFamilyAttribute,
+    HasTag,
     HasSource,
     HasSourceOf,
-    HasMarkerOf,
     HaveAltFamilies,
     HavePhotos,
     HaveChildren,
@@ -136,6 +145,7 @@ editor_rule_list = [
     NeverMarried,
     MultipleMarriages,
     NoBirthdate,
+    NoDeathdate,
     PersonWithIncompleteEvent,
     FamilyWithIncompleteEvent,
     ProbablyAlive,
@@ -145,6 +155,7 @@ editor_rule_list = [
     IsDescendantFamilyOf,
     IsLessThanNthGenerationAncestorOfDefaultPerson,
     IsDescendantOfFilterMatch,
+    IsDuplicatedAncestorOf,
     IsLessThanNthGenerationDescendantOf,
     IsMoreThanNthGenerationDescendantOf,
     IsAncestorOf,
@@ -156,6 +167,7 @@ editor_rule_list = [
     HasCommonAncestorWithFilterMatch,
     MatchesFilter,
     MatchesEventFilter,
+    MatchesSourceConfidence,
     MissingParent,
     IsChildOfFilterMatch,
     IsParentOfFilterMatch,
