@@ -3,7 +3,7 @@
 # Standard Python modules
 #
 #-------------------------------------------------------------------------
-from gettext import gettext as _
+from gen.ggettext import gettext as _
 
 #-------------------------------------------------------------------------
 #
@@ -20,7 +20,8 @@ from Filters.Rules._ChangedSinceBase import ChangedSinceBase
 class ChangedSince(ChangedSinceBase):
     """Rules that checks for places changed since a specific time."""
 
-    name    = ('Places changed after <date time>')
+    labels      = [ _('Changed after:'), _('but before:') ]
+    name    = _('Places changed after <date time>')
     description = _("Matches place records changed after a specified "
                     "date-time (yyyy-mm-dd hh:mm:ss) or in the range, if a second "
-                    "date-time is given")
+                    "date-time is given.")
