@@ -18,22 +18,23 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: __init__.py 6521 2006-05-03 01:02:54Z rshura $
+# $Id: __init__.py 17411 2011-05-05 08:23:28Z romjerome $
 
 """
 Package providing filter rules for GRAMPS.
 """
 
-__author__ = "Don Allingham"
-
 from _AllRepos import AllRepos
 from _HasIdOf import HasIdOf
 from _RegExpIdOf import RegExpIdOf
 from _HasNoteRegexp import HasNoteRegexp
 from _HasNoteMatchingSubstringOf import HasNoteMatchingSubstringOf
+from _HasReferenceCountOf import HasReferenceCountOf
 from _RepoPrivate import RepoPrivate
 from _MatchesFilter import MatchesFilter
 from _HasRepo import HasRepo
+from _ChangedSince import ChangedSince
+from _MatchesNameSubstringOf import MatchesNameSubstringOf
 
 editor_rule_list = [
     AllRepos,
@@ -41,6 +42,9 @@ editor_rule_list = [
     RegExpIdOf,
     HasNoteRegexp,
     HasNoteMatchingSubstringOf,
+    HasReferenceCountOf,
     RepoPrivate,
     MatchesFilter,
+    ChangedSince,
+    MatchesNameSubstringOf
 ]
