@@ -19,7 +19,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: _Rule.py 14240 2010-02-05 23:56:36Z dsblank $
+# $Id: _Rule.py 16205 2010-11-18 12:22:06Z dsblank $
 
 #-------------------------------------------------------------------------
 #
@@ -28,6 +28,14 @@
 #-------------------------------------------------------------------------
 from gen.ggettext import gettext as _
 
+#-------------------------------------------------------------------------
+#
+# enable logging for error handling
+#
+#-------------------------------------------------------------------------
+import logging
+log = logging.getLogger(".")
+
 #-------------------------------------------------------------------------
 #
 # Rule
@@ -43,22 +51,55 @@ class Rule(object):
 
     def __init__(self, arg):
         self.set_list(arg)
+        self.nrprepare = 0
 
     def is_empty(self):
         return False
     
+    def requestprepare(self, db):
+        """
+        Request that the prepare method of the rule is executed if needed
+        
+        Special: Custom Filters have fixed values, so only one instance needs to
+        exists during a search. It is stored in a FilterStore, and initialized
+        once.
+        As filters are can be grouped in a group
+        filter, we request a prepare. Only the first time prepare will be
+        called
+        """
+        if self.nrprepare == 0:
+            self.prepare(db)
+        self.nrprepare += 1
+
     def prepare(self, db):
+        """prepare so the rule can be executed efficiently"""
         pass
 
+    def requestreset(self):
+        """
+        Request that the reset method of the rule is executed if possible
+        
+        Special: Custom Filters have fixed values, so only one instance needs to
+        exists during a search. It is stored in a FilterStore, and initialized
+        once.
+        As filters are can be grouped in a group
+        filter, we request a reset. Only the last time reset will be
+        called
+        """
+        self.nrprepare -= 1
+        if self.nrprepare == 0:
+            self.reset()
+
     def reset(self):
+        """remove no longer needed memory"""
         pass
  
     def set_list(self, arg):
         assert isinstance(arg, list) or arg is None, "Argument is not a list"
-        assert len(arg) == len(self.labels), \
-               "ERROR: Number of arguments does not match number of labels.\n"\
-               "       list:   %s\n"\
-               "       labels: %s" % (arg,self.labels)
+        if len(arg) != len(self.labels):
+               log.warning(("Number of arguments does not match number of labels.\n" +
+                            "   list:   %s\n" +
+                            "   labels: %s") % (arg,self.labels))
         self.list = arg
 
     def values(self):
