@@ -3,6 +3,7 @@
 #
 # Copyright (C) 2002-2006  Donald N. Allingham
 # Copyright (C) 2008       Gary Burton
+# Copyright (C) 2010       Nick Hall
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,14 +20,14 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: _PlaceSidebarFilter.py 10898 2008-07-21 20:39:19Z gburto01 $
+# $Id: _PlaceSidebarFilter.py 16612 2011-02-12 19:52:27Z nick-h $
 
 #-------------------------------------------------------------------------
 #
 # Python modules
 #
 #-------------------------------------------------------------------------
-from gettext import gettext as _
+from gen.ggettext import gettext as _
 
 #-------------------------------------------------------------------------
 #
@@ -60,17 +61,18 @@ class PlaceSidebarFilter(SidebarFilter):
         self.filter_id = gtk.Entry()
         self.filter_title = gtk.Entry()       
         self.filter_street = gtk.Entry()
-        self.filter_parish = gtk.Entry()
-        self.filter_zip = gtk.Entry()
+        self.filter_locality = gtk.Entry()
         self.filter_city = gtk.Entry()
         self.filter_county = gtk.Entry()
         self.filter_state = gtk.Entry()
         self.filter_country = gtk.Entry()
+        self.filter_zip = gtk.Entry()
+        self.filter_parish = gtk.Entry()
         self.filter_note = gtk.Entry()
         self.filter_regex = gtk.CheckButton(_('Use regular expressions'))
         self.generic = gtk.ComboBox()
 
-        SidebarFilter.__init__(self, dbstate, uistate)
+        SidebarFilter.__init__(self, dbstate, uistate, "Place")
 
     def create_widget(self):
         cell = gtk.CellRendererText()
@@ -83,26 +85,28 @@ class PlaceSidebarFilter(SidebarFilter):
         self.add_text_entry(_('ID'), self.filter_id)
         self.add_text_entry(_('Place Name'), self.filter_title)
         self.add_text_entry(_('Street'), self.filter_street)
-        self.add_text_entry(_('Church parish'), self.filter_parish)
-        self.add_text_entry(_('Zip/Postal code'), self.filter_zip)
+        self.add_text_entry(_('Locality'), self.filter_locality)
         self.add_text_entry(_('City'), self.filter_city)
         self.add_text_entry(_('County'), self.filter_county)
         self.add_text_entry(_('State'), self.filter_state)
         self.add_text_entry(_('Country'), self.filter_country)
+        self.add_text_entry(_('ZIP/Postal code'), self.filter_zip)
+        self.add_text_entry(_('Church parish'), self.filter_parish)
         self.add_text_entry(_('Note'), self.filter_note)
-        self.add_entry(_('Custom filter'), self.generic)
+        self.add_filter_entry(_('Custom filter'), self.generic)
         self.add_entry(None, self.filter_regex)
 
     def clear(self, obj):
         self.filter_id.set_text('')
         self.filter_title.set_text('')
         self.filter_street.set_text('')
-        self.filter_parish.set_text('')
-        self.filter_zip.set_text('')
+        self.filter_locality.set_text('')
         self.filter_city.set_text('')
         self.filter_county.set_text('')
         self.filter_state.set_text('')
         self.filter_country.set_text('')
+        self.filter_zip.set_text('')
+        self.filter_parish.set_text('')
         self.filter_note.set_text('')
         self.generic.set_active(0)
 
@@ -110,18 +114,19 @@ class PlaceSidebarFilter(SidebarFilter):
         gid = unicode(self.filter_id.get_text()).strip()
         title = unicode(self.filter_title.get_text()).strip()
         street = unicode(self.filter_street.get_text()).strip()
-        parish = unicode(self.filter_parish.get_text()).strip()
-        zipc = unicode(self.filter_zip.get_text()).strip()
+        locality = unicode(self.filter_locality.get_text()).strip()
         city = unicode(self.filter_city.get_text()).strip()
         county = unicode(self.filter_county.get_text()).strip()
         state = unicode(self.filter_state.get_text()).strip()
         country = unicode(self.filter_country.get_text()).strip()
+        zipc = unicode(self.filter_zip.get_text()).strip()
+        parish = unicode(self.filter_parish.get_text()).strip()
         note = unicode(self.filter_note.get_text()).strip()
         regex = self.filter_regex.get_active()
         gen = self.generic.get_active() > 0
 
-        empty = not (gid or title or street or parish or zipc or city or county
-                     or state or country or note or regex or gen)
+        empty = not (gid or title or street or locality or city or county or
+                     state or country or zipc or parish or note or regex or gen)
         if empty:
             generic_filter = None
         else:
@@ -133,7 +138,8 @@ class PlaceSidebarFilter(SidebarFilter):
                     rule = HasIdOf([gid])
                 generic_filter.add_rule(rule)
 
-            rule = HasPlace([title, street, parish, zipc, city, county, state, country])
+            rule = HasPlace([title, street, locality, city, county, state,
+                             country, zipc, parish])
             generic_filter.add_rule(rule)
                 
             if note:
@@ -146,7 +152,7 @@ class PlaceSidebarFilter(SidebarFilter):
             if self.generic.get_active() != 0:
                 model = self.generic.get_model()
                 node = self.generic.get_active_iter()
-                obj = model.get_value(node, 0)
+                obj = unicode(model.get_value(node, 0))
                 rule = MatchesFilter([obj])
                 generic_filter.add_rule(rule)
 
