@@ -18,7 +18,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: _RepoSidebarFilter.py 14248 2010-02-06 16:08:10Z dsblank $
+# $Id: _RepoSidebarFilter.py 16612 2011-02-12 19:52:27Z nick-h $
 
 #-------------------------------------------------------------------------
 #
@@ -141,7 +141,7 @@ class RepoSidebarFilter(SidebarFilter):
         if self.generic.get_active() != 0:
             model = self.generic.get_model()
             node = self.generic.get_active_iter()
-            obj = model.get_value(node, 0)
+            obj = unicode(model.get_value(node, 0))
             rule = MatchesFilter([obj])
             generic_filter.add_rule(rule)
 
