@@ -18,14 +18,14 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: _SourceSidebarFilter.py 10338 2008-03-19 11:26:08Z bmcage $
+# $Id: _SourceSidebarFilter.py 16612 2011-02-12 19:52:27Z nick-h $
 
 #-------------------------------------------------------------------------
 #
 # Python modules
 #
 #-------------------------------------------------------------------------
-from gettext import gettext as _
+from gen.ggettext import gettext as _
 
 #-------------------------------------------------------------------------
 #
@@ -65,7 +65,7 @@ class SourceSidebarFilter(SidebarFilter):
 
         self.generic = gtk.ComboBox()
 
-        SidebarFilter.__init__(self, dbstate, uistate)
+        SidebarFilter.__init__(self, dbstate, uistate, "Source")
 
     def create_widget(self):
         cell = gtk.CellRendererText()
@@ -80,7 +80,7 @@ class SourceSidebarFilter(SidebarFilter):
         self.add_text_entry(_('Author'), self.filter_author)
         self.add_text_entry(_('Publication'), self.filter_pub)
         self.add_text_entry(_('Note'), self.filter_note)
-        self.add_entry(_('Custom filter'), self.generic)
+        self.add_filter_entry(_('Custom filter'), self.generic)
         self.add_entry(None, self.filter_regex)
 
     def clear(self, obj):
@@ -125,7 +125,7 @@ class SourceSidebarFilter(SidebarFilter):
         if self.generic.get_active() != 0:
             model = self.generic.get_model()
             node = self.generic.get_active_iter()
-            obj = model.get_value(node, 0)
+            obj = unicode(model.get_value(node, 0))
             rule = MatchesFilter([obj])
             generic_filter.add_rule(rule)
 
