@@ -61,8 +61,10 @@ class GenericFilter(object):
             return False
 
     def is_empty(self):
-        return len(self.flist) == 0 or \
-              (len(self.flist) == 1 and self.flist[0].is_empty())
+        return (len(self.flist) == 0 or 
+                (len(self.flist) == 1 and 
+                 ((self.flist[0].is_empty() and not self.invert) or
+                  (not self.flist[0].is_empty() and self.invert))))
 
     def set_logical_op(self, val):
         if val in GenericFilter.logical_functions:
@@ -216,7 +218,7 @@ class GenericFilter(object):
         If progress given, it will be used to indicate progress of the
         Filtering
         
-        If typleind is given, id_list is supposed to consist of a list of 
+        If tupleind is given, id_list is supposed to consist of a list of 
         tuples, with the handle being index tupleind. So 
         handle_0 = id_list[0][tupleind]
         
@@ -227,10 +229,10 @@ class GenericFilter(object):
         """
         m = self.get_check_func()
         for rule in self.flist:
-            rule.prepare(db)
+            rule.requestprepare(db)
         res = m(db, id_list, progress, tupleind)
         for rule in self.flist:
-            rule.reset()
+            rule.requestreset()
         return res
 
 class GenericFamilyFilter(GenericFilter):
