@@ -19,7 +19,7 @@
 #
 import gobject
 
-# $Id: _SearchBar.py 14986 2010-04-01 13:24:48Z dsblank $
+# $Id: _SearchBar.py 16160 2010-11-08 15:50:51Z nick-h $
 
 """
 Package providing filtering framework for GRAMPS.
@@ -48,6 +48,7 @@ class SearchBar(object):
         self.dbstate = dbstate
         self.uistate = uistate
         self.apply_text = ''
+        self.visible = False
 
         self.filterbar = gtk.HBox()
         self.filter_text = gtk.Entry()
@@ -57,7 +58,15 @@ class SearchBar(object):
         self.filter_model = gtk.ListStore(gobject.TYPE_STRING, 
                                           gobject.TYPE_INT, 
                                           gobject.TYPE_BOOLEAN)
-        
+
+    def destroy(self):
+        """Unset all things that can block garbage collection.
+        """
+        self.on_apply_callback = None
+        self.apply_done_callback = None
+        self.dbstate = None
+        self.uistate =  None
+
     def build( self):
         self.filterbar.set_spacing(4)
         self.filter_list.connect('changed', self.filter_changed)
@@ -159,6 +168,11 @@ class SearchBar(object):
 
     def show(self):
         self.filterbar.show()
+        self.visible = True
 
     def hide(self):
         self.filterbar.hide()
+        self.visible = False
+
+    def is_visible(self):
+        return self.visible
