@@ -18,7 +18,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
-# $Id: _GrampsLocale.py 16386 2011-01-15 09:43:34Z romjerome $
+# $Id: _GrampsLocale.py 17860 2011-06-28 23:07:09Z jralls $
 
 import locale
 import constfunc
@@ -132,7 +132,7 @@ try:
 except:
     import time
 
-    if constfunc.win():
+    if constfunc.win() or constfunc.mac():
         codeset = locale.getlocale()[1]
     else:
         codeset = locale.getpreferredencoding()
