@@ -1,14 +1,21 @@
-from gettext import gettext as _
+from gen.ggettext import gettext as _
 import Assistant
 import const
 import gtk
-import sys, os,bsddb
+import pygtk
+import gobject
+import cairo
+import sys, os
 
+import config
+if config.get('preferences.use-bsddb3'):
+    import bsddb3 as bsddb
+else:
+    import bsddb
 
+class ErrorReportAssistant(object):
 
-class ErrorReportAssistant:
-
-    def __init__(self,error_detail,rotate_handler):
+    def __init__(self,error_detail,rotate_handler, ownthread=False):
         self._error_detail = error_detail
         self._rotate_handler = rotate_handler
 
@@ -18,6 +25,9 @@ class ErrorReportAssistant:
         self._final_report_text_buffer = None
         
         self.w = Assistant.Assistant(None,None,self.complete)
+        #connect our extra close to close by x, and close by cancel click
+        self.w.window.connect('delete-event', self.close)
+        self.w.cancel.connect('clicked', self.close)      
 
         self.w.add_text_page(
             _('Report a bug'),
@@ -45,20 +55,30 @@ class ErrorReportAssistant:
 
         self.w.add_text_page(
             _('Complete'),
-            _('GRAMPS is an Open Source project. Its success '
+            _('Gramps is an Open Source project. Its success '
               'depends on its users. User feedback is important. '
               'Thank you for taking the time to submit a bug report.'))
 
         self.w.connect('page-changed',self.on_page_changed)
 
         self.w.show()
+        
+        self.ownthread = ownthread
+        if self.ownthread:
+            gtk.main()
 
+    def close(self, *obj):
+        if self.ownthread:
+            gtk.main_quit()
+            
     def on_page_changed(self, obj,page,data=None):
-        if self.cb.has_key(page):
+        if page in self.cb:
             self.cb[page]()
             
     def complete(self):
-        pass
+        if self.ownthread:
+            #stop the thread we started
+            gtk.main_quit()
 
     def _copy_to_clipboard(self, obj=None):
         clipboard = gtk.Clipboard()
@@ -75,7 +95,8 @@ class ErrorReportAssistant:
 
     def _start_email_client(self, obj=None):
         import GrampsDisplay
-        GrampsDisplay.url('mailto:gramps-bugs@lists.sourceforge.net?subject="bug report"&body="%s"' \
+        GrampsDisplay.url('mailto:gramps-bugs@lists.sourceforge.net?subject='
+                          '"bug report"&body="%s"' \
                           % self._final_report_text_buffer.get_text(
                                self._final_report_text_buffer.get_start_iter(),
                                self._final_report_text_buffer.get_end_iter()))
@@ -97,18 +118,27 @@ class ErrorReportAssistant:
                "Gramps version: %s \n"\
                "LANG: %s\n"\
                "OS: %s\n"\
-               "Distribution: %s\n"\
+               "Distribution: %s\n\n"\
+               "GTK version    : %s\n"\
+               "pygtk version  : %s\n"\
+               "gobject version: %s\n"\
+               "cairo version  : %s"\
                % (str(sys.version).replace('\n',''),
-                  str(bsddb.__version__),
+                  str(bsddb.__version__) + " " + str(bsddb.db.version()),
                   str(const.VERSION),
                   os.environ.get('LANG',''),
                   operatingsystem,
-                  distribution)
+                  distribution,
+                  gtk.gtk_version,
+                  gtk.pygtk_version,
+                  gobject.pygobject_version,
+                  cairo.version_info)
 
     def _reset_error_details_text_buffer(self, obj=None):
         self._error_details_text_buffer.set_text(
-	    "\n".join(self._rotate_handler.get_formatted_log(self._error_detail.get_record())) +
-	    self._error_detail.get_formatted_log())
+            "\n".join(self._rotate_handler.get_formatted_log(
+                        self._error_detail.get_record()))
+            + self._error_detail.get_formatted_log())
 
     def _clear_error_details_text_buffer(self, obj=None):
         self._error_details_text_buffer.delete(
@@ -372,7 +402,7 @@ class ErrorReportAssistant:
 #		side_label = gtk.Label(_("This is the completed bug report. The next page "\
 #                                 "of the assistant will help you to send the report "\
 #                                 "to the bug report mailing list."))
-		
+
         side_label = gtk.Label(_("This is the completed bug report. The next page "\
                                  "of the assistant will help you to file a bug "\
                                  " on the Gramps bug tracking system website."))
@@ -484,10 +514,10 @@ class ErrorReportAssistant:
         outer_frame.get_label_widget().set_use_markup(True)
 
         outer_frame.add(inner_align)
-		
+
 #		side_label = gtk.Label(_("This is the final step. Use the buttons on this "
 #                                 "page to transfer the bug report to your email client."))
-		
+
         side_label = gtk.Label(_("This is the final step. Use the buttons on this "
                                  "page to start a web browser and file a bug "
                                  "report on the Gramps bug tracking system."))
