@@ -38,6 +38,13 @@ import tempfile
 #
 #-------------------------------------------------------------------------
 
+#-------------------------------------------------------------------------
+#
+# Gramps modules
+#
+#-------------------------------------------------------------------------
+import Utils
+
 #-------------------------------------------------------------------------
 #
 # resize_to_jpeg
@@ -110,6 +117,8 @@ def resize_to_jpeg_buffer(source, width, height):
     filed, dest = tempfile.mkstemp()
     img = gtk.gdk.pixbuf_new_from_file(source)
     scaled = img.scale_simple(int(width), int(height), gtk.gdk.INTERP_BILINEAR)
+    os.close(filed)
+    dest = Utils.get_unicode_path_from_env_var(dest)
     scaled.save(dest, 'jpeg')
     ofile = open(dest, mode='rb')
     data = ofile.read()
