@@ -18,11 +18,16 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: __init__.py 6169 2006-03-18 01:30:23Z dallingham $
+# $Id: __init__.py 15645 2010-07-22 02:16:32Z dsblank $
 
 """
 """
 
-from _MergePerson import *
-from _MergePlace import *
-from _MergeSource import *
+from mergeperson import *
+from mergefamily import *
+from mergeevent import *
+from mergeplace import *
+from mergesource import *
+from mergerepository import *
+from mergemedia import *
+from mergenote import *
