@@ -2,6 +2,7 @@
 # Gramps - a GTK+/GNOME based genealogy program
 #
 # Copyright (C) 2007  Donald N. Allingham
+# Copyright (C) 2010       Jakim Friant
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -21,18 +22,25 @@
 """
 Provide a simplified database access interface to the GRAMPS database.
 """
-
+from __future__ import with_statement
 from types import NoneType
 
 import gen.lib
 import DateHandler
 import Utils
+import gen.utils
+from gen.plug.report.utils import place_name
 
-from BasicUtils import name_displayer
-from ReportBase import ReportUtils
+from gen.display.name import displayer as name_displayer
 from gen.lib import EventType
-import Config
+import config
+from gen.ggettext import gettext as _
 
+#-------------------------------------------------------------------------
+#
+# Local functions
+#
+#-------------------------------------------------------------------------
 class SimpleAccess(object):
     """
     Provide a simplified database access system. This system has been designed to
@@ -92,6 +100,8 @@ class SimpleAccess(object):
          Child         :  Garner, Bertha P.
          Child         :  Garner, Elizabeth
          Event         :  Burial 7/1/1911 Twin Falls, ID
+
+    One can also alternatively supply a handle rather than a person.
     """
 
     def __init__(self, dbase):
@@ -99,7 +109,7 @@ class SimpleAccess(object):
         Initialize the SimpleAccess object with the database that will be used.
         
         @param dbase: GRAMPS database object
-        @type dbase: GrampsDbBase
+        @type dbase: DbBase
         """
         self.dbase = dbase
 
@@ -112,6 +122,8 @@ class SimpleAccess(object):
         @return: Returns the name of the person based of the program preferences
         @rtype: unicode
         """
+        if type(person) in [str, unicode]: 
+            person = self.dbase.get_person_from_handle(person)
         assert(isinstance(person, (gen.lib.Person, NoneType)))
         if person:
             return name_displayer.display(person)
@@ -127,10 +139,12 @@ class SimpleAccess(object):
         @return: Returns the name of the person based of the program preferences
         @rtype: unicode
         """
+        if type(person) in [str, unicode]: 
+            person = self.dbase.get_person_from_handle(person)
         assert(isinstance(person, (gen.lib.Person, NoneType)))
         if person:
             surname = person.get_primary_name().get_surname()
-            return surname or Config.get(Config.NO_SURNAME_TEXT)
+            return surname or config.get('preferences.no-surname-text')
         else:
             return u''
         
@@ -143,6 +157,8 @@ class SimpleAccess(object):
         @return: Returns the first name of the person based of the program preferences
         @rtype: unicode
         """
+        if type(person) in [str, unicode]: 
+            person = self.dbase.get_person_from_handle(person)
         assert(isinstance(person, (gen.lib.Person, NoneType)))
         if person:
             return person.get_primary_name().get_first_name()
@@ -158,7 +174,6 @@ class SimpleAccess(object):
         @return: Returns the GRAMPS Id value of the person or family
         @rtype: unicode
         """
-        assert(isinstance(obj, (gen.lib.Person, gen.lib.Family, NoneType)))
         if obj:
             return obj.get_gramps_id()
         else:
@@ -173,6 +188,8 @@ class SimpleAccess(object):
         @return: Returns a string indentifying the person's gender
         @rtype: unicode
         """
+        if type(person) in [str, unicode]: 
+            person = self.dbase.get_person_from_handle(person)
         assert(isinstance(person, (gen.lib.Person, NoneType)))
         if person:
             return Utils.gender[person.get_gender()]
@@ -288,7 +305,7 @@ class SimpleAccess(object):
                 if event_handle:
                     event = self.dbase.get_event_from_handle(event_handle)
                     place_handle = event.get_place_handle()
-                    return ReportUtils.place_name(self.dbase, place_handle)
+                    return place_name(self.dbase, place_handle)
         return u''
 
     def spouse(self, person):
@@ -300,6 +317,8 @@ class SimpleAccess(object):
         @return: The spouse identified as the person's primary spouse
         @rtype: L{gen.lib.Person}
         """
+        if type(person) in [str, unicode]: 
+            person = self.dbase.get_person_from_handle(person)
         assert(isinstance(person, (gen.lib.Person, NoneType)))
 
         if person:
@@ -326,6 +345,8 @@ class SimpleAccess(object):
         his/per primary spouse.
         @rtype: unicode
         """
+        if type(person) in [str, unicode]: 
+            person = self.dbase.get_person_from_handle(person)
         assert(isinstance(person, (gen.lib.Person, NoneType)))
 
         if person:
@@ -348,6 +369,8 @@ class SimpleAccess(object):
         where married.
         @rtype: unicode
         """
+        if type(person) in [str, unicode]: 
+            person = self.dbase.get_person_from_handle(person)
         assert(isinstance(person, (gen.lib.Person, NoneType)))
 
         if person:
@@ -364,7 +387,7 @@ class SimpleAccess(object):
                                    if event.type == EventType.MARRIAGE ]
                         if events:
                             place_handle = events[0].get_place_handle()
-                            return ReportUtils.place_name(self.dbase, place_handle)
+                            return place_name(self.dbase, place_handle)
         return u''
 
     def marriage_date(self, person):
@@ -378,6 +401,8 @@ class SimpleAccess(object):
         where married.
         @rtype: unicode
         """
+        if type(person) in [str, unicode]: 
+            person = self.dbase.get_person_from_handle(person)
         assert(isinstance(person, (gen.lib.Person, NoneType)))
 
         if person:
@@ -467,6 +492,8 @@ class SimpleAccess(object):
         @return: Returns a string indicating the date when the person's birth.
         @rtype: unicode
         """
+        if type(person) in [str, unicode]: 
+            person = self.dbase.get_person_from_handle(person)
         return self.__event_date(person, gen.lib.Person.get_birth_ref)
 
     def birth_date_obj(self, person):
@@ -478,8 +505,31 @@ class SimpleAccess(object):
         @return: Returns the date when the person's birth.
         @rtype: L{gen.lib.Date}
         """
+        if type(person) in [str, unicode]: 
+            person = self.dbase.get_person_from_handle(person)
         return self.__event_date_obj(person, gen.lib.Person.get_birth_ref)
 
+
+    def birth_or_fallback(self, person, get_event=False):
+        """
+        Return the date of the person's birth or fallback event.
+
+        @param person: Person object
+        @type person: L{gen.lib.Person}
+        @return: Returns the date when the person's birth or fallback.
+        @rtype: L{gen.lib.Date}
+        """
+        if type(person) in [str, unicode]: 
+            person = self.dbase.get_person_from_handle(person)
+        event = gen.utils.get_birth_or_fallback(self.dbase, 
+                                                person, "<i>%s</i>")
+        if get_event:
+            return event
+        elif event:
+            return event.date
+        else:
+            return gen.lib.Date()
+
     def birth_place(self, person):
         """
         Return a string indicating the place of the person's birth.
@@ -489,6 +539,8 @@ class SimpleAccess(object):
         @return: Returns a string indicating the place of the person's birth.
         @rtype: unicode
         """
+        if type(person) in [str, unicode]: 
+            person = self.dbase.get_person_from_handle(person)
         return self.__event_place(person, gen.lib.Person.get_birth_ref)
 
     def death_date(self, person):
@@ -500,6 +552,8 @@ class SimpleAccess(object):
         @return: Returns a string indicating the date when the person's death.
         @rtype: unicode
         """
+        if type(person) in [str, unicode]: 
+            person = self.dbase.get_person_from_handle(person)
         return self.__event_date(person, gen.lib.Person.get_death_ref)
 
     def death_date_obj(self, person):
@@ -511,8 +565,30 @@ class SimpleAccess(object):
         @return: Returns the date when the person's death.
         @rtype: L{gen.lib.Date}
         """
+        if type(person) in [str, unicode]: 
+            person = self.dbase.get_person_from_handle(person)
         return self.__event_date_obj(person, gen.lib.Person.get_death_ref)
 
+    def death_or_fallback(self, person, get_event=False):
+        """
+        Return the date of the person's death or fallback event.
+
+        @param person: Person object
+        @type person: L{gen.lib.Person}
+        @return: Returns the date of the person's death or fallback.
+        @rtype: L{gen.lib.Date}
+        """
+        if type(person) in [str, unicode]: 
+            person = self.dbase.get_person_from_handle(person)
+        event = gen.utils.get_death_or_fallback(self.dbase, 
+                                                person, "<i>%s</i>")
+        if get_event:
+            return event
+        elif event:
+            return event.date
+        else:
+            return gen.lib.Date()
+
     def death_place(self, person):
         """
         Return a string indicating the place of the person's death.
@@ -522,6 +598,8 @@ class SimpleAccess(object):
         @return: Returns a string indicating the place of the person's death.
         @rtype: unicode
         """
+        if type(person) in [str, unicode]: 
+            person = self.dbase.get_person_from_handle(person)
         return self.__event_place(person, gen.lib.Person.get_death_ref)
 
     def event_place(self, event):
@@ -537,7 +615,7 @@ class SimpleAccess(object):
 
         if event:
             place_handle = event.get_place_handle()
-            return ReportUtils.place_name(self.dbase, place_handle)
+            return place_name(self.dbase, place_handle)
         else:
             return u''
 
@@ -581,7 +659,8 @@ class SimpleAccess(object):
         @rtype: unicode
         """
         assert(isinstance(event, (gen.lib.Event, NoneType)))
-        return event.get_date_object()
+        if event:
+            return event.get_date_object()
 
     def event_type(self, event):
         """
@@ -606,9 +685,9 @@ class SimpleAccess(object):
         @param obj: Person or Family
         @type obj: L{gen.lib.Person} or L{gen.lib.Family}
         @param restrict: Optional list of strings that will limit the types
-        of events to those of the specfied types.
+        of events to those of the specified types.
         @type restrict: list
-        @return: list of events assocated with the object
+        @return: list of events associated with the object
         @rtype: list
         """
         assert(isinstance(obj, (gen.lib.Person, gen.lib.Family, NoneType)))
@@ -633,14 +712,14 @@ class SimpleAccess(object):
 
         @param obj: Person or Family
         @type obj: L{gen.lib.Person} or L{gen.lib.Family}
-        @return: list of events assocated with the object
+        @return: list of events associated with the object
         @rtype: list
         """
         assert(isinstance(obj, (gen.lib.Person, gen.lib.Family, gen.lib.Event, NoneType)))
 
         if obj:
             handles = [ ref.ref for ref in obj.get_source_references() ]
-            return [ self.dbase.get_source_from_handle(h) for h in handles ]
+            return map(self.dbase.get_source_from_handle, handles)
         else:
             return []
 
@@ -654,6 +733,8 @@ class SimpleAccess(object):
            as a parent.
         @rtype: list
         """
+        if type(person) in [str, unicode]: 
+            person = self.dbase.get_person_from_handle(person)
         assert(isinstance(person, (gen.lib.Person, NoneType)))
         
         if person:
@@ -671,6 +752,8 @@ class SimpleAccess(object):
            as a child.
         @rtype: list
         """
+        if type(person) in [str, unicode]: 
+            person = self.dbase.get_person_from_handle(person)
         assert(isinstance(person, (gen.lib.Person, NoneType)))
         
         if person:
@@ -686,16 +769,10 @@ class SimpleAccess(object):
         @return: list of objects of a particular type in the database
         @rtype: list
         """
-        slist = []
-        cursor = gen_cursor()
-        data = cursor.first()
-        while data:
-            slist.append(data[0])
-            data = cursor.next()
-        cursor.close()
-        for info in slist:
-            obj = get_object(info)
-            yield obj
+        
+        with gen_cursor() as cursor:
+            for key, data in cursor:
+                yield get_object(key)
 
     def all_people(self):
         """
@@ -708,14 +785,10 @@ class SimpleAccess(object):
         @return: list of people in the database
         @rtype: list
         """
-        slist = []
-        cursor = self.dbase.get_person_cursor()
-        data = cursor.first()
-        while data:
-            slist.append((data[1][3][3], data[0]))
-            data = cursor.next()
-        cursor.close()
-        slist.sort()
+        
+        with self.dbase.get_person_cursor() as cursor:
+            slist = sorted((data[3][3], key) for key, data in cursor)
+
         for info in slist:
             obj = self.dbase.get_person_from_handle(info[1])
             yield obj
@@ -745,7 +818,7 @@ class SimpleAccess(object):
         @return: list of events in the database
         @rtype: list
         """
-        return self.__all_objects(self.dbase.get_events_cursor, 
+        return self.__all_objects(self.dbase.get_event_cursor, 
                                   self.dbase.get_event_from_handle)
 
     def all_sources(self):
@@ -787,6 +860,117 @@ class SimpleAccess(object):
             return source.get_author()
         return u''
 
+    def person(self, handle):
+        assert(type(handle) in [str, unicode])
+        return self.dbase.get_person_from_handle(handle)
+
+    def event(self, handle):
+        assert(type(handle) in [str, unicode])
+        return self.dbase.get_event_from_handle(handle)
+
+    def family(self, handle):
+        assert(type(handle) in [str, unicode])
+        return self.dbase.get_family_from_handle(handle)
+
+    def display(self, object_class, prop, value):
+        """
+        Given a object_class, prop, and value return a display string
+        describing object.  
+        object_class is "Person", "Source", etc.
+        prop is "gramps_id", or "handle"
+        value is a gramps_id or handle.
+        """
+        if object_class in self.dbase.get_table_names():
+            obj = self.dbase.get_table_metadata(object_class)\
+                           [prop + "_func"](value)
+            if obj:
+                if isinstance(obj, gen.lib.Person):
+                    return "%s: %s [%s]" % (_(object_class), 
+                                            self.name(obj), 
+                                            self.gid(obj))
+                elif isinstance(obj, gen.lib.Event):
+                    return "%s: %s [%s]" % (_(object_class), 
+                                            self.event_type(obj),
+                                            self.gid(obj))
+                elif isinstance(obj, gen.lib.Family):
+                    return "%s: %s/%s [%s]" % (_(object_class), 
+                                            self.name(self.mother(obj)), 
+                                            self.name(self.father(obj)), 
+                                            self.gid(obj))
+                elif isinstance(obj, gen.lib.MediaObject):
+                    return "%s: %s [%s]" % (_(object_class), 
+                                            obj.desc, 
+                                            self.gid(obj))
+                elif isinstance(obj, gen.lib.Source):
+                    return "%s: %s [%s]" % (_(object_class), 
+                                            self.title(obj), 
+                                            self.gid(obj))
+                elif isinstance(obj, gen.lib.Place):
+                    return "%s: %s [%s]" % (_(object_class), 
+                                            place_name(self.dbase, 
+                                                       obj.handle), 
+                                            self.gid(obj))
+                elif isinstance(obj, gen.lib.Repository):
+                    return "%s: %s [%s]" % (_(object_class), 
+                                            obj.type, 
+                                            self.gid(obj))
+                elif isinstance(obj, gen.lib.Note):
+                    return "%s: %s [%s]" % (_(object_class), 
+                                            obj.type, 
+                                            self.gid(obj))
+                else:
+                    return "Error: incorrect object class: '%s'" % type(obj)
+            else:
+                return "Error: missing object"
+        else:
+            return "Error: invalid object class: '%s'" % object_class
+
+    def describe(self, obj):
+        """
+        Given a object, return a string describing the object.  
+        """
+        if isinstance(obj, gen.lib.Person):
+            return self.name(obj)
+        elif isinstance(obj, gen.lib.Event):
+            return self.event_type(obj)
+        elif isinstance(obj, gen.lib.Family):
+            father = self.father(obj)
+            mother = self.mother(obj)
+            if father:
+                father_text = self.name(father)
+            else:
+                father_text = _("Unknown father")
+            if mother:
+                mother_text = self.name(mother)
+            else:
+                mother_text = _("Unknown mother")
+            return "%s and %s" % (mother_text, father_text)
+        elif isinstance(obj, gen.lib.MediaObject):
+            return obj.desc
+        elif isinstance(obj, gen.lib.Source):
+            return self.title(obj)
+        elif isinstance(obj, gen.lib.Place):
+            return place_name(self.dbase, obj.handle)
+        elif isinstance(obj, gen.lib.Repository):
+            return obj.type
+        elif isinstance(obj, gen.lib.Note):
+            return obj.type
+        elif obj is None:
+            return ""
+        else:
+            return "Error: incorrect object class: '%s'" % type(obj)
+
+    def get_link(self, object_class, prop, value):
+        """
+        Given a object_class, prop, and value return the object.
+        object_class is "Person", "Source", etc.
+        prop is "gramps_id", or "handle"
+        value is a gramps_id or handle.
+        """
+        if object_class in self.dbase.get_table_names():
+            return self.dbase.get_table_metadata(object_class) \
+                [prop + "_func"](value)
+
 def by_date(event1, event2):
     """
     Sort function that will compare two events by their dates.
@@ -799,4 +983,11 @@ def by_date(event1, event2):
        1 if they are the same.
     @rtype: int
     """
-    return cmp(event1.get_date_object() , event2.get_date_object())
+    if event1 and event2:
+        return cmp(event1.get_date_object() , event2.get_date_object())
+    elif event1:
+        return -1
+    elif event2:
+        return 1
+    else:
+        return 0
