@@ -19,7 +19,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: Utils.py 17007 2011-03-31 12:13:29Z romjerome $
+# $Id: Utils.py 17860 2011-06-28 23:07:09Z jralls $
 
 """
 Non GUI/GTK related utility functions
@@ -129,7 +129,10 @@ def fix_encoding(value):
             return unicode(value)
         except:
             try:
-                codeset = locale.getpreferredencoding()
+                if constfunc.mac():
+                    codeset = locale.getlocale()[1]
+                else:
+                    codeset = locale.getpreferredencoding()
             except:
                 codeset = "UTF-8"
             return unicode(value, codeset)
@@ -308,7 +311,9 @@ def find_file( filename):
     
     # Build list of alternate encodings
     encodings = set()
-
+    #Darwin returns "mac roman" for preferredencoding, but since it
+    #returns "UTF-8" for filesystemencoding, and that's first, this
+    #works.
     for enc in [sys.getfilesystemencoding, locale.getpreferredencoding]:
         try:
             encodings.add(enc)
