@@ -6,6 +6,7 @@
 # Copyright (C) 2008 Lukasz Rymarczyk
 # Copyright (C) 2008 Raphael Ackermann
 # Copyright (C) 2008 Brian G. Matherly
+# Copyright (C) 2010 Jakim Friant
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -22,7 +23,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: arghandler.py 15983 2010-10-14 11:31:15Z ldnp $
+# $Id: arghandler.py 17690 2011-06-06 02:02:13Z pez4brian $
 
 """
 Module responsible for handling the command line arguments for GRAMPS.
@@ -47,9 +48,92 @@ import Utils
 import gen
 from clidbman import CLIDbManager, NAME_FILE, find_locker_name
 
-from PluginUtils import Tool
 from gen.plug import BasePluginManager
-from ReportBase import CATEGORY_BOOK, CATEGORY_CODE, cl_report
+from gen.plug.report import CATEGORY_BOOK, CATEGORY_CODE
+from cli.plug import cl_report
+
+#-------------------------------------------------------------------------
+#
+# private functions
+#
+#-------------------------------------------------------------------------
+def _split_options(options_str):
+    """
+    Split the options for the action.
+    
+    Rules:
+        * Entries in the list of options are separated by commas without spaces
+          between entries
+        * List values must be inclosed in brackets ("[" and "]")
+        * Entries within a list value are separated by commas
+        * Text values (as a value or as entries in a list) do not have to be
+          enclosed in quotes unless they include commas or quotation marks.
+        * Text containing double quotes must be contained in single quotes
+        * Text containing single quotes must be contained in double quotes
+        * Text cannot include both single and double quotes
+    
+    Examples:
+        * Multiple options specified:
+            report -p 'name=ancestor_chart,father_disp=["$n born $b"]'
+        * Using text with commas and quotes:
+            title="This is some text with ,s and 's"
+            title='This is some text with ,s and "s'
+        * Using a list of text
+            textlist=[row1,row2,"row3 with ' and ,"]
+    """
+    name = ""
+    value = ""
+    parsing_value = False
+    in_quotes = False
+    in_list = False
+    quote_type = ""
+    options_str_dict = {}
+    
+    for char in options_str:
+        if not parsing_value: 
+            # Parsing the name of the option
+            if char == "=":
+                #print char, "This value ends the name"
+                parsing_value = True
+            else:
+                #print char, "This value is part of the name"
+                name += char
+        else:
+            # Parsing the value of the option
+            if value == "" and char == '[':
+                #print char, "This character begins a list"
+                in_list = True
+                value += char
+            elif in_list == True and char == ']':
+                #print char, "This character ends the list"
+                in_list = False
+                value += char
+            elif not in_quotes and ( char == '"' or char == "'"):
+                #print char, "This character starts a quoted string"
+                in_quotes = True
+                quote_type = char
+                value += char
+            elif in_quotes and char == quote_type:
+                #print char, "This character ends a quoted string"
+                in_quotes = False
+                value += char
+            elif not in_quotes and not in_list and char == ",":
+                #print char, "This character ends the value of the option"
+                options_str_dict[name] = value
+                name = ""
+                value = ""
+                parsing_value = False
+                in_quotes = False
+                in_list = False
+            else:
+                #print char, "This character is part of the value"
+                value += char
+
+    if parsing_value and not in_quotes and not in_list:
+        # Add the last option
+        options_str_dict[name] = value
+
+    return options_str_dict
 
 #-------------------------------------------------------------------------
 # ArgHandler
@@ -87,16 +171,18 @@ class ArgHandler(object):
         self.open = self.__handle_open_option(parser.open)
         self.sanitize_args(parser.imports, parser.exports)
     
-    def __error(self, string):
+    def __error(self, msg1, msg2=None):
         """
         Output an error. Uses errorfunc if given, otherwise a simple print.
         """
         if self.errorfunc:
-            self.errorfunc(string)
+            self.errorfunc(msg1)
         else:
-            # Need to convert to str before printing
-            # For non latin characters in Windows path/file/user names
-            print string.encode(sys.getfilesystemencoding())
+            # Need to convert to system file encoding  before printing
+            # For non latin characters in path/file/user names
+            print >> sys.stderr, msg1.encode(sys.getfilesystemencoding())
+            if msg2 is not None:
+                print >> sys.stderr, msg2.encode(sys.getfilesystemencoding())
 
     #-------------------------------------------------------------------------
     # Argument parser: sorts out given arguments
@@ -119,6 +205,7 @@ class ArgHandler(object):
             return None
         value = Utils.get_unicode_path_from_env_var(value)
         db_path = self.__deduce_db_path(value)
+
         if db_path:
             # We have a potential database path.
             # Check if it is good.
@@ -136,12 +223,12 @@ class ArgHandler(object):
         Handle the "-i" or "--import" option.
         Only Files supported by a plugin can be imported, so not Family Trees.
         """
-        # Need to convert path/filename to unicode before openingh
-		# For non latin characters in Windows path/file/user names
+        # Need to convert path/filename to unicode before opening
+        # For non latin characters in Windows path/file/user names
         value = Utils.get_unicode_path_from_env_var(value)
         fname = value
         fullpath = os.path.abspath(os.path.expanduser(fname))
-        if not os.path.exists(fullpath):
+        if fname != '-' and not os.path.exists(fullpath):
             self.__error(_('Error: Import file %s not found.') % fname)
             sys.exit(0)
         
@@ -174,23 +261,26 @@ class ArgHandler(object):
         if self.gui:
             return
         # Need to covert path/filename to unicode before openingh
-		# For non latin characters in Windows path/file/user names
+        # For non latin characters in Windows path/file/user names
         value = Utils.get_unicode_path_from_env_var(value)
         fname = value
-        fullpath = os.path.abspath(os.path.expanduser(fname))
-        if os.path.exists(fullpath):
-            self.__error(_("WARNING: Output file already exists!\n"
-                    "WARNING: It will be overwritten:\n   %(name)s") % \
-                    {'name' : fullpath})
-            answer = None
-            while not answer:
-                answer = raw_input(_('OK to overwrite? (yes/no) ') \
-                                    .encode(sys.getfilesystemencoding()))
-            if answer.upper() in ('Y', 'YES', _('YES').upper()):
-                self.__error( _("Will overwrite the existing file: %s") 
-                                % fullpath)
-            else:
-                sys.exit(0)
+        if fname == '-':
+            fullpath = '-'
+        else:
+            fullpath = os.path.abspath(os.path.expanduser(fname))
+            if os.path.exists(fullpath):
+                self.__error(_("WARNING: Output file already exists!\n"
+                        "WARNING: It will be overwritten:\n   %(name)s") % \
+                        {'name' : fullpath})
+                answer = None
+                while not answer:
+                    answer = raw_input(_('OK to overwrite? (yes/no) ') \
+                                        .encode(sys.getfilesystemencoding()))
+                if answer.upper() in ('Y', 'YES', _('YES').upper()):
+                    self.__error( _("Will overwrite the existing file: %s") 
+                                    % fullpath)
+                else:
+                    sys.exit(0)
 
         if family_tree_format is None:
             # Guess the file format based on the file extension.
@@ -295,16 +385,16 @@ class ArgHandler(object):
 
         if self.list:
             print 'List of known family trees in your database path\n'
-            for name, dirname in self.dbman.family_tree_list():
-                print dirname, ', with name ', name.encode(sys.getfilesystemencoding()) 
+            for name, dirname in sorted(self.dbman.family_tree_list(), key=lambda pair: pair[0].lower()):
+                print "%s with name \"%s\"" % (dirname, name.encode(sys.getfilesystemencoding()))
             sys.exit(0)
             
         if self.list_more:
             print 'Gramps Family Trees:'
             summary_list = self.dbman.family_tree_summary()
-            for summary in summary_list:
+            for summary in sorted(summary_list, key=lambda summary: summary["Family tree"].lower()):
                 print "Family Tree \"%s\":" % summary["Family tree"]
-                for item in summary:
+                for item in sorted(summary):
                     if item != "Family tree":
                         print "   %s: %s" % (item, summary[item])
             sys.exit(0)
@@ -313,9 +403,9 @@ class ArgHandler(object):
         self.__import_action()
             
         for (action, options_str) in self.actions:
-            print "Performing action: %s." % action
+            print >> sys.stderr, "Performing action: %s." % action
             if options_str:
-                print "Using options string: %s" % options_str
+                print >> sys.stderr, "Using options string: %s" % options_str
             self.cl_action(action, options_str)
 
         for expt in self.exports:
@@ -323,16 +413,16 @@ class ArgHandler(object):
             # For non latin characters in Windows path/file/user names
             fn = expt[0].encode(sys.getfilesystemencoding())
             fmt = str(expt[1])
-            print "Exporting: file %s, format %s." % (fn, fmt)
+            print >> sys.stderr, "Exporting: file %s, format %s." % (fn, fmt)
             self.cl_export(expt[0], expt[1])
 
         if cleanup:
             self.cleanup()
-            print "Exiting."
+            print >> sys.stderr, "Exiting."
             sys.exit(0)
 
     def cleanup(self):
-        print "Cleaning up."
+        print >> sys.stderr, "Cleaning up."
         # remove files in import db subdir after use
         self.dbstate.db.close()
         if self.imp_db_path:
@@ -355,25 +445,23 @@ class ArgHandler(object):
                 if self.gui:
                     self.imp_db_path, title = self.dbman.create_new_db_cli()
                 else:
-                    self.imp_db_path = Utils.get_empty_tempdir("import_dbdir")
-                
+                    self.imp_db_path = Utils.get_empty_tempdir("import_dbdir") \
+					                      .encode(sys.getfilesystemencoding())
                     newdb = gen.db.DbBsddb()
                     newdb.write_version(self.imp_db_path)
                 
                 try:
                     self.sm.open_activate(self.imp_db_path)
-                    print "Created empty family tree successfully"
+                    print >> sys.stderr, "Created empty family tree successfully"
                 except:
-                    print "Error opening the file." 
-                    print "Exiting..." 
+                    print >> sys.stderr, "Error opening the file." 
+                    print >> sys.stderr, "Exiting..." 
                     sys.exit(0)
 
             for imp in self.imports:
-                # Need to covert path/filename to str before printing
-		        # For non latin characters in Windows path/file/user names
                 fn = imp[0].encode(sys.getfilesystemencoding())
                 fmt = str(imp[1])
-                print "Importing: file %s, format %s." % (fn , fmt)
+                print >> sys.stderr, "Importing: file %s, format %s." % (fn, fmt)
                 self.cl_import(imp[0], imp[1])
 
     def __open_action(self):
@@ -389,10 +477,10 @@ class ArgHandler(object):
             # we load this file for use
             try:
                 self.sm.open_activate(self.open)
-                print "Opened successfully!"
+                print >> sys.stderr, "Opened successfully!"
             except:
-                print "Error opening the file." 
-                print "Exiting..." 
+                print >> sys.stderr, "Error opening the file." 
+                print >> sys.stderr, "Exiting..." 
                 sys.exit(0)
 
     def check_db(self, dbpath, force_unlock = False):
@@ -446,7 +534,7 @@ class ArgHandler(object):
         for plugin in pmgr.get_export_plugins():
             if family_tree_format == plugin.get_extension():
                 export_function = plugin.get_export_function()
-                export_function(self.dbstate.db, filename)
+                export_function(self.dbstate.db, filename, self.__error)
 
     #-------------------------------------------------------------------------
     #
@@ -460,11 +548,10 @@ class ArgHandler(object):
         pmgr = BasePluginManager.get_instance()
         if action == "report":
             try:
-                options_str_dict = dict( [ tuple(chunk.split('='))
-                    for chunk in options_str.split(',') ] )
+                options_str_dict = _split_options(options_str)
             except:
                 options_str_dict = {}
-                print "Ignoring invalid options string."
+                print >> sys.stderr, "Ignoring invalid options string."
 
             name = options_str_dict.pop('name', None)
             _cl_list = pmgr.get_reg_reports(gui=False)
@@ -489,25 +576,27 @@ class ArgHandler(object):
                 # name exists, but is not in the list of valid report names
                 msg = "Unknown report name."
             else:
-                msg = "Report name not given. Please use -p name=reportname."
+                msg = "Report name not given. Please use one of [-p|--options] name=reportname."
             
-            print "%s\n Available names are:" % msg
-            for pdata in _cl_list:
+            print >> sys.stderr, "%s\n Available names are:" % msg
+            for pdata in sorted(_cl_list, key= lambda pdata: pdata.id.lower()):
                 # Print cli report name ([item[0]) and GUI report name (item[4])
                 if len(pdata.id) <= 25:
-                    print "   %s%s- %s" % ( pdata.id, 
-                                            " " * (26 - len(pdata.id)),
-                                            pdata.name.encode(sys.getfilesystemencoding()))
+                    print >> sys.stderr, \
+                        "   %s%s- %s" % ( pdata.id, " " * (26 - len(pdata.id)),
+                                 pdata.name.encode(sys.getfilesystemencoding()))
                 else:
-                    print "   %s\t- %s" % (pdata.id, pdata.name.encode(sys.getfilesystemencoding()) )
+                    print >> sys.stderr, "   %s\t- %s" % (pdata.id,
+                                 pdata.name.encode(sys.getfilesystemencoding()))
 
         elif action == "tool":
+            from gui.plug import tool
             try:
                 options_str_dict = dict( [ tuple(chunk.split('=')) for
                                            chunk in options_str.split(',') ] )
             except:
                 options_str_dict = {}
-                print "Ignoring invalid options string."
+                print >> sys.stderr, "Ignoring invalid options string."
 
             name = options_str_dict.pop('name', None)
             _cli_tool_list = pmgr.get_reg_tools(gui=False)
@@ -521,22 +610,23 @@ class ArgHandler(object):
                         category = pdata.category
                         tool_class = eval('mod.' + pdata.toolclass)
                         options_class = eval('mod.' + pdata.optionclass)
-                        Tool.cli_tool(self.dbstate, name, category, tool_class, 
+                        tool.cli_tool(self.dbstate, name, category, tool_class,
                                       options_class, options_str_dict)
                         return
                 msg = "Unknown tool name."
             else:
-                msg = "Tool name not given. Please use -p name=toolname."
+                msg = "Tool name not given. Please use one of [-p|--options] name=toolname."
             
-            print "%s\n Available names are:" % msg
-            for pdata in _cli_tool_list:
+            print >> sys.stderr, "%s\n Available names are:" % msg
+            for pdata in sorted(_cli_tool_list, key=lambda pdata: pdata.id.lower()):
                 # Print cli report name ([item[0]) and GUI report name (item[4])
                 if len(pdata.id) <= 25:
-                    print "   %s%s- %s" % ( pdata.id, 
-                                            " " * (26 - len(pdata.id)),
-                                            pdata.name.encode(sys.getfilesystemencoding()))
+                    print >> sys.stderr, \
+                        "   %s%s- %s" % ( pdata.id, " " * (26 - len(pdata.id)),
+                                 pdata.name.encode(sys.getfilesystemencoding()))
                 else:
-                    print "   %s\t- %s" % (pdata.id, pdata.name.encode(sys.getfilesystemencoding()))
+                    print >> sys.stderr, "   %s\t- %s" % (pdata.id,
+                                 pdata.name.encode(sys.getfilesystemencoding()))
         else:
-            print "Unknown action: %s." % action
+            print >> sys.stderr, "Unknown action: %s." % action
             sys.exit(0)
