@@ -1,11 +1,12 @@
 #
 # Gramps - a GTK+/GNOME based genealogy program
 #
-# Copyright (C) 2001-2007  Donald N. Allingham
-# Copyright (C) 2008 Lukasz Rymarczyk
-# Copyright (C) 2008 Raphael Ackermann
-# Copyright (C) 2008 Brian G. Matherly
+# Copyright (C) 2001-2007 Donald N. Allingham
+# Copyright (C) 2008      Lukasz Rymarczyk
+# Copyright (C) 2008      Raphael Ackermann
+# Copyright (C) 2008-2011 Brian G. Matherly
 # Copyright (C) 2010      Jakim Friant
+# Copyright (C) 2011      Paul Franklin
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -24,7 +25,7 @@
 #
 # cli.plug.__init__
 #
-# $Id: __init__.py 17692 2011-06-06 03:09:43Z pez4brian $
+# $Id: __init__.py 17806 2011-06-19 02:46:33Z pez4brian $
 
 
 #-------------------------------------------------------------------------
@@ -99,12 +100,16 @@ def _convert_str_to_match_type(str_val, type_val):
     elif ret_type == bool:
         if str_val == str(True):
             return True
-        return False
+        elif str_val == str(False):
+            return False
+        else:
+            print "%s is not a boolean-- try 'True' or 'False'" % str_val
+            return False
     
     elif ret_type == list:
         ret_val = []
         if not ( str_val.startswith("[") and str_val.endswith("]") ):
-            print "%s is not a list" % str_val
+            print "%s is not a list-- try: [%s]" % (str_val, str_val)
             return ret_val
         
         entry = ""
@@ -220,7 +225,12 @@ class CommandLineReport(object):
         if category == CATEGORY_GRAPHVIZ:
             # Need to include GraphViz options
             self.__gvoptions = graphdoc.GVOptions()
-            self.__gvoptions.add_menu_options(self.option_class.menu)
+            menu = self.option_class.menu
+            self.__gvoptions.add_menu_options(menu)
+            for name in menu.get_all_option_names():
+                if name not in self.option_class.options_dict:
+                    self.option_class.options_dict[name] = \
+                      menu.get_option_by_name(name).get_value()
         self.option_class.load_previous_values()
         _validate_options(self.option_class, database)
         self.show = options_str_dict.pop('show', None)
