@@ -19,7 +19,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: const.py.in 17399 2011-05-03 21:32:32Z nick-h $
+# $Id: const.py.in 18114 2011-09-06 14:42:33Z nick-h $
 
 """
 Provides constants for other modules
@@ -41,11 +41,11 @@ from gen.ggettext import sgettext as _
 #
 #-------------------------------------------------------------------------
 PROGRAM_NAME   = "Gramps"
-if "3.3.0-1" == "@" + "VERSIONSTRING" + "@":
-    VERSION        = "3.3.0"
+if "3.3.1-1" == "@" + "VERSIONSTRING" + "@":
+    VERSION        = "3.3.1"
 else:
-    VERSION        = "3.3.0-1"
-VERSION_TUPLE  = (3, 3, 0)
+    VERSION        = "3.3.1-1"
+VERSION_TUPLE  = (3, 3, 1)
 
 #-------------------------------------------------------------------------
 #
@@ -145,7 +145,7 @@ IMAGE_DIR      = os.path.join(ROOT_DIR, "images")
 VERSION_DIR    = os.path.join(
     HOME_DIR, "gramps%s%s" % (VERSION_TUPLE[0], VERSION_TUPLE[1]))
 
-CUSTOM_FILTERS = os.path.join(HOME_DIR, "custom_filters.xml")
+CUSTOM_FILTERS = os.path.join(VERSION_DIR, "custom_filters.xml")
 REPORT_OPTIONS = os.path.join(HOME_DIR, "report_options.xml")
 TOOL_OPTIONS   = os.path.join(HOME_DIR, "tool_options.xml")
 
