@@ -19,7 +19,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: base.py 17538 2011-05-19 20:38:03Z m_d_n $
+# $Id: base.py 18021 2011-08-09 20:35:04Z m_d_n $
 
 """
 Base class for the GRAMPS databases. All database interfaces should inherit
@@ -40,6 +40,7 @@ from gen.ggettext import gettext as _
 #-------------------------------------------------------------------------
 import gen.lib
 from txn import DbTxn
+from exceptions import DbTransactionCancel
 
 class DbReadBase(object):
     """
@@ -1689,6 +1690,11 @@ class DbWriteBase(object):
         elif family.get_mother_handle() == person_handle:
             msg = _("Remove mother from family")
             family.set_mother_handle(None)
+        else:
+            raise DbTransactionCancel("The relation between the person and "
+                "the family you try to remove is not consistent, please fix "
+                "that first, for example from the family editor or by running "
+                "the database repair tool, before removing the family.")
     
         if (not family.get_father_handle() and not family.get_mother_handle()
                 and not family.get_child_ref_list()):
