@@ -2,6 +2,7 @@
 # Gramps - a GTK+/GNOME based genealogy program
 #
 # Copyright (C) 2000-2007  Donald N. Allingham
+# Copyright (C) 2010       Nick Hall
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,7 +19,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: read.py 15399 2010-05-13 11:07:38Z dsblank $
+# $Id: read.py 17648 2011-06-01 21:33:49Z m_d_n $
 
 """
 Read classes for the GRAMPS databases.
@@ -35,8 +36,14 @@ import random
 import locale
 import os
 from sys import maxint
-from bsddb import db
+
+import config
+if config.get('preferences.use-bsddb3'):
+    from bsddb3 import db
+else:
+    from bsddb import db
 from gen.ggettext import gettext as _
+import re
 
 import logging
 
@@ -46,10 +53,11 @@ import logging
 #
 #-------------------------------------------------------------------------
 from gen.lib import (MediaObject, Person, Family, Source, Event, Place, 
-                     Repository, Note, GenderStats, Researcher)
+                     Repository, Note, Tag, GenderStats, Researcher, 
+                     NameOriginType)
 from gen.db.dbconst import *
 from gen.utils.callback import Callback
-from gen.db import (GrampsCursor, DbReadBase)
+from gen.db import (BsddbBaseCursor, DbReadBase)
 from Utils import create_id
 import Errors
 
@@ -62,11 +70,46 @@ LOG = logging.getLogger(DBLOGNAME)
 from gen.db.dbconst import *
 
 _SIGBASE = ('person', 'family', 'source', 'event', 
-            'media', 'place', 'repository', 'reference', 'note')
+            'media', 'place', 'repository', 'reference', 'note', 'tag')
 
 DBERRS      = (db.DBRunRecoveryError, db.DBAccessError, 
                db.DBPageNotFoundError, db.DBInvalidArgError)
 
+#-------------------------------------------------------------------------
+#
+# Helper functions
+#
+#-------------------------------------------------------------------------  
+def find_surname(key, data):
+    """
+    Creating a surname from raw data of a person, to use for sort and index
+    """
+    return __index_surname(data[3][5])
+
+def find_surname_name(key, data):
+    """
+    Creating a surname from raw name, to use for sort and index
+    """
+    return __index_surname(data[5])
+
+def __index_surname(surn_list):
+    """
+    All non pa/matronymic surnames are used in indexing.
+    pa/matronymic not as they change for every generation!
+    """
+    if surn_list:
+        surn = " ".join([x[0] for x in surn_list if not (x[3][0] in [
+                    NameOriginType.PATRONYMIC, NameOriginType.MATRONYMIC]) ])
+    else:
+        surn = ""
+    return str(surn)
+    
+
+#-------------------------------------------------------------------------
+#
+# class DbBookmarks
+#
+#-------------------------------------------------------------------------  
 class DbBookmarks(object):
     def __init__(self, default=[]):
         self.bookmarks = list(default) # want a copy (not an alias)
@@ -92,15 +135,18 @@ class DbBookmarks(object):
     def insert(self, pos, item):
         self.bookmarks.insert(pos, item)
 
+    def close(self):
+        del self.bookmarks
+
 #-------------------------------------------------------------------------
 #
 # GrampsDBReadCursor
 #
 #-------------------------------------------------------------------------
-class DbReadCursor(GrampsCursor):
+class DbReadCursor(BsddbBaseCursor):
 
     def __init__(self, source, txn=None, **kwargs):
-        GrampsCursor.__init__(self, txn=txn, **kwargs)
+        BsddbBaseCursor.__init__(self, txn=txn, **kwargs)
         self.cursor = source.db.cursor(txn)
         self.source = source
 
@@ -169,7 +215,6 @@ class DbBsddbRead(DbReadBase, Callback):
         """
         Create a new DbBsddbRead instance. 
         """
-        
         DbReadBase.__init__(self)
         Callback.__init__(self)
 
@@ -230,6 +275,13 @@ class DbBsddbRead(DbReadBase, Callback):
                 "class_func": Note,
                 "cursor_func": self.get_note_cursor,
                 },
+            'Tag':
+                {
+                "handle_func": self.get_tag_from_handle, 
+                "gramps_id_func": None,
+                "class_func": Tag,
+                "cursor_func": self.get_tag_cursor,
+                },
             }
 
         self.set_person_id_prefix('I%04d')
@@ -257,11 +309,11 @@ class DbBsddbRead(DbReadBase, Callback):
         self.individual_event_names = set()
         self.individual_attributes = set()
         self.family_attributes = set()
-        self.marker_names = set()
         self.child_ref_types = set()
         self.family_rel_types = set()
         self.event_role_names = set()
         self.name_types = set()
+        self.origin_types = set()
         self.repository_types = set()
         self.note_types = set()
         self.source_media_types = set()
@@ -280,6 +332,7 @@ class DbBsddbRead(DbReadBase, Callback):
         self.rid_trans = {}
         self.nid_trans = {}
         self.eid_trans = {}
+        self.tag_trans = {}
         self.env = None
         self.person_map = {}
         self.family_map = {}
@@ -319,14 +372,15 @@ class DbBsddbRead(DbReadBase, Callback):
 
     def set_prefixes(self, person, media, family, source, place, event,
                      repository, note):
-        self.person_prefix = self._validated_id_prefix(person, 'I')
-        self.mediaobject_prefix = self._validated_id_prefix(media, 'M')
-        self.family_prefix = self._validated_id_prefix(family, 'F')
-        self.source_prefix = self._validated_id_prefix(source, 'S')
-        self.place_prefix = self._validated_id_prefix(place, 'P')
-        self.event_prefix = self._validated_id_prefix(event, 'E')
-        self.repository_prefix = self._validated_id_prefix(repository, 'R')
-        self.note_prefix = self._validated_id_prefix(note, 'N')
+        self.set_person_id_prefix(person)
+        self.set_object_id_prefix(media)
+        self.set_family_id_prefix(family)
+        self.set_source_id_prefix(source)
+        self.set_place_id_prefix(place)
+        self.set_event_id_prefix(event)
+        self.set_repository_id_prefix(repository)
+        self.set_note_id_prefix(note)
+        #self.set_tag_id_prefix(tag)
 
     def version_supported(self):
         """Return True when the file has a supported version."""
@@ -373,14 +427,35 @@ class DbBsddbRead(DbReadBase, Callback):
     def get_note_cursor(self, *args, **kwargs):
         return self.get_cursor(self.note_map, *args, **kwargs)
 
+    def get_tag_cursor(self, *args, **kwargs):
+        return self.get_cursor(self.tag_map, *args, **kwargs)
+
     def close(self):
         """
         Close the specified database. 
         
         The method needs to be overridden in the derived class.
         """
-        pass
-        
+        #remove circular dependance
+        self.basedb = None
+        #remove links to functions
+        self.disconnect_all()
+        for key in self._tables:
+            for subkey in self._tables[key]:
+                self._tables[key][subkey] = None
+            del self._tables[key][subkey]
+            self._tables[key] = None
+        del self._tables
+##        self.bookmarks = None
+##        self.family_bookmarks = None
+##        self.event_bookmarks = None
+##        self.place_bookmarks = None
+##        self.source_bookmarks = None
+##        self.repo_bookmarks = None
+##        self.media_bookmarks = None
+##        self.note_bookmarks = None
+
+
     def is_open(self):
         """
         Return 1 if the database has been opened.
@@ -400,14 +475,14 @@ class DbBsddbRead(DbReadBase, Callback):
         self.emit('event-rebuild')
         self.emit('repository-rebuild')
         self.emit('note-rebuild')
+        self.emit('tag-rebuild')
 
-    @staticmethod
-    def __find_next_gramps_id(prefix, map_index, trans):
+    def __find_next_gramps_id(self, prefix, map_index, trans):
         """
         Helper function for find_next_<object>_gramps_id methods
         """
         index = prefix % map_index
-        while trans.has_key(str(index)):
+        while trans.get(str(index), txn=self.txn) is not None:
             map_index += 1
             index = prefix % map_index
         map_index += 1
@@ -524,7 +599,7 @@ class DbBsddbRead(DbReadBase, Callback):
 
     def get_person_from_handle(self, handle):
         """
-        Find a Person in the database from the passed gramps' ID.
+        Find a Person in the database from the passed handle.
         
         If no such Person exists, None is returned.
         """
@@ -532,7 +607,7 @@ class DbBsddbRead(DbReadBase, Callback):
 
     def get_source_from_handle(self, handle):
         """
-        Find a Source in the database from the passed gramps' ID.
+        Find a Source in the database from the passed handle.
         
         If no such Source exists, None is returned.
         """
@@ -540,7 +615,7 @@ class DbBsddbRead(DbReadBase, Callback):
 
     def get_object_from_handle(self, handle):
         """
-        Find an Object in the database from the passed gramps' ID.
+        Find an Object in the database from the passed handle.
         
         If no such Object exists, None is returned.
         """
@@ -548,7 +623,7 @@ class DbBsddbRead(DbReadBase, Callback):
 
     def get_place_from_handle(self, handle):
         """
-        Find a Place in the database from the passed gramps' ID.
+        Find a Place in the database from the passed handle.
         
         If no such Place exists, None is returned.
         """
@@ -556,7 +631,7 @@ class DbBsddbRead(DbReadBase, Callback):
 
     def get_event_from_handle(self, handle):
         """
-        Find a Event in the database from the passed gramps' ID.
+        Find a Event in the database from the passed handle.
         
         If no such Event exists, None is returned.
         """
@@ -564,7 +639,7 @@ class DbBsddbRead(DbReadBase, Callback):
 
     def get_family_from_handle(self, handle):
         """
-        Find a Family in the database from the passed gramps' ID.
+        Find a Family in the database from the passed handle.
         
         If no such Family exists, None is returned.
         """
@@ -572,7 +647,7 @@ class DbBsddbRead(DbReadBase, Callback):
 
     def get_repository_from_handle(self, handle):
         """
-        Find a Repository in the database from the passed gramps' ID.
+        Find a Repository in the database from the passed handle.
         
         If no such Repository exists, None is returned.
         """
@@ -580,16 +655,24 @@ class DbBsddbRead(DbReadBase, Callback):
 
     def get_note_from_handle(self, handle):
         """
-        Find a Note in the database from the passed gramps' ID.
+        Find a Note in the database from the passed handle.
         
         If no such Note exists, None is returned.
         """
         return self.get_from_handle(handle, Note, self.note_map)
 
+    def get_tag_from_handle(self, handle):
+        """
+        Find a Tag in the database from the passed handle.
+        
+        If no such Tag exists, None is returned.
+        """
+        return self.get_from_handle(handle, Tag, self.tag_map)
+
     def __get_obj_from_gramps_id(self, val, tbl, class_, prim_tbl):
         try:
-            if tbl.has_key(str(val)):
-                data = tbl.get(str(val), txn=self.txn)
+            data = tbl.get(str(val), txn=self.txn)
+            if data is not None:
                 obj = class_()
                 ### FIXME: this is a dirty hack that works without no
                 ### sensible explanation. For some reason, for a readonly
@@ -678,23 +761,34 @@ class DbBsddbRead(DbReadBase, Callback):
         """
         return self.__get_obj_from_gramps_id(val, self.nid_trans, Note,
                                               self.note_map)
+
+    def get_tag_from_name(self, val):
+        """
+        Find a Tag in the database from the passed Tag name.
+        
+        If no such Tag exists, None is returned.
+        """
+        return self.__get_obj_from_gramps_id(val, self.tag_trans, Tag,
+                                              self.tag_map)
  
-    def get_name_group_mapping(self, name):
+    def get_name_group_mapping(self, surname):
         """
         Return the default grouping name for a surname.
         """
-        return unicode(self.name_group.get(str(name), name))
+        return unicode(self.name_group.get(str(surname), surname))
 
     def get_name_group_keys(self):
         """
         Return the defined names that have been assigned to a default grouping.
         """
-        return [unicode(k) for k in self.name_group.keys()]
+        return map(unicode, self.name_group.keys())
 
     def has_name_group_key(self, name):
         """
         Return if a key exists in the name_group table.
         """
+        # The use of has_key seems allright because there is no write lock
+        # on the name_group table when this is called.
         return self.name_group.has_key(str(name))
 
     def get_number_of_records(self, table):
@@ -753,10 +847,16 @@ class DbBsddbRead(DbReadBase, Callback):
         """
         return self.get_number_of_records(self.note_map)
 
+    def get_number_of_tags(self):
+        """
+        Return the number of tags currently in the database.
+        """
+        return self.get_number_of_records(self.tag_map)
+
     def all_handles(self, table):
-        return table.keys()
+        return table.keys(txn=self.txn)
         
-    def get_person_handles(self, sort_handles=True):
+    def get_person_handles(self, sort_handles=False):
         """
         Return a list of database handles, one handle for each Person in
         the database. 
@@ -770,17 +870,7 @@ class DbBsddbRead(DbReadBase, Callback):
             return handle_list
         return []
 
-        # Old code: does the same as above, but more complicated
-        if self.db_is_open:
-            if sort_handles:
-                with self.get_person_cursor() as cursor:
-                    slist = sorted((data[3][5], key) for key, data in cursor)
-                return [x[1] for x in slist]
-            else:
-                return self.all_handles(self.person_map)
-        return []
-
-    def get_place_handles(self, sort_handles=True):
+    def get_place_handles(self, sort_handles=False):
         """
         Return a list of database handles, one handle for each Place in
         the database. 
@@ -795,17 +885,7 @@ class DbBsddbRead(DbReadBase, Callback):
             return handle_list
         return []
 
-        # Old code: does the same as above, but more complicated
-        if self.db_is_open:
-            if sort_handles:
-                with self.get_place_cursor() as cursor:
-                    slist = sorted((data[2], key) for key, data in cursor)
-                return [x[1] for x in slist]
-            else:
-                return self.all_handles(self.place_map)
-        return []
-        
-    def get_source_handles(self, sort_handles=True):
+    def get_source_handles(self, sort_handles=False):
         """
         Return a list of database handles, one handle for each Source in
         the database.
@@ -819,7 +899,7 @@ class DbBsddbRead(DbReadBase, Callback):
             return handle_list
         return []
         
-    def get_media_object_handles(self, sort_handles=True):
+    def get_media_object_handles(self, sort_handles=False):
         """
         Return a list of database handles, one handle for each MediaObject in
         the database. 
@@ -869,6 +949,20 @@ class DbBsddbRead(DbReadBase, Callback):
             return self.all_handles(self.note_map)
         return []
 
+    def get_tag_handles(self, sort_handles=False):
+        """
+        Return a list of database handles, one handle for each Tag in
+        the database.
+        
+         If sort_handles is True, the list is sorted by Tag name.
+        """
+        if self.db_is_open:
+            handle_list = self.all_handles(self.tag_map)
+            if sort_handles:
+                handle_list.sort(key=self.__sortbytag_key)
+            return handle_list
+        return []
+
     def _f(curs_):
         """
         Closure that returns an iterator over handles in the database.
@@ -889,6 +983,7 @@ class DbBsddbRead(DbReadBase, Callback):
     iter_media_object_handles = _f(get_media_cursor)
     iter_repository_handles   = _f(get_repository_cursor)
     iter_note_handles         = _f(get_note_cursor)
+    iter_tag_handles          = _f(get_tag_cursor)
     del _f
     
     def _f(curs_, obj_):
@@ -913,6 +1008,7 @@ class DbBsddbRead(DbReadBase, Callback):
     iter_media_objects = _f(get_media_cursor, MediaObject)
     iter_repositories  = _f(get_repository_cursor, Repository)
     iter_notes         = _f(get_note_cursor, Note)
+    iter_tags          = _f(get_tag_cursor, Tag)
     del _f
 
     def get_gramps_ids(self, obj_key):
@@ -944,7 +1040,7 @@ class DbBsddbRead(DbReadBase, Callback):
 
         table = key2table[obj_key]
         #return str(gramps_id) in table
-        return table.has_key(str(gramps_id))
+        return table.get(str(gramps_id), txn=self.txn) is not None
 
     def find_initial_person(self):
         person = self.get_default_person()
@@ -961,12 +1057,46 @@ class DbBsddbRead(DbReadBase, Callback):
                 str_ = val % 1
             except TypeError:           # missing conversion specifier
                 prefix_var = val + "%d"
+            except ValueError:          # incomplete format
+                prefix_var = default+"%04d"
             else:
                 prefix_var = val        # OK as given
         else:
             prefix_var = default+"%04d" # not a string or empty string
         return prefix_var
 
+    @staticmethod
+    def __id2user_format(id_pattern):
+        """
+        Return a method that accepts a Gramps ID and adjusts it to the users
+        format.
+        """
+        pattern_match = re.match(r"(.*)%[0 ](\d+)[diu]$", id_pattern)
+        if pattern_match:
+            str_prefix = pattern_match.group(1)
+            nr_width = pattern_match.group(2)
+            def closure_func(gramps_id):
+                if gramps_id and gramps_id.startswith(str_prefix):
+                    id_number = gramps_id[len(str_prefix):]
+                    if id_number.isdigit():
+                        id_value = int(id_number, 10)
+                        if len(str(id_value)) > nr_width:
+                            # The ID to be imported is too large to fit in the
+                            # users format. For now just create a new ID,
+                            # because that is also what happens with IDs that
+                            # are identical to IDs already in the database. If
+                            # the problem of colliding import and already
+                            # present IDs is solved the code here also needs
+                            # some solution.
+                            gramps_id = id_pattern % 1
+                        else:
+                            gramps_id = id_pattern % id_value
+                return gramps_id
+        else:
+            def closure_func(gramps_id):
+                return gramps_id
+        return closure_func
+
     def set_person_id_prefix(self, val):
         """
         Set the naming template for GRAMPS Person ID values. 
@@ -976,6 +1106,7 @@ class DbBsddbRead(DbReadBase, Callback):
         such as I%d or I%04d.
         """
         self.person_prefix = self._validated_id_prefix(val, "I")
+        self.id2user_format = self.__id2user_format(self.person_prefix)
 
     def set_source_id_prefix(self, val):
         """
@@ -986,6 +1117,7 @@ class DbBsddbRead(DbReadBase, Callback):
         such as S%d or S%04d.
         """
         self.source_prefix = self._validated_id_prefix(val, "S")
+        self.sid2user_format = self.__id2user_format(self.source_prefix)
             
     def set_object_id_prefix(self, val):
         """
@@ -996,6 +1128,7 @@ class DbBsddbRead(DbReadBase, Callback):
         such as O%d or O%04d.
         """
         self.mediaobject_prefix = self._validated_id_prefix(val, "O")
+        self.oid2user_format = self.__id2user_format(self.mediaobject_prefix)
 
     def set_place_id_prefix(self, val):
         """
@@ -1006,6 +1139,7 @@ class DbBsddbRead(DbReadBase, Callback):
         such as P%d or P%04d.
         """
         self.place_prefix = self._validated_id_prefix(val, "P")
+        self.pid2user_format = self.__id2user_format(self.place_prefix)
 
     def set_family_id_prefix(self, val):
         """
@@ -1015,6 +1149,7 @@ class DbBsddbRead(DbReadBase, Callback):
         or F%04d.
         """
         self.family_prefix = self._validated_id_prefix(val, "F")
+        self.fid2user_format = self.__id2user_format(self.family_prefix)
 
     def set_event_id_prefix(self, val):
         """
@@ -1025,6 +1160,7 @@ class DbBsddbRead(DbReadBase, Callback):
         such as E%d or E%04d.
         """
         self.event_prefix = self._validated_id_prefix(val, "E")
+        self.eid2user_format = self.__id2user_format(self.event_prefix)
 
     def set_repository_id_prefix(self, val):
         """
@@ -1035,6 +1171,7 @@ class DbBsddbRead(DbReadBase, Callback):
         such as R%d or R%04d.
         """
         self.repository_prefix = self._validated_id_prefix(val, "R")
+        self.rid2user_format = self.__id2user_format(self.repository_prefix)
 
     def set_note_id_prefix(self, val):
         """
@@ -1045,6 +1182,7 @@ class DbBsddbRead(DbReadBase, Callback):
         such as N%d or N%04d.
         """
         self.note_prefix = self._validated_id_prefix(val, "N")
+        self.nid2user_format = self.__id2user_format(self.note_prefix)
 
     def set_undo_callback(self, callback):
         """
@@ -1166,12 +1304,6 @@ class DbBsddbRead(DbReadBase, Callback):
         """
         return list(self.family_event_names)
 
-    def get_marker_types(self):
-        """
-        Return a list of all marker types available in the database.
-        """
-        return list(self.marker_names)
-        
     def get_media_attribute_types(self):
         """
         Return a list of all Attribute types assocated with Media and MediaRef 
@@ -1207,6 +1339,13 @@ class DbBsddbRead(DbReadBase, Callback):
         """
         return list(self.name_types)
 
+    def get_origin_types(self):
+        """
+        Return a list of all custom origin types assocated with Person/Surname
+        instances in the database.
+        """
+        return list(self.origin_types)
+
     def get_repository_types(self):
         """
         Return a list of all custom repository types assocated with Repository 
@@ -1271,7 +1410,10 @@ class DbBsddbRead(DbReadBase, Callback):
 
     def get_raw_note_data(self, handle):
         return self.__get_raw_data(self.note_map, handle)
-        
+
+    def get_raw_tag_data(self, handle):
+        return self.__get_raw_data(self.tag_map, handle)
+
     def __has_handle(self, table, handle):
         """
         Helper function for has_<object>_handle methods
@@ -1330,8 +1472,15 @@ class DbBsddbRead(DbReadBase, Callback):
         """
         return self.__has_handle(self.source_map, handle)
 
+    def has_tag_handle(self, handle):
+        """
+        Return True if the handle exists in the current Tag database.
+        """
+        return self.__has_handle(self.tag_map, handle)
+
     def __sortbyperson_key(self, person):
-        return locale.strxfrm(self.person_map.get(str(person))[3][5])
+        return locale.strxfrm(find_surname(str(person), 
+                                           self.person_map.get(str(person))))
 
     def __sortbyplace(self, first, second):
         return locale.strcoll(self.place_map.get(str(first))[2], 
@@ -1358,6 +1507,15 @@ class DbBsddbRead(DbReadBase, Callback):
         media = self.media_map[str(key)][4]
         return locale.strxfrm(media)
 
+    def __sortbytag(self, first, second):
+        tag1 = self.tag_map[str(first)][1]
+        tag2 = self.tag_map[str(second)][1]
+        return locale.strcoll(tag1, tag2)
+
+    def __sortbytag_key(self, key):
+        tag = self.tag_map[str(key)][1]
+        return locale.strxfrm(tag)
+
     def set_mediapath(self, path):
         """Set the default media path for database, path should be utf-8."""
         if (self.metadata is not None) and (not self.readonly):
@@ -1427,6 +1585,10 @@ class DbBsddbRead(DbReadBase, Callback):
                 'cursor_func': self.get_note_cursor, 
                 'class_func': Note,
                 },
+            'Tag':   {
+                'cursor_func': self.get_tag_cursor, 
+                'class_func': Tag,
+                },
             }
 
         # Find which tables to iterate over
@@ -1466,3 +1628,23 @@ class DbBsddbRead(DbReadBase, Callback):
         """
         return self._bm_changes > 0
 
+    def get_dbid(self):
+        """
+        In BSDDB, we use the file directory name as the unique ID for
+        this database on this computer.
+        """
+        return None
+
+    def get_dbname(self):
+        """
+        In BSDDB, the database is in a text file at the path
+        """
+        filepath = os.path.join(self.path, "name.txt")
+        try:
+            name_file = open(filepath, "r")
+            name = name_file.read()
+            name_file.close()
+        except (OSError, IOError), msg:
+            self.__log_error()
+            name = None
+        return name
