@@ -2,6 +2,8 @@
 # Gramps - a GTK+/GNOME based genealogy program
 #
 # Copyright (C) 2000-2007  Donald N. Allingham
+# Copyright (C) 2010       Michiel D. Nauta
+# Copyright (C) 2010       Nick Hall
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,7 +20,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: family.py 10103 2008-02-24 13:55:55Z acraphae $
+# $Id: family.py 16425 2011-01-21 16:27:54Z gbritton $
 
 """
 Family object for GRAMPS.
@@ -30,7 +32,6 @@ Family object for GRAMPS.
 #
 #-------------------------------------------------------------------------
 from warnings import warn
-from types import InstanceType
 
 #-------------------------------------------------------------------------
 #
@@ -44,9 +45,10 @@ from gen.lib.mediabase import MediaBase
 from gen.lib.attrbase import AttributeBase
 from gen.lib.eventref import EventRef
 from gen.lib.ldsordbase import LdsOrdBase
+from gen.lib.tagbase import TagBase
 from gen.lib.childref import ChildRef
 from gen.lib.familyreltype import FamilyRelType
-from gen.lib.markertype import MarkerType
+from gen.lib.const import IDENTICAL, EQUAL, DIFFERENT
 
 #-------------------------------------------------------------------------
 #
@@ -54,16 +56,12 @@ from gen.lib.markertype import MarkerType
 #
 #-------------------------------------------------------------------------
 class Family(SourceBase, NoteBase, MediaBase, AttributeBase, LdsOrdBase,
-             PrimaryObject):
+             TagBase, PrimaryObject):
     """
-    Introduction
-    ============
     The Family record is the GRAMPS in-memory representation of the
     relationships between people. It contains all the information
     related to the relationship.
     
-    Usage
-    =====
     Family objects are usually created in one of two ways.
 
     1. Creating a new Family object, which is then initialized and
@@ -89,6 +87,7 @@ class Family(SourceBase, NoteBase, MediaBase, AttributeBase, LdsOrdBase,
         MediaBase.__init__(self)
         AttributeBase.__init__(self)
         LdsOrdBase.__init__(self)
+        TagBase.__init__(self)
         self.father_handle = None
         self.mother_handle = None
         self.child_ref_list = []
@@ -104,15 +103,15 @@ class Family(SourceBase, NoteBase, MediaBase, AttributeBase, LdsOrdBase,
         This method is used to convert the object into a form that can easily 
         be saved to a database.
 
-        These elements may be primative Python types (string, integers),
+        These elements may be primitive Python types (string, integers),
         complex Python types (lists or tuples, or Python objects. If the
-        target database cannot handle complex types (such as objectes or
+        target database cannot handle complex types (such as objects or
         lists), the database is responsible for converting the data into
         a form that it can use.
 
-        @returns: Returns a python tuple containing the data that should
+        :returns: Returns a python tuple containing the data that should
             be considered persistent.
-        @rtype: tuple
+        :rtype: tuple
         """
         return (self.handle, self.gramps_id, self.father_handle,
                 self.mother_handle,
@@ -124,7 +123,7 @@ class Family(SourceBase, NoteBase, MediaBase, AttributeBase, LdsOrdBase,
                 LdsOrdBase.serialize(self),
                 SourceBase.serialize(self),
                 NoteBase.serialize(self),
-                self.change, self.marker.serialize(), self.private)
+                self.change, TagBase.serialize(self), self.private)
 
     def unserialize(self, data):
         """
@@ -134,34 +133,33 @@ class Family(SourceBase, NoteBase, MediaBase, AttributeBase, LdsOrdBase,
         (self.handle, self.gramps_id, self.father_handle, self.mother_handle,
          child_ref_list, the_type, event_ref_list, media_list,
          attribute_list, lds_seal_list, source_list, note_list,
-         self.change, marker, self.private) = data
+         self.change, tag_list, self.private) = data
 
-        self.marker = InstanceType(MarkerType)
-        self.marker.unserialize(marker)
-        self.type = InstanceType(FamilyRelType)
+        self.type = FamilyRelType()
         self.type.unserialize(the_type)
-        self.event_ref_list = [InstanceType(EventRef).unserialize(er)
+        self.event_ref_list = [EventRef().unserialize(er)
                                for er in event_ref_list]
-        self.child_ref_list = [InstanceType(ChildRef).unserialize(cr)
+        self.child_ref_list = [ChildRef().unserialize(cr)
                                for cr in child_ref_list]
         MediaBase.unserialize(self, media_list)
         AttributeBase.unserialize(self, attribute_list)
         SourceBase.unserialize(self, source_list)
         NoteBase.unserialize(self, note_list)
         LdsOrdBase.unserialize(self, lds_seal_list)
+        TagBase.unserialize(self, tag_list)
 
     def _has_handle_reference(self, classname, handle):
         """
         Return True if the object has reference to a given handle of given 
         primary object type.
         
-        @param classname: The name of the primary object class.
-        @type classname: str
-        @param handle: The handle to be checked.
-        @type handle: str
-        @return: Returns whether the object has reference to this handle of 
+        :param classname: The name of the primary object class.
+        :type classname: str
+        :param handle: The handle to be checked.
+        :type handle: str
+        :returns: Returns whether the object has reference to this handle of 
                 this object type.
-        @rtype: bool
+        :rtype: bool
         """
         if classname == 'Event':
             return handle in [ref.ref for ref in self.event_ref_list]
@@ -176,18 +174,18 @@ class Family(SourceBase, NoteBase, MediaBase, AttributeBase, LdsOrdBase,
         """
         Remove all references in this object to object handles in the list.
 
-        @param classname: The name of the primary object class.
-        @type classname: str
-        @param handle_list: The list of handles to be removed.
-        @type handle_list: str
+        :param classname: The name of the primary object class.
+        :type classname: str
+        :param handle_list: The list of handles to be removed.
+        :type handle_list: str
         """
         if classname == 'Event':
-            new_list = [ ref for ref in self.event_ref_list \
-                                        if ref.ref not in handle_list ]
+            new_list = [ref for ref in self.event_ref_list
+                            if ref.ref not in handle_list]
             self.event_ref_list = new_list
         elif classname == 'Person':
-            new_list = [ ref for ref in self.child_ref_list \
-                                        if ref.ref not in handle_list ]
+            new_list = [ref for ref in self.child_ref_list
+                            if ref.ref not in handle_list]
             self.child_ref_list = new_list
             if self.father_handle in handle_list:
                 self.father_handle = None
@@ -202,25 +200,49 @@ class Family(SourceBase, NoteBase, MediaBase, AttributeBase, LdsOrdBase,
         """
         Replace all references to old handle with those to the new handle.
 
-        @param classname: The name of the primary object class.
-        @type classname: str
-        @param old_handle: The handle to be replaced.
-        @type old_handle: str
-        @param new_handle: The handle to replace the old one with.
-        @type new_handle: str
+        :param classname: The name of the primary object class.
+        :type classname: str
+        :param old_handle: The handle to be replaced.
+        :type old_handle: str
+        :param new_handle: The handle to replace the old one with.
+        :type new_handle: str
         """
         if classname == 'Event':
-            handle_list = [ref.ref for ref in self.event_ref_list]
-            while old_handle in handle_list:
-                ix = handle_list.index(old_handle)
-                self.event_ref_list[ix].ref = new_handle
-                handle_list[ix] = ''
+            refs_list = [ ref.ref for ref in self.event_ref_list ]
+            new_ref = None
+            if new_handle in refs_list:
+                new_ref = self.event_ref_list[refs_list.index(new_handle)]
+            n_replace = refs_list.count(old_handle)
+            for ix_replace in xrange(n_replace):
+                idx = refs_list.index(old_handle)
+                self.event_ref_list[idx].ref = new_handle
+                refs_list[idx] = new_handle
+                if new_ref:
+                    evt_ref = self.event_ref_list[idx]
+                    equi = new_ref.is_equivalent(evt_ref)
+                    if equi != DIFFERENT:
+                        if equi == EQUAL:
+                            new_ref.merge(evt_ref)
+                        self.event_ref_list.pop(idx)
+                        refs_list.pop(idx)
         elif classname == 'Person':
-            handle_list = [ref.ref for ref in self.child_ref_list]
-            while old_handle in handle_list:
-                ix = handle_list.index(old_handle)
-                self.child_ref_list[ix].ref = new_handle
-                handle_list[ix] = ''
+            refs_list = [ ref.ref for ref in self.child_ref_list ]
+            new_ref = None
+            if new_handle in refs_list:
+                new_ref = self.child_ref_list[refs_list.index(new_handle)]
+            n_replace = refs_list.count(old_handle)
+            for ix_replace in xrange(n_replace):
+                idx = refs_list.index(old_handle)
+                self.child_ref_list[idx].ref = new_handle
+                refs_list[idx] = new_handle
+                if new_ref:
+                    child_ref = self.child_ref_list[idx]
+                    equi = new_ref.is_equivalent(child_ref)
+                    if equi != DIFFERENT:
+                        if equi == EQUAL:
+                            new_ref.merge(child_ref)
+                        self.child_ref_list.pop(idx)
+                        refs_list.pop(idx)
             if self.father_handle == old_handle:
                 self.father_handle = new_handle
             if self.mother_handle == old_handle:
@@ -234,8 +256,8 @@ class Family(SourceBase, NoteBase, MediaBase, AttributeBase, LdsOrdBase,
         """
         Return the list of all textual attributes of the object.
 
-        @return: Returns the list of all textual attributes of the object.
-        @rtype: list
+        :returns: Returns the list of all textual attributes of the object.
+        :rtype: list
         """
         return [self.gramps_id]
 
@@ -243,10 +265,10 @@ class Family(SourceBase, NoteBase, MediaBase, AttributeBase, LdsOrdBase,
         """
         Return the list of child objects that may carry textual data.
 
-        @return: Returns the list of child objects that may carry textual data.
-        @rtype: list
+        :returns: Returns the list of child objects that may carry textual data.
+        :rtype: list
         """
-        add_list = [item for item in self.lds_ord_list if item]
+        add_list = filter(None, self.lds_ord_list)
         return self.media_list + self.attribute_list + \
                 self.source_list + add_list
 
@@ -254,24 +276,26 @@ class Family(SourceBase, NoteBase, MediaBase, AttributeBase, LdsOrdBase,
         """
         Return the list of child secondary objects that may refer sources.
 
-        @return: Returns the list of child secondary child objects that may 
+        :returns: Returns the list of child secondary child objects that may 
                 refer sources.
-        @rtype: list
+        :rtype: list
         """
         check_list = self.media_list + self.attribute_list + \
-            self.lds_ord_list + self.child_ref_list
+            self.lds_ord_list + self.child_ref_list + \
+            self.event_ref_list
         return check_list
 
     def get_note_child_list(self):
         """
         Return the list of child secondary objects that may refer notes.
 
-        @return: Returns the list of child secondary child objects that may 
+        :returns: Returns the list of child secondary child objects that may 
                 refer notes.
-        @rtype: list
+        :rtype: list
         """
         check_list = self.media_list + self.attribute_list + \
-            self.lds_ord_list + self.child_ref_list + self.source_list
+            self.lds_ord_list + self.child_ref_list + self.source_list + \
+            self.event_ref_list
         return check_list
 
     def get_referenced_handles(self):
@@ -279,15 +303,15 @@ class Family(SourceBase, NoteBase, MediaBase, AttributeBase, LdsOrdBase,
         Return the list of (classname, handle) tuples for all directly
         referenced primary objects.
         
-        @return: List of (classname, handle) tuples for referenced objects.
-        @rtype: list
+        :returns: List of (classname, handle) tuples for referenced objects.
+        :rtype: list
         """
         ret = self.get_referenced_note_handles()
-        ret += [('Event', ref.ref) for ref in self.event_ref_list]
         ret += [('Person', handle) for handle
                 in ([ref.ref for ref in self.child_ref_list] +
                     [self.father_handle, self.mother_handle])
                 if handle]
+        ret += self.get_referenced_tag_handles()
         return ret
 
     def get_handle_referents(self):
@@ -295,10 +319,31 @@ class Family(SourceBase, NoteBase, MediaBase, AttributeBase, LdsOrdBase,
         Return the list of child objects which may, directly or through their 
         children, reference primary objects..
         
-        @return: Returns the list of objects refereincing primary objects.
-        @rtype: list
+        :returns: Returns the list of objects referencing primary objects.
+        :rtype: list
         """
-        return self.get_sourcref_child_list() + self.source_list
+        return self.get_sourcref_child_list() + self.source_list 
+
+    def merge(self, acquisition):
+        """
+        Merge the content of acquisition into this family.
+
+        Lost: handle, id, relation, father, mother of acquisition.
+
+        :param acquisition: The family to merge with the present family.
+        :rtype acquisition: Family
+        """
+        if self.type != acquisition.type and self.type == FamilyRelType.UNKNOWN:
+            self.set_relationship(acquisition.get_relationship())
+        self._merge_privacy(acquisition)
+        self._merge_event_ref_list(acquisition)
+        self._merge_lds_ord_list(acquisition)
+        self._merge_media_list(acquisition)
+        self._merge_child_ref_list(acquisition)
+        self._merge_attribute_list(acquisition)
+        self._merge_note_list(acquisition)
+        self._merge_source_reference_list(acquisition)
+        self._merge_tag_list(acquisition)
 
     def set_relationship(self, relationship_type):
         """
@@ -322,9 +367,9 @@ class Family(SourceBase, NoteBase, MediaBase, AttributeBase, LdsOrdBase,
                 between the two individuals does not match any of the
                 other types.
 
-        @param relationship_type: (int,str) tuple of the relationship type
+        :param relationship_type: (int,str) tuple of the relationship type
             between the father and mother of the relationship.
-        @type relationship_type: tuple
+        :type relationship_type: tuple
         """
         self.type.set(relationship_type)
 
@@ -334,59 +379,59 @@ class Family(SourceBase, NoteBase, MediaBase, AttributeBase, LdsOrdBase,
         father and mother in the relationship.
         """
         return self.type
-    
+
     def set_father_handle(self, person_handle):
         """
-        Set the database handle for L{Person} that corresponds to male of the 
+        Set the database handle for :class:`~gen.lib.person.Person` that corresponds to male of the 
         relationship. 
         
         For a same sex relationship, this can represent either of people 
         involved in the relationship.
 
-        @param person_handle: L{Person} database handle
-        @type person_handle: str
+        :param person_handle: :class:`~gen.lib.person.Person` database handle
+        :type person_handle: str
         """
         self.father_handle = person_handle
 
     def get_father_handle(self):
         """
-        Return the database handle of the L{Person} identified as the father 
+        Return the database handle of the :class:`~gen.lib.person.Person` identified as the father 
         of the Family.
 
-        @returns: L{Person} database handle
-        @rtype: str
+        :returns: :class:`~gen.lib.person.Person` database handle
+        :rtype: str
         """
         return self.father_handle
 
     def set_mother_handle(self, person_handle):
         """
-        Set the database handle for L{Person} that corresponds to male of the 
+        Set the database handle for :class:`~gen.lib.person.Person` that corresponds to male of the 
         relationship. 
         
         For a same sex relationship, this can represent either of people 
         involved in the relationship.
 
-        @param person_handle: L{Person} database handle
-        @type person_handle: str
+        :param person_handle: :class:`~gen.lib.person.Person` database handle
+        :type person_handle: str
         """
         self.mother_handle = person_handle
 
     def get_mother_handle(self):
         """
-        Return the database handle of the L{Person} identified as the mother 
+        Return the database handle of the :class:`~gen.lib.person.Person` identified as the mother 
         of the Family.
 
-        @returns: L{Person} database handle
-        @rtype: str
+        :returns: :class:`~gen.lib.person.Person` database handle
+        :rtype: str
         """
         return self.mother_handle
 
     def add_child_ref(self, child_ref):
         """
-        Add the database handle for L{Person} to the Family's list of children.
+        Add the database handle for :class:`~gen.lib.person.Person` to the Family's list of children.
 
-        @param child_ref: Child Reference instance
-        @type  child_ref: ChildRef
+        :param child_ref: Child Reference instance
+        :type  child_ref: ChildRef
         """
         if not isinstance(child_ref, ChildRef):
             raise ValueError("expecting ChildRef instance")
@@ -394,14 +439,14 @@ class Family(SourceBase, NoteBase, MediaBase, AttributeBase, LdsOrdBase,
             
     def remove_child_ref(self, child_ref):
         """
-        Remove the database handle for L{Person} to the Family's list of 
-        children if the L{Person} is already in the list.
+        Remove the database handle for :class:`~gen.lib.person.Person` to the Family's list of 
+        children if the :class:`~gen.lib.person.Person` is already in the list.
 
-        @param child_ref: Child Reference instance
-        @type child_ref: ChildRef
-        @return: True if the handle was removed, False if it was not
+        :param child_ref: Child Reference instance
+        :type child_ref: ChildRef
+        :returns: True if the handle was removed, False if it was not
             in the list.
-        @rtype: bool
+        :rtype: bool
         """
         if not isinstance(child_ref, ChildRef):
             raise ValueError("expecting ChildRef instance")
@@ -411,14 +456,14 @@ class Family(SourceBase, NoteBase, MediaBase, AttributeBase, LdsOrdBase,
 
     def remove_child_handle(self, child_handle):
         """
-        Remove the database handle for L{Person} to the Family's list of 
-        children if the L{Person} is already in the list.
+        Remove the database handle for :class:`~gen.lib.person.Person` to the Family's list of 
+        children if the :class:`~gen.lib.person.Person` is already in the list.
 
-        @param child_handle: L{Person} database handle
-        @type  child_handle: str
-        @return: True if the handle was removed, False if it was not
+        :param child_handle: :class:`~gen.lib.person.Person` database handle
+        :type  child_handle: str
+        :returns: True if the handle was removed, False if it was not
             in the list.
-        @rtype: bool
+        :rtype: bool
         """
         new_list = [ref for ref in self.child_ref_list
                     if ref.ref != child_handle ]
@@ -426,12 +471,12 @@ class Family(SourceBase, NoteBase, MediaBase, AttributeBase, LdsOrdBase,
 
     def get_child_ref_list(self):
         """
-        Return the list of L{ChildRef} handles identifying the children of the 
+        Return the list of :class:`~gen.lib.childref.ChildRef` handles identifying the children of the 
         Family.
 
-        @return: Returns the list of L{ChildRef} handles assocated with
+        :returns: Returns the list of :class:`~gen.lib.childref.ChildRef` handles associated with
             the Family.
-        @rtype: list
+        :rtype: list
         """
         return self.child_ref_list
 
@@ -439,22 +484,42 @@ class Family(SourceBase, NoteBase, MediaBase, AttributeBase, LdsOrdBase,
         """
         Assign the passed list to the Family's list children.
 
-        @param child_ref_list: List of Child Reference instances to be
+        :param child_ref_list: List of Child Reference instances to be
                                associated as the Family's list of children.
-        @type child_ref_list: list of L{ChildRef} instances
+        :type child_ref_list: list of :class:`~gen.lib.childref.ChildRef` instances
         """
         self.child_ref_list = child_ref_list
 
+    def _merge_child_ref_list(self, acquisition):
+        """
+        Merge the list of child references from acquisition with our own.
+
+        :param acquisition: the childref list of this family will be merged
+            with the current childref list.
+        :rtype acquisition: Family
+        """
+        childref_list = self.child_ref_list[:]
+        for addendum in acquisition.get_child_ref_list():
+            for childref in childref_list:
+                equi = childref.is_equivalent(addendum)
+                if equi == IDENTICAL:
+                    break
+                elif equi == EQUAL:
+                    childref.merge(addendum)
+                    break
+            else:
+                self.child_ref_list.append(addendum)
+
     def add_event_ref(self, event_ref):
         """
-        Add the L{EventRef} to the Family instance's L{EventRef} list.
+        Add the :class:`~gen.lib.eventref.EventRef` to the Family instance's :class:`~gen.lib.eventref.EventRef` list.
         
-        This is accomplished by assigning the L{EventRef} for the valid
-        L{Event}in the current database.
+        This is accomplished by assigning the :class:`~gen.lib.eventref.EventRef` for the valid
+        :class:`~gen.lib.event.Event` in the current database.
         
-        @param event_ref: the L{EventRef} to be added to the
-            Person's L{EventRef} list.
-        @type event_ref: EventRef
+        :param event_ref: the :class:`~gen.lib.eventref.EventRef` to be added to the
+            Person's :class:`~gen.lib.eventref.EventRef` list.
+        :type event_ref: EventRef
         """
         if event_ref and not isinstance(event_ref, EventRef):
             raise ValueError("Expecting EventRef instance")
@@ -472,20 +537,41 @@ class Family(SourceBase, NoteBase, MediaBase, AttributeBase, LdsOrdBase,
 
     def get_event_ref_list(self) :
         """
-        Return the list of L{EventRef} objects associated with L{Event}
+        Return the list of :class:`~gen.lib.eventref.EventRef` objects associated with :class:`~gen.lib.event.Event`
         instances.
 
-        @returns: Returns the list of L{EventRef} objects associated with
+        :returns: Returns the list of :class:`~gen.lib.eventref.EventRef` objects associated with
             the Family instance.
-        @rtype: list
+        :rtype: list
         """
         return self.event_ref_list
 
     def set_event_ref_list(self, event_ref_list) :
         """
-        Set the Family instance's L{EventRef} list to the passed list.
+        Set the Family instance's :class:`~gen.lib.eventref.EventRef` list to the passed list.
 
-        @param event_ref_list: List of valid L{EventRef} objects
-        @type event_ref_list: list
+        :param event_ref_list: List of valid :class:`~gen.lib.eventref.EventRef` objects
+        :type event_ref_list: list
         """
         self.event_ref_list = event_ref_list
+
+    def _merge_event_ref_list(self, acquisition):
+        """
+        Merge the list of event references from acquisition with our own.
+
+        :param acquisition: the event references list of this object will be
+            merged with the current event references list.
+        :rtype acquisition: Person
+        """
+        eventref_list = self.event_ref_list[:]
+        for addendum in acquisition.get_event_ref_list():
+            for eventref in eventref_list:
+                equi = eventref.is_equivalent(addendum)
+                if equi == IDENTICAL:
+                    break
+                elif equi == EQUAL:
+                    eventref.merge(addendum)
+                    break
+            else:
+                self.event_ref_list.append(addendum)
+
