@@ -2,6 +2,8 @@
 # Gramps - a GTK+/GNOME based genealogy program
 #
 # Copyright (C) 2000-2007  Donald N. Allingham
+# Copyright (C) 2010       Michiel D. Nauta
+# Copyright (C) 2010       Nick Hall
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,7 +20,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: mediaobj.py 12984 2009-08-13 08:52:05Z acraphae $
+# $Id: mediaobj.py 16025 2010-10-22 23:22:33Z nick-h $
 
 """
 Media object for GRAMPS.
@@ -41,7 +43,7 @@ from gen.lib.srcbase import SourceBase
 from gen.lib.notebase import NoteBase
 from gen.lib.datebase import DateBase
 from gen.lib.attrbase import AttributeBase
-from gen.lib.markertype import MarkerType
+from gen.lib.tagbase import TagBase
 
 #-------------------------------------------------------------------------
 #
@@ -49,9 +51,9 @@ from gen.lib.markertype import MarkerType
 #
 #-------------------------------------------------------------------------
 class MediaObject(SourceBase, NoteBase, DateBase, AttributeBase,
-                  PrimaryObject):
+                  TagBase, PrimaryObject):
     """
-    Containter for information about an image file, including location,
+    Container for information about an image file, including location,
     description and privacy.
     """
     
@@ -70,6 +72,7 @@ class MediaObject(SourceBase, NoteBase, DateBase, AttributeBase,
         NoteBase.__init__(self, source)
         DateBase.__init__(self, source)
         AttributeBase.__init__(self, source)
+        TagBase.__init__(self)
 
         if source:
             self.path = source.path
@@ -90,9 +93,9 @@ class MediaObject(SourceBase, NoteBase, DateBase, AttributeBase,
         This method is used to convert the object into a form that can easily 
         be saved to a database.
 
-        These elements may be primative Python types (string, integers),
+        These elements may be primitive Python types (string, integers),
         complex Python types (lists or tuples, or Python objects. If the
-        target database cannot handle complex types (such as objectes or
+        target database cannot handle complex types (such as objects or
         lists), the database is responsible for converting the data into
         a form that it can use.
 
@@ -106,7 +109,7 @@ class MediaObject(SourceBase, NoteBase, DateBase, AttributeBase,
                 NoteBase.serialize(self),
                 self.change,
                 DateBase.serialize(self, no_text_date),
-                self.marker.serialize(),
+                TagBase.serialize(self),
                 self.private)
 
     def unserialize(self, data):
@@ -119,14 +122,13 @@ class MediaObject(SourceBase, NoteBase, DateBase, AttributeBase,
         """
         (self.handle, self.gramps_id, self.path, self.mime, self.desc,
          attribute_list, source_list, note_list, self.change,
-         date, marker, self.private) = data
+         date, tag_list, self.private) = data
 
-        self.marker = MarkerType()
-        self.marker.unserialize(marker)
         AttributeBase.unserialize(self, attribute_list)
         SourceBase.unserialize(self, source_list)
         NoteBase.unserialize(self, note_list)
         DateBase.unserialize(self, date)
+        TagBase.unserialize(self, tag_list)
 
     def get_text_data_list(self):
         """
@@ -174,7 +176,8 @@ class MediaObject(SourceBase, NoteBase, DateBase, AttributeBase,
         :returns: List of (classname, handle) tuples for referenced objects.
         :rtype: list
         """
-        return self.get_referenced_note_handles()
+        return self.get_referenced_note_handles() + \
+               self.get_referenced_tag_handles()
 
     def get_handle_referents(self):
         """
@@ -186,6 +189,21 @@ class MediaObject(SourceBase, NoteBase, DateBase, AttributeBase,
         """
         return self.attribute_list + self.source_list
 
+    def merge(self, acquisition):
+        """
+        Merge the content of acquisition into this media object.
+
+        Lost: handle, id, file, date of acquisition.
+
+        :param acquisition: The media object to merge with the present object.
+        :rtype acquisition: MediaObject
+        """
+        self._merge_privacy(acquisition)
+        self._merge_attribute_list(acquisition)
+        self._merge_note_list(acquisition)
+        self._merge_source_reference_list(acquisition)
+        self._merge_tag_list(acquisition)
+
     def set_mime_type(self, mime_type):
         """
         Set the MIME type associated with the MediaObject.
