@@ -2,6 +2,8 @@
 # Gramps - a GTK+/GNOME based genealogy program
 #
 # Copyright (C) 2000-2007  Donald N. Allingham
+# Copyright (C) 2010       Michiel D. Nauta
+# Copyright (C) 2010       Nick Hall
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,7 +20,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: person.py 14146 2010-01-27 06:05:24Z dsblank $
+# $Id: person.py 16592 2011-02-09 16:03:52Z m_d_n $
 
 """
 Person object for GRAMPS.
@@ -37,12 +39,14 @@ from gen.lib.attrbase import AttributeBase
 from gen.lib.addressbase import AddressBase
 from gen.lib.ldsordbase import LdsOrdBase
 from gen.lib.urlbase import UrlBase
+from gen.lib.tagbase import TagBase
 from gen.lib.name import Name
 from gen.lib.eventref import EventRef
 from gen.lib.personref import PersonRef
 from gen.lib.attrtype import AttributeType
 from gen.lib.eventroletype import EventRoleType
-from gen.lib.markertype import MarkerType
+from gen.lib.attribute import Attribute
+from gen.lib.const import IDENTICAL, EQUAL, DIFFERENT
 
 #-------------------------------------------------------------------------
 #
@@ -50,7 +54,7 @@ from gen.lib.markertype import MarkerType
 #
 #-------------------------------------------------------------------------
 class Person(SourceBase, NoteBase, AttributeBase, MediaBase,
-             AddressBase, UrlBase, LdsOrdBase, PrimaryObject):
+             AddressBase, UrlBase, LdsOrdBase, TagBase, PrimaryObject):
     """
     The Person record is the GRAMPS in-memory representation of an
     individual person. It contains all the information related to
@@ -88,8 +92,8 @@ class Person(SourceBase, NoteBase, AttributeBase, MediaBase,
         AddressBase.__init__(self)
         UrlBase.__init__(self)
         LdsOrdBase.__init__(self)
+        TagBase.__init__(self)
         self.primary_name = Name()
-        self.marker = MarkerType()
         self.event_ref_list = []
         self.family_list = []
         self.parent_family_list = []
@@ -107,6 +111,9 @@ class Person(SourceBase, NoteBase, AttributeBase, MediaBase,
 
     def __eq__(self, other):
         return isinstance(other, Person) and self.handle == other.handle
+
+    def __ne__(self, other):
+        return not self == other
         
     def serialize(self):
         """
@@ -116,9 +123,9 @@ class Person(SourceBase, NoteBase, AttributeBase, MediaBase,
         This method is used to convert the object into a form that can easily 
         be saved to a database.
 
-        These elements may be primative Python types (string, integers), 
+        These elements may be primitive Python types (string, integers), 
         complex Python types (lists or tuples, or Python objects. If the
-        target database cannot handle complex types (such as objectes or
+        target database cannot handle complex types (such as objects or
         lists), the database is responsible for converting the data into
         a form that it can use.
 
@@ -145,7 +152,7 @@ class Person(SourceBase, NoteBase, AttributeBase, MediaBase,
             SourceBase.serialize(self),                          # 15
             NoteBase.serialize(self),                            # 16
             self.change,                                         # 17
-            self.marker.serialize(),                             # 18
+            TagBase.serialize(self),                             # 18
             self.private,                                        # 19
             [pr.serialize() for pr in self.person_ref_list]      # 20
             )
@@ -177,13 +184,11 @@ class Person(SourceBase, NoteBase, AttributeBase, MediaBase,
          source_list,             # 15
          note_list,               # 16
          self.change,             # 17
-         marker,                  # 18
+         tag_list,                # 18
          self.private,            # 19
          person_ref_list,         # 20
          ) = data
 
-        self.marker = MarkerType()
-        self.marker.unserialize(marker)
         self.primary_name = Name()
         self.primary_name.unserialize(primary_name)
         self.alternate_names = [Name().unserialize(name)
@@ -199,6 +204,7 @@ class Person(SourceBase, NoteBase, AttributeBase, MediaBase,
         UrlBase.unserialize(self, urls)
         SourceBase.unserialize(self, source_list)
         NoteBase.unserialize(self, note_list)
+        TagBase.unserialize(self, tag_list)
         return self
             
     def _has_handle_reference(self, classname, handle):
@@ -220,7 +226,8 @@ class Person(SourceBase, NoteBase, AttributeBase, MediaBase,
             return any(ref.ref == handle for ref in self.person_ref_list)
         elif classname == 'Family':
             return any(ref == handle 
-                for ref in self.family_list + self.parent_family_list)
+                for ref in self.family_list + self.parent_family_list +
+                [ordinance.famc for ordinance in self.lds_ord_list])
         elif classname == 'Place':
             return any(ordinance.place == handle 
                 for ordinance in self.lds_ord_list)
@@ -236,12 +243,12 @@ class Person(SourceBase, NoteBase, AttributeBase, MediaBase,
                         if ref.ref not in handle_list]
             # If deleting removing the reference to the event
             # to which birth or death ref_index points, unset the index
-            if (self.birth_ref_index != -1) \
-                   and (self.event_ref_list[self.birth_ref_index].ref
+            if (self.birth_ref_index != -1
+                   and self.event_ref_list[self.birth_ref_index].ref
                         in handle_list):
                 self.set_birth_ref(None)
-            if (self.death_ref_index != -1) \
-                   and (self.event_ref_list[self.death_ref_index].ref
+            if (self.death_ref_index != -1
+                   and self.event_ref_list[self.death_ref_index].ref
                         in handle_list):
                 self.set_death_ref(None)
             self.event_ref_list = new_list
@@ -253,15 +260,18 @@ class Person(SourceBase, NoteBase, AttributeBase, MediaBase,
                 self.set_death_ref(death_ref)
         elif classname == 'Person':
             new_list = [ref for ref in self.person_ref_list
-                        if ref.ref not in handle_list]
+                            if ref.ref not in handle_list]
             self.person_ref_list = new_list
         elif classname == 'Family':
-            new_list = [ handle for handle in self.family_list
-                         if handle not in handle_list ]
+            new_list = [handle for handle in self.family_list
+                            if handle not in handle_list]
             self.family_list = new_list
-            new_list = [ handle for handle in self.parent_family_list \
-                                        if handle not in handle_list ]
+            new_list = [handle for handle in self.parent_family_list
+                            if handle not in handle_list]
             self.parent_family_list = new_list
+            for ordinance in self.lds_ord_list:
+                if ordinance.famc in handle_list:
+                    ordinance.famc = None
         elif classname == 'Place':
             for ordinance in self.lds_ord_list:
                 if ordinance.place in handle_list:
@@ -269,26 +279,66 @@ class Person(SourceBase, NoteBase, AttributeBase, MediaBase,
 
     def _replace_handle_reference(self, classname, old_handle, new_handle):
         if classname == 'Event':
-            handle_list = [ref.ref for ref in self.event_ref_list]
-            while old_handle in handle_list:
-                ix = handle_list.index(old_handle)
-                self.event_ref_list[ix].ref = new_handle
-                handle_list[ix] = ''
+            refs_list = [ref.ref for ref in self.event_ref_list]
+            new_ref = None
+            if new_handle in refs_list:
+                new_ref = self.event_ref_list[refs_list.index(new_handle)]
+            n_replace = refs_list.count(old_handle)
+            for ix_replace in xrange(n_replace):
+                idx = refs_list.index(old_handle)
+                self.event_ref_list[idx].ref = new_handle
+                refs_list[idx] = new_handle
+                if new_ref:
+                    evt_ref = self.event_ref_list[idx]
+                    equi = new_ref.is_equivalent(evt_ref)
+                    if equi != DIFFERENT:
+                        if equi == EQUAL:
+                            new_ref.merge(evt_ref)
+                        self.event_ref_list.pop(idx)
+                        refs_list.pop(idx)
+                        if idx < self.birth_ref_index:
+                            self.birth_ref_index -= 1
+                        elif idx == self.birth_ref_index:
+                            self.birth_ref_index = -1
+                            # birth_ref_index should be recalculated which
+                            # needs database access!
+                        if idx < self.death_ref_index:
+                            self.death_ref_index -= 1
+                        elif idx == self.death_ref_index:
+                            self.death_ref_index = -1
+                            # death_ref_index should be recalculated which
+                            # needs database access!
         elif classname == 'Person':
-            handle_list = [ref.ref for ref in self.person_ref_list]
-            while old_handle in handle_list:
-                ix = handle_list.index(old_handle)
-                self.person_ref_list[ix].ref = new_handle
-                handle_list[ix] = ''
+            refs_list = [ ref.ref for ref in self.person_ref_list ]
+            new_ref = None
+            if new_handle in refs_list:
+                new_ref = self.person_ref_list[refs_list.index(new_handle)]
+            n_replace = refs_list.count(old_handle)
+            for ix_replace in xrange(n_replace):
+                idx = refs_list.index(old_handle)
+                self.person_ref_list[idx].ref = new_handle
+                refs_list[idx] = new_handle
+                if new_ref:
+                    person_ref = self.person_ref_list[idx]
+                    equi = new_ref.is_equivalent(person_ref)
+                    if equi != DIFFERENT:
+                        if equi == EQUAL:
+                            new_ref.merge(person_ref)
+                        self.person_ref_list.pop(idx)
+                        refs_list.pop(idx)
         elif classname == 'Family':
             while old_handle in self.family_list:
                 ix = self.family_list.index(old_handle)
                 self.family_list[ix] = new_handle
-
             while old_handle in self.parent_family_list:
                 ix = self.parent_family_list.index(old_handle)
                 self.parent_family_list[ix] = new_handle
-        elif classname == 'Place':
+            handle_list = [ordinance.famc for ordinance in self.lds_ord_list]
+            while old_handle in handle_list:
+                ix = handle_list.index(old_handle)
+                self.lds_ord_list[ix].famc = new_handle
+                handle_list[ix] = ''
+        elif classname == "Place":
             handle_list = [ordinance.place for ordinance in self.lds_ord_list]
             while old_handle in handle_list:
                 ix = handle_list.index(old_handle)
@@ -313,11 +363,17 @@ class Person(SourceBase, NoteBase, AttributeBase, MediaBase,
         """
         check_list = self.lds_ord_list
         add_list = filter(None, check_list)
-        return [self.primary_name] + self.media_list + \
-                    self.alternate_names + self.address_list + \
-                    self.attribute_list + self.urls + \
-                    self.source_list + self.event_ref_list + add_list + \
-                    self.person_ref_list
+        return ([self.primary_name] +
+                 self.media_list +
+                 self.alternate_names +
+                 self.address_list +
+                 self.attribute_list +
+                 self.urls +
+                 self.source_list +
+                 self.event_ref_list +
+                 add_list +
+                 self.person_ref_list
+                ) 
 
     def get_sourcref_child_list(self):
         """
@@ -327,10 +383,15 @@ class Person(SourceBase, NoteBase, AttributeBase, MediaBase,
                 refer sources.
         :rtype: list
         """
-        return [self.primary_name] + self.media_list + \
-                    self.alternate_names + self.address_list + \
-                    self.attribute_list + self.lds_ord_list + \
-                    self.person_ref_list + self.event_ref_list
+        return ([self.primary_name] +
+                 self.media_list +
+                 self.alternate_names +
+                 self.address_list +
+                 self.attribute_list +
+                 self.lds_ord_list +
+                 self.person_ref_list +
+                 self.event_ref_list
+                )
 
     def get_note_child_list(self):
         """
@@ -340,11 +401,16 @@ class Person(SourceBase, NoteBase, AttributeBase, MediaBase,
                 refer notes.
         :rtype: list
         """
-        return [self.primary_name] + self.media_list + \
-                    self.alternate_names + self.address_list + \
-                    self.attribute_list + self.lds_ord_list + \
-                    self.person_ref_list + self.source_list + \
-                    self.event_ref_list
+        return ([self.primary_name] +
+                 self.media_list +
+                 self.alternate_names +
+                 self.address_list +
+                 self.attribute_list +
+                 self.lds_ord_list +
+                 self.person_ref_list +
+                 self.source_list +
+                 self.event_ref_list
+                )
 
     def get_referenced_handles(self):
         """
@@ -355,8 +421,10 @@ class Person(SourceBase, NoteBase, AttributeBase, MediaBase,
         :rtype: list
         """
         return [('Family', handle) for handle in
-                (self.family_list + self.parent_family_list)] \
-                + self.get_referenced_note_handles()
+                (self.family_list + self.parent_family_list)] + (
+                 self.get_referenced_note_handles() +
+                 self.get_referenced_tag_handles()
+                )
 
     def get_handle_referents(self):
         """
@@ -369,6 +437,38 @@ class Person(SourceBase, NoteBase, AttributeBase, MediaBase,
         #don't count double, notes can be found in sourcref
         return self.get_sourcref_child_list() + self.source_list
 
+    def merge(self, acquisition):
+        """
+        Merge the content of acquisition into this person.
+
+        :param acquisition: The person to merge with the present person.
+        :rtype acquisition: Person
+        """
+        acquisition_id = acquisition.get_gramps_id()
+        if acquisition_id:
+            attr = Attribute()
+            attr.set_type("Merged Gramps ID")
+            attr.set_value(acquisition.get_gramps_id())
+            self.add_attribute(attr)
+
+        self._merge_privacy(acquisition)
+        acquisition.alternate_names.insert(0, acquisition.get_primary_name())
+        self._merge_alternate_names(acquisition)
+        self._merge_event_ref_list(acquisition)
+        self._merge_lds_ord_list(acquisition)
+        self._merge_media_list(acquisition)
+        self._merge_address_list(acquisition)
+        self._merge_attribute_list(acquisition)
+        self._merge_url_list(acquisition)
+        self._merge_person_ref_list(acquisition)
+        self._merge_note_list(acquisition)
+        self._merge_source_reference_list(acquisition)
+        self._merge_tag_list(acquisition)
+
+        map(self.add_parent_family_handle,
+            acquisition.get_parent_family_handle_list())
+        map(self.add_family_handle, acquisition.get_family_handle_list())
+
     def set_primary_name(self, name):
         """
         Set the primary name of the Person to the specified :class:`~gen.lib.name.Name` instance.
@@ -405,6 +505,29 @@ class Person(SourceBase, NoteBase, AttributeBase, MediaBase,
         """
         self.alternate_names = alt_name_list
 
+    def _merge_alternate_names(self, acquisition):
+        """
+        Merge the list of alternate names from acquisition with our own.
+
+        :param acquisition: the list of alternate names of this object will be
+            merged with the current alternate name list.
+        :rtype acquisition: Person
+        """
+        name_list = self.alternate_names[:]
+        primary_name = self.get_primary_name()
+        if primary_name and not primary_name.is_empty():
+            name_list.insert(0, primary_name)
+        for addendum in acquisition.get_alternate_names():
+            for name in name_list:
+                equi = name.is_equivalent(addendum)
+                if equi == IDENTICAL:
+                    break
+                elif equi == EQUAL:
+                    name.merge(addendum)
+                    break
+            else:
+                self.alternate_names.append(addendum)
+
     def add_alternate_name(self, name):
         """
         Add a :class:`~gen.lib.name.Name` instance to the list of alternative names.
@@ -415,6 +538,9 @@ class Person(SourceBase, NoteBase, AttributeBase, MediaBase,
         self.alternate_names.append(name)
 
     def get_nick_name(self):
+        for name in [self.get_primary_name()] + self.get_alternate_names():
+            if name.get_nick_name():
+                return name.get_nick_name()
         for attr in self.attribute_list:
             if int(attr.type) == AttributeType.NICKNAME:
                 return attr.get_value()
@@ -579,6 +705,32 @@ class Person(SourceBase, NoteBase, AttributeBase, MediaBase,
         """
         self.event_ref_list = event_ref_list
 
+    def _merge_event_ref_list(self, acquisition):
+        """
+        Merge the list of event references from acquisition with our own.
+
+        :param acquisition: the event references list of this object will be
+            merged with the current event references list.
+        :rtype acquisition: Person
+        """
+        eventref_list = self.event_ref_list[:]
+        for idx, addendum in enumerate(acquisition.get_event_ref_list()):
+            for eventref in eventref_list:
+                equi = eventref.is_equivalent(addendum)
+                if equi == IDENTICAL:
+                    break
+                elif equi == EQUAL:
+                    eventref.merge(addendum)
+                    break
+            else:
+                self.event_ref_list.append(addendum)
+                if (self.birth_ref_index == -1 and 
+                    idx == acquisition.birth_ref_index):
+                    self.birth_ref_index = len(self.event_ref_list) - 1
+                if (self.death_ref_index == -1 and
+                    idx == acquisition.death_ref_index):
+                    self.death_ref_index = len(self.event_ref_list) - 1
+
     def add_family_handle(self, family_handle):
         """
         Add the :class:`~gen.lib.family.Family` handle to the Person instance's :class:`~gen.lib.family.Family` list.
@@ -807,3 +959,23 @@ class Person(SourceBase, NoteBase, AttributeBase, MediaBase,
         :type person_ref_list: list
         """
         self.person_ref_list = person_ref_list
+
+    def _merge_person_ref_list(self, acquisition):
+        """
+        Merge the list of person references from acquisition with our own.
+
+        :param acquisition: the list of person references of this person will b
+            merged with the current person references list.
+        :rtype acquisition: Person
+        """
+        personref_list = self.person_ref_list[:]
+        for addendum in acquisition.get_person_ref_list():
+            for personref in personref_list:
+                equi = personref.is_equivalent(addendum)
+                if equi == IDENTICAL:
+                    break
+                elif equi == EQUAL:
+                    personref.merge(addendum)
+                    break
+            else:
+                self.person_ref_list.append(addendum)
