@@ -2,6 +2,7 @@
 # Gramps - a GTK+/GNOME based genealogy program
 #
 # Copyright (C) 2000-2007  Donald N. Allingham
+# Copyright (C) 2010       Michiel D. Nauta
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,7 +19,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: place.py 12984 2009-08-13 08:52:05Z acraphae $
+# $Id: place.py 16636 2011-02-16 20:06:40Z gbritton $
 
 """
 Place object for GRAMPS.
@@ -35,7 +36,6 @@ from gen.lib.notebase import NoteBase
 from gen.lib.mediabase import MediaBase
 from gen.lib.urlbase import UrlBase
 from gen.lib.location import Location
-from gen.lib.markertype import MarkerType
 
 _EMPTY_LOC = Location().serialize()
 
@@ -68,7 +68,7 @@ class Place(SourceBase, NoteBase, MediaBase, UrlBase, PrimaryObject):
             self.lat = source.lat
             self.title = source.title
             self.main_loc = Location(source.main_loc)
-            self.alt_loc = [Location(loc) for loc in source.alt_loc]
+            self.alt_loc = map(Location, source.alt_loc)
         else:
             self.long = ""
             self.lat = ""
@@ -84,9 +84,9 @@ class Place(SourceBase, NoteBase, MediaBase, UrlBase, PrimaryObject):
         This method is used to convert the object into a form that can easily 
         be saved to a database.
 
-        These elements may be primative Python types (string, integers),
+        These elements may be primitive Python types (string, integers),
         complex Python types (lists or tuples, or Python objects. If the
-        target database cannot handle complex types (such as objectes or
+        target database cannot handle complex types (such as objects or
         lists), the database is responsible for converting the data into
         a form that it can use.
 
@@ -106,7 +106,7 @@ class Place(SourceBase, NoteBase, MediaBase, UrlBase, PrimaryObject):
                 MediaBase.serialize(self),
                 SourceBase.serialize(self),
                 NoteBase.serialize(self),
-                self.change, self.marker.serialize() ,self.private)
+                self.change, self.private)
 
     def unserialize(self, data):
         """
@@ -119,15 +119,13 @@ class Place(SourceBase, NoteBase, MediaBase, UrlBase, PrimaryObject):
         """
         (self.handle, self.gramps_id, self.title, self.long, self.lat,
          main_loc, alt_loc, urls, media_list, source_list, note_list,
-         self.change, marker, self.private) = data
+         self.change, self.private) = data
 
         if main_loc is None:
             self.main_loc = None
         else:
             self.main_loc = Location().unserialize(main_loc)
         self.alt_loc = [Location().unserialize(al) for al in alt_loc]
-        self.marker = MarkerType()
-        self.marker.unserialize(marker)
         UrlBase.unserialize(self, urls)
         MediaBase.unserialize(self, media_list)
         SourceBase.unserialize(self, source_list)
@@ -194,6 +192,19 @@ class Place(SourceBase, NoteBase, MediaBase, UrlBase, PrimaryObject):
         """
         return self.get_referenced_note_handles()
 
+    def merge(self, acquisition):
+        """ Merge the content of acquisition into this place.
+
+        :param acquisition: The place to merge with the present place.
+        :rtype acquisition: Place
+        """
+        self._merge_privacy(acquisition)
+        self._merge_locations(acquisition)
+        self._merge_media_list(acquisition)
+        self._merge_url_list(acquisition)
+        self._merge_note_list(acquisition)
+        self._merge_source_reference_list(acquisition)
+
     def set_title(self, title):
         """
         Set the descriptive title of the Place object.
@@ -307,6 +318,28 @@ class Place(SourceBase, NoteBase, MediaBase, UrlBase, PrimaryObject):
         if location not in self.alt_loc:
             self.alt_loc.append(location)
 
+    def _merge_locations(self, acquisition):
+        """
+        Add the main and alternate locations of acquisition to the alternate
+        location list.
+
+        :param acquisition: instance to merge
+        :type acquisition: :class:'~gen.lib.place.Place
+        """
+        altloc_list = self.alt_loc[:]
+        if self.main_loc and not self.main_loc.is_empty():
+            altloc_list.insert(0, self.main_loc)
+        add_list = acquisition.get_alternate_locations()
+        acq_main_loc = acquisition.get_main_location()
+        if acq_main_loc and not acq_main_loc.is_empty():
+            add_list.insert(0, acquisition.get_main_location())
+        for addendum in add_list:
+            for altloc in altloc_list:
+                if altloc.is_equal(addendum):
+                    break
+            else:
+                self.alt_loc.append(addendum)
+
     def get_display_info(self):
         """
         Get the display information associated with the object.
