@@ -19,7 +19,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
-# $Id: _manager.py 14915 2010-03-23 13:27:40Z bmcage $
+# $Id: _manager.py 16979 2011-03-30 01:50:08Z pez4brian $
 
 """
 The core of the GRAMPS plugin system. This module provides capability to load
@@ -96,7 +96,8 @@ class BasePluginManager(object):
         self.__registereddir_set = set()
         self.__loaded_plugins = {}
 
-    def reg_plugins(self, direct):
+    def reg_plugins(self, direct, dbstate=None, uistate=None, 
+                    append=True, load_on_reg=False):
         """
         Searches the specified directory, and registers python plugin that
         are being defined in gpr.py files. 
@@ -118,15 +119,64 @@ class BasePluginManager(object):
                 if dirname.startswith(".") or dirname in ["po", "locale"]:
                     dirnames.remove(dirname)
             # add the directory to the python search path
-            sys.path.append(dirpath)
+            if append:
+                sys.path.append(dirpath)
             # if the path has not already been loaded, save it in the 
             # registereddir_list list for use on reloading.
             self.__registereddir_set.add(dirpath)
             self.__pgr.scan_dir(dirpath)
 
-        # load plugins that request to be loaded on startup
-        for plugin in self.__pgr.filter_load_on_reg():
-            mod = self.load_plugin(plugin)
+        if load_on_reg:
+            # Run plugins that request to be loaded on startup and
+            # have a load_on_reg callable.
+            # first, remove hidden
+            plugins_to_load = []
+            for plugin in self.__pgr.filter_load_on_reg():
+                if plugin.id in config.get("plugin.hiddenplugins"):
+                    continue
+                plugins_to_load.append(plugin)
+            # next, sort on dependencies
+            # Probably a more effecient method to get dependency graph:
+            plugins_sorted = []
+            count = 0
+            max_count = len(plugins_to_load)
+            while plugins_to_load:
+                for plugin in plugins_to_load[:]: # copy of list
+                    delay = False
+                    for depend in plugin.depends_on:
+                        if depend not in [p.id for p in plugins_sorted]:
+                            delay = True
+                            break
+                    if delay:
+                        pass # wait till next loop
+                    else:
+                        if plugin not in plugins_sorted:
+                            plugins_sorted.append(plugin)
+                        if plugin in plugins_to_load:
+                            plugins_to_load.remove(plugin)
+                count += 1
+                if count > max_count:
+                    print "Cannot resolve the following plugin dependencies:"
+                    for plugin in plugins_to_load:
+                        print "   Plugin '%s' requires: %s" % (
+                            plugin.id, plugin.depends_on)
+                    break
+            # now load them:
+            for plugin in plugins_sorted:
+                mod = self.load_plugin(plugin)
+                if hasattr(mod, "load_on_reg"):
+                    try:
+                        results = mod.load_on_reg(dbstate, uistate, plugin)
+                    except:
+                        import traceback
+                        traceback.print_exc()
+                        print "Plugin '%s' did not run; continuing..." % plugin.name
+                        continue
+                    try:
+                        iter(results)
+                        plugin.data += results
+                    except:
+                        plugin.data = results
 
     def is_loaded(self, pdata_id):
         """
@@ -165,24 +215,47 @@ class BasePluginManager(object):
                 del self.__failmsg_list[index]
         else:
             self.__attempt_list.append(filename)
-        plugin = pdata.mod_name
         try: 
-            _module = __import__(plugin)
+            _module = self.import_plugin(pdata)
             if need_reload:
                 # For some strange reason second importing of a failed plugin
                 # results in success. Then reload reveals the actual error.
                 # Looks like a bug in Python.
-                reload(_module)
-            self.__success_list.append((filename, _module, pdata))
-            self.__modules[filename] = _module
-            self.__loaded_plugins[pdata.id] = _module
-            self.__mod2text[_module.__name__] = pdata.description
+                _module = self.reload(_module, pdata)
+            if _module:
+                self.__success_list.append((filename, _module, pdata))
+                self.__modules[filename] = _module
+                self.__loaded_plugins[pdata.id] = _module
+                self.__mod2text[_module.__name__] = pdata.description
             return _module
         except:
+            import traceback
+            print traceback.print_exc()
             self.__failmsg_list.append((filename, sys.exc_info(), pdata))
 
         return None
 
+    def import_plugin(self, pdata):
+        """
+        Rather than just __import__(id), this will add the pdata.fpath
+        to sys.path first (if needed), import, and then reset path.
+        """
+        module = None
+        if isinstance(pdata, basestring):
+            pdata = self.get_plugin(pdata)
+        if not pdata:
+            return None
+        if pdata.fpath not in sys.path:
+            if pdata.mod_name:
+                sys.path.insert(0, pdata.fpath)
+                module = __import__(pdata.mod_name)
+                sys.path.pop(0)
+            else:
+                print "WARNING: module cannot be loaded"
+        else:
+            module = __import__(pdata.mod_name)
+        return module
+
     def empty_managed_plugins(self):
         """ For some plugins, managed Plugin are used. These are only 
         reobtained from the registry if this method is called
@@ -216,7 +289,7 @@ class BasePluginManager(object):
                 #module already reloaded, a second plugin in same module
                 continue
             try: 
-                reload(plugin[1])
+                self.reload(plugin[1], pdata)
                 self.__modules[filename] = plugin[1]
                 self.__loaded_plugins[pdata.id] = plugin[1]
             except:
@@ -235,6 +308,18 @@ class BasePluginManager(object):
         for (filename, message, pdata) in oldfailmsg:
             self.load_plugin(pdata)
 
+    def reload(self, module, pdata):
+        """
+        Reloads modules that might not be in the path.
+        """
+        try:
+            reload(module)
+        except:
+            if pdata.mod_name in sys.modules:
+                del sys.modules[pdata.mod_name]
+            module = self.import_plugin(pdata)
+        return module
+
     def get_fail_list(self):
         """ Return the list of failed plugins. """
         return self.__failmsg_list
@@ -288,6 +373,11 @@ class BasePluginManager(object):
         """
         return self.__pgr.gramplet_plugins()
     
+    def get_reg_sidebars(self):
+        """ Return list of registered sidebars.
+        """
+        return self.__pgr.sidebar_plugins()
+    
     def get_external_opt_dict(self):
         """ Return the dictionary of external options. """
         return self.__external_opt_dict
@@ -311,6 +401,66 @@ class BasePluginManager(object):
         """
         return self.__pgr.docgen_plugins()
     
+    def get_reg_general(self, category=None):
+        """ Return list of registered general libs
+        """
+        return self.__pgr.general_plugins(category)
+
+    def load_plugin_category(self, category):
+        """
+        Make sure all plugins of a type are loaded.
+        """
+        for plugin in self.__pgr.general_plugins(category):
+            if not self.is_loaded(plugin):
+                self.load_plugin(plugin)
+
+    def get_plugin_data(self, category):
+        """
+        Gets all of the data from general plugins of type category.
+        plugin.data maybe a single item, an iterable, or a callable.
+
+        >>> PLUGMAN.get_plugin_data('CSS')
+        <a list of raw data items>
+        """
+        retval = []
+        data = None
+        for plugin in self.__pgr.general_plugins(category):
+            data = plugin.data
+            try:
+                iter(data)
+                retval.extend(data)
+            except:
+                retval.append(data)
+        return retval
+    
+    def process_plugin_data(self, category):
+        """
+        Gathers all of the data from general plugins of type category,
+        and pass it to a single process function from one of those
+        plugins.
+
+        >>> PLUGMAN.process_plugin_data('CSS')
+        <a list of processed data items>
+        """
+        retval = []
+        data = None
+        process = None
+        for plugin in self.__pgr.general_plugins(category):
+            if plugin.process is not None:
+                mod = self.load_plugin(plugin)
+                if hasattr(mod, plugin.process):
+                    process = getattr(mod, plugin.process)
+            data = plugin.data
+            if data:
+                try:
+                    iter(data)
+                    retval.extend(data)
+                except:
+                    retval.append(data)
+        if process:
+            return process(retval)
+        return retval
+    
     def get_import_plugins(self):
         """
         Get the list of import plugins.
