@@ -17,7 +17,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
-# $Id: _pluginreg.py 14411 2010-02-18 14:54:19Z ldnp $
+# $Id: _pluginreg.py 17198 2011-04-22 01:45:19Z dsblank $
 
 """
 This module provides the base class for plugin registration. 
@@ -68,8 +68,9 @@ MAPSERVICE  = 7
 VIEW        = 8
 RELCALC     = 9
 GRAMPLET    = 10
-PTYPE       = [ REPORT , QUICKREPORT, TOOL, IMPORT,
-                EXPORT, DOCGEN, GENERAL, MAPSERVICE, VIEW, RELCALC, GRAMPLET]
+SIDEBAR     = 11
+PTYPE       = [REPORT , QUICKREPORT, TOOL, IMPORT, EXPORT, DOCGEN, GENERAL,
+               MAPSERVICE, VIEW, RELCALC, GRAMPLET, SIDEBAR]
 PTYPE_STR   = {
         REPORT: _('Report') , 
         QUICKREPORT: _('Quickreport'), 
@@ -82,6 +83,7 @@ PTYPE_STR   = {
         VIEW: _('Gramps View'), 
         RELCALC: _('Relationships'), 
         GRAMPLET: _('Gramplet'),
+        SIDEBAR: _('Sidebar'),
         }
 
 #possible report categories
@@ -135,6 +137,22 @@ END   = 2
 # Functions and classes
 #
 #-------------------------------------------------------------------------
+def myint(s):
+    """
+    Protected version of int()
+    """
+    try:
+        v = int(s)
+    except:
+        v = s
+    return v
+
+def version(sversion):
+    """
+    Return the tuple version of a string version.
+    """
+    return tuple([myint(x or "0") for x in (sversion + "..").split(".")])
+
 def valid_plugin_version(plugin_version_string):
     """
     Checks to see if string is a valid version string for this version
@@ -143,12 +161,10 @@ def valid_plugin_version(plugin_version_string):
     if not isinstance(plugin_version_string, basestring): return False
     dots = plugin_version_string.count(".")
     if dots == 1:
-        plugin_version  = tuple([int(n) for n in 
-                                 plugin_version_string.split(".", 1)])
+        plugin_version  = tuple(map(int, plugin_version_string.split(".", 1)))
         return plugin_version == VERSION_TUPLE[:2]
     elif dots == 2:
-        plugin_version  = tuple([int(n) for n in 
-                                 plugin_version_string.split(".", 2)])
+        plugin_version  = tuple(map(int, plugin_version_string.split(".", 2)))
         return (plugin_version[:2] == VERSION_TUPLE[:2] and 
                 plugin_version <= VERSION_TUPLE)
     return False
@@ -168,6 +184,10 @@ class PluginData(object):
        settings.
     .. attribute:: name
        A friendly name to call this plugin (normally translated)
+    .. attribute:: name_accell
+       A friendly name to call this plugin (normally translated), with an 
+       accellerator present (eg '_Descendant report', with D to be accellerator
+       key
     .. attribute:: description
        A friendly description of what the plugin does
     .. attribute:: version
@@ -284,7 +304,9 @@ class PluginData(object):
     .. attribute:: expand
        If the attributed should be expanded on start, default False
     .. attribute:: gramplet_title
-       Title to use for the gramplet, default = 'Gramplet'
+       Title to use for the gramplet, default = _('Gramplet')
+    .. attribute:: navtypes
+       Navigation types that the gramplet is appropriate for, default = []
     .. attribute:: help_url
        The URL where documentation for the URL can be found
 
@@ -292,11 +314,21 @@ class PluginData(object):
     .. attribute:: viewclass
        A class of type ViewCreator that holds the needed info of the
        view to be created: icon, viewclass that derives from pageview, ...
+    .. attribute:: stock_icon
+       The icon in the toolbar or sidebar used to select the view
+
+    Attributes for SIDEBAR plugins
+    .. attribute:: sidebarclass
+       The class that defines the sidebar.
+    .. attribute:: menu_label
+       A label to use on the seltion menu.
+       
+    Attributes for VIEW and SIDEBAR plugins
     .. attribute:: order
        order can be START or END. Default is END. For END, on registering, 
-       the view is appended to the list of views. If START, then the view is
-       prepended. Only set START if you want a view to be the first in the
-       order of views
+       the plugin is appended to the list of plugins. If START, then the
+       plugin is prepended. Only set START if you want a plugin to be the
+       first in the order of plugins
     """
 
     def __init__(self):
@@ -305,6 +337,7 @@ class PluginData(object):
         #base attributes
         self._id = None
         self._name = None
+        self._name_accell = None
         self._version = None
         self._gramps_target_version = None
         self._description = None
@@ -318,6 +351,7 @@ class PluginData(object):
         self._load_on_reg = False
         self._icons = []
         self._icondir = None
+        self._depends_on = []
         #derived var
         self.mod_name = None
         #RELCALC attr
@@ -327,8 +361,9 @@ class PluginData(object):
         self._reportclass = None
         self._require_active = True
         self._report_modes = [REPORT_MODE_GUI]
-        #REPORT and TOOL attr
+        #REPORT and TOOL and GENERAL attr
         self._category = None
+        #REPORT and TOOL attr
         self._optionclass = None
         #TOOL attr
         self._toolclass = None
@@ -355,10 +390,20 @@ class PluginData(object):
         self._detached_width = 400
         self._expand = False
         self._gramplet_title = _('Gramplet')
+        self._navtypes = []
+        self._orientation = None
         self._help_url = None
         #VIEW attr
         self._viewclass = None
+        self._stock_icon = None
+        #SIDEBAR attr
+        self._sidebarclass = None
+        self._menu_label = ''
+        #VIEW and SIDEBAR attr
         self._order = END
+        #GENERAL attr
+        self._data = []
+        self._process = None
     
     def _set_id(self, id):
        self._id = id
@@ -372,6 +417,15 @@ class PluginData(object):
     def _get_name(self):
         return self._name
     
+    def _set_name_accell(self, name):
+        self._name_accell = name
+
+    def _get_name_accell(self):
+        if self._name_accell is None:
+            return self._name
+        else:
+            return self._name_accell
+
     def _set_description(self, description):
         self._description = description
 
@@ -476,8 +530,17 @@ class PluginData(object):
     def _set_icondir(self, icondir):
         self._icondir = icondir
 
-    id = property(_get_id, _set_id)   
+    def _get_depends_on(self):
+        return self._depends_on
+
+    def _set_depends_on(self, depends):
+        if not isinstance(depends, list):
+            raise ValueError, 'Plugin must have depends_on as a list'
+        self._depends_on = depends
+
+    id = property(_get_id, _set_id)
     name = property(_get_name, _set_name)
+    name_accell = property(_get_name_accell, _set_name_accell)
     description = property(_get_description, _set_description) 
     version = property(_get_version, _set_version) 
     gramps_target_version = property(_get_gramps_target_version, 
@@ -492,6 +555,7 @@ class PluginData(object):
     load_on_reg = property(_get_load_on_reg, _set_load_on_reg)
     icons = property(_get_icons, _set_icons)
     icondir = property(_get_icondir, _set_icondir)
+    depends_on = property(_get_depends_on, _set_depends_on)
     
     def statustext(self):
         return STATUSTEXT[self.status]
@@ -549,12 +613,11 @@ class PluginData(object):
     def _get_report_modes(self):
         return self._report_modes
 
-    #REPORT OR TOOL OR QUICKREPORT attributes
+    #REPORT or TOOL or QUICKREPORT or GENERAL attributes
     def _set_category(self, category):
-        if not (self._ptype == REPORT or self._ptype == TOOL or
-        self._ptype == QUICKREPORT or self._ptype == VIEW):
+        if self._ptype not in [REPORT, TOOL, QUICKREPORT, VIEW, GENERAL]:
             raise ValueError, 'category may only be set for ' \
-                              'REPORT/TOOL/QUICKREPORT/VIEW plugins'
+                              'REPORT/TOOL/QUICKREPORT/VIEW/GENERAL plugins'
         self._category = category
 
     def _get_category(self):
@@ -771,6 +834,22 @@ class PluginData(object):
 
     def _get_help_url(self):
         return self._help_url
+
+    def _set_navtypes(self, navtypes):
+        if not self._ptype == GRAMPLET:
+            raise ValueError, 'navtypes may only be set for GRAMPLET plugins'
+        self._navtypes = navtypes
+
+    def _get_navtypes(self):
+        return self._navtypes
+    
+    def _set_orientation(self, orientation):
+        if not self._ptype == GRAMPLET:
+            raise ValueError, 'orientation may only be set for GRAMPLET plugins'
+        self._orientation = orientation
+
+    def _get_orientation(self):
+        return self._orientation
     
     gramplet = property(_get_gramplet, _set_gramplet)
     height = property(_get_height, _set_height)
@@ -778,6 +857,8 @@ class PluginData(object):
     detached_width = property(_get_detached_width, _set_detached_width)
     expand = property(_get_expand, _set_expand)
     gramplet_title = property(_get_gramplet_title, _set_gramplet_title)
+    navtypes = property(_get_navtypes, _set_navtypes)
+    orientation = property(_get_orientation, _set_orientation)
     help_url = property(_get_help_url, _set_help_url)
 
     def _set_viewclass(self, viewclass):
@@ -788,17 +869,67 @@ class PluginData(object):
     def _get_viewclass(self):
         return self._viewclass
   
-    def _set_order(self, order):
+    def _set_stock_icon(self, stock_icon):
         if not self._ptype == VIEW:
-            raise ValueError, 'order may only be set for VIEW plugins'
+            raise ValueError, 'stock_icon may only be set for VIEW plugins'
+        self._stock_icon = stock_icon
+
+    def _get_stock_icon(self):
+        return self._stock_icon
+       
+    viewclass = property(_get_viewclass, _set_viewclass)
+    stock_icon = property(_get_stock_icon, _set_stock_icon)
+
+    #SIDEBAR attributes
+    def _set_sidebarclass(self, sidebarclass):
+        if not self._ptype == SIDEBAR:
+            raise ValueError, 'sidebarclass may only be set for SIDEBAR plugins'
+        self._sidebarclass = sidebarclass
+
+    def _get_sidebarclass(self):
+        return self._sidebarclass
+        
+    def _set_menu_label(self, menu_label):
+        if not self._ptype == SIDEBAR:
+            raise ValueError, 'menu_label may only be set for SIDEBAR plugins'
+        self._menu_label = menu_label
+
+    def _get_menu_label(self):
+        return self._menu_label
+
+    sidebarclass = property(_get_sidebarclass, _set_sidebarclass)
+    menu_label   = property(_get_menu_label, _set_menu_label)
+
+    #VIEW and SIDEBAR attributes
+    def _set_order(self, order):
+        if not self._ptype in (VIEW, SIDEBAR):
+            raise ValueError, 'order may only be set for VIEW and SIDEBAR plugins'
         self._order = order
 
     def _get_order(self):
         return self._order
 
-    viewclass = property(_get_viewclass, _set_viewclass)
-    order     = property(_get_order, _set_order)
-    
+    order = property(_get_order, _set_order)
+  
+    #GENERAL attr
+    def _set_data(self, data):
+        if not self._ptype in (GENERAL,):
+            raise ValueError, 'data may only be set for GENERAL plugins'
+        self._data = data
+
+    def _get_data(self):
+        return self._data
+
+    def _set_process(self, process):
+        if not self._ptype in (GENERAL,):
+            raise ValueError, 'process may only be set for GENERAL plugins'
+        self._process = process
+
+    def _get_process(self):
+        return self._process
+
+    data = property(_get_data, _set_data)
+    process = property(_get_process, _set_process)
 
 def newplugin():
     """
@@ -850,6 +981,7 @@ def make_environment(**kwargs):
         'VIEW': VIEW,
         'RELCALC': RELCALC,
         'GRAMPLET': GRAMPLET,
+        'SIDEBAR': SIDEBAR,
         'CATEGORY_TEXT': CATEGORY_TEXT,
         'CATEGORY_DRAW': CATEGORY_DRAW,
         'CATEGORY_CODE': CATEGORY_CODE,
@@ -880,7 +1012,7 @@ def make_environment(**kwargs):
         'GRAMPSVERSION': GRAMPSVERSION,
         'START': START,
         'END': END,
-        'IMAGE_DIR': IMAGE_DIR
+        'IMAGE_DIR': IMAGE_DIR,
         }
     env.update(kwargs)
     return env
@@ -940,8 +1072,6 @@ class PluginRegister(object):
             lenpd = len(self.__plugindata)
             full_filename = os.path.join(dir, filename)
             local_gettext = get_addon_translator(full_filename).gettext
-            if type(full_filename) == str:
-                full_filename = unicode(full_filename, sys.getfilesystemencoding())
             try:
                 #execfile(full_filename,
                 execfile(full_filename.encode(sys.getfilesystemencoding()),
@@ -983,6 +1113,8 @@ class PluginRegister(object):
                 and not __debug__:
                     rmlist.append(ind)
                     continue
+                if plugin.fname is None:
+                    continue
                 match = pymod.match(plugin.fname)
                 if not match:
                     rmlist.append(ind)
@@ -1009,15 +1141,17 @@ class PluginRegister(object):
 
     def get_plugin(self, id):
         """Return the PluginData for the plugin with id"""
-        for x in self.__plugindata:
-            if x.id == id:
-                return x
+        matches = [x for x in self.__plugindata if x.id == id]
+        matches.sort(key=lambda x: version(x.version))
+        if len(matches) > 0:
+            return matches[-1]
         return None
 
     def type_plugins(self, ptype):
         """Return a list of PluginData that are of type ptype
         """
-        return [x for x in self.__plugindata if x.ptype == ptype]
+        return [self.get_plugin(id) for id in 
+                set([x.id for x in self.__plugindata if x.ptype == ptype])]
 
     def report_plugins(self, gui=True):
         """Return a list of gui or cli PluginData that are of type REPORT
@@ -1067,10 +1201,14 @@ class PluginRegister(object):
         """
         return self.type_plugins(DOCGEN)
 
-    def general_plugins(self):
+    def general_plugins(self, category=None):
         """Return a list of PluginData that are of type GENERAL
         """
-        return self.type_plugins(GENERAL)
+        plugins = self.type_plugins(GENERAL)
+        if category:
+            return [plugin for plugin in plugins 
+                    if plugin.category == category]
+        return plugins
 
     def mapservice_plugins(self):
         """Return a list of PluginData that are of type MAPSERVICE
@@ -1091,8 +1229,15 @@ class PluginRegister(object):
         """Return a list of PluginData that are of type GRAMPLET
         """
         return self.type_plugins(GRAMPLET)
+        
+    def sidebar_plugins(self):
+        """Return a list of PluginData that are of type SIDEBAR
+        """
+        return self.type_plugins(SIDEBAR)
 
     def filter_load_on_reg(self):
         """Return a list of PluginData that have load_on_reg == True
         """
-        return [x for x in self.__plugindata if x.load_on_reg == True]
+        return [self.get_plugin(id) for id in 
+                set([x.id for x in self.__plugindata 
+                     if x.load_on_reg == True])]
