@@ -20,11 +20,19 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: filter.py 15382 2010-05-09 23:59:22Z dsblank $
+# $Id: filter.py 17538 2011-05-19 20:38:03Z m_d_n $
 
 """
 Proxy class for the GRAMPS databases. Apply filter
 """
+
+#-------------------------------------------------------------------------
+#
+# Python libraries
+#
+#-------------------------------------------------------------------------
+from itertools import imap
+
 #-------------------------------------------------------------------------
 #
 # GRAMPS libraries
@@ -294,7 +302,7 @@ class FilterProxyDb(ProxyDbBase):
         if note:
             return self.get_note_from_handle(note.get_handle())
 
-    def get_person_handles(self, sort_handles=True):
+    def get_person_handles(self, sort_handles=False):
         """
         Return a list of database handles, one handle for each Person in
         the database. If sort_handles is True, the list is sorted by surnames
@@ -311,10 +319,9 @@ class FilterProxyDb(ProxyDbBase):
 
     def iter_people(self):
         """
-        Return an iterator over handles and objects for Persons in the database
+        Return an iterator over objects for Persons in the database
         """
-        for handle in self.plist:
-            yield handle, self.get_person_from_handle(handle)
+        return imap(self.get_person_from_handle, self.plist)
 
     def get_event_handles(self):
         """
@@ -325,10 +332,16 @@ class FilterProxyDb(ProxyDbBase):
         
     def iter_event_handles(self):
         """
-        Return an iterator over database handles, one handle for each Person in
+        Return an iterator over database handles, one handle for each Event in
         the database. 
         """
-        return self.elist        
+        return self.elist
+
+    def iter_events(self):
+        """
+        Return an iterator over objects for Events in the database
+        """
+        return imap(self.get_event_from_handle, self.elist)
 
     def get_family_handles(self):
         """
@@ -339,11 +352,17 @@ class FilterProxyDb(ProxyDbBase):
         
     def iter_family_handles(self):
         """
-        Return an iterator over database handles, one handle for each Person in
+        Return an iterator over database handles, one handle for each Family in
         the database. 
         """
         return self.flist
 
+    def iter_families(self):
+        """
+        Return an iterator over objects for Families in the database
+        """
+        return imap(self.get_family_from_handle, self.flist)        
+
     def get_note_handles(self):
         """
         Return a list of database handles, one handle for each Note in
@@ -353,11 +372,17 @@ class FilterProxyDb(ProxyDbBase):
 
     def iter_note_handles(self):
         """
-        Return an iterator over database handles, one handle for each Person in
+        Return an iterator over database handles, one handle for each Note in
         the database. 
         """
         return self.nlist
 
+    def iter_notes(self):
+        """
+        Return an iterator over objects for Notes in the database
+        """
+        return imap(self.get_note_from_handle, self.nlist)        
+
     def get_default_person(self):
         """returns the default Person of the database"""
         person = self.db.get_default_person()
