@@ -18,7 +18,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: living.py 15382 2010-05-09 23:59:22Z dsblank $
+# $Id: living.py 16251 2010-12-05 21:47:41Z dsblank $
 
 """
 Proxy class for the GRAMPS databases. Filter out all living people.
@@ -29,6 +29,7 @@ Proxy class for the GRAMPS databases. Filter out all living people.
 # Python libraries
 #
 #-------------------------------------------------------------------------
+from itertools import ifilter
 
 #-------------------------------------------------------------------------
 #
@@ -36,7 +37,7 @@ Proxy class for the GRAMPS databases. Filter out all living people.
 #
 #-------------------------------------------------------------------------
 from proxybase import ProxyDbBase
-from gen.lib import Date, Person, Name
+from gen.lib import Date, Person, Name, Surname
 from Utils import probably_alive
 import config
 
@@ -106,6 +107,18 @@ class LivingProxyDb(ProxyDbBase):
         family = self.__remove_living_from_family(family)
         return family
 
+    def iter_people(self):
+        """
+        Protected version of iter_people
+        """
+        for person in ifilter(None, self.db.iter_people()):
+            if not(self.__is_living(person) and
+                   self.mode == self.MODE_EXCLUDE_ALL):
+
+                yield self.__restrict_person(person)
+            else:
+                yield person
+
     def get_person_from_gramps_id(self, val):
         """
         Finds a Person in the database from the passed GRAMPS ID.
@@ -248,7 +261,6 @@ class LivingProxyDb(ProxyDbBase):
         new_name.set_group_as(old_name.get_group_as())
         new_name.set_sort_as(old_name.get_sort_as())
         new_name.set_display_as(old_name.get_display_as())
-        new_name.set_surname_prefix(old_name.get_surname_prefix())
         new_name.set_type(old_name.get_type())
         if self.mode == self.MODE_INCLUDE_LAST_NAME_ONLY:
             new_name.set_first_name(config.get('preferences.private-given-text'))
@@ -256,8 +268,10 @@ class LivingProxyDb(ProxyDbBase):
             new_name.set_first_name(old_name.get_first_name())
             new_name.set_suffix(old_name.get_suffix())
             new_name.set_title(old_name.get_title())
-        new_name.set_patronymic(old_name.get_patronymic())
-        new_name.set_surname(old_name.get_surname())
+        surnlst = []
+        for surn in old_name.get_surname_list():
+            surnlst.append(Surname(source=surn))
+        new_name.set_surname_list(surnlst)
         new_name.set_privacy(old_name.get_privacy())
     
         new_person.set_primary_name(new_name)
@@ -269,5 +283,6 @@ class LivingProxyDb(ProxyDbBase):
         new_person.set_family_handle_list(person.get_family_handle_list())
         new_person.set_parent_family_handle_list( 
                                         person.get_parent_family_handle_list() )
+        new_person.set_tag_list(person.get_tag_list())
     
         return new_person
