@@ -2,6 +2,7 @@
 # Gramps - a GTK+/GNOME based genealogy program
 #
 # Copyright (C) 2007       Brian G. Matherly
+# Copyright (C) 2010       Nick Hall
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,7 +19,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: private.py 15353 2010-05-07 23:45:28Z dsblank $
+# $Id: private.py 16074 2010-10-28 19:56:46Z bmcage $
 
 """
 Proxy class for the GRAMPS databases. Filter out all data marked private.
@@ -38,7 +39,7 @@ from gen.ggettext import gettext as _
 #-------------------------------------------------------------------------
 from gen.lib import (MediaRef, SourceRef, Attribute, Address, EventRef, 
                      Person, Name, Source, RepoRef, MediaObject, Place, Event, 
-                     Family, ChildRef, Repository, LdsOrd)
+                     Family, ChildRef, Repository, LdsOrd, Surname)
 from proxybase import ProxyDbBase
 
 class PrivateProxyDb(ProxyDbBase):
@@ -610,12 +611,13 @@ def sanitize_address(db, address):
     new_address = Address()
     
     new_address.set_street(address.get_street())
+    new_address.set_locality(address.get_locality())
     new_address.set_city(address.get_city())
-    new_address.set_postal_code(address.get_postal_code())
-    new_address.set_phone(address.get_phone())
+    new_address.set_county(address.get_county())
     new_address.set_state(address.get_state())
     new_address.set_country(address.get_country())
-    new_address.set_county(address.get_county())
+    new_address.set_postal_code(address.get_postal_code())
+    new_address.set_phone(address.get_phone())
     
     new_address.set_date_object(address.get_date_object())
     copy_source_ref_list(db, address, new_address)
@@ -642,14 +644,14 @@ def sanitize_name(db, name):
     new_name.set_sort_as(name.get_sort_as())
     new_name.set_display_as(name.get_display_as())
     new_name.set_call_name(name.get_call_name())
-    new_name.set_surname_prefix(name.get_surname_prefix())
+    new_name.set_nick_name(name.get_nick_name())
+    new_name.set_family_nick_name(name.get_family_nick_name())
     new_name.set_type(name.get_type())
     new_name.set_first_name(name.get_first_name())
-    new_name.set_patronymic(name.get_patronymic())
-    new_name.set_surname(name.get_surname())
     new_name.set_suffix(name.get_suffix())
     new_name.set_title(name.get_title())
     new_name.set_date_object(name.get_date_object())
+    new_name.set_surname_list(name.get_surname_list())
     
     copy_source_ref_list(db, name, new_name)
     copy_notes(db, name, new_name)
@@ -747,13 +749,17 @@ def sanitize_person(db, person):
     new_person.set_gramps_id(person.get_gramps_id())
     new_person.set_handle(person.get_handle())
     new_person.set_change_time(person.get_change_time())
+    new_person.set_tag_list(person.get_tag_list())
     
     # copy names if not private
     name = person.get_primary_name()
     if (name and name.get_privacy()) or (person and person.get_privacy()):
         # Do this so a person always has a primary name of some sort.
         name = Name()
-        name.set_surname(_('Private'))
+        surn = Surname()
+        surn.set_surname(_('Private'))
+        name.set_surname_list([surn])
+        name.set_primary_surname()
     else:
         name = sanitize_name(db, name)
     new_person.set_primary_name(name)
@@ -767,12 +773,13 @@ def sanitize_person(db, person):
     # copy Family reference list
     for handle in person.get_parent_family_handle_list():
         family = db.get_family_from_handle(handle)
-        if family and family.get_privacy():
+        if not family:
+            continue
+        elif family.get_privacy():
             continue
         child_ref_list = family.get_child_ref_list()
         for child_ref in child_ref_list:
-            if (child_ref and
-              child_ref.get_reference_handle() == person.get_handle()):
+            if child_ref.get_reference_handle() == person.get_handle():
                 if child_ref and not child_ref.get_privacy():
                     new_person.add_parent_family_handle(handle)
                 break
@@ -781,9 +788,6 @@ def sanitize_person(db, person):
         if name and not name.get_privacy():
             new_person.add_alternate_name(sanitize_name(db, name))
 
-    # set complete flag
-    new_person.set_marker(person.get_marker())
-
     # copy event list
     for event_ref in person.get_event_ref_list():
         if event_ref and not event_ref.get_privacy():
@@ -840,7 +844,6 @@ def sanitize_source(db, source):
     new_source.set_gramps_id(source.get_gramps_id())
     new_source.set_handle(source.get_handle())
     new_source.set_change_time(source.get_change_time())
-    new_source.set_marker(source.get_marker())
     new_source.set_data_map(source.get_data_map())
     
     for repo_ref in source.get_reporef_list():
@@ -878,12 +881,12 @@ def sanitize_media(db, media):
     new_media.set_handle(media.get_handle())
     new_media.set_change_time(media.get_change_time())
     new_media.set_date_object(media.get_date_object())
-    new_media.set_marker(media.get_marker())
+    new_media.set_tag_list(media.get_tag_list())
 
     copy_source_ref_list(db, media, new_media)
     copy_attributes(db, media, new_media)
     copy_notes(db, media, new_media)
-    
+
     return new_media
 
 def sanitize_place(db, place):
@@ -910,7 +913,6 @@ def sanitize_place(db, place):
     new_place.set_latitude(place.get_latitude())
     new_place.set_main_location(place.get_main_location())
     new_place.set_alternate_locations(place.get_alternate_locations())
-    new_place.set_marker(place.get_marker())
 
     copy_source_ref_list(db, place, new_place)
     copy_notes(db, place, new_place)
@@ -940,7 +942,6 @@ def sanitize_event(db, event):
     new_event.set_gramps_id(event.get_gramps_id())
     new_event.set_handle(event.get_handle())
     new_event.set_date_object(event.get_date_object())
-    new_event.set_marker(event.get_marker())
     new_event.set_change_time(event.get_change_time())
 
     copy_source_ref_list(db, event, new_event)
@@ -973,9 +974,9 @@ def sanitize_family(db, family):
     
     new_family.set_gramps_id(family.get_gramps_id())
     new_family.set_handle(family.get_handle())
-    new_family.set_marker(family.get_marker())    
     new_family.set_relationship(family.get_relationship())
     new_family.set_change_time(family.get_change_time())
+    new_family.set_tag_list(family.get_tag_list())
     
     # Copy the father handle.
     father_handle = family.get_father_handle()
@@ -1044,7 +1045,6 @@ def sanitize_repository(db, repository):
     new_repository.set_gramps_id(repository.get_gramps_id())
     new_repository.set_handle(repository.get_handle())
     new_repository.set_change_time(repository.get_change_time())
-    new_repository.set_marker(repository.get_marker())
 
     copy_notes(db, repository, new_repository)
     copy_addresses(db, repository, new_repository)
