@@ -18,7 +18,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: proxybase.py 15295 2010-05-01 14:29:52Z dsblank $
+# $Id: proxybase.py 17538 2011-05-19 20:38:03Z m_d_n $
 
 """
 Proxy class for the GRAMPS databases. Filter out all data marked private.
@@ -99,9 +99,10 @@ class ProxyDbBase(DbReadBase):
     include_media_object = \
     include_repository = \
     include_note = \
+    include_tag = \
         None
         
-    def get_person_handles(self, sort_handles=True):
+    def get_person_handles(self, sort_handles=False):
         """
         Return a list of database handles, one handle for each Person in
         the database. 
@@ -131,7 +132,7 @@ class ProxyDbBase(DbReadBase):
         else:
             return []
 
-    def get_source_handles(self, sort_handles=True):
+    def get_source_handles(self, sort_handles=False):
         """
         Return a list of database handles, one handle for each Source in
         the database. 
@@ -141,7 +142,7 @@ class ProxyDbBase(DbReadBase):
         else:
             return []
         
-    def get_place_handles(self, sort_handles=True):
+    def get_place_handles(self, sort_handles=False):
         """
         Return a list of database handles, one handle for each Place in
         the database. 
@@ -151,7 +152,7 @@ class ProxyDbBase(DbReadBase):
         else:
             return []
         
-    def get_media_object_handles(self, sort_handles=True):
+    def get_media_object_handles(self, sort_handles=False):
         """
         Return a list of database handles, one handle for each MediaObject in
         the database. 
@@ -181,6 +182,16 @@ class ProxyDbBase(DbReadBase):
         else:
             return []
 
+    def get_tag_handles(self, sort_handles=False):
+        """
+        Return a list of database handles, one handle for each Tag in
+        the database. 
+        """
+        if self.db.is_open:
+            return list(self.iter_tag_handles())
+        else:
+            return []
+
     def get_default_person(self):
         """returns the default Person of the database"""
         return self.db.get_default_person()
@@ -245,6 +256,13 @@ class ProxyDbBase(DbReadBase):
         """
         return ifilter(self.include_note, self.db.iter_note_handles())
 
+    def iter_tag_handles(self):
+        """
+        Return an iterator over database handles, one handle for each Tag in
+        the database.
+        """
+        return ifilter(self.include_tag, self.db.iter_tag_handles())
+
     @staticmethod
     def __iter_object(filter, method):
         """ Helper function to return an iterator over an object class """
@@ -301,6 +319,12 @@ class ProxyDbBase(DbReadBase):
         """
         return self.__iter_object(self.include_note, self.db.iter_notes)       
         
+    def iter_tags(self):
+        """
+        Return an iterator over Tag objects in the database
+        """
+        return self.__iter_object(self.include_tag, self.db.iter_tags)       
+        
     @staticmethod
     def gfilter(predicate, obj):
         """
@@ -312,6 +336,8 @@ class ProxyDbBase(DbReadBase):
 
     def __getattr__(self, name):
         """ Handle unknown attribute lookups """
+        if name == "readonly":
+            return True
         sname = name.split('_')
         if sname[:2] == ['get', 'unfiltered']:
             """
@@ -396,6 +422,14 @@ class ProxyDbBase(DbReadBase):
         return self.gfilter(self.include_note,
                             self.db.get_note_from_handle(handle))
         
+    def get_tag_from_handle(self, handle):
+        """
+        Finds a Tag in the database from the passed gramps handle.
+        If no such Tag exists, None is returned.
+        """
+        return self.gfilter(self.include_tag,
+                            self.db.get_tag_from_handle(handle))
+        
     def get_person_from_gramps_id(self, val):
         """
         Finds a Person in the database from the passed GRAMPS ID.
@@ -460,11 +494,19 @@ class ProxyDbBase(DbReadBase):
         return self.gfilter(self.include_note,
                 self.db.get_note_from_gramps_id(val))
 
-    def get_name_group_mapping(self, name):
+    def get_tag_from_name(self, val):
+        """
+        Finds a Tag in the database from the passed tag name.
+        If no such Tag exists, None is returned.
+        """
+        return self.gfilter(self.include_tag,
+                self.db.get_tag_from_name(val))
+
+    def get_name_group_mapping(self, surname):
         """
         Return the default grouping name for a surname
         """
-        return self.db.get_name_group_mapping(name)
+        return self.db.get_name_group_mapping(surname)
 
     def has_name_group_key(self, name):
         """
@@ -526,6 +568,12 @@ class ProxyDbBase(DbReadBase):
         """
         return self.db.get_number_of_notes()
 
+    def get_number_of_tags(self):
+        """
+        Return the number of tags currently in the database.
+        """
+        return self.db.get_number_of_tags()
+
     def get_save_path(self):
         """returns the save path of the file, or "" if one does not exist"""
         return self.db.get_save_path()
@@ -550,10 +598,6 @@ class ProxyDbBase(DbReadBase):
         instances in the database"""
         return self.db.get_family_event_types()
 
-    def get_marker_types(self):
-        """return a list of all marker types available in the database"""
-        return self.db.get_marker_types()
-        
     def get_media_attribute_types(self):
         """returns a list of all Attribute types associated with Media
         and MediaRef instances in the database"""
@@ -579,6 +623,11 @@ class ProxyDbBase(DbReadBase):
         instances in the database"""
         return self.db.get_name_types()
 
+    def get_origin_types(self):
+        """returns a list of all custom origin types associated with Person/Surname
+        instances in the database"""
+        return self.db.get_origin_types()
+
     def get_repository_types(self):
         """returns a list of all custom repository types associated with
         Repository instances in the database"""
@@ -623,61 +672,71 @@ class ProxyDbBase(DbReadBase):
     def get_raw_note_data(self, handle):
         return self.db.get_raw_note_data(handle)
 
+    def get_raw_tag_data(self, handle):
+        return self.db.get_raw_tag_data(handle)
+
     def has_person_handle(self, handle):
         """
         Returns True if the handle exists in the current Person database.
         """
         return self.gfilter(self.include_person,
-                self.db.get_person_from_gramps_id(val)) is not None
+                self.db.get_person_from_handle(handle)) is not None
 
     def has_family_handle(self, handle):
         """
         Returns True if the handle exists in the current Family database.
         """
         return self.gfilter(self.include_family,
-                self.db.get_family_from_gramps_id(val)) is not None
+                self.db.get_family_from_handle(handle)) is not None
 
     def has_event_handle(self, handle):
         """
         returns True if the handle exists in the current Event database.
         """
         return self.gfilter(self.include_event,
-                self.db.get_event_from_gramps_id(val)) is not None
+                self.db.get_event_from_handle(handle)) is not None
 
     def has_source_handle(self, handle):
         """
         returns True if the handle exists in the current Source database.
         """
         return self.gfilter(self.include_source,
-                self.db.get_source_from_gramps_id(val)) is not None
+                self.db.get_source_from_handle(handle)) is not None
 
     def has_place_handle(self, handle):
         """
         returns True if the handle exists in the current Place database.
         """
         return self.gfilter(self.include_place,
-                self.db.get_place_from_gramps_id(val)) is not None
+                self.db.get_place_from_handle(handle)) is not None
 
     def has_object_handle(self, handle):
         """
         returns True if the handle exists in the current MediaObjectdatabase.
         """
         return self.gfilter(self.include_media_object,
-                self.db.get_object_from_gramps_id(val)) is not None
+                self.db.get_object_from_handle(handle)) is not None
 
     def has_repository_handle(self, handle):
         """
         returns True if the handle exists in the current Repository database.
         """
         return self.gfilter(self.include_repository,
-                self.db.get_repository_from_gramps_id(val)) is not None
+                self.db.get_repository_from_handle(handle)) is not None
 
     def has_note_handle(self, handle):
         """
         returns True if the handle exists in the current Note database.
         """
         return self.gfilter(self.include_note,
-                self.db.get_note_from_gramps_id(val)) is not None
+                self.db.get_note_from_handle(handle)) is not None
+        
+    def has_tag_handle(self, handle):
+        """
+        returns True if the handle exists in the current Tag database.
+        """
+        return self.gfilter(self.include_tag,
+                self.db.get_tag_from_handle(handle)) is not None
         
     def get_mediapath(self):
         """returns the default media path of the database"""
@@ -720,3 +779,4 @@ class ProxyDbBase(DbReadBase):
     def get_note_bookmarks(self):
         """returns the list of Note handles in the bookmarks"""
         return self.note_bookmarks
+
