@@ -21,7 +21,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: gramps.py 17434 2011-05-07 13:07:35Z bmcage $
+# $Id: gramps.py 17799 2011-06-16 10:44:50Z dsblank $
 
 #-------------------------------------------------------------------------
 #
@@ -149,7 +149,7 @@ def build_user_paths():
         if os.path.islink(path):
             pass # ok
         elif not os.path.isdir(path):
-            os.mkdir(path)
+            os.makedirs(path)
 
 def show_settings():
     """
