@@ -4,6 +4,7 @@
 # Copyright (C) 2000-2007  Donald N. Allingham
 # Copyright (C) 2008       Raphael Ackermann
 # Copyright (C) 2010       Benny Malengier
+# Copyright (C) 2010       Nick Hall
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,7 +21,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: configure.py 15939 2010-09-30 10:02:41Z ldnp $
+# $Id: configure.py 17663 2011-06-02 10:00:45Z romjerome $
 
 #-------------------------------------------------------------------------
 #
@@ -52,7 +53,7 @@ from gen.display.name import displayer as _nd
 from gen.display.name import NameDisplayError
 import Utils
 import gen.lib
-from gen.lib import Name
+from gen.lib import Name, Surname, NameOriginType
 import ManagedWindow
 from gui.widgets import MarkupLabel, BasicLabel
 from QuestionDialog import ErrorDialog, QuestionDialog2, OkDialog
@@ -95,23 +96,29 @@ class DisplayNameEditor(ManagedWindow.ManagedWindow):
                        buttons=(gtk.STOCK_CLOSE, gtk.RESPONSE_CLOSE)), 
             None, _('Display Name Editor'), None)
         table = self.dialog._build_custom_name_ui()
-        label = gtk.Label(_("""The following keywords will be replaced with the name:
+        label = gtk.Label(_("""The following keywords are replaced with the appropriate name parts:
 <tt>  
-  <b>Given</b>      - given name (first name)
-  <b>Surname</b>    - surname (last name)
-  <b>Title</b>      - title (Dr., Mrs.)
-  <b>Prefix</b>     - prefix (von, de, de la)
-  <b>Suffix</b>     - suffix (Jr., Sr.)
-  <b>Call</b>       - call name, or nickname
-  <b>Common</b>     - call name, otherwise first part of Given
-  <b>Patronymic</b> - patronymic (father's name)
-  <b>Initials</b>   - persons's first letters of given names
+  <b>Given</b>      - given name (first name) <b>Surname</b>      - surnames (with prefix and connectors)
+  <b>Title</b>      - title (Dr., Mrs.)       <b>Suffix</b>       - suffix (Jr., Sr.)
+  <b>Call</b>       - call name               <b>Nickname</b>     - nick name
+  <b>Initials</b>   - first letters of Given  <b>Common</b>       - nick name, otherwise first of Given
+  <b>Primary, Primary[pre] or [sur] or [con]</b>- full primary surname, prefix, surname only, connector   
+  <b>Patronymic, or [pre] or [sur] or [con]</b> - full pa/matronymic surname, prefix, surname only, connector 
+  <b>Familynick</b> - family nick name        <b>Prefix</b>       - all prefixes (von, de)  
+  <b>Rest</b>       - non primary surnames    <b>Notpatronymic</b>- all surnames, except pa/matronymic &amp; primary
+  <b>Rawsurnames</b>- surnames (no prefixes and connectors)
+
 </tt>
-Use the same keyword in UPPERCASE to force to upper. Parentheses and commas
-will be removed around empty fields. Other text will appear literally."""))
+UPPERCASE keyword forces uppercase. Extra parentheses, commas are removed. Other text appears literally.
+
+<b>Example</b>: 'Dr. Edwin Jose von der Smith and Weston Wilson Sr ("Ed") - Underhills'
+     <i>Edwin Jose</i> is given name, <i>von der</i> is the prefix, <i>Smith</i> and <i>Weston</i> surnames, 
+     <i>and</i> a connector, <i>Wilson</i> patronymic surname, <i>Dr.</i> title, <i>Sr</i> suffix, <i>Ed</i> nick name, 
+     <i>Underhills</i> family nick name, <i>Jose</i> callname.
+"""))
         label.set_use_markup(True)
-        self.window.vbox.add(label)        
-        self.window.vbox.add(table)
+        self.window.vbox.pack_start(label, expand=False)        
+        self.window.vbox.pack_start(table)
         self.window.set_default_size(600, 550)
         self.window.connect('response', self.close)
         self.show()
@@ -228,15 +235,33 @@ class ConfigureDialog(ManagedWindow.ManagedWindow):
     def update_radiobox(self, obj, constant):
         self.__config.set(constant, obj.get_active())
 
-    def add_checkbox(self, table, label, index, constant, start=1, stop=9, config=None):
+    def update_combo(self, obj, constant):
+        """
+        :param obj: the ComboBox object
+        :param constant: the config setting to which the value must be saved
+        """
+        self.__config.set(constant, obj.get_active())
+
+    def update_slider(self, obj, constant):
+        """
+        :param obj: the HScale object
+        :param constant: the config setting to which the value must be saved
+        """
+        self.__config.set(constant, int(obj.get_value()))
+
+    def add_checkbox(self, table, label, index, constant, start=1, stop=9,
+                     config=None, extra_callback=None):
         if not config:
             config = self.__config
         checkbox = gtk.CheckButton(label)
         checkbox.set_active(config.get(constant))
         checkbox.connect('toggled', self.update_checkbox, constant, config)
+        if extra_callback:
+            checkbox.connect('toggled', extra_callback)
         table.attach(checkbox, start, stop, index, index+1, yoptions=0)
 
-    def add_radiobox(self, table, label, index, constant, group, column, config=None):
+    def add_radiobox(self, table, label, index, constant, group, column,
+                     config=None):
         if not config:
             config = self.__config
         radiobox = gtk.RadioButton(group,label)
@@ -281,7 +306,8 @@ class ConfigureDialog(ManagedWindow.ManagedWindow):
                      xoptions=gtk.FILL)
         table.attach(hbox, 2, 3, index, index+1, yoptions=0)
 
-    def add_entry(self, table, label, index, constant, callback=None, config=None):
+    def add_entry(self, table, label, index, constant, callback=None,
+                  config=None):
         if not config:
             config = self.__config
         if not callback:
@@ -294,7 +320,8 @@ class ConfigureDialog(ManagedWindow.ManagedWindow):
                      xoptions=gtk.FILL)
         table.attach(entry, 1, 2, index, index+1, yoptions=0)
 
-    def add_pos_int_entry(self, table, label, index, constant, callback=None, config=None):
+    def add_pos_int_entry(self, table, label, index, constant, callback=None,
+                          config=None, col_attach=1):
         """ entry field for positive integers
         """
         if not config:
@@ -304,9 +331,10 @@ class ConfigureDialog(ManagedWindow.ManagedWindow):
         entry.set_text(str(config.get(constant)))
         if callback:
             entry.connect('changed', callback, constant)
-        table.attach(lwidget, 1, 2, index, index+1, yoptions=0, 
-                     xoptions=gtk.FILL)
-        table.attach(entry, 2, 3, index, index+1, yoptions=0)
+        table.attach(lwidget, col_attach, col_attach+1, index, index+1,
+                     yoptions=0, xoptions=gtk.FILL)
+        table.attach(entry, col_attach+1, col_attach+2, index, index+1, 
+                     yoptions=0)
 
     def add_color(self, table, label, index, constant, config=None):
         if not config:
@@ -323,6 +351,54 @@ class ConfigureDialog(ManagedWindow.ManagedWindow):
         table.attach(color_hex_label, 2, 3, index, index+1, yoptions=0)
         return entry
 
+    def add_combo(self, table, label, index, constant, opts, callback=None,
+                  config=None):
+        """
+        A drop-down list allowing selection from a number of fixed options.
+        :param opts: A list of options.  Each option is a tuple containing an
+        integer code and a textual description.
+        """
+        if not config:
+            config = self.__config
+        if not callback:
+            callback = self.update_combo
+        lwidget = BasicLabel("%s: " % label)
+        store = gtk.ListStore(int, str)
+        for item in opts:
+            store.append(item)
+        combo = gtk.ComboBox(store)
+        cell = gtk.CellRendererText()
+        combo.pack_start(cell, True)
+        combo.add_attribute(cell, 'text', 1)  
+        combo.set_active(config.get(constant))
+        combo.connect('changed', callback, constant)
+        table.attach(lwidget, 1, 2, index, index+1, yoptions=0, 
+                     xoptions=gtk.FILL)
+        table.attach(combo, 2, 3, index, index+1, yoptions=0)
+        return combo
+
+    def add_slider(self, table, label, index, constant, range, callback=None,
+                   config=None):
+        """
+        A slider allowing the selection of an integer within a specified range.
+        :param range: A tuple containing the minimum and maximum allowed values.
+        """
+        if not config:
+            config = self.__config
+        if not callback:
+            callback = self.update_slider
+        lwidget = BasicLabel("%s: " % label)
+        adj = gtk.Adjustment(config.get(constant), range[0], range[1], 1, 0, 0)
+        slider = gtk.HScale(adj)
+        slider.set_update_policy(gtk.UPDATE_DISCONTINUOUS)
+        slider.set_digits(0)
+        slider.set_value_pos(gtk.POS_BOTTOM)
+        slider.connect('value-changed', callback, constant)
+        table.attach(lwidget, 1, 2, index, index+1, yoptions=0, 
+                     xoptions=gtk.FILL)
+        table.attach(slider, 2, 3, index, index+1, yoptions=0)
+        return slider
+
 #-------------------------------------------------------------------------
 #
 # GrampsPreferences
@@ -333,14 +409,13 @@ class GrampsPreferences(ConfigureDialog):
     def __init__(self, uistate, dbstate):
         page_funcs = (
             self.add_behavior_panel,
-            self.add_database_panel,
+            self.add_famtree_panel,
             self.add_formats_panel,
             self.add_text_panel,
             self.add_prefix_panel,
             self.add_date_panel,
             self.add_advanced_panel,
             self.add_researcher_panel,
-            self.add_color_panel
             )
         ConfigureDialog.__init__(self, uistate, dbstate, page_funcs, 
                                  GrampsPreferences, config,
@@ -353,12 +428,13 @@ class GrampsPreferences(ConfigureDialog):
         table.set_row_spacings(6)
         self.add_entry(table, _('Name'), 0, 'researcher.researcher-name')
         self.add_entry(table, _('Address'), 1, 'researcher.researcher-addr')
-        self.add_entry(table, _('City'), 2, 'researcher.researcher-city')
-        self.add_entry(table, _('State/Province'), 3, 'researcher.researcher-state')
-        self.add_entry(table, _('Country'), 4, 'researcher.researcher-country')
-        self.add_entry(table, _('ZIP/Postal Code'), 5, 'researcher.researcher-postal')
-        self.add_entry(table, _('Phone'), 6, 'researcher.researcher-phone')
-        self.add_entry(table, _('Email'), 7, 'researcher.researcher-email')
+        self.add_entry(table, _('Locality'), 2, 'researcher.researcher-locality')
+        self.add_entry(table, _('City'), 3, 'researcher.researcher-city')
+        self.add_entry(table, _('State/County'), 4, 'researcher.researcher-state')
+        self.add_entry(table, _('Country'), 5, 'researcher.researcher-country')
+        self.add_entry(table, _('ZIP/Postal Code'), 6, 'researcher.researcher-postal')
+        self.add_entry(table, _('Phone'), 7, 'researcher.researcher-phone')
+        self.add_entry(table, _('Email'), 8, 'researcher.researcher-email')
         return _('Researcher'), table
 
     def add_prefix_panel(self, configdialog):
@@ -411,40 +487,6 @@ class GrampsPreferences(ConfigureDialog):
         
         return _('Warnings'), table
 
-    def add_color_panel(self, configdialog):
-        table = gtk.Table(3, 8)
-        table.set_border_width(12)
-        table.set_col_spacings(12)
-        table.set_row_spacings(6)
-
-        self.comp_color = self.add_color(table, _("Complete"), 0, 
-                                         'preferences.complete-color')
-        self.todo_color = self.add_color(table, _("ToDo"), 1, 
-                                         'preferences.todo-color')
-        self.custom_color = self.add_color(table, _("Custom"), 2, 
-                                           'preferences.custom-marker-color')
-        
-        button = gtk.Button(stock=gtk.STOCK_REVERT_TO_SAVED)
-        button.connect('clicked', self.reset_colors)
-        table.attach(button, 1, 2, 3, 4, yoptions=0, xoptions=0)
-        return _('Marker Colors'), table
-
-    def reset_colors(self, obj):
-
-        def_comp = config.get_default('preferences.complete-color')
-        def_todo = config.get_default('preferences.todo-color')
-        def_cust = config.get_default('preferences.custom-marker-color')
-        
-        config.set('preferences.complete-color', def_comp)
-        config.set('preferences.todo-color', def_todo)
-        config.set('preferences.custom-marker-color', def_cust)
-
-        self.comp_color.set_color(gtk.gdk.color_parse(def_comp))
-        self.todo_color.set_color(gtk.gdk.color_parse(def_todo))
-        self.custom_color.set_color(gtk.gdk.color_parse(def_cust))
-        for widget in [self.comp_color, self.todo_color, self.custom_color]:
-            widget.emit('color-set')
-
     def _build_name_format_model(self, active):
         """
         Create a common model for ComboBox and TreeView
@@ -453,7 +495,7 @@ class GrampsPreferences(ConfigureDialog):
                                           gobject.TYPE_STRING, 
                                           gobject.TYPE_STRING, 
                                           gobject.TYPE_STRING)
-        index = 0        
+        index = 0
         the_index = 0
         for num, name, fmt_str, act in _nd.get_name_format():
             translation = fmt_str
@@ -471,40 +513,49 @@ class GrampsPreferences(ConfigureDialog):
         lyst = ["%s, %s %s (%s)" % (_("Surname"), _("Given"), _("Suffix"), 
                                     _("Common")),
                 "%s, %s %s (%s)" % (_("Surname"), _("Given"), _("Suffix"), 
-                                    _("Call")),
+                                    _("Nickname")),
+                "%s, %s %s (%s)" % (_("Surname"), _("Name|Common"), _("Suffix"), 
+                                    _("Nickname")),
+                "%s, %s %s" % (_("Surname"), _("Name|Common"), _("Suffix")),
                 "%s, %s %s (%s)" % (_("SURNAME"), _("Given"), _("Suffix"), 
                                     _("Call")),
-                "%s, %s (%s)" % (_("Surname"), _("Given"), _("Common")),
-                "%s, %s (%s)" % (_("Surname"), _("Given"), _("Call")),
+                "%s, %s (%s)" % (_("Surname"), _("Given"), _("Name|Common")),
+                "%s, %s (%s)" % (_("Surname"), _("Name|Common"), _("Nickname")),
                 "%s %s" % (_("Given"), _("Surname")),
                 "%s %s, %s" % (_("Given"), _("Surname"), _("Suffix")),
-                "%s %s %s" % (_("Given"), _("Surname"), _("Patronymic")),
+                "%s %s %s" % (_("Given"), _("NotPatronymic"), _("Patronymic")),
                 "%s, %s %s (%s)" % (_("SURNAME"), _("Given"), _("Suffix"), 
                                     _("Common")),
-                "%s, %s (%s)" % (_("SURNAME"), _("Given"), _("Common")),
-                "%s, %s (%s)" % (_("SURNAME"), _("Given"), _("Call")),
+                "%s, %s (%s)" % (_("SURNAME"), _("Given"), _("Name|Common")),
+                "%s, %s (%s)" % (_("SURNAME"), _("Given"), _("Nickname")),
                 "%s %s" % (_("Given"), _("SURNAME")),
                 "%s %s, %s" % (_("Given"), _("SURNAME"), _("Suffix")),
                 "%s /%s/" % (_("Given"), _("SURNAME")),
+                "%s %s, %s" % (_("Given"), _("Rawsurnames"), _("Suffix")),
                 ]
-        fmtlyst = ["%s, %s %s (%s)" % ("Surname", "Given", "Suffix", 
-                                       "Common"),
-                   "%s, %s %s (%s)" % ("Surname", "Given", "Suffix", 
-                                       "Call"),
-                   "%s, %s %s (%s)" % ("SURNAME", "Given", "Suffix", 
-                                       "Call"),
-                   "%s, %s (%s)" % ("Surname", "Given", "Common"),
-                   "%s, %s (%s)" % ("Surname", "Given", "Call"),
-                   "%s %s" % ("Given", "Surname"),
-                   "%s %s, %s" % ("Given", "Surname", "Suffix"),
-                   "%s %s %s" % ("Given", "Surname", "Patronymic"),
-                   "%s, %s %s (%s)" % ("SURNAME", "Given", "Suffix", 
-                                       "Common"),
-                   "%s, %s (%s)" % ("SURNAME", "Given", "Common"),
-                   "%s, %s (%s)" % ("SURNAME", "Given", "Call"),
-                   "%s %s" % ("Given", "SURNAME"),
-                   "%s %s, %s" % ("Given", "SURNAME", "Suffix"),
-                   "%s /%s/" % ("Given", "SURNAME"),
+        #repeat above list, but not translated.
+        fmtlyst = ["%s, %s %s (%s)" % (("Surname"), ("Given"), ("Suffix"), 
+                                    ("Common")),
+                "%s, %s %s (%s)" % (("Surname"), ("Given"), ("Suffix"), 
+                                    ("Nickname")),
+                "%s, %s %s (%s)" % (("Surname"), ("Name|Common"), ("Suffix"), 
+                                    ("Nickname")),
+                "%s, %s %s" % (("Surname"), ("Name|Common"), ("Suffix")),
+                "%s, %s %s (%s)" % (("SURNAME"), ("Given"), ("Suffix"), 
+                                    ("Call")),
+                "%s, %s (%s)" % (("Surname"), ("Given"), ("Name|Common")),
+                "%s, %s (%s)" % (("Surname"), ("Name|Common"), ("Nickname")),
+                "%s %s" % (("Given"), ("Surname")),
+                "%s %s, %s" % (("Given"), ("Surname"), ("Suffix")),
+                "%s %s %s" % (("Given"), ("NotPatronymic"), ("Patronymic")),
+                "%s, %s %s (%s)" % (("SURNAME"), ("Given"), ("Suffix"), 
+                                    ("Common")),
+                "%s, %s (%s)" % (("SURNAME"), ("Given"), ("Name|Common")),
+                "%s, %s (%s)" % (("SURNAME"), ("Given"), ("Nickname")),
+                "%s %s" % (("Given"), ("SURNAME")),
+                "%s %s, %s" % (("Given"), ("SURNAME"), ("Suffix")),
+                "%s /%s/" % (("Given"), ("SURNAME")),
+                "%s %s, %s" % (("Given"), ("Rawsurnames"), ("Suffix")),
                    ]
         rand = int(random.random() * len(lyst))
         f = lyst[rand]
@@ -519,13 +570,10 @@ class GrampsPreferences(ConfigureDialog):
         self.edit_button.set_sensitive(False)
         self.remove_button.set_sensitive(False)
         self.insert_button.set_sensitive(False)
-        self.__current_path = path
-        self.__current_text = f
 
     def __edit_name(self, obj):
         store, node = self.format_list.get_selection().get_selected()
         path = self.fmt_model.get_path(node)
-        self.__current_path = path
         self.edit_button.set_sensitive(False)
         self.remove_button.set_sensitive(False)
         self.insert_button.set_sensitive(False)
@@ -552,14 +600,31 @@ class GrampsPreferences(ConfigureDialog):
             iter = model.iter_next(iter)
         return False
 
-    def __cancel_change(self, obj):
-        self.__change_name("", self.__current_path, self.__current_text)
+    def __start_name_editing(self, dummy_renderer, dummy_editable, dummy_path):
+        """
+        Method called at the start of editing a name format.
+        """
+        self.format_list.set_tooltip_text(_("Enter to save, Esc to cancel "
+                                            "editing"))
+
+    def __cancel_change(self, dummy_renderer):
+        """
+        Break off the editing of a name format.
+        """
+        self.format_list.set_tooltip_text('')
+        num = self.selected_fmt[COL_NUM]
+        if any(fmt[COL_NUM] == num for fmt in self.dbstate.db.name_formats):
+            return
+        else: # editing a new format not yet in db, cleanup is needed
+            self.fmt_model.remove(self.iter)
+            _nd.del_name_format(num)
+            self.insert_button.set_sensitive(True)
 
     def __change_name(self, text, path, new_text):
         """
-        If the new string is empty, do nothing. Otherwise, renaming the
-        database is simply changing the contents of the name file.
+        Called when a name format changed and needs to be stored in the db.
         """
+        self.format_list.set_tooltip_text('')
         if len(new_text) > 0 and text != new_text:
             # build a pattern from translated pattern:
             pattern = new_text
@@ -629,6 +694,7 @@ class GrampsPreferences(ConfigureDialog):
                                          name_renderer, 
                                          text=COL_NAME)
         name_renderer.set_property('editable', False)
+        name_renderer.connect('editing-started', self.__start_name_editing)
         name_renderer.connect('edited', self.__change_name)
         name_renderer.connect('editing-canceled', self.__cancel_change)
         self.name_renderer = name_renderer
@@ -655,11 +721,9 @@ class GrampsPreferences(ConfigureDialog):
 
         self.insert_button = gtk.Button(stock=gtk.STOCK_ADD)
         self.insert_button.connect('clicked', self.__new_name)
-                              #self.cb_insert_fmt_str)
 
         self.edit_button = gtk.Button(stock=gtk.STOCK_EDIT)
         self.edit_button.connect('clicked', self.__edit_name)
-                                 #self.cb_edit_fmt_str)
         self.edit_button.set_sensitive(False)
 
         self.remove_button = gtk.Button(stock=gtk.STOCK_REMOVE)
@@ -695,7 +759,14 @@ class GrampsPreferences(ConfigureDialog):
         config.set('preferences.name-format', new_idx)
         _nd.set_default_format(new_idx)
         self.uistate.emit('nameformat-changed')
-        
+
+    def cb_pa_sur_changed(self,*args):
+        """
+        checkbox patronymic as surname changed, propagate to namedisplayer
+        """
+        _nd.change_pa_sur()
+        self.uistate.emit('nameformat-changed')
+
     def cb_format_tree_select(self, tree_selection):
         """
         Name format editor TreeView callback
@@ -714,44 +785,6 @@ class GrampsPreferences(ConfigureDialog):
         self.edit_button.set_sensitive(idx)
         self.name_renderer.set_property('editable', idx)
 
-
-    def cb_edit_fmt_str(self, obj):
-        """
-        Name format editor Edit button callback
-        """
-        num, name, fmt = self.selected_fmt[COL_NUM:COL_EXPL]
-        dlg = NameFormatEditDlg(name, fmt, self.examplename)
-        dlg.dlg.set_transient_for(self.window)
-        (res, name, fmt) = dlg.run()
-
-        if res == gtk.RESPONSE_OK and (name != self.selected_fmt[COL_NAME] or 
-                                       fmt != self.selected_fmt[COL_FMT]):
-            exmpl = _nd.format_str(self.examplename, fmt)
-            self.fmt_model.set(self.iter, COL_NAME, name, 
-                               COL_FMT, fmt, 
-                               COL_EXPL, exmpl)
-            self.selected_fmt = (num, name, fmt, exmpl)
-            _nd.edit_name_format(num, name, fmt)
-
-            self.dbstate.db.name_formats = _nd.get_name_format(only_custom=True, 
-                                                               only_active=False)
-        
-    def cb_insert_fmt_str(self, obj):
-        """
-        Name format editor Insert button callback
-        """
-        dlg = NameFormatEditDlg('', '', self.examplename)
-        dlg.dlg.set_transient_for(self.window)
-        (res, n, f) = dlg.run()
-
-        if res == gtk.RESPONSE_OK:
-            i = _nd.add_name_format(n, f)
-            self.fmt_model.append(row=[i, n, f, 
-                                       _nd.format_str(self.examplename, f)])
-
-        self.dbstate.db.name_formats = _nd.get_name_format(only_custom=True, 
-                                                           only_active=False)
-        
     def cb_del_fmt_str(self, obj):
         """
         Name format editor Remove button callback
@@ -775,13 +808,26 @@ class GrampsPreferences(ConfigureDialog):
 
         # Display name:
         self.examplename = Name()
+        examplesurname = Surname()
+        examplesurnamesecond = Surname()
+        examplesurnamepat = Surname()
         self.examplename.set_title('Dr.')
         self.examplename.set_first_name('Edwin Jose')
-        self.examplename.set_surname_prefix('von der')
-        self.examplename.set_surname('Smith')
+        examplesurname.set_prefix('von der')
+        examplesurname.set_surname('Smith')
+        examplesurname.set_connector('and')
+        self.examplename.add_surname(examplesurname)
+        examplesurnamesecond.set_surname('Weston')
+        self.examplename.add_surname(examplesurnamesecond)
+        examplesurnamepat.set_surname('Wilson')
+        examplesurnamepat.set_origintype(
+                                    NameOriginType(NameOriginType.PATRONYMIC))
+        self.examplename.add_surname(examplesurnamepat)
+        self.examplename.set_primary_surname(0)
         self.examplename.set_suffix('Sr')
-        self.examplename.set_patronymic('Wilson')
-        self.examplename.set_call_name('Ed')
+        self.examplename.set_call_name('Jose')
+        self.examplename.set_nick_name('Ed')
+        self.examplename.set_family_nick_name('Underhills')
         # get the model for the combo and the treeview
         active = _nd.get_default_format()
         self.fmt_model, active = self._build_name_format_model(active)
@@ -806,6 +852,13 @@ class GrampsPreferences(ConfigureDialog):
         table.attach(lwidget, 0, 1, row, row+1, yoptions=0)
         table.attach(hbox,    1, 3, row, row+1, yoptions=0)
         row += 1
+        
+        # Pa/Matronymic surname handling
+        self.add_checkbox(table, 
+                          _("Consider single pa/matronymic as surname"), 
+                          row, 'preferences.patronimic-surname', stop=3,
+                          extra_callback=self.cb_pa_sur_changed)
+        row += 1
 
         # Date format:
         obox = gtk.combo_box_new_text()
@@ -846,6 +899,13 @@ class GrampsPreferences(ConfigureDialog):
         table.attach(lwidget, 0, 1, row, row+1, yoptions=0)
         table.attach(obox, 1, 3, row, row+1, yoptions=0)
         row += 1
+        
+        #height multiple surname table 
+        self.add_pos_int_entry(table, 
+                _('Height multiple surname box (pixels)'),
+                row, 'interface.surname-box-height', self.update_surn_height,
+                col_attach=0)
+        row += 1
 
         # Status bar:
         obox = gtk.combo_box_new_text()
@@ -903,15 +963,31 @@ class GrampsPreferences(ConfigureDialog):
         self.old_format = the_list.get_value(the_iter, COL_FMT)
         win = DisplayNameEditor(self.uistate, self.dbstate, self.track, self)
 
-    def date_format_changed(self, obj):
+    def check_for_type_changed(self, obj):
+        active = obj.get_active()
+        if active == 0:  # update
+            config.set('behavior.check-for-update-types', ["update"])
+        elif active == 1:  # update
+            config.set('behavior.check-for-update-types', ["new"])
+        elif active == 2:  # update
+            config.set('behavior.check-for-update-types', ["update", "new"])
 
+    def toggle_hide_previous_addons(self, obj):
+        active = obj.get_active()
+        config.set('behavior.do-not-show-previously-seen-updates',  
+                   bool(active))
+
+    def check_for_updates_changed(self, obj):
+        active = obj.get_active()
+        config.set('behavior.check-for-updates', active)
+
+    def date_format_changed(self, obj):
         config.set('preferences.date-format', obj.get_active())
         OkDialog(_('Change is not immediate'), 
                  _('Changing the data format will not take '
                    'effect until the next time Gramps is started.'))
 
-    def date_calendar_changed(self, obj):
- 
+    def date_calendar_changed(self, obj): 
         config.set('preferences.calendar-format-report', obj.get_active())
     
     def add_date_panel(self, configdialog):
@@ -974,22 +1050,65 @@ class GrampsPreferences(ConfigureDialog):
                 5, self.path_entry, self.dbstate.db.get_mediapath(),
                 self.set_mediapath, self.select_mediapath)
 
+        # Check for updates:
+        obox = gtk.combo_box_new_text()
+        formats = [_("Never"), 
+                   _("Once a month"), 
+                   _("Once a week"), 
+                   _("Once a day"), 
+                   _("Always"), ]
+        map(obox.append_text, formats)
+        active = config.get('behavior.check-for-updates')
+        obox.set_active(active)
+        obox.connect('changed', self.check_for_updates_changed)
+        lwidget = BasicLabel("%s: " % _('Check for updates'))
+        table.attach(lwidget, 1, 2, 6, 7, yoptions=0)
+        table.attach(obox,    2, 3, 6, 7, yoptions=0)
+
+        self.whattype_box = gtk.combo_box_new_text()
+        formats = [_("Updated addons only"), 
+                   _("New addons only"), 
+                   _("New and updated addons"),]
+        map(self.whattype_box.append_text, formats)
+        whattype = config.get('behavior.check-for-update-types')
+        if "new" in whattype and "update" in whattype:
+            self.whattype_box.set_active(2)
+        elif "new" in whattype:
+            self.whattype_box.set_active(1)
+        elif "update" in whattype:
+            self.whattype_box.set_active(0)
+        self.whattype_box.connect('changed', self.check_for_type_changed)
+        lwidget = BasicLabel("%s: " % _('What to check'))
+        table.attach(lwidget, 1, 2, 7, 8, yoptions=0)
+        table.attach(self.whattype_box, 2, 3, 7, 8, yoptions=0)
+
+        checkbutton = gtk.CheckButton(
+            _("Do not ask about previously notified addons"))
+        checkbutton.set_active(config.get('behavior.do-not-show-previously-seen-updates'))
+        checkbutton.connect("toggled", self.toggle_hide_previous_addons)
+
+        table.attach(checkbutton, 0, 3, 8, 9, yoptions=0)
+        button = gtk.Button(_("Check now"))
+        button.connect("clicked", lambda obj: \
+                  self.uistate.viewmanager.check_for_updates(force=True))
+        table.attach(button, 3, 4, 8, 9, yoptions=0)
+
         return _('General'), table
 
-    def add_database_panel(self, configdialog):
+    def add_famtree_panel(self, configdialog):
         table = gtk.Table(2, 2)
         table.set_border_width(12)
         table.set_col_spacings(6)
         table.set_row_spacings(6)
 
         self.add_entry(table, 
-                _('Database path'), 
+                _('Family Tree Database path'), 
                 0, 'behavior.database-path')
         self.add_checkbox(table, 
-                _('Automatically load last database'), 
+                _('Automatically load last family tree'), 
                 1, 'behavior.autoload')
                 
-        return _('Database'), table
+        return _('Family Tree'), table
 
     def set_mediapath(self, *obj):
         if self.path_entry.get_text().strip():
@@ -1000,7 +1119,7 @@ class GrampsPreferences(ConfigureDialog):
     def select_mediapath(self, *obj):
         f = gtk.FileChooserDialog(
             _("Select media directory"),
-            action=gtk.FILE_CHOOSER_ACTION_CREATE_FOLDER,
+            action=gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER,
             buttons=(gtk.STOCK_CANCEL,
                      gtk.RESPONSE_CANCEL,
                      gtk.STOCK_APPLY,
@@ -1048,6 +1167,23 @@ class GrampsPreferences(ConfigureDialog):
         else:
             obj.set_text(str(intval))
 
+    def update_surn_height(self, obj, constant):
+        ok = True
+        if not obj.get_text():
+            return
+        try:
+            intval = int(obj.get_text())
+        except:
+            intval = config.get(constant)
+            ok = False
+        if intval < 0 :
+            intval = config.get(constant)
+            ok = False
+        if ok:
+            config.set(constant, intval)
+        else:
+            obj.set_text(str(intval))
+
     def build_menu_names(self, obj):
         return (_('Preferences'), None)
 
@@ -1060,71 +1196,3 @@ class GrampsPreferences(ConfigureDialog):
         button.add(image)
         button.show()
         return button
-    
-class NameFormatEditDlg(object):
-    """
-    """
-    
-    def __init__(self, fmt_name, fmt_str, name):
-        self.fmt_name = fmt_name
-        self.fmt_str = fmt_str
-        self.name = name
-        self.valid = True
-        self.top = Glade()
-        
-        self.dlg = self.top.get_object('namefmt_edit')
-        ManagedWindow.set_titles(self.dlg, None, _('Name Format Editor'))
-        
-        self.examplelabel = self.top.get_object('example_label')
-        
-        self.nameentry = self.top.get_object('name_entry')
-        self.nameentry.set_text('<span weight="bold">%s</span>' % self.fmt_name)
-        self.nameentry.set_use_markup(True)
-        
-        self.formatentry = self.top.get_object('format_entry')
-        self.formatentry.connect('changed', self.cb_format_changed)
-        self.formatentry.set_text(self.fmt_str)
-        
-    def run(self):
-        running = True
-        while running:
-            self.response = self.dlg.run()
-
-            running = False
-            self.fmt_name = self.nameentry.get_text()
-            self.fmt_str = self.formatentry.get_text()
-            
-            if self.response == gtk.RESPONSE_OK:
-                if not self.valid:
-                    q = QuestionDialog2(
-                        _('The format definition is invalid'), 
-                        _('What would you like to do?'), 
-                        _('_Continue anyway'), _('_Modify format'), 
-                        parent=self.dlg)
-                    running = not q.run()
-                    self.response = gtk.RESPONSE_CANCEL
-                elif self.fmt_name == '' and self.fmt_str == '':
-                    self.response = gtk.RESPONSE_CANCEL
-                elif (self.fmt_name == '') ^ (self.fmt_str == ''):
-                    ErrorDialog(
-                        _('Both Format name and definition have to be defined.'), 
-                        parent=self.dlg)
-                    running = True
-                                    
-        self.dlg.destroy()
-        return (self.response, self.fmt_name, self.fmt_str)
-
-    def cb_format_changed(self, obj):
-        try:
-            t = (_nd.format_str(self.name, escape(obj.get_text())))
-            sample = '<span weight="bold" style="italic">%s</span>' % t
-            self.valid = True
-        except NameDisplayError:
-            t = _("Invalid or incomplete format definition.")
-            sample = '<span foreground="#FF0000">%s</span>' % t
-            self.valid = False
-
-        self.examplelabel.set_text(sample)
-        self.examplelabel.set_use_markup(True)
-        self.nameentry.set_text('<span weight="bold">%s</span>' % obj.get_text())
-        self.nameentry.set_use_markup(True)
