@@ -20,7 +20,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: dbman.py 14282 2010-02-08 17:19:15Z ldnp $
+# $Id: dbman.py 16631 2011-02-16 18:38:01Z m_d_n $
 
 """
 Provide the management of databases. This includes opening, renaming,
@@ -68,7 +68,7 @@ import pango
 # gramps modules
 #
 #-------------------------------------------------------------------------
-from QuestionDialog import ErrorDialog, QuestionDialog
+from QuestionDialog import ErrorDialog, QuestionDialog, QuestionDialog2
 from gen.db import DbBsddb
 from gui.pluginmanager import GuiPluginManager
 from cli.clidbman import CLIDbManager, NAME_FILE, time_val
@@ -77,6 +77,7 @@ import RecentFiles
 from glade import Glade
 from gen.db.backup import restore
 from gen.db.exceptions import DbException
+from Utils import get_unicode_path_from_env_var
 
 
 _RETURN = gtk.gdk.keyval_from_name("Return")
@@ -114,6 +115,8 @@ class DbManager(CLIDbManager):
                 CLIDbManager.ICON_OPEN : gtk.STOCK_OPEN,
                }
 
+    ERROR = ErrorDialog
+        
     def __init__(self, dbstate, parent=None):
         """
         Create the top level window from the glade description, and extracts
@@ -233,7 +236,7 @@ class DbManager(CLIDbManager):
                 self.rcs.set_sensitive(False)
 
         if store.get_value(node, STOCK_COL) == gtk.STOCK_DIALOG_ERROR:
-            path = store.get_value(node, PATH_COL)
+            path = get_unicode_path_from_env_var(store.get_value(node, PATH_COL))
             backup = os.path.join(path, "person.gbkp")
             self.repair.set_sensitive(os.path.isfile(backup))
         else:
@@ -255,7 +258,8 @@ class DbManager(CLIDbManager):
         'edited' signal, so that we can change the name when the user changes
         the column.
 
-        The last modified column simply displays the last modification time.
+        The last accessed column simply displays the last time famtree was 
+        opened.
         """
 
         # build the database name column
@@ -278,9 +282,9 @@ class DbManager(CLIDbManager):
                                          stock_id=STOCK_COL)
         self.dblist.append_column(icon_column)
 
-        # build the last modified column
+        # build the last accessed column
         render = gtk.CellRendererText()
-        column = gtk.TreeViewColumn(_('Last modified'), render, text=DATE_COL)
+        column = gtk.TreeViewColumn(_('Last accessed'), render, text=DATE_COL)
         column.set_sort_column_id(DSORT_COL)
         self.dblist.append_column(column)
 
@@ -345,8 +349,8 @@ class DbManager(CLIDbManager):
                     self.top.destroy()
                     del self.selection
                     del self.name_renderer
-                    return (store.get_value(node, PATH_COL),
-                            store.get_value(node, NAME_COL))
+                    path = get_unicode_path_from_env_var(store.get_value(node, PATH_COL))
+                    return (path, store.get_value(node, NAME_COL))
             else:
                 self.top.destroy()
                 del self.selection
@@ -380,7 +384,7 @@ class DbManager(CLIDbManager):
         try:
             self.break_lock(self.lock_file)
             store, node = self.selection.get_selected()
-            dbpath = store.get_value(node, PATH_COL)
+            dbpath = get_unicode_path_from_env_var(store.get_value(node, PATH_COL))
             (tval, last) = time_val(dbpath)
             store.set_value(node, OPEN_COL, 0)
             store.set_value(node, STOCK_COL, "")
@@ -445,7 +449,7 @@ class DbManager(CLIDbManager):
         del proc
 
         if status != 0:
-            ErrorDialog(
+            DbManager.ERROR(
                 _("Rename failed"),
                 _("An attempt to rename a version failed "
                   "with the following message:\n\n%s") % message
@@ -461,23 +465,13 @@ class DbManager(CLIDbManager):
         node = self.model.get_iter(path)
         filename = self.model.get_value(node, FILE_COL)
         if self.existing_name(new_text, skippath=path):
-            ErrorDialog(
-                    _("Could not rename the Family Tree."), 
-                    _("Family Tree already exists, choose a unique name."))
+            DbManager.ERROR(_("Could not rename the Family Tree."), 
+                  _("Family Tree already exists, choose a unique name."))
             return
-        try:
-            name_file = open(filename, "r")
-            old_text=name_file.read()
-            name_file.close()
-            name_file = open(filename, "w")
-            name_file.write(new_text)
-            name_file.close()
+        old_text, new_text = self.rename_database(filename, new_text)
+        if not (old_text is None):
             RecentFiles.rename_filename(old_text, new_text)
             self.model.set_value(node, NAME_COL, new_text)
-        except (OSError, IOError), msg:
-            ErrorDialog(
-                _("Could not rename family tree"),
-                str(msg))
 
     def __rcs(self, obj):
         """
@@ -574,7 +568,7 @@ class DbManager(CLIDbManager):
                     os.unlink(os.path.join(top, filename))
             os.rmdir(self.data_to_delete[1])
         except (IOError, OSError), msg:
-            ErrorDialog(_("Could not delete family tree"),
+            DbManager.ERROR(_("Could not delete family tree"),
                                        str(msg))
         # rebuild the display
         self.__populate()
@@ -598,7 +592,7 @@ class DbManager(CLIDbManager):
         del proc
 
         if status != 0:
-            ErrorDialog(
+            DbManager.ERROR(
                 _("Deletion failed"),
                 _("An attempt to delete a version failed "
                   "with the following message:\n\n%s") % message
@@ -625,6 +619,36 @@ class DbManager(CLIDbManager):
         """
         store, node = self.selection.get_selected()
         dirname = store[node][1]
+        
+        #First ask user if he is really sure :-)
+        yes_no = QuestionDialog2(
+            _("Repair family tree?"),
+            _(
+"If you click <b>Proceed</b>, Gramps will attempt to recover your family tree"
+" from the last good backup. There are several ways this can cause unwanted"
+" effects, so <b>backup</b> the family tree first.\n"
+"The Family tree you have selected is stored in %s.\n\n"
+"Before doing a repair, verify that the Family Tree can really no longer be "
+" opened, as the database back-end can recover from some errors automatically.\n\n"
+"<b>Details:</b> Repairing a Family Tree actually uses the last backup of"
+" the Family Tree, which Gramps stored on last use. If you have worked for"
+" several hours/days without closing Gramps, then all this information will"
+" be lost! If the repair fails, then the original family tree will be lost"
+" forever, hence a backup is needed. If the repair fails, or too much"
+" information is lost, you can fix the original family tree manually."
+" For details, see the webpage\n"
+"http://gramps-project.org/wiki/index.php?title=Recover_corrupted_family_tree\n"
+"Before doing a repair, try to open the family tree in the normal manner."
+" Several errors that trigger the repair button can be fixed automatically."
+" If this is the case, you can disable the repair button by removing the"
+" file <i>need_recover</i> in the family tree directory."
+) % dirname,
+            _("Proceed, I have taken a backup"),
+            _("Stop"))
+        prompt = yes_no.run()
+        if not prompt:
+            return
+        
         opened = store[node][5]
         if opened:
             self.dbstate.no_database()
@@ -648,7 +672,7 @@ class DbManager(CLIDbManager):
         try:
             restore(dbase)
         except DbException, msg:
-            ErrorDialog(_("Error restoring backup data"), msg)
+            DbManager.ERROR(_("Error restoring backup data"), msg)
 
         self.__end_cursor()
 
@@ -683,7 +707,7 @@ class DbManager(CLIDbManager):
         try:
             self._create_new_db()
         except (OSError, IOError), msg:
-            ErrorDialog(_("Could not create family tree"),
+            DbManager.ERROR(_("Could not create family tree"),
                                        str(msg))
         self.new.set_sensitive(True)
 
