@@ -19,7 +19,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: gallerytab.py 17720 2011-06-08 19:51:42Z gburto01 $
+# $Id: gallerytab.py 17872 2011-06-30 20:40:42Z nick-h $
 
 #-------------------------------------------------------------------------
 #
@@ -287,6 +287,16 @@ class GalleryTab(ButtonTab, DbGUIElement):
         self.changed = True
         self.rebuild()
 
+    def __blocked_text(self):
+        """
+        Return the common text used when mediaref cannot be edited
+        """
+        return _("This media reference cannot be edited at this time. "
+                    "Either the associated media object is already being "
+                    "edited or another media reference that is associated with "
+                    "the same media object is being edited.\n\nTo edit this "
+                    "media reference, you need to close the media object.")
+
     def share_button_clicked(self, obj):
         """
         Function called when the Share button is clicked. 
@@ -305,7 +315,9 @@ class GalleryTab(ButtonTab, DbGUIElement):
                 EditMediaRef(self.dbstate, self.uistate, self.track, 
                              src, sref, self.add_callback)
             except Errors.WindowActiveError:
-                pass
+                from QuestionDialog import WarningDialog
+                WarningDialog(_("Cannot share this reference"),
+                              self.__blocked_text())
 
     def del_button_clicked(self, obj):
         ref = self.get_selected()
@@ -323,7 +335,9 @@ class GalleryTab(ButtonTab, DbGUIElement):
                 EditMediaRef(self.dbstate, self.uistate, self.track, 
                              obj, ref, self.edit_callback)
             except Errors.WindowActiveError:
-                pass
+                from QuestionDialog import WarningDialog
+                WarningDialog(_("Cannot edit this reference"),
+                              self.__blocked_text())
 
     def edit_callback(self, media_ref, media):
         """
