@@ -19,7 +19,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: editevent.py 16118 2010-11-02 18:57:06Z bmcage $
+# $Id: editevent.py 16680 2011-02-20 10:52:06Z bmcage $
 
 #-------------------------------------------------------------------------
 #
@@ -42,6 +42,7 @@ import gtk
 #-------------------------------------------------------------------------
 import const
 import gen.lib
+from gen.db import DbTxn
 import GrampsDisplay
 from editprimary import EditPrimary
 from objectentries import PlaceEntry
@@ -77,7 +78,8 @@ class EditEvent(EditPrimary):
         self._init_event()
 
     def _init_event(self):
-        self.commit_event = self.db.commit_event
+        if not self.db.readonly:
+            self.commit_event = self.db.commit_event
 
     def empty_object(self):
         return gen.lib.Event()
@@ -98,8 +100,9 @@ class EditEvent(EditPrimary):
         return dialog_title
 
     def get_custom_events(self):
-        return self.dbstate.db.get_person_event_types() + \
-               self.dbstate.db.get_family_event_types()
+        return (self.dbstate.db.get_person_event_types() +
+                self.dbstate.db.get_family_event_types()
+               )
 
     def _local_init(self):
         self.width_key = 'interface.event-width'
@@ -251,17 +254,17 @@ class EditEvent(EditPrimary):
             return
 
         if not self.obj.handle:
-            trans = self.db.transaction_begin()
-            self.db.add_event(self.obj, trans)
-            self.db.transaction_commit(trans, _("Add Event"))
+            with DbTxn(_("Add Event (%s)") % self.obj.get_gramps_id(),
+                       self.db) as trans:
+                self.db.add_event(self.obj, trans)
         else:
             orig = self.get_from_handle(self.obj.handle)
             if cmp(self.obj.serialize(), orig.serialize()):
-                trans = self.db.transaction_begin()
-                if not self.obj.get_gramps_id():
-                    self.obj.set_gramps_id(self.db.find_next_event_gramps_id())
-                self.commit_event(self.obj, trans)
-                self.db.transaction_commit(trans, _("Edit Event"))
+                with DbTxn(_("Edit Event (%s)") % self.obj.get_gramps_id(),
+                           self.db) as trans:
+                    if not self.obj.get_gramps_id():
+                        self.obj.set_gramps_id(self.db.find_next_event_gramps_id())
+                    self.commit_event(self.obj, trans)
 
         if self.callback:
             self.callback(self.obj)
@@ -297,7 +300,8 @@ class EditPersonEvent(EditEvent):
         EditEvent.__init__(self, dbstate, uistate, track, event, callback)
 
     def _init_event(self):
-        self.commit_event = self.db.commit_personal_event
+        if not self.db.readonly:
+            self.commit_event = self.db.commit_personal_event
 
     def get_custom_events(self):
         return self.dbstate.db.get_person_event_types()
@@ -308,7 +312,8 @@ class EditFamilyEvent(EditEvent):
         EditEvent.__init__(self, dbstate, uistate, track, event, callback)
 
     def _init_event(self):
-        self.commit_event = self.db.commit_family_event
+        if not self.db.readonly:
+            self.commit_event = self.db.commit_family_event
 
     def get_custom_events(self):
         return self.dbstate.db.get_family_event_types()
@@ -327,22 +332,21 @@ class DeleteEventQuery(object):
         self.family_list = family_list
 
     def query_response(self):
-        trans = self.db.transaction_begin()
-        self.db.disable_signals()
+        with DbTxn(_("Delete Event (%s)") % self.event.get_gramps_id(),
+                   self.db) as trans:
+            self.db.disable_signals()
         
-        ev_handle_list = [self.event.get_handle()]
+            ev_handle_list = [self.event.get_handle()]
 
-        for handle in self.person_list:
-            person = self.db.get_person_from_handle(handle)
-            person.remove_handle_references('Event', ev_handle_list)
-            self.db.commit_person(person, trans)
+            for handle in self.person_list:
+                person = self.db.get_person_from_handle(handle)
+                person.remove_handle_references('Event', ev_handle_list)
+                self.db.commit_person(person, trans)
 
-        for handle in self.family_list:
-            family = self.db.get_family_from_handle(handle)
-            family.remove_handle_references('Event', ev_handle_list)
-            self.db.commit_family(family, trans)
+            for handle in self.family_list:
+                family = self.db.get_family_from_handle(handle)
+                family.remove_handle_references('Event', ev_handle_list)
+                self.db.commit_family(family, trans)
 
-        self.db.enable_signals()
-        self.db.remove_event(self.event.get_handle(), trans)
-        self.db.transaction_commit(
-            trans,_("Delete Event (%s)") % self.event.get_gramps_id())
+            self.db.enable_signals()
+            self.db.remove_event(self.event.get_handle(), trans)
