@@ -20,7 +20,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: editfamily.py 17220 2011-04-23 08:54:01Z m_d_n $
+# $Id: editfamily.py 18084 2011-08-29 19:06:59Z m_d_n $
 
 #-------------------------------------------------------------------------
 #
@@ -391,8 +391,9 @@ class ChildEmbedList(EmbeddedList):
             mothername = gen.lib.Name()
             preset_name(mother, mothername)
             preset_name(father, name)
-            name.set_surname_list(name.get_surname_list[:1].append(
-                                            mothername.get_surname_list[:1]))
+            mothersurname = mothername.get_surname_list()[0]
+            mothersurname.set_primary(False)
+            name.set_surname_list([name.get_surname_list()[0], mothersurname])
             return name
         else:
             return name
