@@ -20,7 +20,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: editmediaref.py 16104 2010-11-01 20:48:23Z bmcage $
+# $Id: editmediaref.py 16680 2011-02-20 10:52:06Z bmcage $
 
 #-------------------------------------------------------------------------
 #
@@ -47,6 +47,7 @@ import gen.mime
 import ThumbNails
 import Utils
 from gen.lib import NoteType
+from gen.db import DbTxn
 from glade import Glade
 from displaytabs import (SourceEmbedList, AttrEmbedList, MediaBackRefList, 
                          NoteTab)
@@ -98,25 +99,53 @@ class EditMediaRef(EditReference):
                               _('_General'), tblref)
         self.track_ref_for_deletion("primtab")
 
+    def setup_filepath(self):
+        self.select = self.top.get_object('file_select')
+        self.track_ref_for_deletion("select")
+        self.file_path = self.top.get_object("path")
+        self.track_ref_for_deletion("file_path")
+
+        self.file_path.set_text(self.source.get_path())
+        self.select.connect('clicked', self.select_file)
+
+    def determine_mime(self):
+        descr = gen.mime.get_description(self.source.get_mime_type())
+        if descr:
+            self.mimetext.set_text(descr)
+
+        path = self.file_path.get_text()
+        path_full = Utils.media_path_full(self.db, path)
+        if path != self.source.get_path() and path_full != self.source.get_path():
+            #redetermine mime
+            mime = gen.mime.get_type(Utils.find_file(path_full))
+            self.source.set_mime_type(mime)
+            descr = gen.mime.get_description(mime)
+            if descr:
+                self.mimetext.set_text(descr)
+            else:
+                self.mimetext.set_text(_('Unknown'))
+        #if mime type not set, is note
+        if not self.source.get_mime_type():
+            self.mimetext.set_text(_('Note'))
+
     def draw_preview(self):
         """
         Draw the two preview images. This method can be called on eg change of
         the path.
         """
-        self.mtype = self.source.get_mime_type()
-        fullpath = Utils.media_path_full(self.db, self.source.get_path())
-        self.pix = ThumbNails.get_thumbnail_image(fullpath,
-                                                  self.mtype)
-        self.pixmap.set_from_pixbuf(self.pix)
-        
-        self.subpix = ThumbNails.get_thumbnail_image(fullpath,
-                                                     self.mtype,
-                                                     self.rectangle)
-        self.subpixmap.set_from_pixbuf(self.subpix)
+        mtype = self.source.get_mime_type()
+        if mtype:
+            fullpath = Utils.media_path_full(self.db, self.source.get_path())
+            pb = ThumbNails.get_thumbnail_image(fullpath, mtype)
+            self.pixmap.set_from_pixbuf(pb)
+            subpix = ThumbNails.get_thumbnail_image(fullpath, mtype,
+                                                    self.rectangle)
+            self.subpixmap.set_from_pixbuf(subpix)
+        else:
+            pb = ThumbNails.find_mime_type_pixbuf('text/plain')
+            self.pixmap.set_from_pixbuf(pb)
+            self.subpixmap.set_from_pixbuf(pb)
 
-        mt = gen.mime.get_description(self.mtype)
-        self.top.get_object("type").set_text(mt if mt else "")
-        
     def _setup_fields(self):
         ebox_shared = self.top.get_object('eventbox')
         ebox_shared.connect('button-press-event', self.button_press_event)
@@ -131,6 +160,8 @@ class EditMediaRef(EditReference):
             ebox_ref.add_events(gtk.gdk.BUTTON_RELEASE_MASK)
 
         self.pixmap = self.top.get_object("pixmap")
+        self.mimetext = self.top.get_object("type")
+        self.track_ref_for_deletion("mimetext")
         
         coord = self.source_ref.get_rectangle()
         #upgrade path: set invalid (from eg old db) to none
@@ -146,6 +177,8 @@ class EditMediaRef(EditReference):
         self.subpixmap = self.top.get_object("subpixmap")
         self.track_ref_for_deletion("subpixmap")
 
+        self.setup_filepath()
+        self.determine_mime()
         self.draw_preview()
         
         corners = ["corner1_x", "corner1_y", "corner2_x", "corner2_y"]
@@ -477,8 +510,16 @@ class EditMediaRef(EditReference):
         """
         for obj in (self.descr_window, self.path_obj):
             obj.update()
+        self.determine_mime()
         self.draw_preview()
 
+    def select_file(self, val):
+        self.determine_mime()
+        path = self.file_path.get_text()
+        self.source.set_path(Utils.get_unicode_path_from_file_chooser(path))
+        AddMediaObject(self.dbstate, self.uistate, self.track, self.source, 
+                       self._update_addmedia)
+
     def _connect_signals(self):
         self.define_cancel_button(self.top.get_object('button84'))
         self.define_ok_button(self.top.get_object('button82'),self.save)
@@ -546,15 +587,14 @@ class EditMediaRef(EditReference):
 
     def save(self,*obj):
         #first save primary object
-        trans = self.db.transaction_begin()
         if self.source.handle:
-            self.db.commit_media_object(self.source, trans)
-            self.db.transaction_commit(trans, _("Edit Media Object (%s)"
-                                           ) % self.source.get_description())
+            with DbTxn(_("Edit Media Object (%s)") %
+                       self.source.get_description(), self.db) as trans:
+                self.db.commit_media_object(self.source, trans)
         else:
-            self.db.add_object(self.source, trans)
-            self.db.transaction_commit(trans,_("Add Media Object (%s)"
-                                           ) % self.source.get_description())
+            with DbTxn(_("Add Media Object (%s)") % 
+                       self.source.get_description(), self.db) as trans:
+                self.db.add_object(self.source, trans)
 
         #save reference object in memory
         coord = (
