@@ -4,6 +4,7 @@
 # Copyright (C) 2000-2007  Donald N. Allingham
 #               2008-2009  Benny Malengier
 #               2009       Gary Burton
+#               2010       Michiel D. Nauta
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,14 +21,16 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: editname.py 14091 2010-01-18 04:42:17Z pez4brian $
+# $Id: editname.py 16435 2011-01-22 15:07:29Z bmcage $
 
 #-------------------------------------------------------------------------
 #
 # Standard python modules
 #
 #-------------------------------------------------------------------------
+import gobject
 from gen.ggettext import gettext as _
+from copy import copy
 
 #-------------------------------------------------------------------------
 #
@@ -41,13 +44,15 @@ import gtk
 # gramps modules
 #
 #-------------------------------------------------------------------------
+import config
 from gen.display.name import displayer as name_displayer
 from editsecondary import EditSecondary
 from gen.lib import NoteType
-from displaytabs import GrampsTab,SourceEmbedList,NoteTab
+from displaytabs import GrampsTab, SourceEmbedList, NoteTab, SurnameTab
 from gui.widgets import (MonitoredEntry, MonitoredMenu, MonitoredDate, 
                      MonitoredDataType, PrivacyButton)
 from glade import Glade                     
+from Errors import ValidationError
 
 #-------------------------------------------------------------------------
 #
@@ -110,12 +115,15 @@ class EditName(EditSecondary):
         
         self.top = Glade()
         
-        self.set_window(self.top.toplevel,
-                        self.top.get_object("title"),
-                        _("Name Editor"))
+        self.set_window(self.top.toplevel, None, _("Name Editor"))
 
         tblgnam =  self.top.get_object('table23')
         notebook = self.top.get_object('notebook')
+        hbox_surn = self.top.get_object('hboxmultsurnames')
+        hbox_surn.set_size_request(-1, 
+                            int(config.get('interface.surname-box-height')))
+        hbox_surn.pack_start(SurnameTab(self.dbstate, self.uistate, self.track, 
+                                        self.obj, top_label=None))
         #recreate start page as GrampsTab
         notebook.remove_page(0)
         self.gennam = GeneralNameTab(self.dbstate, self.uistate, self.track, 
@@ -123,7 +131,7 @@ class EditName(EditSecondary):
 
         self.original_group_as = self.obj.get_group_as()
         self.original_group_set = not (self.original_group_as == '')
-        srn = self.obj.get_surname()
+        srn = self.obj.get_primary_surname().get_surname()
         self._get_global_grouping(srn)
 
         self.group_over = self.top.get_object('group_over')
@@ -135,13 +143,30 @@ class EditName(EditSecondary):
     def _post_init(self):
         """if there is override, set the override toggle active
         """
-        if self.original_group_set or self.global_group_set :
+        if self.original_group_set:
             self.group_over.set_active(True)
+        else:
+            # The glade file correctly sets group_as widget editable=False.
+            # At this stage of initialization self.group_as.obj.get_editable()
+            # is however still true, correct that.
+            self.group_as.enable(False)
 
     def _connect_signals(self):
         self.define_cancel_button(self.top.get_object('button119'))
         self.define_help_button(self.top.get_object('button131'))
-        self.define_ok_button(self.top.get_object('button118'),self.save)
+        self.define_ok_button(self.top.get_object('button118'), self.save)
+
+    def _validate_call(self, widget, text):
+        """ a callname must be a part of the given name, see if this is the 
+            case """
+        validcall = self.given_field.obj.get_text().split()
+        dummy = copy(validcall)
+        for item in dummy:
+            validcall += item.split('-')
+        if text in validcall:
+            return
+        return ValidationError(_("Call name must be the given name that "
+                                     "is normally used."))
 
     def _setup_fields(self):
         self.group_as = MonitoredEntry(
@@ -154,7 +179,7 @@ class EditName(EditSecondary):
             if self.global_group_set :
                 self.group_as.force_value(self.global_group_as)
             else :
-                self.group_as.force_value(self.obj.get_surname())
+                self.group_as.force_value(self.obj.get_primary_surname().get_surname())
             
         format_list = [(name, number) for (number, name,fmt_str,act)
                        in name_displayer.get_name_format(also_default=True)]
@@ -174,7 +199,7 @@ class EditName(EditSecondary):
             self.db.readonly)
 
         self.given_field = MonitoredEntry(
-            self.top.get_object("alt_given"),
+            self.top.get_object("given_name"),
             self.obj.set_first_name,
             self.obj.get_first_name,
             self.db.readonly)
@@ -184,39 +209,42 @@ class EditName(EditSecondary):
             self.obj.set_call_name,
             self.obj.get_call_name,
             self.db.readonly)
+        self.call_field.connect("validate", self._validate_call)
+        #force validation now with initial entry
+        self.call_field.obj.validate(force=True)
 
         self.title_field = MonitoredEntry(
-            self.top.get_object("alt_title"),
+            self.top.get_object("title_field"),
             self.obj.set_title,
             self.obj.get_title,
             self.db.readonly)
 
         self.suffix_field = MonitoredEntry(
-            self.top.get_object("alt_suffix"),
+            self.top.get_object("suffix"),
             self.obj.set_suffix,
             self.obj.get_suffix,
             self.db.readonly)
 
-        self.patronymic_field = MonitoredEntry(
-            self.top.get_object("patronymic"),
-            self.obj.set_patronymic,
-            self.obj.get_patronymic,
+        self.nick = MonitoredEntry(
+            self.top.get_object("nickname"), 
+            self.obj.set_nick_name, 
+            self.obj.get_nick_name, 
             self.db.readonly)
 
-        self.surname_field = MonitoredEntry(
-            self.top.get_object("alt_surname"),
-            self.obj.set_surname,
-            self.obj.get_surname,
-            self.db.readonly,
-            autolist=self.db.get_surname_list(),
-            changed=self.update_group_as)
-
-        self.prefix_field = MonitoredEntry(
-            self.top.get_object("alt_prefix"),
-            self.obj.set_surname_prefix,
-            self.obj.get_surname_prefix,
+        self.famnick = MonitoredEntry(
+            self.top.get_object("familynickname"), 
+            self.obj.set_family_nick_name, 
+            self.obj.get_family_nick_name, 
             self.db.readonly)
 
+        #self.surname_field = MonitoredEntry(
+        #    self.top.get_object("alt_surname"),
+        #    self.obj.set_surname,
+        #    self.obj.get_surname,
+        #    self.db.readonly,
+        #    autolist=self.db.get_surname_list() if not self.db.readonly else [],
+        #    changed=self.update_group_as)
+
         self.date = MonitoredDate(
             self.top.get_object("date_entry"),
             self.top.get_object("date_stat"), 
@@ -226,7 +254,7 @@ class EditName(EditSecondary):
             self.db.readonly)
 
         self.obj_combo = MonitoredDataType(
-            self.top.get_object("name_type"),
+            self.top.get_object("ntype"),
             self.obj.set_type,
             self.obj.get_type,
             self.db.readonly,
@@ -242,16 +270,17 @@ class EditName(EditSecondary):
         notebook = self.top.get_object("notebook")
 
         self._add_tab(notebook, self.gennam)
+        self.track_ref_for_deletion("gennam")
 
-        self.srcref_list = self._add_tab(
-            notebook,
-            SourceEmbedList(self.dbstate,self.uistate,self.track,self.obj))
+        self.srcref_list = SourceEmbedList(self.dbstate,self.uistate,self.track,self.obj)
+        self._add_tab(notebook, self.srcref_list)
+        self.track_ref_for_deletion("srcref_list")
         
-        self.note_tab = self._add_tab(
-            notebook,
-            NoteTab(self.dbstate, self.uistate, self.track,
+        self.note_tab = NoteTab(self.dbstate, self.uistate, self.track,
                     self.obj.get_note_list(),
-                    notetype=NoteType.PERSONNAME))
+                    notetype=NoteType.PERSONNAME)
+        self._add_tab(notebook, self.note_tab)
+        self.track_ref_for_deletion("note_tab")
 
         self._setup_notebook_tabs( notebook)
         
@@ -280,7 +309,7 @@ class EditName(EditSecondary):
         """Callback if surname changes on GUI
             If overwrite is not set, we change the group name too
         """
-        name = self.obj.get_surname()
+        name = self.obj.get_primary_surname().get_surname()
         if not self.group_over.get_active():
             self.group_as.force_value(name)
         #new surname, so perhaps now a different grouping?
@@ -303,17 +332,20 @@ class EditName(EditSecondary):
         
     def on_group_over_toggled(self, obj):
         """ group over changes, if activated, enable edit,
-            if unactivated, go back to surname.
+            if unactivated, go back to surname/global_group_as.
         """
         self.toggle_dirty = False
         #enable group as box
         self.group_as.enable(obj.get_active())
         
         if not obj.get_active():
-            surname = self.obj.get_surname()
-            self.group_as.set_text(surname)
+            if self.global_group_set:
+                self.group_as.set_text(self.global_group_as)
+            else:
+                surname = self.obj.get_primary_surname().get_surname()
+                self.group_as.set_text(surname)
 
-    def save(self,*obj):
+    def save(self, *obj):
         """Save the name setting. All is ok, except grouping. We need to 
            consider: 
             1/     global set, not local set --> unset (ask if global unset)
@@ -326,7 +358,7 @@ class EditName(EditSecondary):
             6/     local set,     global set --> set (set to global if possible)
         """
         closeit = True
-        surname = self.obj.get_surname()
+        surname = self.obj.get_primary_surname().get_surname()
         group_as= self.obj.get_group_as()
         grouping_active = self.group_over.get_active()
         
@@ -414,4 +446,18 @@ class EditName(EditSecondary):
         if closeit:
             if self.callback:
                 self.callback(self.obj)
+            self.callback = None
             self.close()
+
+    def _cleanup_on_exit(self):
+        """
+        Somehow it was decided that a database value of group="" is represented
+        in the GUI by a widget with a group="surname" which is disabled. So if
+        the group_as widget is disabled then remove the group from the name
+        otherwise gramps thinks the name has changed resulting in asking if
+        data must be saved, and also bug 1892 occurs on reopening of the editor.
+        """
+        # can't use group_over, see Note in gen/lib/name/Name.set_group_as().
+        if not self.group_as.obj.get_editable():
+            self.obj.set_group_as("")
+        EditSecondary._cleanup_on_exit(self)
\ No newline at end of file
