@@ -206,6 +206,16 @@ UIDEFAULT = '''<ui>
 <accelerator action="F9"/>
 <accelerator action="F11"/>
 <accelerator action="F12"/>
+<accelerator action="<CONTROL>1"/>
+<accelerator action="<CONTROL>2"/>
+<accelerator action="<CONTROL>3"/>
+<accelerator action="<CONTROL>4"/>
+<accelerator action="<CONTROL>5"/>
+<accelerator action="<CONTROL>6"/>
+<accelerator action="<CONTROL>7"/>
+<accelerator action="<CONTROL>8"/>
+<accelerator action="<CONTROL>9"/>
+<accelerator action="<CONTROL>0"/>
 <accelerator action="<CONTROL>BackSpace"/>
 <accelerator action="<CONTROL>J"/>
 <accelerator action="<CONTROL>N"/>
@@ -777,6 +787,17 @@ class ViewManager(CLIManager):
             ('F8', None, 'F9', "F8", None, self.__keypress),
             ('F9', None, 'F9', "F9", None, self.__keypress),
             ('F11', None, 'F11', "F11", None, self.__keypress),
+            ('<CONTROL>1', None, '<CONTROL>1', "<CONTROL>1", None, self.__gocat),
+            ('<CONTROL>2', None, '<CONTROL>2', "<CONTROL>2", None, self.__gocat),
+            ('<CONTROL>3', None, '<CONTROL>3', "<CONTROL>3", None, self.__gocat),
+            ('<CONTROL>4', None, '<CONTROL>4', "<CONTROL>4", None, self.__gocat),
+            ('<CONTROL>5', None, '<CONTROL>5', "<CONTROL>5", None, self.__gocat),
+            ('<CONTROL>6', None, '<CONTROL>6', "<CONTROL>6", None, self.__gocat),
+            ('<CONTROL>7', None, '<CONTROL>7', "<CONTROL>7", None, self.__gocat),
+            ('<CONTROL>8', None, '<CONTROL>8', "<CONTROL>8", None, self.__gocat),
+            ('<CONTROL>9', None, '<CONTROL>9', "<CONTROL>9", None, self.__gocat),
+            ('<CONTROL>0', None, '<CONTROL>0', "<CONTROL>0", None, self.__gocat),
+            # NOTE: CTRL+ALT+NUMBER is set in src/plugins/sidebar/cat...py
             ('<CONTROL>BackSpace', None, '<CONTROL>BackSpace',
              "<CONTROL>BackSpace", None, self.__keypress),
             ('<CONTROL>Delete', None, '<CONTROL>Delete',
@@ -807,7 +828,7 @@ class ViewManager(CLIManager):
             ]
 
         self._file_toggle_action_list = [
-            ('Navigator', None, _('_Navigator'), None, None,
+            ('Navigator', None, _('_Navigator'), "<control>m", None,
              self.navigator_toggle, self.show_navigator ),
             ('Toolbar', None, _('_Toolbar'), None, None, self.toolbar_toggle,
              self.show_toolbar ),
@@ -843,31 +864,55 @@ class ViewManager(CLIManager):
             self.uistate.push_message(self.dbstate,
                                       _("Key %s is not bound") % name)
 
+    def __gocat(self, action):
+        """
+        Callback that is called on ctrl+number press. It moves to the 
+        requested category like __next_view/__prev_view. 0 is 10
+        """
+        cat = int(action.get_name()[-1])
+        if cat == 0:
+            cat = 10
+        cat -= 1
+        if cat >= len(self.current_views):
+            #this view is not present
+            return False
+        self.goto_page(cat, None)
+
     def __next_view(self, action):
         """
-        Callback that is called when the next view action is selected.
-        It selects the next view as the active view. If we reach the end of
-        the list of views, we wrap around to the first view.
+        Callback that is called when the next category action is selected.
+        It selects the next category as the active category. If we reach the end, 
+        we wrap around to the first.
         """
-        current_page = self.notebook.get_current_page()
-        if current_page == len(self.pages)-1:
-            new_page = 0
+        curpage = self.notebook.get_current_page()
+        #find cat and view of the current page
+        for key in self.page_lookup:
+            if self.page_lookup[key] == curpage:
+                cat_num, view_num = key
+                break
+        #now go to next category
+        if cat_num >= len(self.current_views)-1:
+            self.goto_page(0, None)
         else:
-            new_page = current_page + 1
-        self.notebook.set_current_page(new_page)
+            self.goto_page(cat_num+1, None)
 
     def __prev_view(self, action):
         """
-        Callback that is called when the previous view action is selected.
-        It selects the previous view as the active view. If we reach the
-        beginning of the list of views, we wrap around to the last view.
+        Callback that is called when the previous category action is selected.
+        It selects the previous category as the active category. If we reach the
+        beginning of the list, we wrap around to the last.
         """
-        current_page = self.notebook.get_current_page()
-        if current_page == 0:
-            new_page = len(self.pages)-1
+        curpage = self.notebook.get_current_page()
+        #find cat and view of the current page
+        for key in self.page_lookup:
+            if self.page_lookup[key] == curpage:
+                cat_num, view_num = key
+                break
+        #now go to next category
+        if cat_num > 0:
+            self.goto_page(cat_num-1, None)
         else:
-            new_page = current_page - 1
-        self.notebook.set_current_page(new_page)
+            self.goto_page(len(self.current_views)-1, None)
 
     def init_interface(self):
         """
