@@ -20,7 +20,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: listview.py 14986 2010-04-01 13:24:48Z dsblank $
+# $Id: listview.py 17084 2011-04-09 14:08:29Z nick-h $
 
 """
 Provide the base classes for GRAMPS' DataView classes
@@ -50,6 +50,7 @@ import pango
 # GRAMPS 
 #
 #----------------------------------------------------------------
+from gui.views.pageview import PageView
 from gui.views.navigationview import NavigationView
 from gui.columnorder import ColumnOrder
 import config
@@ -62,6 +63,7 @@ import Utils
 from QuestionDialog import QuestionDialog, QuestionDialog2
 from gui.filtereditor import FilterEditor
 from gen.ggettext import sgettext as _
+from DdTargets import DdTargets
 
 #----------------------------------------------------------------
 #
@@ -87,13 +89,14 @@ class ListView(NavigationView):
     ADD_MSG = ""
     EDIT_MSG = ""
     DEL_MSG = ""
+    MERGE_MSG = ""
     FILTER_TYPE = None  # Set in inheriting class
     QR_CATEGORY = -1
 
-    def __init__(self, title, dbstate, uistate, columns, handle_col, 
+    def __init__(self, title, pdata, dbstate, uistate, columns, handle_col, 
                  make_model, signal_map, get_bookmarks, bm_type, nav_group,
                  multiple=False, filter_class=None, markup=None):
-        NavigationView.__init__(self, title, dbstate, uistate, 
+        NavigationView.__init__(self, title, pdata, dbstate, uistate, 
                               get_bookmarks, bm_type, nav_group)
         #default is listviews keep themself in sync with database
         self._dirty_on_change_inactive = False
@@ -111,11 +114,9 @@ class ListView(NavigationView):
         self.signal_map = signal_map
         self.multiple_selection = multiple
         self.generic_filter = None
-        if markup:
-            self.markup_columns = markup
-        else:
-            self.markup_columns = []
+        self.markup_columns = markup or []
         dbstate.connect('database-changed', self.change_db)
+        self.connect_signals()
 
     def type_list(self):
         """
@@ -132,7 +133,6 @@ class ListView(NavigationView):
         contains the interface. This containter will be inserted into
         a gtk.Notebook page.
         """
-        self.init_config()
         self.vbox = gtk.VBox()
         self.vbox.set_border_width(4)
         self.vbox.set_spacing(4)
@@ -153,9 +153,17 @@ class ListView(NavigationView):
         else:
             # Tree
             self.list.connect('key-press-event', self._key_press_tree)
+
         if self.drag_info():
             self.list.connect('drag_data_get', self.drag_data_get)
             self.list.connect('drag_begin', self.drag_begin)
+        if self.drag_dest_info():
+            self.list.connect('drag_data_received', self.drag_data_received)
+            self.list.drag_dest_set(gtk.DEST_DEFAULT_MOTION |
+                                    gtk.DEST_DEFAULT_DROP, 
+                                    [self.drag_dest_info().target()], 
+                                    gtk.gdk.ACTION_MOVE |
+                                    gtk.gdk.ACTION_COPY)
 
         scrollwindow = gtk.ScrolledWindow()
         scrollwindow.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
@@ -176,11 +184,7 @@ class ListView(NavigationView):
         self.selection.connect('changed', self.row_changed)
 
         self.setup_filter()
-
-        if self.filter_class:
-            return self.build_filter_container(self.vbox, self.filter_class)
-        else:
-            return self.vbox
+        return self.vbox
 
     def define_actions(self):
         """
@@ -198,6 +202,8 @@ class ListView(NavigationView):
                     self.ADD_MSG, self.add), 
                 ('Remove', gtk.STOCK_REMOVE, _("_Remove"), "<control>Delete", 
                     self.DEL_MSG, self.remove), 
+                ('Merge', 'gramps-merge', _('_Merge...'), None,
+                    self.MERGE_MSG, self.merge),
                 ('ExportTab', None, _('Export View...'), None, None,
                     self.export), 
                 ])
@@ -221,9 +227,8 @@ class ListView(NavigationView):
 
             column = gtk.TreeViewColumn(name, self.renderer)
             
-            if self.model and self.model.marker_column() is not None:
-                mcol = self.model.marker_column()
-                column.add_attribute(self.renderer, 'foreground', mcol)
+            if self.model and self.model.color_column() is not None:
+                column.set_cell_data_func(self.renderer, self.foreground_color)
 
             if pair[1] in self.markup_columns:
                 column.add_attribute(self.renderer, 'markup', pair[1])
@@ -239,6 +244,14 @@ class ListView(NavigationView):
             self.list.append_column(column)
             index += 1
 
+    def foreground_color(self, column, renderer, model, iter_):
+        '''
+        Set the foreground color of the cell renderer.  We use a cell data
+        function because we don't want to set the color of untagged rows.
+        '''
+        fg_color = model.get_value(iter_, model.color_column())
+        renderer.set_property('foreground', fg_color)
+
     def set_active(self):
         NavigationView.set_active(self)
         self.uistate.show_filter_results(self.dbstate, 
@@ -248,16 +261,19 @@ class ListView(NavigationView):
     def __build_tree(self):
         Utils.profile(self._build_tree)
 
-    def build_tree(self):
+    def build_tree(self, force_sidebar=False):
         if self.active:
             cput0 = time.clock()
-            if config.get('interface.filter'):
+            if not self.search_bar.is_visible():
                 filter_info = (True, self.generic_filter, False)
             else:
                 value = self.search_bar.get_value()
                 filter_info = (False, value, value[0] in self.exact_search())
 
             if self.dirty or not self.model:
+                if self.model:
+                    self.list.set_model(None)
+                    self.model.destroy()
                 self.model = self.make_model(self.dbstate.db, self.sort_col, 
                                              search=filter_info,
                                              sort_map=self.column_order())
@@ -276,9 +292,9 @@ class ListView(NavigationView):
             self.__display_column_sort()
             self.goto_active(None)
 
-            if const.USE_TIPS and self.model.tooltip_column is not None:
+            if const.USE_TIPS and self.model.tooltip_column() is not None:
                 self.tooltips = TreeTips.TreeTips(
-                    self.list, self.model.tooltip_column, True)
+                    self.list, self.model.tooltip_column(), True)
             self.dirty = False
             cput4 = time.clock()
             self.uistate.show_filter_results(self.dbstate, 
@@ -304,50 +320,11 @@ class ListView(NavigationView):
         """
         return ()
 
-    ####################################################################
-    # Filter
-    ####################################################################
-    def build_filter_container(self, box, filter_class):
-        self.filter_sidebar = filter_class(self.dbstate, self.uistate, 
-                                           self.filter_clicked)
-        self.filter_pane = self.filter_sidebar.get_widget()
-
-        hpaned = gtk.HBox()
-        hpaned.pack_start(self.vbox, True, True)
-        hpaned.pack_end(self.filter_pane, False, False)
-        self.filter_toggle(None, None, None, None)
-        return hpaned
-
-    def filter_toggle(self, client, cnxn_id, entry, data):
-        """
-        Callback on change interface.filter, inheriting methods connect to 
-        change in ini file
-        """
-        if config.get('interface.filter'):
-            self.search_bar.hide()
-            self.filter_pane.show()
-        else:
-            self.search_bar.show()
-            self.filter_pane.hide()
-
-    def post(self):
-        if self.filter_class:
-            if config.get('interface.filter'):
-                self.search_bar.hide()
-                self.filter_pane.show()
-            else:
-                self.search_bar.show()
-                self.filter_pane.hide()
-
     def get_viewtype_stock(self):
         """Type of view in category, default listview is a flat list
         """
         return 'gramps-tree-list'
 
-    def filter_clicked(self):
-        self.generic_filter = self.filter_sidebar.get_filter()
-        self.build_tree()
-
     def filter_editor(self, obj):
         try:
             FilterEditor(self.FILTER_TYPE , const.CUSTOM_FILTERS, 
@@ -361,6 +338,15 @@ class ListView(NavigationView):
             [(self.colinfo[pair[1]], pair[1], pair[1] in self.exact_search())
                 for pair in self.column_order() if pair[0]])
 
+    def sidebar_toggled(self, active):
+        """
+        Called when the sidebar is toggled.
+        """
+        if active:
+            self.search_bar.hide()
+        else:
+            self.search_bar.show()
+
     ####################################################################
     # Navigation
     ####################################################################
@@ -431,9 +417,21 @@ class ListView(NavigationView):
     ####################################################################
 
     def drag_info(self):
+        """
+        Specify the drag type for a single selected row
+        """
         return None
 
     def drag_list_info(self):
+        """
+        Specify the drag type for a multiple selected rows
+        """
+        return DdTargets.LINK_LIST
+
+    def drag_dest_info(self):
+        """
+        Specify the drag type for objects dropped on the view
+        """
         return None
 
     def drag_begin(self, widget, context):
@@ -443,9 +441,15 @@ class ListView(NavigationView):
     def drag_data_get(self, widget, context, sel_data, info, time):
         selected_ids = self.selected_handles()
 
-        if selected_ids:
+        if len(selected_ids) == 1:
             data = (self.drag_info().drag_type, id(self), selected_ids[0], 0)
-            sel_data.set(sel_data.target, 8 , pickle.dumps(data))
+            sel_data.set(sel_data.target, 8, pickle.dumps(data))
+        elif len(selected_ids) > 1:
+            data = (self.drag_list_info().drag_type, id(self), 
+                    [(self.drag_info().drag_type, handle)
+                        for handle in selected_ids], 
+                    0)
+            sel_data.set(sel_data.target, 8, pickle.dumps(data))
         return True
 
     def set_column_order(self):
@@ -475,12 +479,9 @@ class ListView(NavigationView):
         order = self._config.get('columns.rank')
         size = self._config.get('columns.size')
         vis =  self._config.get('columns.visible')
-        colord = []
-        for val, size in zip(order, size):
-            if val in vis:
-                colord.append((1, val, size))
-            else:
-                colord.append((0, val, size))
+
+        colord = [(1 if val in vis else 0, val, size)
+            for val, size in zip(order, size)]
         return colord
 
     def get_column_widths(self):
@@ -578,14 +579,11 @@ class ListView(NavigationView):
         self.sort_order = order
         handle = self.first_selected()
 
-        if config.get('interface.filter'):
+        if not self.search_bar.is_visible():
             filter_info = (True, self.generic_filter, False)
         else:
             value = self.search_bar.get_value()
-            if value[0] in self.exact_search():
-                filter_info = (False, value, True)
-            else:
-                filter_info = (False, value, False)
+            filter_info = (False, value, value[0] in self.exact_search())
 
         if same_col:
             self.model.reverse_order()
@@ -612,18 +610,23 @@ class ListView(NavigationView):
                     str(time.clock() - cput) + ' sec')
 
     def __display_column_sort(self):
-        for i in xrange(len(self.columns)):
-            enable_sort_flag = (i==self.sort_col)
-            self.columns[i].set_sort_indicator(enable_sort_flag)
+        for i, c in enumerate(self.columns):
+            c.set_sort_indicator(i == self.sort_col)
         self.columns[self.sort_col].set_sort_order(self.sort_order)
 
+    def connect_signals(self):
+        """
+        Connect database signals defined in the signal map.
+        """
+        for sig in self.signal_map:
+            self.callman.add_db_signal(sig, self.signal_map[sig])
+        
     def change_db(self, db):
         """
         Called when the database is changed.
         """
         self._change_db(db)
-        for sig in self.signal_map:
-            self.callman.add_db_signal(sig, self.signal_map[sig])
+        self.connect_signals()
 
         self.bookmarks.update_bookmarks(self.get_bookmarks())
         if self.active:
@@ -717,8 +720,19 @@ class ListView(NavigationView):
         """
         self.dirty = True
         if self.active:
+            # Save the currently selected handles, if any:
+            selected_ids = self.selected_handles()
             self.bookmarks.redraw()
             self.build_tree()
+            # Reselect one, if it still exists after rebuild:
+            nav_type = self.navigation_type()
+            lookup_handle = self.dbstate.db.get_table_metadata(nav_type)['handle_func']
+            for handle in selected_ids:
+                # Still exist?
+                if lookup_handle(handle):
+                    # Select it, and stop selecting:
+                    self.change_active(handle)
+                    break
 
     def _button_press(self, obj, event):
         """
@@ -726,7 +740,8 @@ class ListView(NavigationView):
         """
         if not self.dbstate.open:
             return False
-        from QuickReports import create_quickreport_menu
+        from QuickReports import (create_quickreport_menu, 
+                                  create_web_connect_menu)
         if event.type == gtk.gdk._2BUTTON_PRESS and event.button == 1:
             if self.type_list() == LISTFLAT:
                 self.edit(obj)
@@ -764,6 +779,23 @@ class ListView(NavigationView):
                         add_menuitem(qr_menu, action[2], None, action[5])
                     self.uistate.uimanager.get_widget('/Popup/QuickReport').\
                             set_submenu(qr_menu)
+            if menu and self.get_active():
+                popup = self.uistate.uimanager.get_widget('/Popup/WebConnect')
+                if popup:
+                    qr_menu = popup.get_submenu()
+                    webconnects = []
+                    if qr_menu:
+                        popup.remove_submenu()
+                        webconnects = create_web_connect_menu(
+                            self.dbstate, 
+                            self.uistate,
+                            self.navigation_type(),
+                            self.first_selected())
+                    if len(webconnects) > 1 :
+                        qr_menu = gtk.Menu()
+                        for action in webconnects[1:] :
+                            add_menuitem(qr_menu, action[2], None, action[5])
+                        popup.set_submenu(qr_menu)
             if menu:
                 menu.popup(None, None, None, event.button, event.time)
                 return True
@@ -797,7 +829,7 @@ class ListView(NavigationView):
                 if paths:
                     firstsel = paths[0]
                     firstnode = self.model.on_get_iter(firstsel)
-                    if len(paths)==1 and firstnode.handle is None:
+                    if len(paths) == 1 and firstnode.handle is None:
                         return self.expand_collapse_tree()
                     else:
                         self.edit(obj)
@@ -808,7 +840,7 @@ class ListView(NavigationView):
                 if paths:
                     firstsel = paths[0]
                     firstnode = self.model.on_get_iter(firstsel)
-                    if len(paths)==1 and firstnode.handle is None:
+                    if len(paths) == 1 and firstnode.handle is None:
                         return self.expand_collapse_tree_branch()
             
         return False
@@ -880,7 +912,7 @@ class ListView(NavigationView):
                 index += 1
             newsize.append(size)
         self._config.set('columns.size', newsize)
-        self._config.save()
+        PageView.on_delete(self)
 
     ####################################################################
     # Export data
@@ -909,6 +941,7 @@ class ListView(NavigationView):
         while True:
             value = chooser.run()
             fn = chooser.get_filename()
+            fn = Utils.get_unicode_path_from_file_chooser(fn)
             fl = combobox.get_active()
             if value == gtk.RESPONSE_OK:
                 if fn:
@@ -1010,6 +1043,12 @@ class ListView(NavigationView):
         """
         raise NotImplementedError
 
+    def merge(self, obj):
+        """
+        Template function to allow the merger of two objects.
+        """
+        raise NotImplementedError
+
     def remove_object_from_handle(self, handle):
         """
         Template function to allow the removal of an object by its handle
