@@ -19,7 +19,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: navigationview.py 17618 2011-05-29 16:52:25Z romjerome $
+# $Id: navigationview.py 17783 2011-06-14 21:11:39Z bmcage $
 
 """
 Provide the base classes for GRAMPS' DataView classes
@@ -272,7 +272,7 @@ class NavigationView(PageView):
              '<control>d', None, self.add_bookmark), 
             ('EditBook', 'gramps-bookmark-edit', 
              _("%(title)s...") % {'title': _("Organize Bookmarks")}, 
-             '<shift><control>b', None, 
+             '<shift><control>D', None, 
              self.edit_bookmarks), 
             ])
 
