@@ -18,7 +18,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: pageview.py 17550 2011-05-22 13:23:34Z bmcage $
+# $Id: pageview.py 18066 2011-08-25 20:48:53Z nick-h $
 
 """
 Provide the base class for GRAMPS' DataView classes
@@ -111,10 +111,6 @@ class PageView(DbGUIElement):
               </placeholder>
             </menu>
           </menubar>
-          <popup name="GrampsBarPopup">
-            <menuitem action="AddGramplet"/>
-            <menuitem action="RemoveGramplet"/>
-          </popup>
         </ui>'''
         self.dirty = True
         self.active = False
@@ -408,14 +404,11 @@ class PageView(DbGUIElement):
         self.action_toggle_list in this function. 
         """
         self._add_toggle_action('Sidebar', None, _('_Sidebar'), 
-             None, None, self.__sidebar_toggled,
+             "<shift><control>R", None, self.__sidebar_toggled,
              self.sidebar.get_property('visible'))
         self._add_toggle_action('Bottombar', None, _('_Bottombar'), 
-             None, None, self.__bottombar_toggled,
+             "<shift><control>B", None, self.__bottombar_toggled,
              self.bottombar.get_property('visible'))
-        self._add_action("AddGramplet", gtk.STOCK_ADD, _("Add a gramplet"))
-        self._add_action("RemoveGramplet", gtk.STOCK_REMOVE,
-                         _("Remove a gramplet"))
 
     def __build_action_group(self):
         """
