@@ -17,14 +17,13 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
-# $Id: eventmodel.py 16092 2010-10-31 08:38:05Z bmcage $
+# $Id: eventmodel.py 16089 2010-10-31 07:33:17Z bmcage $
 
 #-------------------------------------------------------------------------
 #
 # python modules
 #
 #-------------------------------------------------------------------------
-import time
 import cgi
 import logging
 log = logging.getLogger(".")
@@ -42,7 +41,6 @@ import gtk
 #
 #-------------------------------------------------------------------------
 import ToolTips
-import GrampsLocale
 import DateHandler
 import gen.lib
 import Utils
@@ -98,7 +96,7 @@ class EventModel(FlatBaseModel):
             self.column_handle,
             self.column_tooltip,
             ]
-        FlatBaseModel.__init__(self, db, scol, order, tooltip_column=9,
+        FlatBaseModel.__init__(self, db, scol, order, tooltip_column=8,
                            search=search, skip=skip, sort_map=sort_map)
 
     def destroy(self):
@@ -165,9 +163,7 @@ class EventModel(FlatBaseModel):
         return "%012x" % data[COLUMN_CHANGE]
 
     def column_change(self,data):
-        return unicode(time.strftime('%x %X',
-                                     time.localtime(data[COLUMN_CHANGE])),
-                       GrampsLocale.codeset)
+        return Utils.format_time(data[COLUMN_CHANGE])
 
     def column_tooltip(self,data):
         try:
