@@ -2,6 +2,7 @@
 # Gramps - a GTK+/GNOME based genealogy program
 #
 # Copyright (C) 2000-2007  Donald N. Allingham
+# Copyright (C) 2010       Nick Hall
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,16 +18,16 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
-# $Id: familymodel.py 16092 2010-10-31 08:38:05Z bmcage $
+# $Id: familymodel.py 16834 2011-03-16 18:33:09Z nick-h $
 
 #-------------------------------------------------------------------------
 #
 # python modules
 #
 #-------------------------------------------------------------------------
-import time
 import logging
 log = logging.getLogger(".")
+import locale
 
 #-------------------------------------------------------------------------
 #
@@ -42,13 +43,15 @@ import gtk
 #-------------------------------------------------------------------------
 import const
 import ToolTips
-import GrampsLocale
+import Utils
 import DateHandler
 from gen.display.name import displayer as name_displayer
 import gen.lib
 from gen.lib import EventRoleType
-
 from gui.views.treemodels.flatbasemodel import FlatBaseModel
+import config
+
+invalid_date_format = config.get('preferences.invalid-date-format')
 
 #-------------------------------------------------------------------------
 #
@@ -57,8 +60,6 @@ from gui.views.treemodels.flatbasemodel import FlatBaseModel
 #-------------------------------------------------------------------------
 class FamilyModel(FlatBaseModel):
 
-    _MARKER_COL = 13
-
     def __init__(self, db, scol=0, order=gtk.SORT_ASCENDING, search=None, 
                  skip=set(), sort_map=None):
         self.gen_cursor = db.get_family_cursor
@@ -69,11 +70,11 @@ class FamilyModel(FlatBaseModel):
             self.column_mother, 
             self.column_type, 
             self.column_marriage, 
+            self.column_tags,
             self.column_change, 
             self.column_handle, 
-            self.column_tooltip, 
-            self.column_marker_text, 
-            self.column_marker_color, 
+            self.column_tag_color,
+            self.column_tooltip,
             ]
         self.smap = [
             self.column_id, 
@@ -81,13 +82,13 @@ class FamilyModel(FlatBaseModel):
             self.sort_mother, 
             self.column_type, 
             self.sort_marriage, 
+            self.column_tags,
             self.sort_change, 
             self.column_handle, 
-            self.column_tooltip, 
-            self.column_marker_text, 
-            self.column_marker_color, 
+            self.column_tag_color,
+            self.column_tooltip,
             ]
-        FlatBaseModel.__init__(self, db, scol, order, tooltip_column=6, 
+        FlatBaseModel.__init__(self, db, scol, order, tooltip_column=9, 
                            search=search, skip=skip, sort_map=sort_map)
 
     def destroy(self):
@@ -101,11 +102,11 @@ class FamilyModel(FlatBaseModel):
         self.smap = None
         FlatBaseModel.destroy(self)
 
-    def marker_column(self):
+    def color_column(self):
         """
-        Return the column for marker colour.
+        Return the color column.
         """
-        return 9
+        return 8
 
     def on_get_n_columns(self):
         return len(self.fmap)+1
@@ -145,18 +146,23 @@ class FamilyModel(FlatBaseModel):
         return unicode(gen.lib.FamilyRelType(data[5]))
 
     def column_marriage(self, data):
-        erlist = [gen.lib.EventRef().unserialize(item) for item in data[6]]
-        erlist = [x for x in erlist if x.get_role()==EventRoleType.FAMILY or 
-                                       x.get_role()==EventRoleType.PRIMARY]
-        event = self.db.marriage_from_eventref_list(erlist)
+        from gen.utils import get_marriage_or_fallback
+        family = self.db.get_family_from_handle(data[0])
+        event = get_marriage_or_fallback(self.db, family, "<i>%s</i>")
         if event:
-            return DateHandler.displayer.display(event.date)
+            if event.date.format:
+                return event.date.format % DateHandler.displayer.display(event.date)
+            elif not DateHandler.get_date_valid(event):
+                return invalid_date_format % DateHandler.displayer.display(event.date)
+            else:
+                return "%s" % DateHandler.displayer.display(event.date)
         else:
             return u''
 
     def sort_marriage(self, data):
-        erlist = [gen.lib.EventRef().unserialize(item) for item in data[6]]
-        event = self.db.marriage_from_eventref_list(erlist)
+        from gen.utils import get_marriage_or_fallback
+        family = self.db.get_family_from_handle(data[0])
+        event = get_marriage_or_fallback(self.db, family)
         if event:
             return "%09d" % event.date.get_sort_value()
         else:
@@ -169,29 +175,7 @@ class FamilyModel(FlatBaseModel):
         return "%012x" % data[12]
     
     def column_change(self, data):
-        return unicode(time.strftime('%x %X', time.localtime(data[12])), 
-                       GrampsLocale.codeset)
-
-    def column_marker_text(self, data):
-        try:
-            if data[FamilyModel._MARKER_COL]:
-                return str(data[FamilyModel._MARKER_COL])
-        except IndexError:
-            return ""
-        return ""
-
-    def column_marker_color(self, data):
-        try:
-            col = data[FamilyModel._MARKER_COL][0]
-            if col == gen.lib.MarkerType.COMPLETE:
-                return self.complete_color
-            elif col == gen.lib.MarkerType.TODO_TYPE:
-                return self.todo_color
-            elif col == gen.lib.MarkerType.CUSTOM:
-                return self.custom_color
-        except IndexError:
-            pass
-        return None
+        return Utils.format_time(data[12])
 
     def column_tooltip(self, data):
         if const.USE_TIPS:
@@ -204,3 +188,30 @@ class FamilyModel(FlatBaseModel):
             return t
         else:
             return u''
+
+    def get_tag_name(self, tag_handle):
+        """
+        Return the tag name from the given tag handle.
+        """
+        return self.db.get_tag_from_handle(tag_handle).get_name()
+        
+    def column_tag_color(self, data):
+        """
+        Return the tag color.
+        """
+        tag_color = None
+        tag_priority = None
+        for handle in data[13]:
+            tag = self.db.get_tag_from_handle(handle)
+            this_priority = tag.get_priority()
+            if tag_priority is None or this_priority < tag_priority:
+                tag_color = tag.get_color()
+                tag_priority = this_priority
+        return tag_color
+
+    def column_tags(self, data):
+        """
+        Return the sorted list of tags.
+        """
+        tag_list = map(self.get_tag_name, data[13])
+        return ', '.join(sorted(tag_list, key=locale.strxfrm))
