@@ -2,7 +2,8 @@
 # Gramps - a GTK+/GNOME based genealogy program
 #
 # Copyright (C) 2000-2006  Donald N. Allingham
-# Copyright (C) 2009  Benny Malengier
+# Copyright (C) 2009       Benny Malengier
+# Copyright (C) 2010       Nick Hall
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,7 +19,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
-# $Id: flatbasemodel.py 14678 2010-03-07 19:48:30Z nick-h $
+# $Id: flatbasemodel.py 17423 2011-05-05 20:03:54Z m_d_n $
 
 """
 This module provides the flat treemodel that is used for all flat treeviews.
@@ -73,7 +74,6 @@ import gtk
 #-------------------------------------------------------------------------
 from Filters import SearchFilter
 from Filters import ExactSearchFilter
-import config
 from Utils import conv_unicode_tosrtkey_ongtk
 
 #-------------------------------------------------------------------------
@@ -122,6 +122,14 @@ class FlatNodeMap(object):
         self._reverse = False
         self.__corr = (0, 1)
 
+    def destroy(self):
+        """
+        Unset all elements that can prevent garbage collection
+        """
+        self._index2hndl = None
+        self._fullhndl = None
+        self._hndl2index = None
+
     def set_path_map(self, index2hndllist, fullhndllist, identical=True,
                      reverse=False):
         """
@@ -303,6 +311,10 @@ class FlatNodeMap(object):
         :Returns: path of the row inserted in the treeview
         :Returns type: integer or None
         """
+        if srtkey_hndl[1] in self._hndl2index:
+            print ('WARNING: Attempt to add row twice to the model (%s)' %
+                    srtkey_hndl[1])
+            return
         if not self._identical:
             bisect.insort_left(self._fullhndl, srtkey_hndl)
             if allkeyonly:
@@ -400,20 +412,22 @@ class FlatBaseModel(gtk.GenericTreeModel):
         self._reverse = (order == gtk.SORT_DESCENDING)
         self._tooltip_column = tooltip_column
 
-        config.connect("preferences.todo-color",
-                          self.__update_todo)
-        config.connect("preferences.custom-marker-color",
-                          self.__update_custom)
-        config.connect("preferences.complete-color",
-                          self.__update_complete)
-
-        self.complete_color = config.get('preferences.complete-color')
-        self.todo_color = config.get('preferences.todo-color')
-        self.custom_color = config.get('preferences.custom-marker-color')
         self.rebuild_data()
         _LOG.debug(self.__class__.__name__ + ' __init__ ' +
                     str(time.clock() - cput) + ' sec')
 
+    def destroy(self):
+        """
+        Unset all elements that prevent garbage collection
+        """
+        self.db = None
+        self.sort_func = None
+        if self.node_map:
+            self.node_map.destroy()
+        self.node_map = None
+        self.rebuild_data = None
+        self.search = None
+
     def set_search(self, search):
         """
         Change the search function that filters the data in the model. 
@@ -446,24 +460,6 @@ class FlatBaseModel(gtk.GenericTreeModel):
             self.search = None
             self.rebuild_data = self._rebuild_search
 
-    def __update_todo(self, client, cnxn_id, entry, data):
-        """
-        Callback if preferences todo color changes
-        """
-        self.todo_color = config.get('preferences.todo-color')
-        
-    def __update_custom(self, client, cnxn_id, entry, data):
-        """
-        Callback if preferences todo color changes
-        """
-        self.custom_color = config.get('preferences.custom-marker-color')
-
-    def __update_complete(self, client, cnxn_id, entry, data):
-        """
-        Callback if preferences todo color changes
-        """
-        self.complete_color = config.get('preferences.complete-color')
-
     def total(self):
         """
         Total number of items that maximally can be shown
@@ -489,9 +485,9 @@ class FlatBaseModel(gtk.GenericTreeModel):
         """
         return self._tooltip_column
         
-    def marker_column(self):
+    def color_column(self):
         """
-        Return the column for marker colour.
+        Return the color column.
         """
         return None
 
@@ -583,6 +579,8 @@ class FlatBaseModel(gtk.GenericTreeModel):
         Add a row. This is called after object with handle is created.
         Row is only added if search/filter data is such that it must be shown
         """
+        if self.node_map.get_path(handle) is not None:
+            return # row is already displayed
         data = self.map(handle)
         insert_val = (map(conv_unicode_tosrtkey_ongtk, self.sort_func(data)),
                             handle)
@@ -601,10 +599,10 @@ class FlatBaseModel(gtk.GenericTreeModel):
         """
         Delete a row, called after the object with handle is deleted
         """
+        if self.node_map.get_path(handle) is None:
+            return # row is not currently displayed
         self.clear_cache(handle)
-        data = self.map(handle)
-        delete_val = (map(conv_unicode_tosrtkey_ongtk, self.sort_func(data)),
-                            handle)
+        delete_val = (self.node_map.get_sortkey(handle), handle)
         delete_path = self.node_map.delete(delete_val)
         #delete_path is an integer from 0 to n-1
         if delete_path is not None: 
@@ -614,6 +612,8 @@ class FlatBaseModel(gtk.GenericTreeModel):
         """
         Update a row, called after the object with handle is changed
         """
+        if self.node_map.get_path(handle) is None:
+            return # row is not currently displayed
         self.clear_cache(handle)
         oldsortkey = self.node_map.get_sortkey(handle)
         newsortkey = map(conv_unicode_tosrtkey_ongtk, self.sort_func(self.map(
@@ -686,7 +686,7 @@ class FlatBaseModel(gtk.GenericTreeModel):
                 self.prev_handle = handle
             return self.fmap[col](self.prev_data)
         except:
-            return u''
+            return None
 
     def on_iter_next(self, handle):
         """
