@@ -3,7 +3,7 @@
 #
 # Copyright (C) 2000-2007  Donald N. Allingham
 # Copyright (C) 2009       Gary Burton
-# Copyright (C) 2009       Nick Hall
+# Copyright (C) 2009-2010  Nick Hall
 # Copyright (C) 2009       Benny Malengier
 #
 # This program is free software; you can redistribute it and/or modify
@@ -21,7 +21,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: peoplemodel.py 16092 2010-10-31 08:38:05Z bmcage $
+# $Id: peoplemodel.py 16849 2011-03-18 15:45:41Z bmcage $
 
 """
 TreeModel for the GRAMPS Person tree.
@@ -33,8 +33,8 @@ TreeModel for the GRAMPS Person tree.
 #
 #-------------------------------------------------------------------------
 from gen.ggettext import gettext as _
-import time
 import cgi
+import locale
 
 #-------------------------------------------------------------------------
 #
@@ -57,11 +57,11 @@ _LOG = logging.getLogger(".")
 #
 #-------------------------------------------------------------------------
 import const
-from gen.lib import Name, EventRef, EventType, EventRoleType, MarkerType
+from gen.lib import Name, EventRef, EventType, EventRoleType
 from gen.display.name import displayer as name_displayer
 import DateHandler
 import ToolTips
-import GrampsLocale
+import Utils
 from Lru import LRU
 from gui.views.treemodels.flatbasemodel import FlatBaseModel
 from gui.views.treemodels.treebasemodel import TreeBaseModel
@@ -80,7 +80,7 @@ COLUMN_BIRTH  = 6
 COLUMN_EVENT  = 7
 COLUMN_FAMILY = 8
 COLUMN_CHANGE = 17
-COLUMN_MARKER = 18
+COLUMN_TAGS   = 18
 
 invalid_date_format = config.get('preferences.invalid-date-format')
 
@@ -95,8 +95,8 @@ class PeopleBaseModel(object):
     """
     _GENDER = [ _(u'female'), _(u'male'), _(u'unknown') ]
 
-    # The following is accessed from PersonView - CHECK
-    COLUMN_INT_ID = 9  # dynamic calculation of column indices
+    # The following is accessed from the Person Selector
+    COLUMN_INT_ID = 10  # dynamic calculation of column indices
     # LRU cache size
     _CACHE_SIZE = 250
 
@@ -104,6 +104,7 @@ class PeopleBaseModel(object):
         """
         Initialize the model building the initial data
         """
+        self.db = db
         self.gen_cursor = db.get_person_cursor
         self.map = db.get_raw_person_data
 
@@ -116,10 +117,10 @@ class PeopleBaseModel(object):
             self.column_death_day,
             self.column_death_place,
             self.column_spouse,
+            self.column_tags,
             self.column_change,
             self.column_int_id,
-            self.column_marker_text,
-            self.column_marker_color,
+            self.column_tag_color,
             self.column_tooltip,
             ]
         self.smap = [
@@ -131,10 +132,10 @@ class PeopleBaseModel(object):
             self.sort_death_day,
             self.column_death_place,
             self.column_spouse,
+            self.column_tags,
             self.sort_change,
             self.column_int_id,
-            self.column_marker_text,
-            self.column_marker_color,
+            self.column_tag_color,
             self.column_tooltip,
             ]
 
@@ -157,9 +158,9 @@ class PeopleBaseModel(object):
         self.smap = None
         self.clear_local_cache()
 
-    def marker_column(self):
+    def color_column(self):
         """
-        Return the column for marker colour.
+        Return the color column.
         """
         return 11
 
@@ -193,16 +194,14 @@ class PeopleBaseModel(object):
         return len(self.fmap)+1
 
     def sort_name(self, data):
-        n = Name()
-        n.unserialize(data[COLUMN_NAME])
-        return (n.get_surname(), n.get_first_name())
+        return name_displayer.raw_sorted_name(data[COLUMN_NAME])
 
     def column_name(self, data):
         handle = data[0]
         if handle in self.lru_name:
             name = self.lru_name[handle]
         else:
-            name = name_displayer.raw_sorted_name(data[COLUMN_NAME])
+            name = name_displayer.raw_display_name(data[COLUMN_NAME])
             if not self._in_build:
                 self.lru_name[handle] = name
         return name
@@ -240,10 +239,7 @@ class PeopleBaseModel(object):
         return "%012x" % data[COLUMN_CHANGE]
 
     def column_change(self, data):
-        return unicode(
-            time.strftime('%x %X',
-                          time.localtime(data[COLUMN_CHANGE])),
-            GrampsLocale.codeset)
+        return Utils.format_time(data[COLUMN_CHANGE])
 
     def column_gender(self, data):
         return PeopleBaseModel._GENDER[data[COLUMN_GENDER]]
@@ -429,24 +425,6 @@ class PeopleBaseModel(object):
                         return "<i>" + cgi.escape(place_title) + "</i>"
         return u""
 
-    def column_marker_text(self, data):
-        if COLUMN_MARKER < len(data):
-            return str(data[COLUMN_MARKER])
-        return ""
-
-    def column_marker_color(self, data):
-        try:
-            if data[COLUMN_MARKER]:
-                if data[COLUMN_MARKER][0] == MarkerType.COMPLETE:
-                    return self.complete_color
-                if data[COLUMN_MARKER][0] == MarkerType.TODO_TYPE:
-                    return self.todo_color
-                if data[COLUMN_MARKER][0] == MarkerType.CUSTOM:
-                    return self.custom_color
-        except IndexError:
-            pass
-        return None
-
     def column_tooltip(self, data):
         if const.USE_TIPS:
             return ToolTips.TipFromFunction(
@@ -459,6 +437,33 @@ class PeopleBaseModel(object):
     def column_int_id(self, data):
         return data[0]
 
+    def get_tag_name(self, tag_handle):
+        """
+        Return the tag name from the given tag handle.
+        """
+        return self.db.get_tag_from_handle(tag_handle).get_name()
+        
+    def column_tag_color(self, data):
+        """
+        Return the tag color.
+        """
+        tag_color = None
+        tag_priority = None
+        for handle in data[COLUMN_TAGS]:
+            tag = self.db.get_tag_from_handle(handle)
+            this_priority = tag.get_priority()
+            if tag_priority is None or this_priority < tag_priority:
+                tag_color = tag.get_color()
+                tag_priority = this_priority
+        return tag_color
+
+    def column_tags(self, data):
+        """
+        Return the sorted list of tags.
+        """
+        tag_list = map(self.get_tag_name, data[COLUMN_TAGS])
+        return ', '.join(sorted(tag_list, key=locale.strxfrm))
+
 class PersonListModel(PeopleBaseModel, FlatBaseModel):
     """
     Listed people model.
@@ -467,7 +472,7 @@ class PersonListModel(PeopleBaseModel, FlatBaseModel):
                  skip=set(), sort_map=None):
         PeopleBaseModel.__init__(self, db)
         FlatBaseModel.__init__(self, db, search=search, skip=skip,
-                                tooltip_column=12,
+                                tooltip_column=13,
                                 scol=scol, order=order, sort_map=sort_map)
 
     def clear_cache(self, handle=None):
@@ -489,7 +494,7 @@ class PersonTreeModel(PeopleBaseModel, TreeBaseModel):
                  skip=set(), sort_map=None):
 
         PeopleBaseModel.__init__(self, db)
-        TreeBaseModel.__init__(self, db, 12, search=search, skip=skip,
+        TreeBaseModel.__init__(self, db, 13, search=search, skip=skip,
                                 scol=scol, order=order, sort_map=sort_map)
 
     def destroy(self):
@@ -531,7 +536,6 @@ class PersonTreeModel(PeopleBaseModel, TreeBaseModel):
         data        The object data.
         """
         ngn = name_displayer.name_grouping_data
-        nsn = name_displayer.raw_sorted_name
         
         name_data = data[COLUMN_NAME]
         group_name = ngn(self.db, name_data)
