@@ -30,7 +30,6 @@ Place Model.
 # python modules
 #
 #-------------------------------------------------------------------------
-import time
 import cgi
 import logging
 _LOG = logging.getLogger(".gui.views.treemodels.placemodel")
@@ -49,7 +48,8 @@ import gtk
 #-------------------------------------------------------------------------
 import const
 import ToolTips
-import GrampsLocale
+import Utils
+from PlaceUtils import conv_lat_lon
 from gui.views.treemodels.flatbasemodel import FlatBaseModel
 from gui.views.treemodels.treebasemodel import TreeBaseModel
 
@@ -77,7 +77,7 @@ COUNTRYLEVELS = {
 #-------------------------------------------------------------------------
 class PlaceBaseModel(object):
 
-    HANDLE_COL = 13
+    HANDLE_COL = 14
 
     def __init__(self, db):
         self.gen_cursor = db.get_place_cursor
@@ -85,16 +85,17 @@ class PlaceBaseModel(object):
         self.fmap = [
             self.column_name,
             self.column_id,
-            self.column_parish,
-            self.column_postal_code,
+            self.column_street,
+            self.column_locality,
             self.column_city,
             self.column_county,
             self.column_state,
             self.column_country,
+            self.column_postal_code,
+            self.column_parish,
             self.column_latitude,
             self.column_longitude,
             self.column_change,
-            self.column_street,
             self.column_place_name,
             self.column_handle,
             self.column_tooltip
@@ -102,20 +103,31 @@ class PlaceBaseModel(object):
         self.smap = [
             self.column_name,
             self.column_id,
-            self.column_parish,
-            self.column_postal_code,
+            self.column_street,
+            self.column_locality,
             self.column_city,
             self.column_county,
             self.column_state,
             self.column_country,
+            self.column_postal_code,
+            self.column_parish,
             self.sort_latitude,
             self.sort_longitude,
             self.sort_change,
-            self.column_street,
             self.column_place_name,
             self.column_handle,
             ]
 
+    def destroy(self):
+        """
+        Unset all elements that can prevent garbage collection
+        """
+        self.db = None
+        self.gen_cursor = None
+        self.map = None
+        self.fmap = None
+        self.smap = None
+
     def on_get_n_columns(self):
         return len(self.fmap)+1
 
@@ -125,43 +137,17 @@ class PlaceBaseModel(object):
     def column_place_name(self, data):
         return unicode(data[2])
 
-    def __format_degrees(self, angle, sign_str):
-        """
-        Format a decimal as degrees, minutes and seconds.
-        If the value is not a decimal leave it unformatted.
-        """
-        try:
-            angle = float(angle)
-        except ValueError:
-            return angle
-
-        if angle >= 0:
-            sign = sign_str[0]
-        else:
-            sign = sign_str[1]
-        seconds = abs(int(angle * 60 * 60))
-        minutes = seconds / 60
-        seconds %= 60
-        degrees = minutes / 60
-        minutes %= 60
-
-        string = unicode(degrees) + u'\u00b0 ' + \
-                 unicode(minutes) + u'\u2032 ' + \
-                 unicode(seconds) + u'\u2033 ' + unicode(sign)
-
-        return string
-        
     def column_longitude(self, data):
-        return self.__format_degrees(data[3], _('EW'))
+        return conv_lat_lon('0', data[3], format='DEG')[1]
 
     def column_latitude(self, data):
-        return self.__format_degrees(data[4], _('NS'))
+        return conv_lat_lon(data[4], '0', format='DEG')[0]
 
     def sort_longitude(self, data):
-        return unicode(data[3])
+        return conv_lat_lon('0', data[3], format='ISO-DMS') if data[3] else u''
 
     def sort_latitude(self, data):
-        return unicode(data[4])
+        return conv_lat_lon(data[4], '0', format='ISO-DMS') if data[4] else u''
 
     def column_id(self, data):
         return unicode(data[1])
@@ -178,33 +164,39 @@ class PlaceBaseModel(object):
         except:
             return u''
 
-    def column_city(self, data):
+    def column_locality(self, data):
         try:
             return data[5][0][1]
         except:
             return u''
+
+    def column_city(self, data):
+        try:
+            return data[5][0][2]
+        except:
+            return u''
         
     def column_county(self, data):
         try:
-            return data[5][0][2]
+            return data[5][0][3]
         except:
             return u''
     
     def column_state(self, data):
         try:
-            return data[5][0][3]
+            return data[5][0][4]
         except:
             return u''
 
     def column_country(self, data):
         try:
-            return data[5][0][4]
+            return data[5][0][5]
         except:
             return u''
 
     def column_postal_code(self, data):
         try:
-            return data[5][0][5]
+            return data[5][0][6]
         except:
             return u''
 
@@ -212,8 +204,7 @@ class PlaceBaseModel(object):
         return "%012x" % data[11]
     
     def column_change(self, data):
-        return unicode(time.strftime('%x %X',time.localtime(data[11])),
-                            GrampsLocale.codeset)
+        return Utils.format_time(data[11])
 
     def column_tooltip(self, data):
         if const.USE_TIPS:
@@ -240,9 +231,16 @@ class PlaceListModel(PlaceBaseModel, FlatBaseModel):
                  skip=set(), sort_map=None):
 
         PlaceBaseModel.__init__(self, db)
-        FlatBaseModel.__init__(self, db, scol, order, tooltip_column=14,
+        FlatBaseModel.__init__(self, db, scol, order, tooltip_column=15,
                            search=search, skip=skip, sort_map=sort_map)
 
+    def destroy(self):
+        """
+        Unset all elements that can prevent garbage collection
+        """
+        PlaceBaseModel.destroy(self)
+        FlatBaseModel.destroy(self)
+
     def column_name(self, data):
         return unicode(data[2])
 
@@ -260,17 +258,26 @@ class PlaceTreeModel(PlaceBaseModel, TreeBaseModel):
 
         PlaceBaseModel.__init__(self, db)
         TreeBaseModel.__init__(self, db, scol=scol, order=order,
-                                tooltip_column=14,
+                                tooltip_column=15,
                                 search=search, skip=skip, sort_map=sort_map,
                                 nrgroups = 3,
                                 group_can_have_handle = True)
 
+    def destroy(self):
+        """
+        Unset all elements that can prevent garbage collection
+        """
+        PlaceBaseModel.destroy(self)
+        self.hmap = None
+        self.number_items = None
+        TreeBaseModel.destroy(self)
+
     def _set_base_data(self):
         """See TreeBaseModel, for place, most have been set in init of
         PlaceBaseModel
         """
         self.number_items = self.db.get_number_of_places
-        self.hmap = [self.column_header] + [None]*13
+        self.hmap = [self.column_header] + [None]*14
 
     def get_tree_levels(self):
         """
@@ -287,17 +294,17 @@ class PlaceTreeModel(PlaceBaseModel, TreeBaseModel):
         """
         if data[5] is None:
             # No primary location
-            level = [''] * 5
+            level = [''] * 6
         else:
-            #country, state, county, city, street
-            level = [data[5][0][i] for i in range(4,-1,-1)]
+            #country, state, county, city, locality, street
+            level = [data[5][0][i] for i in range(5,-1,-1)]
 
         node1 = (level[0], )
         node2 = (level[1], level[0])
         node3 = (level[2], level[1], level[0])
         sort_key = self.sort_func(data)
 
-        if not (level[3] or level[4]):
+        if not (level[3] or level[4] or level[5]):
             if level[2]:
                 self.add_node(None, node1, level[0], None, add_parent=False)
                 self.add_node(node1, node2, level[1], None, add_parent=False)
@@ -325,13 +332,11 @@ class PlaceTreeModel(PlaceBaseModel, TreeBaseModel):
     def column_name(self, data):
         name = ''
         if data[5] is not None:
-            level = [data[5][0][i] for i in range(4,-1,-1)]
-            if not (level[3] or level[4]):
+            level = [data[5][0][i] for i in range(5,-1,-1)]
+            if not (level[3] or level[4] or level[5]):
                 name = unicode(level[2] or level[1] or level[0])
-            elif level[3] and level[4]:
-                name = unicode(level[3] + ', ' + level[4])
-            elif level[3] or level[4]:
-                name = unicode(level[3] or level[4])
+            else:
+                name = ', '.join([item for item in level[3:] if item])
         if not name:
             name = unicode(data[2])
 
