@@ -24,7 +24,6 @@
 # python modules
 #
 #-------------------------------------------------------------------------
-import time
 import logging
 log = logging.getLogger(".")
 
@@ -42,7 +41,7 @@ import gtk
 #-------------------------------------------------------------------------
 import const
 import ToolTips
-import GrampsLocale
+import Utils
 from gui.views.treemodels.flatbasemodel import FlatBaseModel
 
 #-------------------------------------------------------------------------
@@ -110,8 +109,7 @@ class SourceModel(FlatBaseModel):
         return unicode(data[4])
 
     def column_change(self,data):
-        return unicode(time.strftime('%x %X',time.localtime(data[8])),
-                       GrampsLocale.codeset)
+        return Utils.format_time(data[8])
     
     def sort_change(self,data):
         return "%012x" % data[8]
@@ -123,6 +121,6 @@ class SourceModel(FlatBaseModel):
                                              self.db.get_source_from_handle(data[0]))
             except:
                 log.error("Failed to create tooltip.",exc_info=True)
-                return t
+            return t
         else:
             return u''
