@@ -19,7 +19,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: __init__.py 13807 2009-12-15 05:56:12Z pez4brian $
+# $Id: __init__.py 17332 2011-04-28 22:05:54Z nick-h $
 
 """Custom widgets."""
 
@@ -27,6 +27,7 @@ from buttons import *
 from expandcollapsearrow import *
 from labels import *
 from linkbox import *
+from photo import *
 from monitoredwidgets import *
 from shortlistcomboentry import *
 from springseparator import *
@@ -34,6 +35,9 @@ from statusbar import Statusbar
 from styledtextbuffer import *
 from styledtexteditor import *
 from toolcomboentry import *
+from undoablebuffer import *
+from undoableentry import *
+from undoablestyledbuffer import *
 from validatedcomboentry import *
 from validatedmaskedentry import *
 from valueaction import *
