@@ -2,6 +2,7 @@
 # Gramps - a GTK+/GNOME based genealogy program
 #
 # Copyright (C) 2000-2006  Donald N. Allingham
+# Copyright (C) 2010       Nick Hall
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,12 +19,13 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: monitoredwidgets.py 12739 2009-06-30 14:05:57Z gbritton $
+# $Id: monitoredwidgets.py 17694 2011-06-06 16:06:59Z gburto01 $
 
-__all__ = ["MonitoredCheckbox", "MonitoredEntry", "MonitoredSpinButton",
+__all__ = ["MonitoredCheckbox", "MonitoredEntry", 
+           "MonitoredEntryIndicator", "MonitoredSpinButton",
            "MonitoredText", "MonitoredType", "MonitoredDataType",
            "MonitoredMenu", "MonitoredStrMenu", "MonitoredDate",
-           "MonitoredComboSelectedEntry"]
+           "MonitoredComboSelectedEntry", "MonitoredTagList"]
 
 #-------------------------------------------------------------------------
 #
@@ -41,14 +43,26 @@ import locale
 #-------------------------------------------------------------------------
 import gobject
 import gtk
+import pango
 
 #-------------------------------------------------------------------------
 #
 # Gramps modules
 #
 #-------------------------------------------------------------------------
+from gen.ggettext import gettext as _
 import AutoComp
 import DateEdit
+from gui.widgets.tageditor import TagEditor
+
+#-------------------------------------------------------------------------
+#
+# constants
+#
+#------------------------------------------------------------------------
+
+_RETURN = gtk.gdk.keyval_from_name("Return")
+_KP_ENTER = gtk.gdk.keyval_from_name("KP_Enter")
 
 #-------------------------------------------------------------------------
 #
@@ -57,7 +71,8 @@ import DateEdit
 #-------------------------------------------------------------------------
 class MonitoredCheckbox(object):
 
-    def __init__(self, obj, button, set_val, get_val, on_toggle=None, readonly = False):
+    def __init__(self, obj, button, set_val, get_val, on_toggle=None,
+                 readonly = False):
         self.button = button
         self.button.connect('toggled', self._on_toggle)
         self.on_toggle = on_toggle
@@ -71,6 +86,14 @@ class MonitoredCheckbox(object):
         self.set_val(obj.get_active())
         if self.on_toggle:
             self.on_toggle(self.get_val())
+
+##    def destroy(self):
+##        """
+##        Unset all elements that can prevent garbage collection
+##        """
+##        self.set_val = None
+##        self.get_val = None
+##        self.obj = None
         
 #-------------------------------------------------------------------------
 #
@@ -94,6 +117,14 @@ class MonitoredEntry(object):
         if autolist:
             AutoComp.fill_entry(obj, autolist)
 
+##    def destroy(self):
+##        """
+##        Unset all elements that can prevent garbage collection
+##        """
+##        self.set_val = None
+##        self.get_val = None
+##        self.obj = None
+
     def reinit(self, set_val, get_val):
         self.set_val = set_val
         self.get_val = get_val
@@ -127,6 +158,49 @@ class MonitoredEntry(object):
         if self.get_val() is not None:
             self.obj.set_text(self.get_val())
 
+#-------------------------------------------------------------------------
+#
+# MonitoredEntryIndicator class
+#
+#-------------------------------------------------------------------------
+class MonitoredEntryIndicator(MonitoredEntry):
+    """
+    Show an Entry box with an indicator in it that disappears when 
+    entry becomes active
+    """
+    def __init__(self, obj, set_val, get_val, indicator, read_only=False, 
+                 autolist=None, changed=None):
+        MonitoredEntry.__init__(self, obj, set_val, get_val, read_only,
+                                autolist, changed)
+        if get_val():
+            self.indicatorshown = False
+        else:
+            self.indicatorshown = True
+            self.indicator = indicator
+            self.obj.set_text(indicator)
+            self.obj.modify_text(gtk.STATE_NORMAL, 
+                                 gtk.gdk.color_parse('grey'))
+            self.obj.modify_font(pango.FontDescription('sans italic'))
+            self.fockey = self.obj.connect('focus-in-event', 
+                                               self._obj_focus)
+    
+    def _on_change(self, obj):
+        if not self.indicatorshown:
+            self.set_val(unicode(obj.get_text()))
+            if self.changed:
+                self.changed(obj)
+
+    def _obj_focus(self, widg, eve):
+        """
+        callback for when prefix obtains focus
+        """
+        self.set_text('')
+        self.obj.modify_text(gtk.STATE_NORMAL, gtk.gdk.color_parse('black'))
+        self.obj.modify_font(pango.FontDescription('normal'))
+        self.obj.disconnect(self.fockey)
+        self.indicatorshown = False
+        return False
+
 #-------------------------------------------------------------------------
 #
 # MonitoredSpinButton class
@@ -161,6 +235,14 @@ class MonitoredSpinButton(object):
         if autolist:
             AutoComp.fill_entry(obj,autolist)
 
+##    def destroy(self):
+##        """
+##        Unset all elements that can prevent garbage collection
+##        """
+##        self.set_val = None
+##        self.get_val = None
+##        self.obj = None
+
     def reinit(self, set_val, get_val):
         """
         Reinitialize class with the specified callback functions.
@@ -265,6 +347,14 @@ class MonitoredText(object):
         self.buf.connect('changed', self.on_change)
         obj.set_editable(not read_only)
 
+##    def destroy(self):
+##        """
+##        Unset all elements that can prevent garbage collection
+##        """
+##        self.set_val = None
+##        self.get_val = None
+##        self.buf = None
+
     def on_change(self, obj):
         s, e = self.buf.get_bounds()
         self.set_val(unicode(self.buf.get_text(s, e, False)))
@@ -296,6 +386,14 @@ class MonitoredType(object):
         self.obj.set_sensitive(not readonly)
         self.obj.connect('changed', self.on_change)
 
+##    def destroy(self):
+##        """
+##        Unset all elements that can prevent garbage collection
+##        """
+##        self.set_val = None
+##        self.get_val = None
+##        self.obj = None
+
     def reinit(self, set_val, get_val):
         self.set_val = set_val
         self.get_val = get_val
@@ -364,6 +462,14 @@ class MonitoredDataType(object):
         self.obj.set_sensitive(not readonly)
         self.obj.connect('changed', self.on_change)
 
+##    def destroy(self):
+##        """
+##        Unset all elements that can prevent garbage collection
+##        """
+##        self.set_val = None
+##        self.get_val = None
+##        self.obj = None
+
     def reinit(self, set_val, get_val):
         self.set_val = set_val
         self.get_val = get_val
@@ -405,6 +511,14 @@ class MonitoredMenu(object):
         self.obj.connect('changed', self.on_change)
         self.obj.set_sensitive(not readonly)
 
+##    def destroy(self):
+##        """
+##        Unset all elements that can prevent garbage collection
+##        """
+##        self.set_val = None
+##        self.get_val = None
+##        self.obj = None
+
     def force(self, value):
         self.obj.set_active(value)
 
@@ -460,6 +574,15 @@ class MonitoredStrMenu(object):
         self.obj.connect('changed', self.on_change)
         self.obj.set_sensitive(not readonly)
 
+##    def destroy(self):
+##        """
+##        Unset all elements that can prevent garbage collection
+##        """
+##        self.set_val = None
+##        self.get_val = None
+##        self.obj = None
+##        self.model = None
+
     def on_change(self, obj):
         self.set_val(self.data[obj.get_active()])
 
@@ -526,6 +649,15 @@ class MonitoredComboSelectedEntry(object):
         #set correct editable
         self.enable(not read_only)
 
+##    def destroy(self):
+##        """
+##        Unset all elements that can prevent garbage collection
+##        """
+##        self.set_val_list = None
+##        self.get_val_list = None
+##        self.objcombo = None
+##        self.objentry = None
+
     def __fill(self):
         """
         Fill combo with data
@@ -597,3 +729,77 @@ class MonitoredComboSelectedEntry(object):
         Eg: name editor save brings you back to person editor that must update
         """
         self.entry_reinit()
+
+#-------------------------------------------------------------------------
+#
+# MonitoredTagList class
+#
+#-------------------------------------------------------------------------
+class MonitoredTagList(object):
+    """
+    A MonitoredTagList consists of a label to display a list of tags and a
+    button to invoke the tag editor.
+    """
+    def __init__(self, label, button, set_list, get_list, db,
+                 uistate, track, readonly=False):
+
+        self.uistate = uistate
+        self.track = track
+        self.db = db
+        self.set_list = set_list
+
+        self.tag_list = []
+        for handle in get_list():
+            tag = self.db.get_tag_from_handle(handle)
+            self.tag_list.append((handle, tag.get_name()))
+
+        self.all_tags = []
+        for tag in self.db.iter_tags():
+            self.all_tags.append((tag.get_handle(), tag.get_name()))
+
+        self.label = label
+        self.label.set_alignment(0, 0.5)
+        self.label.set_ellipsize(pango.ELLIPSIZE_END)
+        image = gtk.Image()
+        image.set_from_stock('gramps-tag', gtk.ICON_SIZE_MENU)
+        button.set_image (image)
+        button.set_tooltip_text(_('Edit the tag list'))
+        button.connect('button-press-event', self.cb_edit)
+        button.connect('key-press-event', self.cb_edit)
+        button.set_sensitive(not readonly)
+
+        self._display()
+
+##    def destroy(self):
+##        """
+##        Unset all elements that can prevent garbage collection
+##        """
+##        self.uistate = None
+##        self.track = None
+##        self.db = None
+##        self.set_list = None
+
+    def _display(self):
+        """
+        Display the tag list.
+        """
+        tag_text = ', '.join(item[1] for item in self.tag_list)
+        self.label.set_text(tag_text)
+        self.label.set_tooltip_text(tag_text)
+
+    def cb_edit(self, button, event):
+        """
+        Invoke the tag editor.
+        """
+        if not event.state or event.state in (gtk.gdk.MOD2_MASK,):
+            if (event.type == gtk.gdk.BUTTON_PRESS or
+               (event.type == gtk.gdk.KEY_PRESS and
+                event.keyval in (_RETURN, _KP_ENTER))):
+                editor = TagEditor(self.tag_list, self.all_tags,
+                                   self.uistate, self.track)
+                if editor.return_list is not None:
+                    self.tag_list = editor.return_list
+                    self._display()
+                    self.set_list([item[0] for item in self.tag_list])
+                return True
+        return False
