@@ -18,9 +18,9 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: styledtextbuffer.py 15331 2010-05-05 23:15:22Z dsblank $
+# $Id: styledtextbuffer.py 17108 2011-04-12 14:27:05Z romjerome $
 
-"Text buffer subclassed from gtk.TextBuffer handling L{StyledText}."
+"""Text buffer subclassed from gtk.TextBuffer handling L{StyledText}."""
 
 __all__ = ["ALLOWED_STYLES", "MATCH_START", "MATCH_END", "MATCH_FLAVOR",
            "MATCH_STRING", "StyledTextBuffer"]
@@ -43,6 +43,7 @@ _LOG = logging.getLogger(".widgets.styledtextbuffer")
 #-------------------------------------------------------------------------
 import gobject
 import gtk
+from gui.widgets.undoablebuffer import UndoableBuffer
 from pango import WEIGHT_BOLD, STYLE_ITALIC, UNDERLINE_SINGLE
 
 #-------------------------------------------------------------------------
@@ -65,6 +66,7 @@ ALLOWED_STYLES = (
     StyledTextTagType.HIGHLIGHT,
     StyledTextTagType.FONTFACE,
     StyledTextTagType.FONTSIZE,
+    StyledTextTagType.LINK,
 )
 
 STYLE_TO_PROPERTY = {
@@ -75,6 +77,7 @@ STYLE_TO_PROPERTY = {
     StyledTextTagType.HIGHLIGHT: 'background',
     StyledTextTagType.FONTFACE: 'family',
     StyledTextTagType.FONTSIZE: 'size-points',
+    StyledTextTagType.LINK: 'link',
 }
 
 (MATCH_START,
@@ -82,6 +85,25 @@ STYLE_TO_PROPERTY = {
  MATCH_FLAVOR,
  MATCH_STRING,) = range(4)
 
+#-------------------------------------------------------------------------
+#
+# LinkTag class
+#
+#-------------------------------------------------------------------------
+class LinkTag(gtk.TextTag):
+    """
+    Class for keeping track of link data.
+    """
+    lid = 0
+    def __init__(self, buffer, data, **properties):
+        LinkTag.lid += 1
+        self.data = data
+        gtk.TextTag.__init__(self, "link-%d" % LinkTag.lid)
+        tag_table = buffer.get_tag_table()
+        for property in properties:
+            self.set_property(property, properties[property])
+        tag_table.add(self)
+
 #-------------------------------------------------------------------------
 #
 # GtkSpellState class
@@ -206,7 +228,7 @@ class GtkSpellState(object):
 # StyledTextBuffer class
 #
 #-------------------------------------------------------------------------
-class StyledTextBuffer(gtk.TextBuffer):
+class StyledTextBuffer(UndoableBuffer):
     """An extended TextBuffer for handling StyledText strings.
     
     StyledTextBuffer is an interface between GRAMPS' L{StyledText} format
@@ -237,7 +259,7 @@ class StyledTextBuffer(gtk.TextBuffer):
     }    
 
     def __init__(self):
-        gtk.TextBuffer.__init__(self)
+        super(StyledTextBuffer, self).__init__()
 
         # Create fix tags.
         # Other tags (e.g. color) have to be created on the fly
@@ -308,7 +330,7 @@ class StyledTextBuffer(gtk.TextBuffer):
     def do_changed(self):
         """Parse for patterns in the text."""
         self.matches = []
-        text = unicode(gtk.TextBuffer.get_text(self,
+        text = unicode(super(StyledTextBuffer, self).get_text(
                                                self.get_start_iter(),
                                                self.get_end_iter()))
         for regex, flavor in self.patterns:
@@ -432,6 +454,29 @@ class StyledTextBuffer(gtk.TextBuffer):
                     self.remove_tag_by_name(tag_name,
                                             self.get_iter_at_offset(start),
                                             self.get_iter_at_offset(end+1))
+
+    def clear_selection(self):
+        """
+        Clear tags from selection.
+        """
+        start, end = self._get_selection()
+        tags = self._get_tag_from_range(start.get_offset(), end.get_offset())
+        removed_something = False
+        for tag_name, tag_data in tags.iteritems():
+            if tag_name.startswith("link"):
+                for start_pos, end_pos in tag_data:
+                    self.remove_tag_by_name(tag_name,
+                                            self.get_iter_at_offset(start_pos),
+                                            self.get_iter_at_offset(end_pos+1))
+                    removed_something = True
+
+        for style in ALLOWED_STYLES:
+            value = self.style_state[style]
+            if value and (value != StyledTextTagType.STYLE_DEFAULT[style]):
+                self.remove_tag(self._find_tag_by_name(style, value),
+                                start, end)
+                removed_something = True
+        return removed_something
                     
     def _get_tag_from_range(self, start=None, end=None):
         """Extract gtk.TextTags from buffer.
@@ -505,11 +550,17 @@ class StyledTextBuffer(gtk.TextBuffer):
         @note: 's_' prefix means StyledText*, while 'g_' prefix means gtk.*.
         
         """
-        gtk.TextBuffer.set_text(self, str(s_text))
+        super(StyledTextBuffer, self).set_text(str(s_text))
+        #self.remove_all_tags(self.get_start_iter(), self.get_end_iter())
     
         s_tags = s_text.get_tags()
         for s_tag in s_tags:
-            g_tag = self._find_tag_by_name(int(s_tag.name), s_tag.value)
+            if s_tag.name == _('Link'):
+                g_tag = LinkTag(self, s_tag.value, 
+                                foreground="blue",
+                                underline=UNDERLINE_SINGLE)
+            else:
+                g_tag = self._find_tag_by_name(int(s_tag.name), s_tag.value)
             if g_tag is not None:
                 for (start, end) in s_tag.ranges:
                     start_iter = self.get_iter_at_offset(start)
@@ -527,7 +578,7 @@ class StyledTextBuffer(gtk.TextBuffer):
         if end is None:
             end = self.get_end_iter()
 
-        txt = gtk.TextBuffer.get_text(self, start, end, include_hidden_chars)
+        txt = super(StyledTextBuffer, self).get_text(start, end, include_hidden_chars)
         txt = unicode(txt)
         
         # extract tags out of the buffer
@@ -535,23 +586,30 @@ class StyledTextBuffer(gtk.TextBuffer):
         s_tags = []
         
         for g_tagname, g_ranges in g_tags.items():
-            style_and_value = g_tagname.split(' ', 1)
+            if g_tagname.startswith('link'):
+                tag = self.get_tag_table().lookup(g_tagname)
+                s_ranges = [(start, end+1) for (start, end) in g_ranges]
+                s_value = tag.data
+                s_tag = StyledTextTag(_('Link'), s_value, s_ranges)
+                s_tags.append(s_tag)
+            else:
+                style_and_value = g_tagname.split(' ', 1)
 
-            try:
-                style = int(style_and_value[0])
-                if len(style_and_value) == 1:
-                    s_value = None
-                else:
-                    s_value = StyledTextTagType.STYLE_TYPE[style]\
-                            (style_and_value[1])
-    
-                if style in ALLOWED_STYLES:
-                    s_ranges = [(start, end+1) for (start, end) in g_ranges]
-                    s_tag = StyledTextTag(style, s_value, s_ranges)
-                                          
-                    s_tags.append(s_tag)
-            except ValueError:
-                _LOG.debug("silently skipping gtk.TextTag '%s'" % g_tagname)
+                try:
+                    style = int(style_and_value[0])
+                    if len(style_and_value) == 1:
+                        s_value = None
+                    else:
+                        s_value = StyledTextTagType.STYLE_TYPE[style]\
+                                (style_and_value[1])
+
+                    if style in ALLOWED_STYLES:
+                        s_ranges = [(start, end+1) for (start, end) in g_ranges]
+                        s_tag = StyledTextTag(style, s_value, s_ranges)
+
+                        s_tags.append(s_tag)
+                except ValueError:
+                    _LOG.debug("silently skipping gtk.TextTag '%s'" % g_tagname)
 
         return StyledText(txt, s_tags)
     
