@@ -3,6 +3,7 @@
 #
 # Copyright (C) 2003-2007  Donald N. Allingham
 # Copyright (C) 2007-2008  Brian G. Matherly
+# Copyright (C) 2010       Jakim Friant
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,7 +20,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: BookReport.py 12560 2009-05-21 17:58:31Z gbritton $
+# $Id: BookReport.py 17022 2011-04-02 03:29:05Z pez4brian $
 
 # Written by Alex Roitman, 
 # largely based on the BaseDoc classes by Don Allingham
@@ -29,8 +30,7 @@
 # Standard Python modules
 #
 #-------------------------------------------------------------------------
-import os
-from gettext import gettext as _
+from gen.ggettext import gettext as _
 
 #------------------------------------------------------------------------
 #
@@ -39,6 +39,7 @@ from gettext import gettext as _
 #------------------------------------------------------------------------
 import logging
 log = logging.getLogger(".BookReport")
+import os
 
 #-------------------------------------------------------------------------
 #
@@ -58,8 +59,7 @@ except:
 #
 #-------------------------------------------------------------------------
 import gtk
-from gtk import glade
-from gtk import RESPONSE_OK
+import gobject
 
 #-------------------------------------------------------------------------
 #
@@ -70,21 +70,22 @@ import const
 import Utils
 import ListModel
 import Errors
-import BaseDoc
+from gui.pluginmanager import GuiPluginManager
+from gen.plug.docgen import StyleSheet, StyleSheetList
 from QuestionDialog import WarningDialog, ErrorDialog
-from gen.plug import PluginManager
 from gen.plug.menu import PersonOption, FilterOption, FamilyOption
 import ManagedWindow
+from glade import Glade
+from gui.utils import open_file_with_default_application
 
 # Import from specific modules in ReportBase
-from ReportBase import CATEGORY_BOOK, book_categories
-from ReportBase._BookFormatComboBox import BookFormatComboBox
-from ReportBase._ReportDialog import ReportDialog
-from ReportBase._DocReportDialog import DocReportDialog
-from ReportBase._CommandLineReport import CommandLineReport
-from ReportBase._ReportOptions import ReportOptions
+from gen.plug.report import CATEGORY_BOOK, book_categories
+from gui.plug.report._reportdialog import ReportDialog
+from gui.plug.report._docreportdialog import DocReportDialog
+from gen.plug.report._options import ReportOptions
+from cli.plug import CommandLineReport
 
-from BasicUtils import name_displayer as _nd
+from gen.display.name import displayer as _nd
 
 #------------------------------------------------------------------------
 #
@@ -98,7 +99,7 @@ _UNSUPPORTED = _("Unsupported")
 # Private Functions
 #
 #------------------------------------------------------------------------
-def _initialize_options(options, dbstate):
+def _initialize_options(options, dbstate, uistate):
     """
     Validates all options by making sure that their values are consistent with
     the database.
@@ -106,27 +107,38 @@ def _initialize_options(options, dbstate):
     menu: The Menu class
     dbase: the database the options will be applied to
     """
-    dbase = dbstate.get_database()
     if not hasattr(options, "menu"):
         return
+    dbase = dbstate.get_database()
     menu = options.menu
     
     for name in menu.get_all_option_names():
         option = menu.get_option_by_name(name)
+        value = option.get_value()
 
         if isinstance(option, PersonOption):
-            person = dbstate.get_active_person()
-            option.set_value(person.get_gramps_id())
+            if not dbase.get_person_from_gramps_id(value):
+                person_handle = uistate.get_active('Person')
+                person = dbase.get_person_from_handle(person_handle)
+                option.set_value(person.get_gramps_id())
         
         elif isinstance(option, FamilyOption):
-            person = dbstate.get_active_person()
-            family_list = person.get_family_handle_list()
-            if family_list:
-                family_handle = family_list[0]
-            else:
-                family_handle = dbase.get_family_handles()[0]
-            family = dbase.get_family_from_handle(family_handle)
-            option.set_value(family.get_gramps_id())
+            if not dbase.get_family_from_gramps_id(value):
+                person_handle = uistate.get_active('Person')
+                person = dbase.get_person_from_handle(person_handle)
+                family_list = person.get_family_handle_list()
+                if family_list:
+                    family_handle = family_list[0]
+                else:
+                    try:
+                        family_handle = dbase.iter_family_handles().next()
+                    except StopIteration:
+                        family_handle = None
+                if family_handle:
+                    family = dbase.get_family_from_handle(family_handle)
+                    option.set_value(family.get_gramps_id())
+                else:
+                    print "No family specified for ", name
 
 def _get_subject(options, dbase):
     """
@@ -138,10 +150,12 @@ def _get_subject(options, dbase):
     dbase: the database for which it corresponds
     """
     if not hasattr(options, "menu"):
-        return _("Not Applicable")
+        return ""
     menu = options.menu
     
     option_names = menu.get_all_option_names()
+    if not option_names:
+        return _("Entire Database")
 
     for name in option_names:
         option = menu.get_option_by_name(name)
@@ -156,6 +170,8 @@ def _get_subject(options, dbase):
         
         elif isinstance(option, FamilyOption):
             family = dbase.get_family_from_gramps_id(option.get_value())
+            if not family:
+                return ""
             family_id = family.get_gramps_id()
             fhandle = family.get_father_handle()
             mhandle = family.get_mother_handle()
@@ -172,10 +188,13 @@ def _get_subject(options, dbase):
             else:
                 mother_name = _("unknown mother")
                 
-            name = _("%s and %s (%s)") % (father_name, mother_name, family_id)
+            name = _("%(father)s and %(mother)s (%(id)s)") % {
+                                                'father' : father_name, 
+                                                'mother' : mother_name, 
+                                                'id' : family_id }
             return name
         
-    return _("Not Applicable")
+    return ""
 
 #------------------------------------------------------------------------
 #
@@ -191,23 +210,25 @@ class BookItem(object):
         """
         Create a new empty BookItem.
         
-        name:   the book item is retreived 
+        name:   the book item is retrieved 
                 from the book item registry using name for lookup
         """
         self.dbase = dbase
         self.style_name = "default"
-        pmgr = PluginManager.get_instance()
+        pmgr = GuiPluginManager.get_instance()
 
-        for item in pmgr.get_book_item_list():
-            if item[4] == name:
-                self.translated_name = item[0]
-                if item[5]:
+        for pdata in pmgr.get_reg_bookitems():
+            if pdata.id == name:
+                self.translated_name = pdata.name
+                if not pdata.supported:
                     self.category = _UNSUPPORTED
                 else:
-                    self.category = book_categories[item[1]]
-                self.write_item = item[2]
-                self.name = item[4]
-                self.option_class = item[3](self.name, self.dbase)
+                    self.category = book_categories[pdata.category]
+                mod = pmgr.load_plugin(pdata)
+                self.write_item = eval('mod.' + pdata.reportclass)
+                self.name = pdata.id
+                oclass = eval('mod.' + pdata.optionclass)
+                self.option_class = oclass(self.name, self.dbase)
                 self.option_class.load_previous_values()
 
     def get_name(self):
@@ -403,14 +424,14 @@ class BookList(object):
         return self.bookmap[name]
 
     def get_book_names(self):
-        "Return a list of all the book names in the BookList"
-        return self.bookmap.keys()
+        "Return a list of all the book names in the BookList, sorted"
+        return sorted(self.bookmap.keys())
 
     def set_book(self, name, book):
         """
         Add or replaces a Book in the BookList. 
 
-        name:   name assocated with the Book to add or replace.
+        name:   name associated with the Book to add or replace.
         book:   definition of the Book
         """
         self.bookmap[name] = book
@@ -422,8 +443,7 @@ class BookList(object):
         f = open(self.file, "w")
         f.write("<?xml version=\"1.0\" encoding=\"utf-8\"?>\n")
         f.write('<booklist>\n')
-
-        for name in self.bookmap.keys():
+        for name in self.bookmap:
             book = self.get_book(name)
             dbname = book.get_dbname()
             f.write('<book name="%s" database="%s">\n' % (name, dbname) )
@@ -431,16 +451,16 @@ class BookList(object):
                 f.write('  <item name="%s" trans_name="%s">\n' % 
                             (item.get_name(),item.get_translated_name() ) )
                 options = item.option_class.handler.options_dict
-                for option_name in options.keys():
-                    option_value = options[option_name]
+                for option_name, option_value in options.iteritems():
                     if isinstance(option_value, (list, tuple)):
                         f.write('  <option name="%s" value="" '
                                 'length="%d">\n' % (
                                     escape(option_name),
                                     len(options[option_name]) ) )
-                        for list_index in range(len(options[option_name])):
-                            option_type = \
-                                Utils.type_name(option_value[list_index])
+                        for list_index in range(len(option_value)):
+                            option_type = Utils.type_name(
+                                option_value[list_index]
+                            )
                             value = escape(unicode(option_value[list_index]))
                             value = value.replace('"', '&quot;')
                             f.write('    <listitem number="%d" type="%s" '
@@ -476,7 +496,7 @@ class BookList(object):
             the_file = open(self.file)
             p.parse(the_file)
             the_file.close()
-        except (IOError, OSError, ValueError, SAXParseException, KeyError, \
+        except (IOError, OSError, ValueError, SAXParseException, KeyError,
                AttributeError):
             pass
 
@@ -571,29 +591,38 @@ class BookListDisplay(object):
         
         self.booklist = booklist
         self.dosave = dosave
-        base = os.path.dirname(__file__)
-        glade_file = os.path.join(base,"book.glade")
-        self.xml = glade.XML(glade_file, "booklist", "gramps")
-        self.top = self.xml.get_widget('booklist')
+        self.xml = Glade()
+        self.top = self.xml.toplevel
+        self.unsaved_changes = False
 
         ManagedWindow.set_titles(self.top,
-            self.xml.get_widget('title'),_('Available Books'))
+            self.xml.get_object('title'),_('Available Books'))
 
         if nodelete:
-            delete_button = self.xml.get_widget("delete_button")
+            delete_button = self.xml.get_object("delete_button")
             delete_button.hide()
-
-        self.xml.signal_autoconnect({
+        self.xml.connect_signals({
             "on_booklist_cancel_clicked" : self.on_booklist_cancel_clicked,
-            "on_booklist_ok_clicked" : self.on_booklist_ok_clicked,
-            "on_booklist_delete_clicked" : self.on_booklist_delete_clicked
+            "on_booklist_ok_clicked"     : self.on_booklist_ok_clicked,
+            "on_booklist_delete_clicked" : self.on_booklist_delete_clicked,
+            "on_book_ok_clicked"         : self.do_nothing,
+            "destroy_passed_object"      : self.do_nothing,
+            "on_setup_clicked"           : self.do_nothing,
+            "on_down_clicked"            : self.do_nothing,
+            "on_up_clicked"              : self.do_nothing,
+            "on_remove_clicked"          : self.do_nothing,
+            "on_add_clicked"             : self.do_nothing,
+            "on_edit_clicked"            : self.do_nothing,
+            "on_open_clicked"            : self.do_nothing,
+            "on_save_clicked"            : self.do_nothing,
+            "on_clear_clicked"           : self.do_nothing
             })
 
-        title_label = self.xml.get_widget('title')
+        title_label = self.xml.get_object('title')
         title_label.set_text(Utils.title(_('Book List')))
         title_label.set_use_markup(True)
         
-        self.blist = ListModel.ListModel(self.xml.get_widget("list"),
+        self.blist = ListModel.ListModel(self.xml.get_object("list"),
                                         [('Name',-1,10)],)
         self.redraw()
         self.selection = None
@@ -632,9 +661,23 @@ class BookListDisplay(object):
         data = self.blist.get_data(the_iter, [0])
         self.booklist.delete_book(unicode(data[0]))
         self.blist.remove(the_iter)
+        self.unsaved_changes = True
         self.top.run()
 
     def on_booklist_cancel_clicked(self, obj):
+        if self.unsaved_changes:
+            from QuestionDialog import QuestionDialog2
+            q = QuestionDialog2(
+                _('Discard Unsaved Changes'),
+                _('You have made changes which have not been saved.'),
+                _('Proceed'),
+                _('Cancel'))
+            if q.run():
+                return                
+            else:
+                self.top.run()
+    
+    def do_nothing(self, object):
         pass
 
 #------------------------------------------------------------------------
@@ -683,15 +726,13 @@ class BookReportSelector(ManagedWindow.ManagedWindow):
 
         ManagedWindow.ManagedWindow.__init__(self, uistate, [], self.__class__)
 
-        base = os.path.dirname(__file__)
-        glade_file = os.path.join(base,"book.glade")
-
-        self.xml = glade.XML(glade_file, "top", "gramps")
-        window = self.xml.get_widget("top")
-        title_label = self.xml.get_widget('title')
+        self.xml = Glade(toplevel="top")
+        window = self.xml.toplevel
+        
+        title_label = self.xml.get_object('title')
         self.set_window(window, title_label, self.title)
-    
-        self.xml.signal_autoconnect({
+        window.show()
+        self.xml.connect_signals({
             "on_add_clicked"        : self.on_add_clicked,
             "on_remove_clicked"     : self.on_remove_clicked,
             "on_up_clicked"         : self.on_up_clicked,
@@ -702,40 +743,47 @@ class BookReportSelector(ManagedWindow.ManagedWindow):
             "on_open_clicked"       : self.on_open_clicked,
             "on_edit_clicked"       : self.on_edit_clicked,
             "on_book_ok_clicked"    : self.on_book_ok_clicked,
-            "destroy_passed_object" : self.close
+            "destroy_passed_object" : self.close,
+
+        # Insert dummy handlers for second top level in the glade file
+            "on_booklist_ok_clicked"    : lambda _:None,
+            "on_booklist_delete_clicked": lambda _:None,
+            "on_booklist_cancel_clicked": lambda _:None,
+            "on_booklist_ok_clicked"    : lambda _:None,
+            "on_booklist_ok_clicked"    : lambda _:None,
             })
 
-        self.avail_tree = self.xml.get_widget("avail_tree")
-        self.book_tree = self.xml.get_widget("book_tree")
-        self.avail_tree.connect('button-press-event', self.av_button_press)
-        self.book_tree.connect('button-press-event', self.bk_button_press)
+        self.avail_tree = self.xml.get_object("avail_tree")
+        self.book_tree = self.xml.get_object("book_tree")
+        self.avail_tree.connect('button-press-event', self.avail_button_press)
+        self.book_tree.connect('button-press-event', self.book_button_press)
 
-        self.name_entry = self.xml.get_widget("name_entry")
+        self.name_entry = self.xml.get_object("name_entry")
         self.name_entry.set_text(_('New Book'))
 
-        avail_label = self.xml.get_widget('avail_label')
+        avail_label = self.xml.get_object('avail_label')
         avail_label.set_text("<b>%s</b>" % _("_Available items"))
         avail_label.set_use_markup(True)
         avail_label.set_use_underline(True)
-        book_label = self.xml.get_widget('book_label')
+        book_label = self.xml.get_object('book_label')
         book_label.set_text("<b>%s</b>" % _("Current _book"))
         book_label.set_use_underline(True)
         book_label.set_use_markup(True)
 
-        av_titles = [ (_('Name'), 0, 150),
-                      (_('Type'), 1, 50 ),
+        avail_titles = [ (_('Name'), 0, 230),
+                      (_('Type'), 1, 80 ),
                       (  '' ,    -1, 0  ) ]
         
-        bk_titles = [ (_('Item name'), -1, 150),
-                      (_('Type'),      -1, 50 ),
+        book_titles = [ (_('Item name'), -1, 230),
+                      (_('Type'),      -1, 80 ),
                       (  '',           -1, 0  ),
                       (_('Subject'),   -1, 50 ) ]
         
-        self.av_ncols = len(av_titles)
-        self.bk_ncols = len(bk_titles)
+        self.avail_nr_cols = len(avail_titles)
+        self.book_nr_cols = len(book_titles)
 
-        self.av_model = ListModel.ListModel(self.avail_tree, av_titles)
-        self.bk_model = ListModel.ListModel(self.book_tree, bk_titles)
+        self.avail_model = ListModel.ListModel(self.avail_tree, avail_titles)
+        self.book_model = ListModel.ListModel(self.book_tree, book_titles)
         self.draw_avail_list()
 
         self.book = Book()
@@ -749,24 +797,25 @@ class BookReportSelector(ManagedWindow.ManagedWindow):
         
         The selections are read from the book item registry.
         """
-        pmgr = PluginManager.get_instance()
-        if not pmgr.get_book_item_list():
+        pmgr = GuiPluginManager.get_instance()
+        regbi = pmgr.get_reg_bookitems()
+        if not regbi:
             return
 
-        for book_item in pmgr.get_book_item_list():
-            if book_item[5]:
+        for pdata in regbi:
+            if not pdata.supported:
                 category = _UNSUPPORTED
             else:
-                category = book_categories[book_item[1]]
+                category = book_categories[pdata.category]
             
-            data = [ book_item[0], category, book_item[4] ] 
-            new_iter = self.av_model.add(data)
+            data = [pdata.name, category, pdata.id ] 
+            new_iter = self.avail_model.add(data)
 
-        self.av_model.connect_model()
+        self.avail_model.connect_model()
 
         if new_iter:
-            self.av_model.selection.select_iter(new_iter)
-            path = self.av_model.model.get_path(new_iter)
+            self.avail_model.selection.select_iter(new_iter)
+            path = self.avail_model.model.get_path(new_iter)
             col = self.avail_tree.get_column(0)
             self.avail_tree.scroll_to_cell(path, col, 1, 1, 0.0)
 
@@ -789,12 +838,22 @@ class BookReportSelector(ManagedWindow.ManagedWindow):
                 % book.get_dbname() )
             
         self.book.clear()
-        self.bk_model.clear()
+        self.book_model.clear()
         for saved_item in book.get_item_list():
             name = saved_item.get_name()
             item = BookItem(self.db, name)
             item.option_class = saved_item.option_class
-            _initialize_options(item.option_class, self.dbstate)
+
+            # The option values were loaded magically by the book parser.
+            # But they still need to be applied to the menu options.
+            opt_dict = item.option_class.handler.options_dict
+            menu = item.option_class.menu
+            for optname in opt_dict:
+                menu_option = menu.get_option_by_name(optname)
+                if menu_option:
+                    menu_option.set_value(opt_dict[optname])
+            
+            _initialize_options(item.option_class, self.dbstate, self.uistate)
             item.set_style_name(saved_item.get_style_name())
             self.book.append_item(item)
             
@@ -802,7 +861,7 @@ class BookReportSelector(ManagedWindow.ManagedWindow):
                      item.get_category(), item.get_name() ]
             
             data[2] = _get_subject(item.option_class, self.db)
-            self.bk_model.add(data)
+            self.book_model.add(data)
 
     def on_add_clicked(self, obj):
         """
@@ -810,45 +869,45 @@ class BookReportSelector(ManagedWindow.ManagedWindow):
         
         Use the selected available item to get the item's name in the registry.
         """
-        store, the_iter = self.av_model.get_selected()
+        store, the_iter = self.avail_model.get_selected()
         if not the_iter:
             return
-        data = self.av_model.get_data(the_iter, range(self.av_ncols))
+        data = self.avail_model.get_data(the_iter, range(self.avail_nr_cols))
         item = BookItem(self.db, data[2])
-        _initialize_options(item.option_class, self.dbstate)
+        _initialize_options(item.option_class, self.dbstate, self.uistate)
         data[2] = _get_subject(item.option_class, self.db)
-        self.bk_model.add(data)
+        self.book_model.add(data)
         self.book.append_item(item)
 
     def on_remove_clicked(self, obj):
         """
         Remove the item from the current list of selections.
         """
-        store, the_iter = self.bk_model.get_selected()
+        store, the_iter = self.book_model.get_selected()
         if not the_iter:
             return
-        row = self.bk_model.get_selected_row()
+        row = self.book_model.get_selected_row()
         self.book.pop_item(row)
-        self.bk_model.remove(the_iter)
+        self.book_model.remove(the_iter)
 
     def on_clear_clicked(self, obj):
         """
         Clear the whole current book.
         """
-        self.bk_model.clear()
+        self.book_model.clear()
         self.book.clear()
 
     def on_up_clicked(self, obj):
         """
         Move the currently selected item one row up in the selection list.
         """
-        row = self.bk_model.get_selected_row()
+        row = self.book_model.get_selected_row()
         if not row or row == -1:
             return
-        store, the_iter = self.bk_model.get_selected()
-        data = self.bk_model.get_data(the_iter, range(self.bk_ncols))
-        self.bk_model.remove(the_iter)
-        self.bk_model.insert(row-1, data, None, 1)
+        store, the_iter = self.book_model.get_selected()
+        data = self.book_model.get_data(the_iter, range(self.book_nr_cols))
+        self.book_model.remove(the_iter)
+        self.book_model.insert(row-1, data, None, 1)
         item = self.book.pop_item(row)
         self.book.insert_item(row-1, item)
 
@@ -856,13 +915,13 @@ class BookReportSelector(ManagedWindow.ManagedWindow):
         """
         Move the currently selected item one row down in the selection list.
         """
-        row = self.bk_model.get_selected_row()
-        if row + 1 >= self.bk_model.count or row == -1:
+        row = self.book_model.get_selected_row()
+        if row + 1 >= self.book_model.count or row == -1:
             return
-        store, the_iter = self.bk_model.get_selected()
-        data = self.bk_model.get_data(the_iter, range(self.bk_ncols))
-        self.bk_model.remove(the_iter)
-        self.bk_model.insert(row+1, data, None, 1)
+        store, the_iter = self.book_model.get_selected()
+        data = self.book_model.get_data(the_iter, range(self.book_nr_cols))
+        self.book_model.remove(the_iter)
+        self.book_model.insert(row+1, data, None, 1)
         item = self.book.pop_item(row)
         self.book.insert_item(row+1, item)
 
@@ -870,25 +929,37 @@ class BookReportSelector(ManagedWindow.ManagedWindow):
         """
         Configure currently selected item.
         """
-        store, the_iter = self.bk_model.get_selected()
+        store, the_iter = self.book_model.get_selected()
         if not the_iter:
             return
-        data = self.bk_model.get_data(the_iter, range(self.bk_ncols))
-        row = self.bk_model.get_selected_row()
+        data = self.book_model.get_data(the_iter, range(self.book_nr_cols))
+        row = self.book_model.get_selected_row()
         item = self.book.get_item(row)
         option_class = item.option_class
         item_dialog = BookItemDialog(self.dbstate, self.uistate, option_class,
                                      item.get_name(),
                                      item.get_translated_name(),
                                      self.track)
-        response = item_dialog.window.run()
-        if response == RESPONSE_OK:
-            subject = _get_subject(option_class, self.db)
-            self.bk_model.model.set_value(the_iter, 2, subject)
-            self.book.set_item(row, item)
-        item_dialog.close()
 
-    def bk_button_press(self, obj, event):
+        while True:
+            response = item_dialog.window.run()
+            if response == gtk.RESPONSE_OK:
+                # dialog will be closed by connect, now continue work while
+                # rest of dialog is unresponsive, release when finished
+                subject = _get_subject(option_class, self.db)
+                self.book_model.model.set_value(the_iter, 2, subject)
+                self.book.set_item(row, item)
+                item_dialog.close()
+                break
+            elif response == gtk.RESPONSE_CANCEL:
+                item_dialog.close()
+                break
+            elif response == gtk.RESPONSE_DELETE_EVENT:
+                #just stop, in ManagedWindow, delete-event is already coupled to
+                #correct action.
+                break
+
+    def book_button_press(self, obj, event):
         """
         Double-click on the current book selection is the same as setup.
         Right click evokes the context menu. 
@@ -896,9 +967,9 @@ class BookReportSelector(ManagedWindow.ManagedWindow):
         if event.type == gtk.gdk._2BUTTON_PRESS and event.button == 1:
             self.on_setup_clicked(obj)
         elif event.type == gtk.gdk.BUTTON_PRESS and event.button == 3:
-            self.build_bk_context_menu(event)
+            self.build_book_context_menu(event)
 
-    def av_button_press(self, obj, event):
+    def avail_button_press(self, obj, event):
         """
         Double-click on the available selection is the same as add.
         Right click evokes the context menu. 
@@ -906,12 +977,12 @@ class BookReportSelector(ManagedWindow.ManagedWindow):
         if event.type == gtk.gdk._2BUTTON_PRESS and event.button == 1:
             self.on_add_clicked(obj)
         elif event.type == gtk.gdk.BUTTON_PRESS and event.button == 3:
-            self.build_av_context_menu(event)
+            self.build_avail_context_menu(event)
 
-    def build_bk_context_menu(self, event):
+    def build_book_context_menu(self, event):
         """Builds the menu with item-centered and book-centered options."""
         
-        store, the_iter = self.bk_model.get_selected()
+        store, the_iter = self.book_model.get_selected()
         if the_iter:
             sensitivity = 1 
         else:
@@ -939,10 +1010,10 @@ class BookReportSelector(ManagedWindow.ManagedWindow):
             menu.append(item)
         menu.popup(None, None, None, event.button, event.time)
 
-    def build_av_context_menu(self, event):
+    def build_avail_context_menu(self, event):
         """Builds the menu with the single Add option."""
         
-        store, the_iter = self.av_model.get_selected()
+        store, the_iter = self.avail_model.get_selected()
         if the_iter:
             sensitivity = 1 
         else:
@@ -977,7 +1048,27 @@ class BookReportSelector(ManagedWindow.ManagedWindow):
         """
         self.book_list = BookList(self.file, self.db)
         name = unicode(self.name_entry.get_text())
-        self.book.set_name(name)
+        if not name:
+            WarningDialog(_('No book name'), _(
+                'You are about to save away a book with no name.\n\n'
+                'Please give it a name before saving it away.')
+                )
+            return
+        if name in self.book_list.get_book_names():
+            from QuestionDialog import QuestionDialog2
+            q = QuestionDialog2(
+                _('Book name already exists'),
+                _('You are about to save away a '
+                  'book with a name which already exists.'
+                ),
+                _('Proceed'),
+                _('Cancel'))
+            if q.run():
+                self.book.set_name(name)
+            else:
+                return
+        else:
+            self.book.set_name(name)
         self.book.set_dbname(self.db.get_save_path())
         self.book_list.set_book(name, self.book)
         self.book_list.save()
@@ -1039,6 +1130,45 @@ class BookItemDialog(ReportDialog):
     def parse_target_frame(self):
         """Target frame is not used."""
         return 1
+    
+#-------------------------------------------------------------------------
+#
+# _BookFormatComboBox
+#
+#-------------------------------------------------------------------------
+class _BookFormatComboBox(gtk.ComboBox):
+
+    def __init__(self, active):
+
+        gtk.ComboBox.__init__(self)
+        
+        pmgr = GuiPluginManager.get_instance()
+        self.__bookdoc_plugins = []
+        for plugin in pmgr.get_docgen_plugins():
+            if plugin.get_text_support() and plugin.get_draw_support():
+                self.__bookdoc_plugins.append(plugin)
+        
+        self.store = gtk.ListStore(gobject.TYPE_STRING)
+        self.set_model(self.store)
+        cell = gtk.CellRendererText()
+        self.pack_start(cell, True)
+        self.add_attribute(cell, 'text', 0)
+
+        index = 0
+        active_index = 0
+        for plugin in self.__bookdoc_plugins:
+            name = plugin.get_name()
+            self.store.append(row=[name])
+            if plugin.get_extension() == active:
+                active_index = index
+            index += 1
+        self.set_active(active_index)
+
+    def get_active_plugin(self):
+        """
+        Get the plugin represented by the currently active selection.
+        """
+        return self.__bookdoc_plugins[self.get_active()]
 
 #------------------------------------------------------------------------
 #
@@ -1053,23 +1183,24 @@ class BookReportDialog(DocReportDialog):
     """
 
     def __init__(self, dbstate, uistate, book, options):
+        self.format_menu = None
         self.options = options
         self.page_html_added = False
         DocReportDialog.__init__(self, dbstate, uistate, options,
                                   'book', _("Book Report"))
         self.book = book
         self.database = dbstate.db
-        self.selected_style = BaseDoc.StyleSheet()
+        self.selected_style = StyleSheet()
 
         for item in self.book.get_item_list():
             # Set up default style
-            default_style = BaseDoc.StyleSheet()
+            default_style = StyleSheet()
             make_default_style = item.option_class.make_default_style
             make_default_style(default_style)
 
             # Read all style sheets available for this item
             style_file = item.option_class.handler.get_stylesheet_savefile()
-            style_list = BaseDoc.StyleSheetList(style_file, default_style)
+            style_list = StyleSheetList(style_file, default_style)
 
             # Get the selected stylesheet
             style_name = item.option_class.handler.get_default_stylesheet_name()
@@ -1092,7 +1223,7 @@ class BookReportDialog(DocReportDialog):
                     this_style_name,style_sheet.get_cell_style(this_style_name))
 
         response = self.window.run()
-        if response == RESPONSE_OK:
+        if response == gtk.RESPONSE_OK:
             try:
                 self.make_report()
             except (IOError,OSError),msg:
@@ -1107,23 +1238,18 @@ class BookReportDialog(DocReportDialog):
         return _("Book Report")
 
     def get_header(self, name):
-        return _("GRAMPS Book")
-
-    def get_stylesheet_savefile(self):
-        """Needed solely for forming sane filename for the output."""
-        return "book.xml"
+        return _("Gramps Book")
 
     def make_doc_menu(self, active=None):
         """Build a menu of document types that are appropriate for
         this text report.  This menu will be generated based upon
         whether the document requires table support, etc."""
-        self.format_menu = BookFormatComboBox()
-        self.format_menu.set(self.doc_type_changed, None, active)
+        self.format_menu = _BookFormatComboBox( active )
 
     def make_document(self):
         """Create a document of the type requested by the user."""
         pstyle = self.paper_frame.get_paper_style()
-        self.doc = self.format(self.selected_style, pstyle, self.template_name)
+        self.doc = self.format(self.selected_style, pstyle)
 
         self.rptlist = []
         for item in self.book.get_item_list():
@@ -1133,9 +1259,6 @@ class BookReportDialog(DocReportDialog):
                                   report_class, item.option_class)
             self.rptlist.append(obj)
         self.doc.open(self.target_path)
-        
-        if self.print_report.get_active():
-            self.doc.open_requested ()
 
     def make_report(self):
         """The actual book report. Start it out, then go through the item list 
@@ -1147,9 +1270,13 @@ class BookReportDialog(DocReportDialog):
             if newpage:
                 self.doc.page_break()
             newpage = 1
-            item.begin_report()
-            item.write_report()
+            if item:
+                item.begin_report()
+                item.write_report()
         self.doc.close()
+        
+        if self.open_with_app.get_active():
+            open_file_with_default_application(self.target_path)
 
 #------------------------------------------------------------------------
 #
@@ -1165,24 +1292,28 @@ def cl_report(database, name, category, options_str_dict):
     if clr.show:
         return
     
-    if 'bookname' not in clr.options_dict:
-        print "Please Specify a book name"
+    if 'bookname' not in clr.options_dict or not clr.options_dict['bookname']:
+        print _("Please specify a book name")
         return
 
     book_list = BookList('books.xml', database)
     book_name = clr.options_dict['bookname']
-    book = book_list.get_book(book_name)
-    selected_style = BaseDoc.StyleSheet()
+    if book_name:
+        book = book_list.get_book(book_name)
+    else:
+        print _("Please specify a book name")
+        return
+    selected_style = StyleSheet()
 
     for item in book.get_item_list():
         # Set up default style
-        default_style = BaseDoc.StyleSheet()
+        default_style = StyleSheet()
         make_default_style = item.option_class.make_default_style
         make_default_style(default_style)
 
         # Read all style sheets available for this item
         style_file = item.option_class.handler.get_stylesheet_savefile()
-        style_list = BaseDoc.StyleSheetList(style_file, default_style)
+        style_list = StyleSheetList(style_file, default_style)
 
         # Get the selected stylesheet
         style_name = item.option_class.handler.get_default_stylesheet_name()
@@ -1233,22 +1364,3 @@ def write_book_item(database, report_class, options_class):
     except:
         log.error("Failed to write book item.", exc_info=True)
     return None
-
-#------------------------------------------------------------------------
-#
-# 
-#
-#------------------------------------------------------------------------
-pmgr = PluginManager.get_instance()
-pmgr.register_report(
-    name = 'book',
-    category = CATEGORY_BOOK,
-    report_class = BookReportSelector,
-    options_class = cl_report,
-    modes = PluginManager.REPORT_MODE_GUI | PluginManager.REPORT_MODE_CLI,
-    translated_name = _("Book Report"),
-    status = _("Stable"),
-    description = _("Produces a book containing several reports."),
-    author_name = "Alex Roitman",
-    author_email = "shura@gramps-project.org"
-    )
