@@ -19,7 +19,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: PdfDoc.py 15939 2010-09-30 10:02:41Z ldnp $
+# $Id: PdfDoc.py 16979 2011-03-30 01:50:08Z pez4brian $
 
 """PDF output generator based on Cairo.
 """
@@ -37,7 +37,6 @@ import sys
 # Gramps modules
 #
 #------------------------------------------------------------------------
-from gui.utils import open_file_with_default_application
 import libcairodoc
 
 #------------------------------------------------------------------------
@@ -124,6 +123,3 @@ class PdfDoc(libcairodoc.CairoDoc):
         # if we don't restore the resolution.
         fontmap.set_resolution(saved_resolution)
 
-        # load the result into an external viewer
-        if self.open_req:
-            open_file_with_default_application(self._backend.filename) 
