@@ -2,6 +2,7 @@
 #
 # Copyright (C) 2000-2007  Donald N. Allingham
 # Copyright (C) 2008-2009  Brian G. Matherly
+# Copyright (C) 2010       Jakim Friant
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,14 +17,17 @@
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-
+#
+# $Id: Calendar.py 16707 2011-02-23 23:39:11Z dsblank $
+#
 #------------------------------------------------------------------------
 #
 # python modules
 #
 #------------------------------------------------------------------------
-from gettext import gettext as _
-from gettext import ngettext
+from gen.ggettext import gettext as _
+from gen.ggettext import ngettext
+from functools import partial
 import datetime
 import time
 
@@ -32,16 +36,23 @@ import time
 # GRAMPS modules
 #
 #------------------------------------------------------------------------
-import BaseDoc
-from BasicUtils import name_displayer
-from gen.plug import PluginManager
-from ReportBase import Report, ReportUtils, MenuReportOptions, CATEGORY_DRAW
-from gen.plug.menu import BooleanOption, StringOption, NumberOption, \
-                         EnumeratedListOption, FilterOption, PersonOption
+from gen.display.name import displayer as _nd 
+from Errors import ReportError
+import Relationship
+from gen.plug.docgen import (FontStyle, ParagraphStyle, GraphicsStyle,
+                             FONT_SERIF, PARA_ALIGN_CENTER,
+                             PARA_ALIGN_LEFT, PARA_ALIGN_RIGHT)
+from gen.plug.docgen.fontscale import string_trim
+from gen.plug.menu import (BooleanOption, StringOption, NumberOption, 
+                         EnumeratedListOption, FilterOption, PersonOption)
+from gui.utils import ProgressMeter
+from gen.plug.report import Report
+from gen.plug.report import utils as ReportUtils
+from gui.plug.report import MenuReportOptions
+from Utils import probably_alive
+from DateHandler import displayer as _dd
 import GrampsLocale
 import gen.lib
-from Utils import probably_alive, ProgressMeter
-from FontScale import string_trim
 
 import libholiday
 from libholiday import g2iso
@@ -66,22 +77,25 @@ class Calendar(Report):
     def __init__(self, database, options_class):
         Report.__init__(self, database, options_class)
         menu = options_class.menu
-
-        self.year = menu.get_option_by_name('year').get_value()
-        self.name_format = menu.get_option_by_name('name_format').get_value()
-        self.country = menu.get_option_by_name('country').get_value()
-        self.anniversaries = menu.get_option_by_name('anniversaries').get_value()
-        self.start_dow = menu.get_option_by_name('start_dow').get_value()
-        self.maiden_name = menu.get_option_by_name('maiden_name').get_value()
-        self.alive = menu.get_option_by_name('alive').get_value()
-        self.birthdays = menu.get_option_by_name('birthdays').get_value()
-        self.text1 = menu.get_option_by_name('text1').get_value()
-        self.text2 = menu.get_option_by_name('text2').get_value()
-        self.text3 = menu.get_option_by_name('text3').get_value()
-        self.filter_option =  menu.get_option_by_name('filter')
+        get_value = lambda name: menu.get_option_by_name(name).get_value()
+        
+        self.year = get_value('year')
+        self.name_format = get_value('name_format')
+        self.country = get_value('country')
+        self.anniversaries = get_value('anniversaries')
+        self.start_dow = get_value('start_dow')
+        self.maiden_name = get_value('maiden_name')
+        self.alive = get_value('alive')
+        self.birthdays = get_value('birthdays')
+        self.text1 = get_value('text1')
+        self.text2 = get_value('text2')
+        self.text3 = get_value('text3')
+        self.filter_option = menu.get_option_by_name('filter')
         self.filter = self.filter_option.get_filter()
-        pid = menu.get_option_by_name('pid').get_value()
+        pid = get_value('pid')
         self.center_person = database.get_person_from_gramps_id(pid)
+        if (self.center_person == None) :
+            raise ReportError(_("Person %s is not in the Database") % pid )
 
     def get_name(self, person, maiden_name = None):
         """ Return person's name, unless maiden_name given, 
@@ -101,11 +115,13 @@ class Calendar(Report):
                 name = gen.lib.Name(married_name)
             else:
                 name = gen.lib.Name(primary_name)
-                name.set_surname(maiden_name)
+                surname = gen.lib.Surname()
+                surname.set_surname(maiden_name)
+                name.set_surname_list([surname])
         else:
             name = gen.lib.Name(primary_name)
         name.set_display_as(self.name_format)
-        return name_displayer.display_name(name)
+        return _nd.display_name(name)
         
     def draw_rectangle(self, style, sx, sy, ex, ey):
         """ This should be in BaseDoc """
@@ -116,11 +132,11 @@ class Calendar(Report):
 
 ### The rest of these all have to deal with calendar specific things
 
-    def add_day_item(self, text, month, day):
+    def add_day_item(self, text, month, day, format="CAL-Text"):
         """ Add an item to a day. """
         month_dict = self.calendar.get(month, {})
         day_list = month_dict.get(day, [])
-        day_list.append(text)
+        day_list.append((format, text))
         month_dict[day] = day_list
         self.calendar[month] = month_dict
 
@@ -133,7 +149,7 @@ class Calendar(Report):
             for day in range(1, 32):
                 holiday_names = holiday_table.get_holidays(month, day) 
                 for holiday_name in holiday_names:
-                    self.add_day_item(holiday_name, month, day)
+                    self.add_day_item(holiday_name, month, day, "CAL-Holiday")
 
     def write_report(self):
         """ The short method that runs through each month and creates a page. """
@@ -173,7 +189,7 @@ class Calendar(Report):
         self.doc.draw_box("CAL-Title", "", 0, 0, width, header)
         self.doc.draw_line("CAL-Border", 0, header, width, header)
         year = self.year
-        title = "%s %d" % (GrampsLocale.long_months[month].capitalize(), year)
+        title = "%s %d" % (_dd.long_months[month].capitalize(), year)
         font_height = pt2cm(ptitle.get_font().get_size())
         self.doc.center_text("CAL-Title", title, width/2, font_height * 0.25)
         cell_width = width / 7
@@ -209,9 +225,10 @@ class Calendar(Report):
                     self.doc.center_text("CAL-Numbers", str(thisday.day), 
                                          day_col * cell_width + cell_width/2, 
                                          header + week_row * cell_height)
-                    list = self.calendar.get(month, {}).get(thisday.day, [])
+                    list_ = self.calendar.get(month, {}).get(thisday.day, [])
+                    list_.sort() # to get CAL-Holiday on bottom
                     position = 0.0 
-                    for p in list:
+                    for (format, p) in list_:
                         lines = p.count("\n") + 1 # lines in the text
                         position += (lines  * spacing)
                         current = 0
@@ -222,7 +239,7 @@ class Calendar(Report):
                                 continue
                             font = ptext.get_font()
                             line = string_trim(font, line, cm2pt(cell_width + 0.2))
-                            self.doc.draw_text("CAL-Text", line, 
+                            self.doc.draw_text(format, line, 
                                               day_col * cell_width + 0.1, 
                                               header + (week_row + 1) * cell_height - position + (current * spacing) - 0.1)
                             current += 1
@@ -230,9 +247,11 @@ class Calendar(Report):
         if not something_this_week:
             last_edge = 0
         font_height = pt2cm(1.5 * ptext1style.get_font().get_size())
-        self.doc.center_text("CAL-Text1style", self.text1, last_edge + (width - last_edge)/2, height - font_height * 3) 
-        self.doc.center_text("CAL-Text2style", self.text2, last_edge + (width - last_edge)/2, height - font_height * 2) 
-        self.doc.center_text("CAL-Text3style", self.text3, last_edge + (width - last_edge)/2, height - font_height * 1) 
+        x = last_edge + (width - last_edge)/2
+        y = height - font_height
+        self.doc.center_text("CAL-Text1style", self.text1, x, y * 3) 
+        self.doc.center_text("CAL-Text2style", self.text2, x, y * 2) 
+        self.doc.center_text("CAL-Text3style", self.text3, x, y * 1) 
         self.doc.end_page()
 
     def collect_data(self):
@@ -240,20 +259,20 @@ class Calendar(Report):
         This method runs through the data, and collects the relevant dates
         and text.
         """
-        people = self.database.get_person_handles(sort_handles=False)
-        self.progress.set_pass(_('Applying Filter...'), len(people))
+        db = self.database
+        people = db.iter_person_handles()
+        self.progress.set_pass(_('Applying Filter...'), db.get_number_of_people())
         people = self.filter.apply(self.database, people, self.progress)
-        pmgr = PluginManager.get_instance()
-        rel_calc = pmgr.get_relationship_calculator()
+        rel_calc = Relationship.get_relationship_calculator()
 
         self.progress.set_pass(_('Reading database...'), len(people))
         for person_handle in people:
             self.progress.step()
-            person = self.database.get_person_from_handle(person_handle)
+            person = db.get_person_from_handle(person_handle)
             birth_ref = person.get_birth_ref()
             birth_date = None
             if birth_ref:
-                birth_event = self.database.get_event_from_handle(birth_ref.ref)
+                birth_event = db.get_event_from_handle(birth_ref.ref)
                 birth_date = birth_event.get_date_object()
 
             if (self.birthdays and birth_date is not None and birth_date.is_valid()):
@@ -269,23 +288,23 @@ class Calendar(Report):
                 if self.maiden_name in ['spouse_first', 'spouse_last']: # get husband's last name:
                     if person.get_gender() == gen.lib.Person.FEMALE:
                         family_list = person.get_family_handle_list()
-                        if len(family_list) > 0:
+                        if family_list:
                             if self.maiden_name == 'spouse_first':
                                 fhandle = family_list[0]
                             else:
                                 fhandle = family_list[-1]
-                            fam = self.database.get_family_from_handle(fhandle)
+                            fam = db.get_family_from_handle(fhandle)
                             father_handle = fam.get_father_handle()
                             mother_handle = fam.get_mother_handle()
                             if mother_handle == person_handle:
                                 if father_handle:
-                                    father = self.database.get_person_from_handle(father_handle)
-                                    if father is not None:
+                                    father = db.get_person_from_handle(father_handle)
+                                    if father:
                                         father_lastname = father.get_primary_name().get_surname()
                 short_name = self.get_name(person, father_lastname)
-                alive = probably_alive(person, self.database, prob_alive_date)
+                alive = probably_alive(person, db, prob_alive_date)
 
-                if (self.alive and alive) or not self.alive:
+                if  not self.alive or alive:
                     if nyears == 0:
                         text = _('%(person)s, birth%(relation)s') % {
                             'person' : short_name,
@@ -300,7 +319,7 @@ class Calendar(Report):
             if self.anniversaries:
                 family_list = person.get_family_handle_list()
                 for fhandle in family_list: 
-                    fam = self.database.get_family_from_handle(fhandle)
+                    fam = db.get_family_from_handle(fhandle)
                     father_handle = fam.get_father_handle()
                     mother_handle = fam.get_mother_handle()
                     if father_handle == person.get_handle():
@@ -309,7 +328,7 @@ class Calendar(Report):
                         continue # with next person if the father is not "person"
                                  # this will keep from duplicating the anniversary
                     if spouse_handle:
-                        spouse = self.database.get_person_from_handle(spouse_handle)
+                        spouse = db.get_person_from_handle(spouse_handle)
                         if spouse:
                             spouse_name = self.get_name(spouse)
                             short_name = self.get_name(person)
@@ -317,17 +336,23 @@ class Calendar(Report):
                             # GRAMPS 3.0 will have a new mechanism for start/stop events
                             are_married = None
                             for event_ref in fam.get_event_ref_list():
-                                event = self.database.get_event_from_handle(event_ref.ref)
-                                if event.type in [gen.lib.EventType.MARRIAGE, 
-                                                             gen.lib.EventType.MARR_ALT]:
+                                event = db.get_event_from_handle(event_ref.ref)
+                                et = gen.lib.EventType
+                                rt = gen.lib.EventRoleType
+                                if event.type in [et.MARRIAGE, 
+                                                  et.MARR_ALT] and \
+                                (event_ref.get_role() == rt.FAMILY or 
+                                event_ref.get_role() == rt.PRIMARY ):
                                     are_married = event
-                                elif event.type in [gen.lib.EventType.DIVORCE, 
-                                                               gen.lib.EventType.ANNULMENT, 
-                                                               gen.lib.EventType.DIV_FILING]:
+                                elif event.type in [et.DIVORCE, 
+                                                    et.ANNULMENT, 
+                                                    et.DIV_FILING] and \
+                                (event_ref.get_role() == rt.FAMILY or 
+                                event_ref.get_role() == rt.PRIMARY ):
                                     are_married = None
                             if are_married is not None:
                                 for event_ref in fam.get_event_ref_list():
-                                    event = self.database.get_event_from_handle(event_ref.ref)
+                                    event = db.get_event_from_handle(event_ref.ref)
                                     event_obj = event.get_date_object()
 
                                     if event_obj.is_valid():
@@ -350,10 +375,10 @@ class Calendar(Report):
                                                        'person' : short_name, 
                                                        'nyears' : nyears})
 
-                                        alive1 = probably_alive(person, self.database, \
-                                            prob_alive_date)
-                                        alive2 = probably_alive(spouse, self.database, \
-                                            prob_alive_date)
+                                        alive1 = probably_alive(person, self.database,
+                                                    prob_alive_date)
+                                        alive2 = probably_alive(spouse, self.database,
+                                                    prob_alive_date)
                                         if ((self.alive and alive1 and alive2) or not self.alive):
                                             self.add_day_item(text, month, day)
 
@@ -372,82 +397,87 @@ class CalendarOptions(MenuReportOptions):
     
     def add_menu_options(self, menu):
         """ Add the options for the graphical calendar """
-        category_name = _("Report Options")
-
+        add_option = partial(menu.add_option, _("Report Options"))
         year = NumberOption(_("Year of calendar"), time.localtime()[0], 
                             1000, 3000)
         year.set_help(_("Year of calendar"))
-        menu.add_option(category_name, "year", year)
+        add_option("year", year)
 
         self.__filter = FilterOption(_("Filter"), 0)
         self.__filter.set_help(
                _("Select filter to restrict people that appear on calendar"))
-        menu.add_option(category_name, "filter", self.__filter)
+        add_option("filter", self.__filter)
         
         self.__pid = PersonOption(_("Center Person"))
         self.__pid.set_help(_("The center person for the report"))
-        menu.add_option(category_name, "pid", self.__pid)
+        add_option("pid", self.__pid)
         self.__pid.connect('value-changed', self.__update_filters)
         
         self.__update_filters()
 
         # We must figure out the value of the first option before we can
         # create the EnumeratedListOption
-        fmt_list = name_displayer.get_name_format()
+        fmt_list = _nd.get_name_format()
         name_format = EnumeratedListOption(_("Name format"), fmt_list[0][0])
         for num, name, fmt_str, act in fmt_list:
             name_format.add_item(num, name)
         name_format.set_help(_("Select the format to display names"))
-        menu.add_option(category_name, "name_format", name_format)
+        add_option("name_format", name_format)
 
         country = EnumeratedListOption(_("Country for holidays"), 0)
         holiday_table = libholiday.HolidayTable()
+        countries = holiday_table.get_countries()
+        countries.sort()
+        if (len(countries) == 0 or 
+            (len(countries) > 0 and countries[0] != '')):
+            countries.insert(0, '')
         count = 0
-        for c in  holiday_table.get_countries():
+        for c in countries:
             country.add_item(count, c)
             count += 1
         country.set_help(_("Select the country to see associated holidays"))
-        menu.add_option(category_name, "country", country)
+        add_option("country", country)
 
         start_dow = EnumeratedListOption(_("First day of week"), 1)
         for count in range(1, 8):
             # conversion between gramps numbering (sun=1) and iso numbering (mon=1) of weekdays below
             start_dow.add_item((count+5) % 7 + 1, GrampsLocale.long_days[count].capitalize()) 
         start_dow.set_help(_("Select the first day of the week for the calendar"))
-        menu.add_option(category_name, "start_dow", start_dow) 
+        add_option("start_dow", start_dow) 
 
         maiden_name = EnumeratedListOption(_("Birthday surname"), "own")
         maiden_name.add_item("spouse_first", _("Wives use husband's surname (from first family listed)"))
         maiden_name.add_item("spouse_last", _("Wives use husband's surname (from last family listed)"))
         maiden_name.add_item("own", _("Wives use their own surname"))
         maiden_name.set_help(_("Select married women's displayed surname"))
-        menu.add_option(category_name, "maiden_name", maiden_name)
+        add_option("maiden_name", maiden_name)
 
         alive = BooleanOption(_("Include only living people"), True)
         alive.set_help(_("Include only living people in the calendar"))
-        menu.add_option(category_name, "alive", alive)
+        add_option("alive", alive)
 
         birthdays = BooleanOption(_("Include birthdays"), True)
         birthdays.set_help(_("Include birthdays in the calendar"))
-        menu.add_option(category_name, "birthdays", birthdays)
+        add_option("birthdays", birthdays)
 
         anniversaries = BooleanOption(_("Include anniversaries"), True)
         anniversaries.set_help(_("Include anniversaries in the calendar"))
-        menu.add_option(category_name, "anniversaries", anniversaries)
+        add_option("anniversaries", anniversaries)
 
         category_name = _("Text Options")
+        add_option = partial(menu.add_option, _("Text Options"))
 
         text1 = StringOption(_("Text Area 1"), _("My Calendar")) 
         text1.set_help(_("First line of text at bottom of calendar"))
-        menu.add_option(category_name, "text1", text1)
+        add_option("text1", text1)
 
-        text2 = StringOption(_("Text Area 2"), _("Produced with GRAMPS"))
+        text2 = StringOption(_("Text Area 2"), _("Produced with Gramps"))
         text2.set_help(_("Second line of text at bottom of calendar"))
-        menu.add_option(category_name, "text2", text2)
+        add_option("text2", text2)
 
         text3 = StringOption(_("Text Area 3"), "http://gramps-project.org/",)
         text3.set_help(_("Third line of text at bottom of calendar"))
-        menu.add_option(category_name, "text3", text3)
+        add_option("text3", text3)
         
     def __update_filters(self):
         """
@@ -459,17 +489,17 @@ class CalendarOptions(MenuReportOptions):
         self.__filter.set_filters(filter_list)
 
     def make_my_style(self, default_style, name, description, 
-                      size=9, font=BaseDoc.FONT_SERIF, justified ="left", 
-                      color=None, align=BaseDoc.PARA_ALIGN_CENTER, 
+                      size=9, font=FONT_SERIF, justified ="left", 
+                      color=None, align=PARA_ALIGN_CENTER, 
                       shadow = None, italic=0, bold=0, borders=0, indent=None):
         """ Create paragraph and graphic styles of the same name """
         # Paragraph:
-        f = BaseDoc.FontStyle()
+        f = FontStyle()
         f.set_size(size)
         f.set_type_face(font)
         f.set_italic(italic)
         f.set_bold(bold)
-        p = BaseDoc.ParagraphStyle()
+        p = ParagraphStyle()
         p.set_font(f)
         p.set_alignment(align)
         p.set_description(description)
@@ -480,14 +510,14 @@ class CalendarOptions(MenuReportOptions):
         if indent:
             p.set(first_indent=indent)
         if justified == "left":
-            p.set_alignment(BaseDoc.PARA_ALIGN_LEFT)       
+            p.set_alignment(PARA_ALIGN_LEFT)       
         elif justified == "right":
-            p.set_alignment(BaseDoc.PARA_ALIGN_RIGHT)       
+            p.set_alignment(PARA_ALIGN_RIGHT)       
         elif justified == "center":
-            p.set_alignment(BaseDoc.PARA_ALIGN_CENTER)       
+            p.set_alignment(PARA_ALIGN_CENTER)       
         default_style.add_paragraph_style(name, p)
         # Graphics:
-        g = BaseDoc.GraphicsStyle()
+        g = GraphicsStyle()
         g.set_paragraph_style(name)
         if shadow:
             g.set_shadow(*shadow)
@@ -508,6 +538,9 @@ class CalendarOptions(MenuReportOptions):
                            bold=1)
         self.make_my_style(default_style, "CAL-Text", 
                            _('Daily text display'), 9)
+        self.make_my_style(default_style, "CAL-Holiday", 
+                           _('Holiday text display'), 9,
+                           bold=1, italic=1)
         self.make_my_style(default_style, "CAL-Daynames", 
                            _('Days of the week text'), 12, 
                            italic=1, bold=1, 
@@ -520,25 +553,3 @@ class CalendarOptions(MenuReportOptions):
                            _('Text at bottom, line 3'), 9)
         self.make_my_style(default_style, "CAL-Border", 
                            _('Borders'), borders=True)
-
-#------------------------------------------------------------------------
-#
-# Register the plugins
-#
-#------------------------------------------------------------------------
-pmgr = PluginManager.get_instance()
-pmgr.register_report(
-    name = 'calendar', 
-    category = CATEGORY_DRAW, 
-    report_class = Calendar, 
-    options_class = CalendarOptions, 
-    modes = PluginManager.REPORT_MODE_GUI | \
-            PluginManager.REPORT_MODE_BKI | \
-            PluginManager.REPORT_MODE_CLI, 
-    translated_name = _("Calendar"), 
-    status = _("Stable"), 
-    author_name = "Douglas S. Blank", 
-    author_email = "dblank@cs.brynmawr.edu", 
-    description = _("Produces a graphical calendar"), 
-    )
-
