@@ -3,6 +3,7 @@
 #
 # Copyright (C) 2003-2006 Donald N. Allingham
 # Copyright (C) 2007-2008 Brian G. Matherly
+# Copyright (C) 2010       Jakim Friant
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,25 +20,35 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: FanChart.py 12362 2009-03-19 04:25:37Z pez4brian $
+# $Id: FanChart.py 16979 2011-03-30 01:50:08Z pez4brian $
 
 #------------------------------------------------------------------------
 #
 # python modules
 #
 #------------------------------------------------------------------------
-from gettext import gettext as _
+from gen.ggettext import gettext as _
+from math import pi, cos, sin, log10
+
+def log2(val):
+	"""
+	Calculate the log base 2 of a value.
+	"""
+	return int(log10(val)/log10(2))
 
 #------------------------------------------------------------------------
 #
 # gramps modules
 #
 #------------------------------------------------------------------------
-import BaseDoc
-from gen.plug import PluginManager
+from Errors import ReportError
+from gen.plug.docgen import (FontStyle, ParagraphStyle, GraphicsStyle,
+                             FONT_SANS_SERIF, PARA_ALIGN_CENTER)
 from gen.plug.menu import EnumeratedListOption, NumberOption, PersonOption
-from ReportBase import Report, ReportUtils, MenuReportOptions, CATEGORY_DRAW
-from SubstKeywords import SubstKeywords
+from gen.plug.report import Report
+from gen.plug.report import utils as ReportUtils
+from gui.plug.report import MenuReportOptions
+import config
 
 #------------------------------------------------------------------------
 #
@@ -56,6 +67,67 @@ RADIAL_ROUNDABOUT = 1
 
 pt2cm = ReportUtils.pt2cm
 
+cal = config.get('preferences.calendar-format-report')
+
+#------------------------------------------------------------------------
+#
+# private functions
+#
+#------------------------------------------------------------------------
+def draw_wedge(doc,  style,  centerx,  centery,  radius,  start_angle, 
+               end_angle,  short_radius=0):
+    """
+    Draw a wedge shape.
+    """
+    while end_angle < start_angle:
+        end_angle += 360
+
+    p = []
+    
+    degreestoradians = pi / 180.0
+    radiansdelta = degreestoradians / 2
+    sangle = start_angle * degreestoradians
+    eangle = end_angle * degreestoradians
+    while eangle < sangle:
+        eangle = eangle + 2 * pi
+    angle = sangle
+
+    if short_radius == 0:
+        if (end_angle - start_angle) != 360:
+            p.append((centerx, centery))
+    else:
+        origx = (centerx + cos(angle) * short_radius)
+        origy = (centery + sin(angle) * short_radius)
+        p.append((origx, origy))
+        
+    while angle < eangle:
+        x = centerx + cos(angle) * radius
+        y = centery + sin(angle) * radius
+        p.append((x, y))
+        angle = angle + radiansdelta
+    x = centerx + cos(eangle) * radius
+    y = centery + sin(eangle) * radius
+    p.append((x, y))
+
+    if short_radius:
+        x = centerx + cos(eangle) * short_radius
+        y = centery + sin(eangle) * short_radius
+        p.append((x, y))
+
+        angle = eangle
+        while angle >= sangle:
+            x = centerx + cos(angle) * short_radius
+            y = centery + sin(angle) * short_radius
+            p.append((x, y))
+            angle -= radiansdelta
+    doc.draw_path(style, p)
+
+    delta = (eangle - sangle) / 2.0
+    rad = short_radius + (radius - short_radius) / 2.0
+
+    return ( (centerx + cos(sangle + delta) * rad), 
+             (centery + sin(sangle + delta) * rad))
+
 #------------------------------------------------------------------------
 #
 # FanChart
@@ -89,6 +161,8 @@ class FanChart(Report):
         self.radial          = menu.get_option_by_name('radial').get_value()
         pid                  = menu.get_option_by_name('pid').get_value()
         self.center_person = database.get_person_from_gramps_id(pid)
+        if (self.center_person == None) :
+            raise ReportError(_("Person %s is not in the Database") % pid )
 
         self.background_style = []
         self.text_style = []
@@ -100,15 +174,14 @@ class FanChart(Report):
             self.background_style.append(background_style_name)
             text_style_name = 'text_style' + '%d' % i
             self.text_style.append(text_style_name)
+        
+        self.calendar = 0
 
         Report.__init__(self, database, options_class)
 
         self.height = 0
-        self.lines = 0
-        self.display = "%n"
         self.map = [None] * 2**self.max_generations
         self.text = {}
-        self.box_width = 0
 
     def apply_filter(self,person_handle,index):
         """traverse the ancestors recursively until either the end
@@ -118,20 +191,7 @@ class FanChart(Report):
         if (not person_handle) or (index >= 2**self.max_generations):
             return
         self.map[index-1] = person_handle
-
-        self.text[index-1] = []
-
-        subst = SubstKeywords(self.database,person_handle)
-        
-        for line in self.display:
-            self.text[index-1].append(subst.replace(line))
-
-        style_sheet = self.doc.get_style_sheet()
-        self.font = style_sheet.get_paragraph_style('text_style').get_font()
-        for line in self.text[index-1]:
-            self.box_width = max(self.box_width,self.doc.string_width(self.font,line))
-
-        self.lines = max(self.lines,len(self.text[index-1]))    
+        self.text[index-1] = self.get_info(person_handle, log2(index))
 
         person = self.database.get_person_from_handle(person_handle)
         family_handle = person.get_main_parents_family_handle()
@@ -179,7 +239,7 @@ class FanChart(Report):
                 style_sheet = self.doc.get_style_sheet()
                 fontsize = pt2cm(style_sheet.get_paragraph_style('FC-Title').get_font().get_size())
                 # y is vertical distance to center of circle, move center down 1 fontsize
-                y = y + fontsize
+                y += fontsize
                 # min_XY is the diamter of the circle, subtract two fontsize
                 # so we dont draw outside bottom of the paper
                 min_xy = min(min_xy,y-2*fontsize)
@@ -202,11 +262,12 @@ class FanChart(Report):
     def get_info(self,person_handle,generation):
         person = self.database.get_person_from_handle(person_handle)
         pn = person.get_primary_name()
+        self.calendar = config.get('preferences.calendar-format-report')
 
         birth_ref = person.get_birth_ref()
         if birth_ref:
             birth = self.database.get_event_from_handle(birth_ref.ref)
-            b = birth.get_date_object().get_year()
+            b = birth.get_date_object().to_calendar(self.calendar).get_year()
             if b == 0:
                 b = ""
         else:
@@ -215,12 +276,11 @@ class FanChart(Report):
         death_ref = person.get_death_ref()
         if death_ref:
             death = self.database.get_event_from_handle(death_ref.ref)
-            d = death.get_date_object().get_year()
+            d = death.get_date_object().to_calendar(self.calendar).get_year()
             if d == 0:
                 d = ""
         else:
             d = ""
-
         if b and d:
             val = "%s - %s" % (str(b),str(d))
         elif b:
@@ -274,14 +334,12 @@ class FanChart(Report):
         for index in range(segments - 1, 2*segments - 1):
             start_angle = end_angle
             end_angle = start_angle + delta
-            (xc,yc) = ReportUtils.draw_wedge(self.doc,background_style, x, y, rad2,
-                                          start_angle, end_angle, rad1)
+            (xc,yc) = draw_wedge(self.doc, background_style, x, y, rad2,
+                                 start_angle, end_angle, rad1)
             if self.map[index]:
                 if (generation == 0) and self.circle == FULL_CIRCLE:
                     yc = y
-                self.doc.rotate_text(text_style,
-                                     self.get_info(self.map[index],
-                                                   generation),
+                self.doc.rotate_text(text_style, self.text[index],
                                      xc, yc, text_angle)
             text_angle += delta
 
@@ -306,19 +364,15 @@ class FanChart(Report):
         for index in range(segments - 1, 2*segments - 1):
             start_angle = end_angle
             end_angle = start_angle + delta
-            (xc,yc) = ReportUtils.draw_wedge(self.doc,background_style, x, y, rad2,
-                                          start_angle, end_angle, rad1)
+            (xc,yc) = draw_wedge(self.doc,background_style, x, y, rad2,
+                                 start_angle, end_angle, rad1)
             text_angle += delta
             if self.map[index]:
                 if self.radial == RADIAL_UPRIGHT and (start_angle >= 90) and (start_angle < 270):
-                    self.doc.rotate_text(text_style,
-                                         self.get_info(self.map[index],
-                                                       generation),
+                    self.doc.rotate_text(text_style, self.text[index],
                                          xc, yc, text_angle + 180)
                 else:
-                    self.doc.rotate_text(text_style,
-                                         self.get_info(self.map[index],
-                                                       generation),
+                    self.doc.rotate_text(text_style, self.text[index],
                                          xc, yc, text_angle)
 
 #------------------------------------------------------------------------
@@ -383,67 +437,46 @@ class FanChartOptions(MenuReportOptions):
                             ]
 
         #Paragraph Styles
-        f = BaseDoc.FontStyle()
+        f = FontStyle()
         f.set_size(20)
         f.set_bold(1)
-        f.set_type_face(BaseDoc.FONT_SANS_SERIF)
-        p = BaseDoc.ParagraphStyle()
+        f.set_type_face(FONT_SANS_SERIF)
+        p = ParagraphStyle()
         p.set_font(f)
-        p.set_alignment(BaseDoc.PARA_ALIGN_CENTER)
+        p.set_alignment(PARA_ALIGN_CENTER)
         p.set_description(_('The style used for the title.'))
         default_style.add_paragraph_style("FC-Title",p)
 
-        f = BaseDoc.FontStyle()
+        f = FontStyle()
         f.set_size(9)
-        f.set_type_face(BaseDoc.FONT_SANS_SERIF)
-        p = BaseDoc.ParagraphStyle()
+        f.set_type_face(FONT_SANS_SERIF)
+        p = ParagraphStyle()
         p.set_font(f)
-        p.set_alignment(BaseDoc.PARA_ALIGN_CENTER)
+        p.set_alignment(PARA_ALIGN_CENTER)
         p.set_description(_('The basic style used for the text display.'))
         default_style.add_paragraph_style("text_style", p)
             
         # GraphicsStyles
-        g = BaseDoc.GraphicsStyle()
+        g = GraphicsStyle()
         g.set_paragraph_style('FC-Title')
         g.set_line_width(0)
         default_style.add_draw_style("t",g)
 
         for i in range (0, self.MAX_GENERATIONS):
-            g = BaseDoc.GraphicsStyle()
+            g = GraphicsStyle()
             g.set_fill_color(BACKGROUND_COLORS[i])
             g.set_paragraph_style('FC-Normal')
             background_style_name = 'background_style' + '%d' % i
             default_style.add_draw_style(background_style_name,g)
 
-            g = BaseDoc.GraphicsStyle()
+            g = GraphicsStyle()
             g.set_fill_color(BACKGROUND_COLORS[i])
             g.set_paragraph_style('text_style')
             g.set_line_width(0)
             text_style_name = 'text_style' + '%d' % i
             default_style.add_draw_style(text_style_name,g)
             
-        g = BaseDoc.GraphicsStyle()
+        g = GraphicsStyle()
         g.set_fill_color((255,255,255))
         g.set_paragraph_style('FC-Normal')
         default_style.add_draw_style('background_style_white',g)
-
-#------------------------------------------------------------------------
-#
-#
-#
-#------------------------------------------------------------------------
-pmgr = PluginManager.get_instance()
-pmgr.register_report(
-    name = 'fan_chart',
-    category = CATEGORY_DRAW,
-    report_class = FanChart,
-    options_class = FanChartOptions,
-    modes = PluginManager.REPORT_MODE_GUI | \
-            PluginManager.REPORT_MODE_BKI | \
-            PluginManager.REPORT_MODE_CLI,
-    translated_name = _("Fan Chart"),
-    status = _("Stable"),
-    author_name = "Donald N. Allingham",
-    author_email = "don@gramps-project.org",
-    description = _("Produces fan charts")
-    )
