@@ -3,6 +3,7 @@
 #
 # Copyright (C) 2003-2007 Donald N. Allingham
 # Copyright (C) 2007-2008 Brian G. Matherly
+# Copyright (C) 2010       Jakim Friant
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,7 +20,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: TimeLine.py 14091 2010-01-18 04:42:17Z pez4brian $
+# $Id: TimeLine.py 16979 2011-03-30 01:50:08Z pez4brian $
 
 """
 Timeline Chart
@@ -38,7 +39,9 @@ from gen.ggettext import sgettext as _
 #
 #------------------------------------------------------------------------
 from gen.plug.menu import PersonOption, FilterOption, EnumeratedListOption
-from ReportBase import Report, ReportUtils, MenuReportOptions
+from gen.plug.report import Report
+from gen.plug.report import utils as ReportUtils
+from gui.plug.report import MenuReportOptions
 pt2cm = ReportUtils.pt2cm
 from gen.plug.docgen import (FontStyle, ParagraphStyle, GraphicsStyle,
                     FONT_SANS_SERIF, DASHED, PARA_ALIGN_CENTER)
