@@ -4,6 +4,7 @@
 # Copyright (C) 2007-2008 Douglas S. Blank
 # Copyright (C) 2004-2007 Donald N. Allingham
 # Copyright (C) 2008      Brian G. Matherly
+# Copyright (C) 2010       Jakim Friant
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,7 +20,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
-# $Id: ExportCsv.py 12560 2009-05-21 17:58:31Z gbritton $
+# $Id: ExportCsv.py 16979 2011-03-30 01:50:08Z pez4brian $
 
 "Export to CSV Spreadsheet."
 
@@ -29,25 +30,18 @@
 #
 #-------------------------------------------------------------------------
 import os
-from TransUtils import sgettext as _
+from gen.ggettext import sgettext as _
 import csv
 import cStringIO
 import codecs
 
-#-------------------------------------------------------------------------
-#
-# GNOME/GTK modules
-#
-#-------------------------------------------------------------------------
-from gtk import glade
-
 #------------------------------------------------------------------------
 #
 # Set up logging
 #
 #------------------------------------------------------------------------
 import logging
-log = logging.getLogger(".ExportCSV")
+LOG = logging.getLogger(".ExportCSV")
 
 #-------------------------------------------------------------------------
 #
@@ -55,20 +49,21 @@ log = logging.getLogger(".ExportCSV")
 #
 #-------------------------------------------------------------------------
 import gen.lib
-from Filters import GenericFilter, Rules, build_filter_menu
+from Filters import GenericFilter, Rules, build_filter_model
+from gen.lib.eventroletype import EventRoleType
+from ExportOptions import WriterOptionBox
 import Utils
-from QuestionDialog import ErrorDialog
-from gen.plug import PluginManager, ExportPlugin
 import gen.proxy
 import DateHandler
+from glade import Glade
 
 #-------------------------------------------------------------------------
 #
 # The function that does the exporting
 #
 #-------------------------------------------------------------------------
-def exportData(database, filename, option_box=None, callback=None):
-    gw = CSVWriter(database, filename, option_box, callback)
+def exportData(database, filename, msg_callback, option_box=None, callback=None):
+    gw = CSVWriter(database, filename, msg_callback, option_box, callback)
     return gw.export_data()
 
 #-------------------------------------------------------------------------
@@ -86,6 +81,24 @@ def sortable_string_representation(text):
             alpha += s
     return alpha + (("0" * 10) + numeric)[-10:]
 
+def get_primary_event_ref_from_type(db, person, event_name):
+    """
+    >>> get_primary_event_ref_from_type(db, Person(), "Baptism"):
+    """
+    for ref in person.event_ref_list:
+        if ref.get_role() == EventRoleType.PRIMARY:
+            event = db.get_event_from_handle(ref.ref)
+            if event and event.type.is_type(event_name):
+                return ref
+    return None
+
+def get_primary_source_title(db, obj):
+    for ref in obj.get_source_references():
+        source = db.get_source_from_handle(ref.ref)
+        if source:
+            return source.get_title()
+    return ""
+
 #-------------------------------------------------------------------------
 #
 # Encoding support for CSV, from http://docs.python.org/lib/csv-examples.html
@@ -147,8 +160,7 @@ class UnicodeWriter(object):
         self.queue.truncate(0)
 
     def writerows(self, rows):
-        for row in rows:
-            self.writerow(row)
+        map(self.writerow, rows)
 
     def close(self):
         self.stream.close()
@@ -158,69 +170,52 @@ class UnicodeWriter(object):
 # CSVWriter Options
 #
 #-------------------------------------------------------------------------
-class CSVWriterOptionBox(object):
+class CSVWriterOptionBox(WriterOptionBox):
     """
     Create a VBox with the option widgets and define methods to retrieve
     the options. 
     
     """
-    def __init__(self,person):
-        self.person = person
+    def __init__(self, person, dbstate, uistate):
+        WriterOptionBox.__init__(self, person, dbstate, uistate)
         self.include_individuals = 1
         self.include_marriages = 1
         self.include_children = 1
         self.translate_headers = 1
+        self.include_individuals_check = None
+        self.include_marriages_check = None
+        self.include_children_check = None
+        self.translate_headers_check = None
 
     def get_option_box(self):
-        glade_file = os.path.join(os.path.dirname(__file__), "ExportCsv.glade")
+        import gtk
+        option_box = WriterOptionBox.get_option_box(self)
 
-        self.topDialog = glade.XML(glade_file,"csvExport","gramps")
-        filter_obj = self.topDialog.get_widget("filter")
+        self.include_individuals_check = gtk.CheckButton(_("Include people"))
+        self.include_marriages_check = gtk.CheckButton(_("Include marriages"))
+        self.include_children_check = gtk.CheckButton(_("Include children"))
+        self.translate_headers_check = gtk.CheckButton(_("Translate headers"))
 
-        all = GenericFilter()
-        all.set_name(_("Entire Database"))
-        all.add_rule(Rules.Person.Everyone([]))
+        self.include_individuals_check.set_active(1) 
+        self.include_marriages_check.set_active(1) 
+        self.include_children_check.set_active(1) 
+        self.translate_headers_check.set_active(1) 
 
-        the_filters = [all]
+        option_box.pack_start(self.include_individuals_check, False)
+        option_box.pack_start(self.include_marriages_check, False)
+        option_box.pack_start(self.include_children_check, False)
+        option_box.pack_start(self.translate_headers_check, False)
 
-        if self.person:
-            des = GenericFilter()
-            des.set_name(_("Descendants of %s") %
-                         self.person.get_primary_name().get_name())
-            des.add_rule(Rules.Person.IsDescendantOf(
-                [self.person.get_gramps_id(),1]))
+        return option_box
 
-            ans = GenericFilter()
-            ans.set_name(_("Ancestors of %s") %
-                         self.person.get_primary_name().get_name())
-            ans.add_rule(Rules.Person.IsAncestorOf(
-                [self.person.get_gramps_id(),1]))
-
-            com = GenericFilter()
-            com.set_name(_("People with common ancestor with %s") %
-                         self.person.get_primary_name().get_name())
-            com.add_rule(Rules.Person.HasCommonAncestorWith(
-                [self.person.get_gramps_id()]))
-
-            the_filters += [des,ans,com]
-
-        from Filters import CustomFilters
-        the_filters.extend(CustomFilters.get_filters('Person'))
-        self.filter_menu = build_filter_menu(the_filters)
-        filter_obj.set_menu(self.filter_menu)
-
-        the_box = self.topDialog.get_widget('vbox1')
-        the_parent = self.topDialog.get_widget('dialog-vbox1')
-        the_parent.remove(the_box)
-        self.topDialog.get_widget("csvExport").destroy()
-        return the_box
 
     def parse_options(self):
-        self.include_individuals = self.topDialog.get_widget("individuals").get_active()
-        self.include_marriages = self.topDialog.get_widget("marriages").get_active()
-        self.include_children = self.topDialog.get_widget("children").get_active()
-        self.translate_headers = self.topDialog.get_widget("translate_headers").get_active()
-        self.cfilter = self.filter_menu.get_active().get_data("filter")
+        WriterOptionBox.parse_options(self)
+        if self.include_individuals_check:
+            self.include_individuals = self.include_individuals_check.get_active()
+            self.include_marriages = self.include_marriages_check.get_active()
+            self.include_children = self.include_children_check.get_active()
+            self.translate_headers = self.translate_headers_check.get_active()
 
 #-------------------------------------------------------------------------
 #
@@ -228,11 +223,12 @@ class CSVWriterOptionBox(object):
 #
 #-------------------------------------------------------------------------
 class CSVWriter(object):
-    def __init__(self, database, filename, option_box=None, callback=None):
+    def __init__(self, database, filename, msg_callback, option_box=None, callback=None):
         self.db = database
         self.option_box = option_box
         self.filename = filename
         self.callback = callback
+        self.msg_callback = msg_callback
         if callable(self.callback): # callback is really callable
             self.update = self.update_real
         else:
@@ -252,31 +248,30 @@ class CSVWriter(object):
             self.translate_headers = 1
         else:
             self.option_box.parse_options()
+            self.db = option_box.get_filtered_database(self.db)
 
             self.include_individuals = self.option_box.include_individuals
             self.include_marriages = self.option_box.include_marriages
             self.include_children = self.option_box.include_children
             self.translate_headers = self.option_box.translate_headers
             
-            if not option_box.cfilter.is_empty():
-                self.db = gen.proxy.FilterProxyDb(self.db, option_box.cfilter)
-
-        for p in self.db.get_person_handles(sort_handles=False):
-            self.plist[p] = 1
+        self.plist = [x for x in self.db.iter_person_handles()]
         # get the families for which these people are spouses:
         self.flist = {}
         for key in self.plist:
             p = self.db.get_person_from_handle(key)
-            for family_handle in p.get_family_handle_list():
-                self.flist[family_handle] = 1
-        # now add the families for which these people are a child:
-        family_handles = self.db.get_family_handles()
-        for family_handle in family_handles:
-            family = self.db.get_family_from_handle(family_handle)
-            for child_ref in family.get_child_ref_list():
-                child_handle = child_ref.ref
-                if child_handle in self.plist.keys():
+            if p:
+                for family_handle in p.get_family_handle_list():
                     self.flist[family_handle] = 1
+        # now add the families for which these people are a child:
+        for family_handle in self.db.iter_family_handles():
+            family = self.db.get_family_from_handle(family_handle)
+            if family:
+                for child_ref in family.get_child_ref_list():
+                    if child_ref:
+                        child_handle = child_ref.ref
+                        if child_handle in self.plist:
+                            self.flist[family_handle] = 1
                         
     def update_empty(self):
         pass
@@ -302,10 +297,10 @@ class CSVWriter(object):
             self.g = UnicodeWriter(self.fp)
         except IOError,msg:
             msg2 = _("Could not create %s") % self.filename
-            ErrorDialog(msg2,str(msg))
+            self.msg_callback(msg2,str(msg))
             return False
         except:
-            ErrorDialog(_("Could not create %s") % self.filename)
+            self.msg_callback(_("Could not create %s") % self.filename)
             return False
         ######################### initialize progress bar
         self.count = 0
@@ -318,8 +313,8 @@ class CSVWriter(object):
         if self.include_children:
             self.total += len(self.flist)
         ######################## 
-        print "Possible people to export:", len(self.plist)
-        print "Possible families to export:", len(self.flist)
+        LOG.debug("Possible people to export: %s", len(self.plist))
+        LOG.debug("Possible families to export: %s", len(self.flist))
         ########################### sort:
         sortorder = []
         for key in self.plist:
@@ -327,40 +322,48 @@ class CSVWriter(object):
             if person:
                 primary_name = person.get_primary_name()
                 first_name = primary_name.get_first_name()
-                surname = primary_name.get_surname()
+                surname_obj = primary_name.get_primary_surname()
+                surname = surname_obj.get_surname()
                 sortorder.append( (surname, first_name, key) )
         sortorder.sort() # will sort on tuples
         plist = [data[2] for data in sortorder]
         ###########################
         if self.include_individuals:
             if self.translate_headers:
-                self.write_csv(_("Person"), _("Surname"), _("Given"), 
-                               _("Call"), _("Suffix"), _("Prefix"), 
-                               _("Person|Title"), _("Gender"), _("Birth date"), 
-                               _("Birth place"), _("Birth source"),
-                               _("Death date"), _("Death place"), 
-                               _("Death source"), _("Note"))
+                self.write_csv(
+                    _("Person"), _("Surname"), _("Given"), 
+                    _("Call"), _("Suffix"), _("Prefix"), 
+                    _("Person|Title"), _("Gender"), 
+                    _("Birth date"), _("Birth place"), _("Birth source"),
+                    _("Baptism date"), _("Baptism place"), _("Baptism source"),
+                    _("Death date"), _("Death place"), _("Death source"), 
+                    _("Burial date"), _("Burial place"), _("Burial source"),
+                    _("Note"))
             else:
-                self.write_csv("Person", "Surname", "Given", 
-                               "Call", "Suffix", "Prefix", 
-                               "Title", "Gender", "Birth date", 
-                               "Birth place", "Birth source",
-                               "Death date", "Death place", 
-                               "Death source", "Note")
+                self.write_csv(
+                    "Person", "Surname", "Given", 
+                    "Call", "Suffix", "Prefix", 
+                    "Title", "Gender", 
+                    "Birth date", "Birth place", "Birth source",
+                    "Baptism date", "Baptism place", "Baptism source",
+                    "Death date", "Death place", "Death source", 
+                    "Burial date", "Burial place", "Burial source",
+                    "Note")
             for key in plist:
                 person = self.db.get_person_from_handle(key)
                 if person:
                     primary_name = person.get_primary_name()
                     first_name = primary_name.get_first_name()
-                    surname = primary_name.get_surname()
-                    prefix = primary_name.get_surname_prefix()
+                    surname_obj = primary_name.get_primary_surname()
+                    surname = surname_obj.get_surname()
+                    prefix = surname_obj.get_prefix()
                     suffix = primary_name.get_suffix()
                     title = primary_name.get_title()
                     grampsid = person.get_gramps_id()
                     grampsid_ref = ""
                     if grampsid != "":
                         grampsid_ref = "[" + grampsid + "]"
-                    note = '' # don't export notes or source
+                    note = '' # don't export notes
                     callname = primary_name.get_call_name()
                     gender = person.get_gender()
                     if gender == gen.lib.Person.MALE:
@@ -372,6 +375,7 @@ class CSVWriter(object):
                     # Birth:
                     birthdate = ""
                     birthplace = ""
+                    birthsource = ""
                     birth_ref = person.get_birth_ref()
                     if birth_ref:
                         birth = self.db.get_event_from_handle(birth_ref.ref)
@@ -381,9 +385,26 @@ class CSVWriter(object):
                             if place_handle:
                                 place = self.db.get_place_from_handle(place_handle)
                                 birthplace = place.get_title()
+                            birthsource = get_primary_source_title(self.db, birth)
+                    # Baptism:
+                    baptismdate = ""
+                    baptismplace = ""
+                    baptismsource = ""
+                    baptism_ref = get_primary_event_ref_from_type(
+                        self.db, person, "Baptism")
+                    if baptism_ref:
+                        baptism = self.db.get_event_from_handle(baptism_ref.ref)
+                        if baptism:
+                            baptismdate = self.format_date( baptism)
+                            place_handle = baptism.get_place_handle()
+                            if place_handle:
+                                place = self.db.get_place_from_handle(place_handle)
+                                baptismplace = place.get_title()
+                            baptismsource = get_primary_source_title(self.db, baptism)
                     # Death:
                     deathdate = ""
                     deathplace = ""
+                    deathsource = ""
                     death_ref = person.get_death_ref()
                     if death_ref:
                         death = self.db.get_event_from_handle(death_ref.ref)
@@ -393,10 +414,29 @@ class CSVWriter(object):
                             if place_handle:
                                 place = self.db.get_place_from_handle(place_handle)
                                 deathplace = place.get_title()
+                            deathsource = get_primary_source_title(self.db, death)
+                    # Burial:
+                    burialdate = ""
+                    burialplace = ""
+                    burialsource = ""
+                    burial_ref = get_primary_event_ref_from_type(
+                        self.db, person, "Burial")
+                    if burial_ref:
+                        burial = self.db.get_event_from_handle(burial_ref.ref)
+                        if burial:
+                            burialdate = self.format_date( burial)
+                            place_handle = burial.get_place_handle()
+                            if place_handle:
+                                place = self.db.get_place_from_handle(place_handle)
+                                burialplace = place.get_title()
+                            burialsource = get_primary_source_title(self.db, burial)
+                    # Write it out:
                     self.write_csv(grampsid_ref, surname, first_name, callname,
                                    suffix, prefix, title, gender,
-                                   birthdate, birthplace, "",
-                                   deathdate, deathplace, "",
+                                   birthdate, birthplace, birthsource,
+                                   baptismdate, baptismplace, baptismsource,
+                                   deathdate, deathplace, deathsource,
+                                   burialdate, burialplace, burialsource,
                                    note)
                 self.update()
             self.writeln()
@@ -406,7 +446,9 @@ class CSVWriter(object):
             family = self.db.get_family_from_handle(key)
             if family:
                 marriage_id = family.get_gramps_id()
-                sortorder.append( (sortable_string_representation(marriage_id), key) )
+                sortorder.append(
+                    (sortable_string_representation(marriage_id), key)
+                    )
         sortorder.sort() # will sort on tuples
         flist = [data[1] for data in sortorder]
         ########################### 
@@ -438,7 +480,7 @@ class CSVWriter(object):
                         if mother_id != "":
                             mother_id = "[" + mother_id + "]"
                     # get mdate, mplace
-                    mdate, mplace = '', ''
+                    mdate, mplace, source = '', '', ''
                     event_ref_list = family.get_event_ref_list()
                     for event_ref in event_ref_list:
                         event = self.db.get_event_from_handle(event_ref.ref)
@@ -448,8 +490,8 @@ class CSVWriter(object):
                             if place_handle:
                                 place = self.db.get_place_from_handle(place_handle)
                                 mplace = place.get_title()
-                                # m_source = self.get_primary_source( event.get_source_references())
-                    source, note = '', ''
+                                source = get_primary_source_title(self.db, event)
+                    note = ''
                     self.write_csv(marriage_id, father_id, mother_id, mdate,
                                    mplace, source, note)
                 self.update()
@@ -480,20 +522,3 @@ class CSVWriter(object):
     
     def format_date(self, date):
         return DateHandler.get_date(date)
-
-#-------------------------------------------------------------------------
-#
-# Register the plugin
-#
-#-------------------------------------------------------------------------
-_name = _('Comma _Separated Values Spreadsheet (CSV)')
-_description = _('CSV is a common spreadsheet format.')
-_config = (_('CSV spreadsheet options'),CSVWriterOptionBox)
-
-pmgr = PluginManager.get_instance()
-plugin = ExportPlugin(name            = _name, 
-                      description     = _description,
-                      export_function = exportData,
-                      extension       = "csv",
-                      config          = _config )
-pmgr.register_plugin(plugin)
