@@ -5,6 +5,8 @@
 # Copyright (C) 2008       Brian G. Matherly
 # Copyright (C) 2008-2009  Gary Burton
 # Copyright (C) 2008       Robert Cheramy <robert@cheramy.net>
+# Copyright (C) 2010       Jakim Friant
+# Copyright (C) 2010       Nick Hall
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,7 +22,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
-# $Id: ExportGedcom.py 15371 2010-05-09 13:55:15Z dsblank $
+# $Id: ExportGedcom.py 17480 2011-05-12 12:43:46Z romjerome $
 
 "Export to GEDCOM"
 
@@ -44,9 +46,7 @@ import libgedcom
 import Errors
 from ExportOptions import WriterOptionBox
 from gen.updatecallback import UpdateCallback
-from Utils import media_path_full, get_unicode_path
-import gen.proxy
-from QuestionDialog import ErrorDialog
+from Utils import media_path_full
 from PlaceUtils import conv_lat_lon
 
 #-------------------------------------------------------------------------
@@ -142,8 +142,9 @@ def sort_handles_by_id(handle_list, handle_to_object):
     sorted_list = []
     for handle in handle_list:
         obj = handle_to_object(handle)
-        data = (obj.get_gramps_id(), handle)
-        sorted_list.append (data)
+        if obj:
+            data = (obj.get_gramps_id(), handle)
+            sorted_list.append (data)
     sorted_list.sort()
     return sorted_list
 
@@ -234,66 +235,7 @@ class GedcomWriter(UpdateCallback):
         """
         if option_box:
             option_box.parse_options()
-
-            # Increment the progress count for each filter type chosen
-            if option_box.private:
-                self.progress_cnt += 1
-
-            if option_box.restrict:
-                self.progress_cnt += 1
-
-            if not option_box.cfilter.is_empty():
-                self.progress_cnt += 1
-
-            if not option_box.nfilter.is_empty():
-                self.progress_cnt += 1
-
-            if option_box.unlinked:
-                self.progress_cnt += 1
-
-            self.set_total(self.progress_cnt)
-            self.progress_cnt = 0
-
-            # If the private flag is set, apply the PrivateProxyDb
-            if option_box.private:
-                self.reset(_("Filtering private data"))
-                self.progress_cnt += 1
-                self.update(self.progress_cnt)
-                self.dbase = gen.proxy.PrivateProxyDb(self.dbase)
-
-            # If the restrict flag is set, apply the LivingProxyDb
-            if option_box.restrict:
-                self.reset(_("Filtering living persons"))
-                self.progress_cnt += 1
-                self.update(self.progress_cnt)
-                self.dbase = gen.proxy.LivingProxyDb(
-                            self.dbase, 
-                            gen.proxy.LivingProxyDb.MODE_INCLUDE_LAST_NAME_ONLY)
-
-            # If the filter returned by cfilter is not empty, apply the 
-            # FilterProxyDb (Person Filter)
-            if not option_box.cfilter.is_empty():
-                self.reset(_("Applying selected person filter"))
-                self.progress_cnt += 1
-                self.update(self.progress_cnt)
-                self.dbase = gen.proxy.FilterProxyDb(
-                    self.dbase, option_box.cfilter)
-            
-            # Apply the Note Filter
-            if not option_box.nfilter.is_empty():
-                self.reset(_("Applying selected note filter"))
-                self.progress_cnt += 1
-                self.update(self.progress_cnt)
-                self.dbase = gen.proxy.FilterProxyDb(
-                    self.dbase, note_filter=option_box.nfilter)
-
-            # Apply the ReferencedProxyDb to remove any objects not referenced
-            # after any of the other proxies have been applied
-            if option_box.unlinked:
-                self.reset(_("Filtering unlinked records"))
-                self.progress_cnt += 1
-                self.update(self.progress_cnt)
-                self.dbase = gen.proxy.ReferencedProxyDb(self.dbase)
+            self.dbase = option_box.get_filtered_database(self.dbase, self)
 
     def write_gedcom_file(self, filename):
         """
@@ -328,11 +270,13 @@ class GedcomWriter(UpdateCallback):
         assert(token)
         if textlines:
             # break the line into multiple lines if a newline is found
+            textlines = textlines.replace('\n\r', '\n')
+            textlines = textlines.replace('\r', '\n')
             textlist = textlines.split('\n')
             token_level = level
             for text in textlist:
                 # make it unicode so that breakup below does the right thin.
-                text = get_unicode_path(text)
+                text = unicode(text)
                 if limit:
                     prefix = "\n%d CONC " % (level + 1)
                     txt = prefix.join(breakup(text, limit))
@@ -420,6 +364,7 @@ class GedcomWriter(UpdateCallback):
         owner = self.dbase.get_researcher()
         name = owner.get_name()
         addr = owner.get_address()
+        adr2 = owner.get_locality()
         city = owner.get_city()
         state = owner.get_state()
         ctry = owner.get_country()
@@ -439,6 +384,10 @@ class GedcomWriter(UpdateCallback):
             self.__writeln(2, "CONT", "%s, %s %s" % (city, state, post))
         else:
             self.__writeln(2, "CONT", u"Not Provided")
+        if addr:
+            self.__writeln(2, "ADR1", addr)
+        if adr2:
+            self.__writeln(2, "ADR2", adr2)
         if city:
             self.__writeln(2, "CITY", city)
         if state:
@@ -469,8 +418,9 @@ class GedcomWriter(UpdateCallback):
         sorted_list = []
         for handle in phandles:
             person = self.dbase.get_person_from_handle(handle)
-            data = (person.get_gramps_id(), handle)
-            sorted_list.append(data)
+            if person:
+                data = (person.get_gramps_id(), handle)
+                sorted_list.append(data)
         sorted_list.sort()
 
         for data in sorted_list:
@@ -504,6 +454,8 @@ class GedcomWriter(UpdateCallback):
         +1 RIN <AUTOMATED_RECORD_ID> {0:1} 
         +1 <<CHANGE_DATE>> {0:1} 
         """
+        if person is None:
+            return
         self.__writeln(0, "@%s@" % person.get_gramps_id(),  "INDI")
 
         self.__names(person)
@@ -532,10 +484,11 @@ class GedcomWriter(UpdateCallback):
         """
         for ref in person.get_person_ref_list():
             person = self.dbase.get_person_from_handle(ref.ref)
-            self.__writeln(level, "ASSO", "@%s@" % person.get_gramps_id())
-            self.__writeln(level+1, "RELA", ref.get_relation())
-            self.__note_references(ref.get_note_list(), level+1)
-            self.__source_references(ref.get_source_references(), level+1)
+            if person:
+                self.__writeln(level, "ASSO", "@%s@" % person.get_gramps_id())
+                self.__writeln(level+1, "RELA", ref.get_relation())
+                self.__note_references(ref.get_note_list(), level+1)
+                self.__source_references(ref.get_source_references(), level+1)
 
     def __note_references(self, notelist, level):
         """
@@ -546,20 +499,19 @@ class GedcomWriter(UpdateCallback):
         """
         for note_handle in notelist:
             note = self.dbase.get_note_from_handle(note_handle)
-            self.__writeln(level, 'NOTE', '@%s@' % note.get_gramps_id())
+            if note:
+                self.__writeln(level, 'NOTE', '@%s@' % note.get_gramps_id())
 
     def __names(self, person):
         """
         Write the names associated with the person to the current level.
          
-        Since nicknames are now separate from the name structure, we search
-        the attribute list to see if we can find a nickname. Because we do
-        not know the mappings, we just take the first nickname we find, and 
-        add it to the primary name.
+        Since nicknames in version < 3.3 are separate from the name structure,
+        we search the attribute list to see if we can find a nickname. 
+        Because we do not know the mappings, we just take the first nickname 
+        we find, and add it to the primary name.
+        If a nickname is present in the name structure, it has precedence
 
-        All other names are assumed to not  have a nickname, even if other 
-        nicknames exist in the attribute list.
-        
         """
         nicknames = [ attr.get_value() for attr in person.get_attribute_list()
                       if int(attr.get_type()) == gen.lib.AttributeType.NICKNAME ]
@@ -610,6 +562,12 @@ class GedcomWriter(UpdateCallback):
             # if the event is a birth or death, skip it.
             if etype in (gen.lib.EventType.BIRTH, gen.lib.EventType.DEATH):
                 continue
+
+            role = int(event_ref.get_role())
+
+            # if the event role is not primary, skip the event.
+            if role != gen.lib.EventRoleType.PRIMARY:
+                continue
                 
             val = libgedcom.personalConstantEvents.get(etype, "").strip()
                         
@@ -654,6 +612,8 @@ class GedcomWriter(UpdateCallback):
 
         for family in [ self.dbase.get_family_from_handle(fh) 
                         for fh in person.get_parent_family_handle_list() ]:
+            if family is None:
+                continue
             for child_ref in [ ref for ref in family.get_child_ref_list()
                                if ref.ref == person.handle ]:
                 if child_ref.mrel == gen.lib.ChildRefType.ADOPTED \
@@ -727,6 +687,10 @@ class GedcomWriter(UpdateCallback):
             self.__writeln(1, 'RESI')
             self.__date(2, addr.get_date_object())
             self.__writeln(2, "ADDR", addr.get_street())
+            if addr.get_street():
+                self.__writeln(3, 'ADR1', addr.get_street())
+            if addr.get_locality():
+                self.__writeln(3, 'ADR2', addr.get_locality())
             if addr.get_city():
                 self.__writeln(3, 'CITY', addr.get_city())
             if addr.get_state():
@@ -760,7 +724,8 @@ class GedcomWriter(UpdateCallback):
                         for hndl in person.get_parent_family_handle_list() ]
 
         for family in family_list:
-            self.__writeln(1, 'FAMC', '@%s@' % family.get_gramps_id())
+            if family:
+                self.__writeln(1, 'FAMC', '@%s@' % family.get_gramps_id())
 
     def __parent_families(self, person):
         """
@@ -773,7 +738,8 @@ class GedcomWriter(UpdateCallback):
                         for hndl in person.get_family_handle_list() ]
 
         for family in family_list:
-            self.__writeln(1, 'FAMS', '@%s@' % family.get_gramps_id())
+            if family:
+                self.__writeln(1, 'FAMS', '@%s@' % family.get_gramps_id())
 
     def __person_sources(self, person):
         """
@@ -830,6 +796,8 @@ class GedcomWriter(UpdateCallback):
         +1 <<LDS_SPOUSE_SEALING>> {0:M}
         +1 REFN <USER_REFERENCE_NUMBER> {0:M}
         """
+        if family is None:
+            return
         gramps_id = family.get_gramps_id()
 
         self.__writeln(0, '@%s@' % gramps_id, 'FAM' )
@@ -852,9 +820,10 @@ class GedcomWriter(UpdateCallback):
         """
         child_list = [ 
             self.dbase.get_person_from_handle(cref.ref).get_gramps_id()
-            for cref in child_ref_list ]
+            for cref in child_ref_list]
 
         for gid in child_list:
+            if gid is None: continue
             self.__writeln(1, 'CHIL', '@%s@' % gid)
 
     def __family_reference(self, token, person_handle):
@@ -867,7 +836,8 @@ class GedcomWriter(UpdateCallback):
         """
         if person_handle:
             person = self.dbase.get_person_from_handle(person_handle)
-            self.__writeln(1, token, '@%s@' % person.get_gramps_id())
+            if person:
+                self.__writeln(1, token, '@%s@' % person.get_gramps_id())
 
     def __family_events(self, family):
         """
@@ -879,6 +849,7 @@ class GedcomWriter(UpdateCallback):
         """
         for event_ref in family.get_event_ref_list():
             event = self.dbase.get_event_from_handle(event_ref.ref)
+            if event is None: continue
             etype = int(event.get_type())
             val = libgedcom.familyConstantEvents.get(etype)
             
@@ -965,6 +936,7 @@ class GedcomWriter(UpdateCallback):
 
         for (source_id, handle) in sorted_list:
             source = self.dbase.get_source_from_handle(handle)
+            if source is None: continue
             self.__writeln(0, '@%s@' % source_id, 'SOUR')
             if source.get_title():
                 self.__writeln(1, 'TITL', source.get_title())
@@ -999,6 +971,7 @@ class GedcomWriter(UpdateCallback):
 
         for note_handle in [hndl[1] for hndl in sorted_list]:
             note = self.dbase.get_note_from_handle(note_handle)
+            if note is None: continue
             self.__note_record(note)
             
     def __note_record(self, note):
@@ -1011,8 +984,8 @@ class GedcomWriter(UpdateCallback):
         +1 RIN <AUTOMATED_RECORD_ID> {0:1} 
         +1 <<CHANGE_DATE>> {0:1} 
         """
-
-        self.__writeln(0, '@%s@' % note.get_gramps_id(),  'NOTE ' + note.get())
+        if note:
+            self.__writeln(0, '@%s@' % note.get_gramps_id(),  'NOTE ' + note.get())
 
     def __repos(self):
         """
@@ -1038,11 +1011,16 @@ class GedcomWriter(UpdateCallback):
 
         for (repo_id, handle) in sorted_list:
             repo = self.dbase.get_repository_from_handle(handle)
+            if repo is None: continue
             self.__writeln(0, '@%s@' % repo_id, 'REPO' )
             if repo.get_name():
                 self.__writeln(1, 'NAME', repo.get_name())
             for addr in repo.get_address_list():
                 self.__writeln(1, "ADDR", addr.get_street())
+                if addr.get_street():
+                    self.__writeln(2, 'ADR1', addr.get_street())
+                if addr.get_locality():
+                    self.__writeln(2, 'ADR2', addr.get_locality())
                 if addr.get_city():
                     self.__writeln(2, 'CITY', addr.get_city())
                 if addr.get_state():
@@ -1053,6 +1031,11 @@ class GedcomWriter(UpdateCallback):
                     self.__writeln(2, 'CTRY', addr.get_country())
                 if addr.get_phone():
                     self.__writeln(1, 'PHON', addr.get_phone())
+            for url in repo.get_url_list():
+                if int(url.get_type()) == gen.lib.UrlType.EMAIL:
+                    self.__writeln(1, 'EMAIL', url.get_path())
+                elif int(url.get_type()) == gen.lib.UrlType.WEB_HOME:
+                    self.__writeln(1, 'WWW', url.get_path())
             self.__note_references(repo.get_note_list(), 1)
 
     def __reporef(self, reporef, level):
@@ -1067,6 +1050,9 @@ class GedcomWriter(UpdateCallback):
             return
 
         repo = self.dbase.get_repository_from_handle(reporef.ref)
+        if repo is None:
+            return
+        
         repo_id = repo.get_gramps_id()
 
         self.__writeln(level, 'REPO', '@%s@' % repo_id )
@@ -1117,6 +1103,15 @@ class GedcomWriter(UpdateCallback):
         """
         dateobj = event.get_date_object()
         self.__date(2, dateobj)
+        if self.__datewritten:
+            # write out TIME if present
+            times = [ attr.get_value() for attr in event.get_attribute_list()
+                      if int(attr.get_type()) == gen.lib.AttributeType.TIME ]
+            # Not legal, but inserted by PhpGedView
+            if len(times) > 0:
+                time = times[0]
+                self.__writeln(3, 'TIME', time)
+
         place = None
 
         if event.get_place_handle():
@@ -1205,6 +1200,7 @@ class GedcomWriter(UpdateCallback):
         Write the 'DATE' GEDCOM token, along with the date in GEDCOM's
         expected format.
         """
+        self.__datewritten = True
         start = date.get_start_date()
         if start != gen.lib.Date.EMPTY:
             cal = date.get_calendar()
@@ -1225,8 +1221,10 @@ class GedcomWriter(UpdateCallback):
             self.__writeln(level, 'DATE', val)
         elif date.get_text():
             self.__writeln(level, 'DATE', date.get_text())
+        else:
+            self.__datewritten = False
 
-    def __person_name(self, name, nick):
+    def __person_name(self, name, attr_nick):
         """
         n NAME <NAME_PERSONAL> {1:1} 
         +1 NPFX <NAME_PIECE_PREFIX> {0:1} 
@@ -1241,13 +1239,21 @@ class GedcomWriter(UpdateCallback):
         gedcom_name = name.get_gedcom_name()
 
         firstname = name.get_first_name().strip()
-        patron = name.get_patronymic().strip()
-        if patron:
-            firstname = "%s %s" % (firstname, patron)
-        surname = name.get_surname().replace('/', '?')
-        surprefix = name.get_surname_prefix().replace('/', '?')
+        surns = []
+        surprefs = []
+        for surn in name.get_surname_list():
+            surns.append(surn.get_surname().replace('/', '?'))
+            if surn.get_connector():
+                #we store connector with the surname
+                surns[-1] = surns[-1] + ' ' + surn.get_connector()
+            surprefs.append(surn.get_prefix().replace('/', '?'))
+        surname = ', '.join(surns)
+        surprefix = ', '.join(surprefs)
         suffix = name.get_suffix()
         title = name.get_title()
+        nick = name.get_nick_name()
+        if nick.strip() == '':
+            nick = attr_nick
 
         self.__writeln(1, 'NAME', gedcom_name)
 
@@ -1257,7 +1263,6 @@ class GedcomWriter(UpdateCallback):
             self.__writeln(2, 'SPFX', surprefix)
         if surname:
             self.__writeln(2, 'SURN', surname)
-                
         if name.get_suffix():
             self.__writeln(2, 'NSFX', suffix)
         if name.get_title():
@@ -1328,7 +1333,7 @@ class GedcomWriter(UpdateCallback):
             note_list = [ self.dbase.get_note_from_handle(h) 
                           for h in ref.get_note_list() ]
             note_list = [ n.handle for n in note_list 
-                          if n.get_type() != gen.lib.NoteType.SOURCE_TEXT]
+                          if n and n.get_type() != gen.lib.NoteType.SOURCE_TEXT]
             self.__note_references(note_list, level+1)
 
     def __photo(self, photo, level):
@@ -1369,8 +1374,9 @@ class GedcomWriter(UpdateCallback):
             +2 LONG <PLACE_LONGITUDE> {1:1}
             +1 <<NOTE_STRUCTURE>> {0:M} 
         """
+        if place is None: return
         place_name = place.get_title()
-        self.__writeln(level, "PLAC", place_name.replace('\r', ' '))
+        self.__writeln(level, "PLAC", place_name.replace('\r', ' '), limit=120)
         longitude = place.get_longitude()
         latitude = place.get_latitude()
         if longitude and latitude:
@@ -1386,6 +1392,10 @@ class GedcomWriter(UpdateCallback):
         location = place.get_main_location()
         if location and not location.is_empty():
             self.__writeln(level, "ADDR", location.get_street())
+            if location.get_street():
+                self.__writeln(level + 1, 'ADR1', location.get_street())
+            if location.get_locality():
+                self.__writeln(level + 1, 'ADR2', location.get_locality())
             if location.get_city():
                 self.__writeln(level + 1, 'CITY', location.get_city())
             if location.get_state():
@@ -1402,7 +1412,7 @@ class GedcomWriter(UpdateCallback):
 #
 #
 #-------------------------------------------------------------------------
-def export_data(database, filename, option_box=None, callback=None):
+def export_data(database, filename, msg_callback, option_box=None, callback=None):
     """
     External interface used to register with the plugin system.
     """
@@ -1412,7 +1422,7 @@ def export_data(database, filename, option_box=None, callback=None):
         ret = ged_write.write_gedcom_file(filename)
     except IOError, msg:
         msg2 = _("Could not create %s") % filename
-        ErrorDialog(msg2, str(msg))
+        msg_callback(msg2, str(msg))
     except Errors.DatabaseError, msg:
-        ErrorDialog(_("Export failed"), str(msg))
+        msg_callback(_("Export failed"), str(msg))
     return ret
