@@ -5,6 +5,7 @@
 # Copyright (C) 2004-2006, 2008 Donald N. Allingham
 # Copyright (C) 2008  Brian G. Matherly
 # Copyright (C) 2009  Gary Burton
+# Copyright (C) 2010       Jakim Friant
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,7 +21,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
-# $Id: ExportGeneWeb.py 12560 2009-05-21 17:58:31Z gbritton $
+# $Id: ExportGeneWeb.py 15937 2010-09-29 14:45:27Z romjerome $
 
 "Export to GeneWeb."
 
@@ -30,14 +31,7 @@
 #
 #-------------------------------------------------------------------------
 import os
-from gettext import gettext as _
-
-#-------------------------------------------------------------------------
-#
-# GNOME/GTK modules
-#
-#-------------------------------------------------------------------------
-from gtk import glade
+from gen.ggettext import gettext as _
 
 #------------------------------------------------------------------------
 #
@@ -53,165 +47,33 @@ log = logging.getLogger(".WriteGeneWeb")
 #
 #-------------------------------------------------------------------------
 import gen.lib
-from Filters import GenericFilter, Rules, build_filter_menu
+from Filters import GenericFilter, Rules, build_filter_model
+from ExportOptions import WriterOptionBox
 #import const
 import Utils
-from QuestionDialog import ErrorDialog
-from gen.plug import PluginManager, ExportPlugin
-
-#-------------------------------------------------------------------------
-#
-#
-#
-#-------------------------------------------------------------------------
-class GeneWebWriterOptionBox(object):
-    """
-    Create a VBox with the option widgets and define methods to retrieve
-    the options.
-     
-    """
-    def __init__(self, person):
-        self.person = person
-
-    def get_option_box(self):
-        self.restrict = 1
-        self.private = 1
-
-        glade_file = os.path.join(os.path.dirname(__file__), 
-                                  "ExportGeneWeb.glade")
-
-        self.topDialog = glade.XML(glade_file, "genewebExport", "gramps")
-        self.topDialog.signal_autoconnect({
-                "on_restrict_toggled": self.on_restrict_toggled
-                })
-
-        filter_obj = self.topDialog.get_widget("filter")
-        self.copy = 0
-
-        all = GenericFilter()
-        all.set_name(_("Entire Database"))
-        all.add_rule(Rules.Person.Everyone([]))
-
-        the_filters = [all]
-
-        if self.person:
-            des = GenericFilter()
-            des.set_name(_("Descendants of %s") %
-                         self.person.get_primary_name().get_name())
-            des.add_rule(Rules.Person.IsDescendantOf(
-                [self.person.get_gramps_id(),1]))
-
-            ans = GenericFilter()
-            ans.set_name(_("Ancestors of %s") %
-                         self.person.get_primary_name().get_name())
-            ans.add_rule(Rules.Person.IsAncestorOf(
-                [self.person.get_gramps_id(),1]))
-
-            com = GenericFilter()
-            com.set_name(_("People with common ancestor with %s") %
-                         self.person.get_primary_name().get_name())
-            com.add_rule(Rules.Person.HasCommonAncestorWith(
-                [self.person.get_gramps_id()]))
-
-            the_filters += [des, ans, com]
-
-        from Filters import CustomFilters
-        the_filters.extend(CustomFilters.get_filters('Person'))
-        self.filter_menu = build_filter_menu(the_filters)
-        filter_obj.set_menu(self.filter_menu)
-
-        the_box = self.topDialog.get_widget('vbox1')
-        the_parent = self.topDialog.get_widget('dialog-vbox1')
-        the_parent.remove(the_box)
-        self.topDialog.get_widget("genewebExport").destroy()
-        return the_box
-
-    def on_restrict_toggled(self, restrict):
-        active = restrict.get_active ()
-        for x in [self.topDialog.get_widget("living"),
-                  self.topDialog.get_widget("notes"),
-                  self.topDialog.get_widget("sources")]:
-            x.set_sensitive(active)
-
-    def parse_options(self):
-        self.restrict = self.topDialog.get_widget("restrict").get_active()
-        self.living = (self.restrict and
-                       self.topDialog.get_widget("living").get_active())
-        self.exclnotes = (self.restrict and
-                          self.topDialog.get_widget("notes").get_active())
-        self.exclsrcs = (self.restrict and
-                         self.topDialog.get_widget("sources").get_active())
-        self.private = self.topDialog.get_widget("private").get_active()
-
-        self.cfilter = self.filter_menu.get_active().get_data("filter")
-
-        self.images = self.topDialog.get_widget ("images").get_active ()
-        if self.images:
-            images_path = self.topDialog.get_widget ("images_path")
-            self.images_path = unicode(images_path.get_text ())
-        else:
-            self.images_path = ""
+from glade import Glade
+import config
 
 class GeneWebWriter(object):
-    def __init__(self, database, filename="", option_box=None, 
+    def __init__(self, database, filename, msg_callback, option_box=None,
                  callback=None):
         self.db = database
-        self.option_box = option_box
         self.filename = filename
+        self.msg_callback = msg_callback
+        self.option_box = option_box
         self.callback = callback
         if callable(self.callback): # callback is really callable
             self.update = self.update_real
         else:
             self.update = self.update_empty
 
-        self.plist = {}
-        self.flist = {}
-        
         self.persons_details_done = []
         self.persons_notes_done = []
         self.person_ids = {}
         
-        if not option_box:
-            self.restrict = 0
-            self.private = 0
-            self.copy = 0
-            self.images = 0
-        else:
+        if option_box:
             self.option_box.parse_options()
-
-            self.restrict = self.option_box.restrict
-            self.living = self.option_box.living
-            self.exclnotes = self.option_box.exclnotes
-            self.exclsrcs = self.option_box.exclsrcs
-            self.private = self.option_box.private
-            self.copy = self.option_box.copy
-            self.images = self.option_box.images
-            self.images_path = self.option_box.images_path
-            
-            if not option_box.cfilter.is_empty():
-                self.db = gen.proxy.FilterProxyDb(self.db, option_box.cfilter)
-
-        for p in self.db.get_person_handles(sort_handles=False):
-            self.plist[p] = 1
-            
-        self.flist = {}
-        for key in self.plist:
-            p = self.db.get_person_from_handle(key)
-            for family_handle in p.get_family_handle_list():
-                self.flist[family_handle] = 1
-
-        # remove families that dont contain father AND mother
-        # because GeneWeb requires both to be present
-        templist = self.flist
-        self.flist = {}
-        for family_handle in templist:
-            family = self.db.get_family_from_handle(family_handle)
-            if family:
-                father_handle = family.get_father_handle()
-                mother_handle = family.get_mother_handle()
-                if father_handle and mother_handle:
-                    self.flist[family_handle] = 1
-
+            self.db = option_box.get_filtered_database(self.db)
 
     def update_empty(self):
         pass
@@ -233,14 +95,15 @@ class GeneWebWriter(object):
             self.g = open(self.filename, "w")
         except IOError,msg:
             msg2 = _("Could not create %s") % self.filename
-            ErrorDialog(msg2, str(msg))
+            self.msg_callback(msg2, str(msg))
             return False
         except:
-            ErrorDialog(_("Could not create %s") % self.filename)
+            self.msg_callback(_("Could not create %s") % self.filename)
             return False
 
+        self.flist = [x for x in self.db.iter_family_handles()]
         if len(self.flist) < 1:
-            ErrorDialog(_("No families matched by selected filter"))
+            self.msg_callback(_("No families matched by selected filter"))
             return False
         
         self.count = 0
@@ -263,18 +126,24 @@ class GeneWebWriter(object):
                 mother_handle = family.get_mother_handle()
                 if mother_handle:
                     mother = self.db.get_person_from_handle(mother_handle)
-                    self.writeln("fam %s %s +%s %s %s" % (self.get_ref_name(father), self.get_full_person_info_fam(father), self.get_wedding_data(family), self.get_ref_name(mother), self.get_full_person_info_fam(mother)))
+                    self.writeln("fam %s %s+%s %s %s" %
+                            (self.get_ref_name(father), 
+                            self.get_full_person_info_fam(father), 
+                            self.get_wedding_data(family), 
+                            self.get_ref_name(mother), 
+                            self.get_full_person_info_fam(mother)
+                            )
+                         )
                     self.write_witness( family)
                     self.write_sources( family.get_source_references())
                     self.write_children( family, father)
                     self.write_notes( family, father, mother)
-                    if not (self.restrict and self.exclnotes):
+                    if True: # FIXME: not (self.restrict and self.exclnotes):
                         notelist = family.get_note_list()
                         note = ""
                         for notehandle in notelist:
                             noteobj = self.db.get_note_from_handle(notehandle)
-                            note += noteobj.get()
-                            note += " "
+                            note += noteobj.get() + " "
                         if note and note != "":
                             note = note.replace('\n\r',' ')
                             note = note.replace('\r\n',' ')
@@ -285,7 +154,7 @@ class GeneWebWriter(object):
     def write_witness(self, family):
         # FIXME: witnesses are not in events anymore
         return
-        
+    
         if self.restrict:
             return
         event_ref_list = family.get_event_ref_list()
@@ -296,19 +165,25 @@ class GeneWebWriter(object):
                 if w_list:
                     for witness in w_list:
                         if witness and witness.type == gen.lib.Event.ID:
-                            person = self.db.get_person_from_handle(witness.get_value())
+                            person = self.db.get_person_from_handle(
+                                                        witness.get_value())
                             if person:
                                 gender = ""
                                 if person.get_gender() == gen.lib.Person.MALE:
                                     gender = "h"
                                 elif person.get_gender() == gen.lib.Person.FEMALE:
                                     gender = "f"
-                                self.writeln("wit %s %s %s" % (gender, self.get_ref_name(person), self.get_full_person_info_fam(person)))
-                            
+                                self.writeln("wit %s %s %s" % 
+                                        (gender, 
+                                        self.get_ref_name(person), 
+                                        self.get_full_person_info_fam(person)
+                                        )
+                                    )
 
     def write_sources(self,reflist):
-        if self.restrict and self.exclnotes:
-            return
+        # FIXME
+        #if self.restrict and self.exclnotes:
+        #    return
             
         if reflist:
             for sr in reflist:
@@ -316,7 +191,9 @@ class GeneWebWriter(object):
                 if sbase:
                     source = self.db.get_source_from_handle(sbase)
                     if source:
-                        self.writeln( "src %s" % (self.rem_spaces(source.get_title())))
+                        self.writeln( "src %s" % 
+                            (self.rem_spaces(source.get_title()))
+                            )
 
     def write_children(self,family, father):
         father_lastname = father.get_primary_name().get_surname()
@@ -331,12 +208,18 @@ class GeneWebWriter(object):
                         gender = "h"
                     elif child.get_gender() == gen.lib.Person.FEMALE:
                         gender = "f"
-                    self.writeln("- %s %s %s" % (gender, self.get_child_ref_name(child, father_lastname), self.get_full_person_info_child(child)))
+                    self.writeln("- %s %s %s" % 
+                            (gender, 
+                            self.get_child_ref_name(child, father_lastname), 
+                            self.get_full_person_info_child(child)
+                            )
+                         )
             self.writeln("end")
 
     def write_notes(self,family, father, mother):
-        if self.restrict and self.exclnotes:
-            return
+        # FIXME:
+        #if self.restrict and self.exclnotes:
+        #    return
             
         self.write_note_of_person(father)
         self.write_note_of_person(mother)
@@ -378,8 +261,9 @@ class GeneWebWriter(object):
                 self.writeln("end notes")
     
     def get_full_person_info(self, person):
-        if self.restrict:
-            return "0 "
+        # FIXME:
+        #if self.restrict:
+        #    return "0 "
             
         retval = ""
 
@@ -450,26 +334,28 @@ class GeneWebWriter(object):
         return str.replace(' ','_')
     
     def get_ref_name(self,person):
+        #missing_surname = config.get("preferences.no-surname-text")
         surname = self.rem_spaces( person.get_primary_name().get_surname())
-        firstname = _("Living")
-        if not (Utils.probably_alive(person,self.db) and self.restrict and self.living):
-            firstname = self.rem_spaces( person.get_primary_name().get_first_name())
+        #firstname = config.get('preferences.private-given-text') 
+        #if not (Utils.probably_alive(person,self.db) and \
+        #  self.restrict and self.living):
+        firstname = self.rem_spaces( person.get_primary_name().get_first_name())
         if person.get_handle() not in self.person_ids:
             self.person_ids[person.get_handle()] = len(self.person_ids)
-        ret = "%s %s.%d" % (surname, firstname, self.person_ids[person.get_handle()])
-        return ret
+        return "%s %s.%d" % (surname, firstname, 
+                             self.person_ids[person.get_handle()])
 
     def get_child_ref_name(self,person,father_lastname):
+        #missing_first_name = config.get("preferences.no-given-text")
         surname = self.rem_spaces( person.get_primary_name().get_surname())
-        firstname = _("Living")
-        if not (Utils.probably_alive(person,self.db) and self.restrict and self.living):
-            firstname = self.rem_spaces( person.get_primary_name().get_first_name())
+        #firstname = config.get('preferences.private-given-text')
+        #if not (Utils.probably_alive(person,self.db) and \
+        #  self.restrict and self.living):
+        firstname = self.rem_spaces( person.get_primary_name().get_first_name())
         if person.get_handle() not in self.person_ids:
             self.person_ids[person.get_handle()] = len(self.person_ids)
-        if surname != father_lastname:
-            ret = "%s.%d %s" % (firstname, self.person_ids[person.get_handle()], surname)
-        else:
-            ret = "%s.%d" % (firstname, self.person_ids[person.get_handle()])
+        ret = "%s.%d" % (firstname, self.person_ids[person.get_handle()])
+        if surname != father_lastname: ret += " " + surname
         return ret
 
     def get_wedding_data(self,family):
@@ -509,34 +395,32 @@ class GeneWebWriter(object):
         if married == 1:
             if m_date != "":
                 ret = ret + m_date
-            ret = ret + " "
-            
-            if m_place != "":
-                ret = ret + "#mp %s " % self.rem_spaces( m_place)
-            
-            if m_source != "":
-                ret = ret + "#ms %s " % self.rem_spaces( m_source)
+            if m_place != "" and m_source != "":
+                ret = ret + " #mp %s #ms %s" % (self.rem_spaces( m_place), self.rem_spaces( m_source))
+            if m_place != "" and m_source == "":
+                ret = ret + " #mp %s" % self.rem_spaces( m_place)
+            if m_source != "" and m_place == "":
+                ret = ret + " #ms %s" % self.rem_spaces( m_source)
         elif engaged == 1:
             """Geneweb only supports either Marriage or engagement"""
             if eng_date != "":
                 ret = ret + eng_date
-            ret = ret + " "
-            
-            if eng_place != "":
-                ret = ret + "#mp %s " % self.rem_spaces( m_place)
-            
-            if eng_source != "":
-                ret = ret + "#ms %s " % self.rem_spaces( m_source)
+            if m_place != "" and m_source != "":
+                ret = ret + " #mp %s #ms %s" % (self.rem_spaces( m_place), self.rem_spaces( m_source))
+            if eng_place != "" and m_source == "":
+                ret = ret + " #mp %s" % self.rem_spaces( m_place)
+            if eng_source != "" and m_place == "":
+                ret = ret + " #ms %s" % self.rem_spaces( m_source)
         else:
             if family.get_relationship() != gen.lib.FamilyRelType.MARRIED:
                 """Not married or engaged"""
-                ret = ret + " #nm "
+                ret = ret + " #nm"
 
         if divorced == 1:
             if div_date != "":
-                ret = ret + "-%s " %div_date
+                ret = ret + " -%s" %div_date
             else:
-                ret = ret + "-0 "
+                ret = ret + " -0"
 
         return ret
 
@@ -607,22 +491,6 @@ class GeneWebWriter(object):
 #
 #
 #-------------------------------------------------------------------------
-def exportData(database, filename, option_box=None, callback=None):
-    gw = GeneWebWriter(database, filename, option_box, callback)
+def exportData(database, filename, msg_callback, option_box=None, callback=None):
+    gw = GeneWebWriter(database, filename, msg_callback, option_box, callback)
     return gw.export_data()
-
-#------------------------------------------------------------------------
-#
-# Register with the plugin system
-#
-#------------------------------------------------------------------------
-_description = _('GeneWeb is a web based genealogy program.')
-_config = (_('GeneWeb export options'), GeneWebWriterOptionBox)
-
-pmgr = PluginManager.get_instance()
-plugin = ExportPlugin(name            = _('_GeneWeb'), 
-                      description     = _description,
-                      export_function = exportData,
-                      extension       = "gw",
-                      config          = _config )
-pmgr.register_plugin(plugin)
