@@ -5,6 +5,7 @@
 # Copyright (C) 2008       Brian G. Matherly
 # Copyright (C) 2008       Gary Burton
 # Copyright (C) 2008       Robert Cheramy <robert@cheramy.net>
+# Copyright (C) 2010       Jakim Friant
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -21,7 +22,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: ExportPkg.py 15939 2010-09-30 10:02:41Z ldnp $
+# $Id: ExportPkg.py 17538 2011-05-19 20:38:03Z m_d_n $
 
 "Export to GRAMPS package"
 
@@ -69,7 +70,7 @@ import constfunc
 # writeData
 #
 #-------------------------------------------------------------------------
-def writeData(database, filename, option_box=None, callback=None):
+def writeData(database, filename, msg_callback, option_box=None, callback=None):
 
 # Rename file, if it exists already, with <filename>.bak
 # as it it for normal XML export.
@@ -83,29 +84,9 @@ def writeData(database, filename, option_box=None, callback=None):
 
     if option_box:
         option_box.parse_options()
-    
-        if option_box.private:
-            database = gen.proxy.PrivateProxyDb(database)
-    
-        if option_box.restrict:
-            database = gen.proxy.LivingProxyDb(
-                database, gen.proxy.LivingProxyDb.MODE_INCLUDE_LAST_NAME_ONLY)
-    
-        # Apply the Person Filter
-        if not option_box.cfilter.is_empty():
-            database = gen.proxy.FilterProxyDb(database, option_box.cfilter)
-    
-        # Apply the Note Filter
-        if not option_box.nfilter.is_empty():
-            database = gen.proxy.FilterProxyDb(
-                database, note_filter=option_box.nfilter)
-        
-        # Apply the ReferencedProxyDb to remove any objects not referenced
-        # after any of the other proxies have been applied
-        if option_box.unlinked:
-            database = gen.proxy.ReferencedProxyDb(database)
+        database = option_box.get_filtered_database(database)
 
-    writer = PackageWriter(database, filename, callback)
+    writer = PackageWriter(database, filename, msg_callback, callback)
     return writer.export()
     
 #-------------------------------------------------------------------------
@@ -115,9 +96,10 @@ def writeData(database, filename, option_box=None, callback=None):
 #-------------------------------------------------------------------------
 class PackageWriter(object):
 
-    def __init__(self, database, filename, callback=None):
+    def __init__(self, database, filename, msg_callback, callback=None):
         self.db = database
         self.callback = callback
+        self.msg_callback = msg_callback
         self.filename = filename
             
     def export(self):
@@ -198,11 +180,11 @@ class PackageWriter(object):
         
         # Write media files first, since the database may be modified 
         # during the process (i.e. when removing object)
-        for m_id in self.db.get_media_object_handles():
+        for m_id in self.db.get_media_object_handles(sort_handles=True):
             mobject = self.db.get_object_from_handle(m_id)
             filename = Utils.media_path_full(self.db, mobject.get_path())
             archname = str(mobject.get_path())
-            if os.path.isfile(filename):
+            if os.path.isfile(filename) and os.access(filename, os.R_OK):
                 archive.add(filename, archname)
 #             else:
 #                 # File is lost => ask what to do
@@ -227,7 +209,7 @@ class PackageWriter(object):
         
         # Write XML now
         g = StringIO()
-        gfile = XmlWriter(self.db, self.callback, 2)
+        gfile = XmlWriter(self.db, self.msg_callback, self.callback, 2)
         gfile.write_handle(g)
         tarinfo = tarfile.TarInfo('data.gramps')
         tarinfo.size = len(g.getvalue())
