@@ -6,6 +6,8 @@
 # Copyright (C) 2008       Gary Burton
 # Copyright (C) 2008       Robert Cheramy <robert@cheramy.net>
 # Copyright (C) 2009       Douglas S. Blank
+# Copyright (C) 2010       Jakim Friant
+# Copyright (C) 2010       Nick Hall
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -22,7 +24,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: ExportXml.py 14999 2010-04-02 09:14:11Z ldnp $
+# $Id: ExportXml.py 17339 2011-04-29 14:02:36Z dsblank $
 
 """
 Contains the interface to allow a database to get written using
@@ -60,7 +62,7 @@ from gen.updatecallback import UpdateCallback
 from gen.db.exceptions import DbWriteFailure
 import const
 import constfunc
-from QuestionDialog import ErrorDialog
+#from QuestionDialog import ErrorDialog
 from ExportOptions import WriterOptionBox
 import gen.proxy
 import libgrampsxml
@@ -77,11 +79,15 @@ try:
 except:
     _gzip_ok = 0
 
-# table for skipping control chars from XML
-strip_dict = dict.fromkeys(range(9)+range(12,20))
+# table for skipping control chars from XML except 09, 0A, 0D
+strip_dict = dict.fromkeys(range(9)+range(11,13)+range(14, 32))
 
 def escxml(d):
-    return escape(d, { '"' : '&quot;' } )
+    return escape(d, 
+                  {'"' : '&quot;',
+                   '<' : '&lt;',
+                   '>' : '&gt;',
+                   }) if d else ""
 
 #-------------------------------------------------------------------------
 #
@@ -93,7 +99,7 @@ class GrampsXmlWriter(UpdateCallback):
     Writes a database to the XML file.
     """
 
-    def __init__(self, db, strip_photos=0, compress=1, version="unknown", 
+    def __init__(self, db, strip_photos=0, compress=1, version="unknown",
                  callback=None):
         """
         Initialize, but does not write, an XML file.
@@ -119,46 +125,52 @@ class GrampsXmlWriter(UpdateCallback):
         """
         Write the database to the specified file.
         """
-        base = os.path.dirname(filename)
-        if os.path.isdir(base):
-            if not os.access(base, os.W_OK) or not os.access(base, os.R_OK):
-                raise DbWriteFailure(
-                         _('Failure writing %s') % filename,
-                         _("The database cannot be saved because you do "
-                           "not have permission to write to the directory. "
-                           "Please make sure you have write access to the "
-                            "directory and try again."))
-                return 0
-            
-        if os.path.exists(filename):
-            if not os.access(filename, os.W_OK):
-                raise DbWriteFailure(
-                        _('Failure writing %s') % filename,
-                        _("The database cannot be saved because you do "
-                           "not have permission to write to the file. "
-                           "Please make sure you have write access to the "
-                           "file and try again."))
-                return 0
+        if filename == '-':
+            import sys
+            g = sys.stdout
+            self.compress = False
+        else:
+            base = os.path.dirname(filename)
+            if os.path.isdir(base):
+                if not os.access(base, os.W_OK) or not os.access(base, os.R_OK):
+                    raise DbWriteFailure(
+                            _('Failure writing %s') % filename,
+                            _("The database cannot be saved because you do "
+                            "not have permission to write to the directory. "
+                            "Please make sure you have write access to the "
+                                "directory and try again."))
+                    return 0
+                
+            if os.path.exists(filename):
+                if not os.access(filename, os.W_OK):
+                    raise DbWriteFailure(
+                            _('Failure writing %s') % filename,
+                            _("The database cannot be saved because you do "
+                            "not have permission to write to the file. "
+                            "Please make sure you have write access to the "
+                            "file and try again."))
+                    return 0
         
-        self.fileroot = os.path.dirname(filename)
-        try:
-            if self.compress and _gzip_ok:
-                try:
-                    g = gzip.open(filename,"wb")
-                except:
+            self.fileroot = os.path.dirname(filename)
+            try:
+                if self.compress and _gzip_ok:
+                    try:
+                        g = gzip.open(filename,"wb")
+                    except:
+                        g = open(filename,"w")
+                else:
                     g = open(filename,"w")
-            else:
-                g = open(filename,"w")
-        except IOError,msg:
-            LOG.warn(str(msg))
-            raise DbWriteFailure((_('Failure writing %s') % filename,
-                                       str(msg)))
-            return 0
+            except IOError,msg:
+                LOG.warn(str(msg))
+                raise DbWriteFailure(_('Failure writing %s') % filename,
+                                        str(msg))
+                return 0
 
         self.g = codecs.getwriter("utf8")(g)
 
         self.write_xml_data()
-        g.close()
+        if filename != '-':
+            g.close()
         return 1
 
     def write_handle(self, handle):
@@ -193,9 +205,11 @@ class GrampsXmlWriter(UpdateCallback):
         repo_len = self.db.get_number_of_repositories()
         obj_len = self.db.get_number_of_media_objects()
         note_len = self.db.get_number_of_notes()        
+        tag_len = self.db.get_number_of_tags()        
         
-        total_steps = person_len + family_len + event_len + source_len \
-                      + place_len + repo_len + obj_len + note_len
+        total_steps = (person_len + family_len + event_len + source_len +
+                       place_len + repo_len + obj_len + note_len + tag_len
+                      )
 
         self.set_total(total_steps)
         
@@ -214,6 +228,7 @@ class GrampsXmlWriter(UpdateCallback):
         self.g.write("    <researcher>\n")
         self.write_line("resname", owner.get_name(),3)
         self.write_line("resaddr", owner.get_address(),3)
+        self.write_line("reslocality", owner.get_locality(),3)
         self.write_line("rescity", owner.get_city(),3)
         self.write_line("resstate", owner.get_state(),3)
         self.write_line("rescountry", owner.get_country(),3)
@@ -228,6 +243,16 @@ class GrampsXmlWriter(UpdateCallback):
         # by the time we get to person's names
         self.write_name_formats()
 
+        # Write table objects
+        if tag_len > 0:
+            self.g.write("  <tags>\n")
+            for key in self.db.get_tag_handles():
+                tag = self.db.get_tag_from_handle(key)
+                self.write_tag(tag, 2)
+                self.update()
+            self.g.write("  </tags>\n")
+
+        # Write primary objects
         if event_len > 0:
             self.g.write("  <events>\n")
             for handle in self.db.get_event_handles():
@@ -335,9 +360,10 @@ class GrampsXmlWriter(UpdateCallback):
         bm_obj_len = len(self.db.media_bookmarks.get())
         bm_note_len = len(self.db.note_bookmarks.get())
 
-        bm_len = bm_person_len + bm_family_len + bm_event_len \
-               + bm_source_len + bm_place_len + bm_repo_len \
-               + bm_obj_len + bm_note_len
+        bm_len = (bm_person_len + bm_family_len + bm_event_len +
+                  bm_source_len + bm_place_len + bm_repo_len +
+                  bm_obj_len + bm_note_len
+                  )
         
         if bm_len > 0:
             self.g.write("  <bookmarks>\n")
@@ -375,9 +401,23 @@ class GrampsXmlWriter(UpdateCallback):
             for number, name,fmt_str,active in self.db.name_formats:
                 self.g.write('%s<format number="%d" name="%s" '
                              'fmt_str="%s" active="%d"/>\n'
-                             % ('    ', number, name, fmt_str, int(active)) )
+                             % ('    ', number, 
+                                escxml(name), escxml(fmt_str), int(active)) )
             self.g.write("  </name-formats>\n")
 
+    def write_tag(self, tag, index=2):
+        """
+        Write a tag definition.
+        """
+        if not tag:
+            return
+
+        self.write_table_tag('tag', tag, index, close=False)
+        self.g.write(' name="%s"' % escxml(tag.get_name()))
+        self.g.write(' color="%s"' % tag.get_color())
+        self.g.write(' priority="%d"' % tag.get_priority())
+        self.g.write('/>\n')
+        
     def fix(self,line):
         try:
             l = unicode(line)
@@ -399,7 +439,7 @@ class GrampsXmlWriter(UpdateCallback):
         ntype = escxml(note.get_type().xml_str())
         format = note.get_format()
         text = note.get_styledtext()
-        tags = text.get_tags()
+        styles = text.get_tags()
         text = str(text)
 
         self.g.write(' type="%s"' % ntype)
@@ -409,27 +449,29 @@ class GrampsXmlWriter(UpdateCallback):
         
         self.write_text('text', text, index + 1)
 
-        if tags:
-            self.write_texttags(tags, index + 1)
+        if styles:
+            self.write_styles(styles, index + 1)
+
+        for tag_handle in note.get_tag_list():
+            self.write_ref("tagref", tag_handle, index+1)
 
         self.g.write('  ' * index + '</note>\n')
-
         
-    def write_texttags(self, tags, index=3):
-        for tag in tags:
-            name = tag.name.xml_str()
-            value = tag.value
+    def write_styles(self, styles, index=3):
+        for style in styles:
+            name = style.name.xml_str()
+            value = style.value
             
-            self.g.write('  ' * index + '<tag name="%s"' % name)
+            self.g.write('  ' * index + '<style name="%s"' % name)
             if value:
                 self.g.write(' value="%s"' % escxml(str(value)))
             self.g.write('>\n')
             
-            for (start, end) in tag.ranges:
+            for (start, end) in style.ranges:
                 self.g.write(('  ' * (index + 1)) +
                              '<range start="%d" end="%d"/>\n' % (start, end))
             
-            self.g.write('  ' * index + '</tag>\n')
+            self.g.write('  ' * index + '</style>\n')
     
     def write_text(self, val, text, indent=0):
         if not text:
@@ -482,6 +524,10 @@ class GrampsXmlWriter(UpdateCallback):
 
         for s in person.get_source_references():
             self.dump_source_ref(s,index+2)
+
+        for tag_handle in person.get_tag_list():
+            self.write_ref("tagref", tag_handle, index+1)
+
         self.g.write("%s</person>\n" % sp)
 
     def write_family(self,family,index=1):
@@ -506,6 +552,10 @@ class GrampsXmlWriter(UpdateCallback):
         self.write_note_list(family.get_note_list(),index+1)
         for s in family.get_source_references():
             self.dump_source_ref(s,index+1)
+
+        for tag_handle in family.get_tag_list():
+            self.write_ref("tagref", tag_handle, index+1)
+
         self.g.write("%s</family>\n" % sp)
 
     def write_source(self,source,index=1):
@@ -544,6 +594,7 @@ class GrampsXmlWriter(UpdateCallback):
             self.g.write('%s<address%s>\n' % (sp,conf_priv(address)))
             self.write_date(address.get_date_object(),index+1)
             self.write_line("street",address.get_street(),index+1)
+            self.write_line("locality",address.get_locality(),index+1)
             self.write_line("city",address.get_city(),index+1)
             self.write_line("county",address.get_county(),index+1)
             self.write_line("state",address.get_state(),index+1)
@@ -683,13 +734,18 @@ class GrampsXmlWriter(UpdateCallback):
             priv = conf_priv(source_ref)
 
             if p == "" and n == [] and d.is_empty() and q == 2:
-                self.g.write('<sourceref hlink="%s"%s/>\n' % ("_"+source.get_handle(), priv))
+                self.g.write('<sourceref hlink="%s"%s/>\n'
+                                % ("_"+source.get_handle(), priv)
+                            )
             else:
                 if q == 2:
-                    self.g.write('<sourceref hlink="%s"%s>\n' % ("_"+source.get_handle(), priv))
+                    self.g.write('<sourceref hlink="%s"%s>\n'
+                                    % ("_"+source.get_handle(), priv)
+                                )
                 else:
-                    self.g.write('<sourceref hlink="%s" conf="%d"%s>\n' % (
-                            "_"+source.get_handle(),q, priv))
+                    self.g.write('<sourceref hlink="%s" conf="%d"%s>\n'
+                                    % ("_"+source.get_handle(), q, priv)
+                                )
                 self.write_line("spage",p,index+1)
                 self.write_note_list(n,index+1)
                 self.write_date(d,index+1)
@@ -705,22 +761,36 @@ class GrampsXmlWriter(UpdateCallback):
             self.g.write('%s<%s hlink="_%s"%s%s>\n'
                          % (sp,tagname, handle,extra_text,close_tag))
 
-    def write_primary_tag(self,tagname, obj,index=1,close=True):
+    def write_primary_tag(self, tagname, obj, index=1, close=True):
+        """
+        Write the tag attributes common to all primary objects.
+        """
         if not obj:
             return
-        sp = "  "*index
-        marker = obj.get_marker().xml_str()
-        if marker:
-            marker_text = ' marker="%s"' % escxml(marker)
-        else:
-            marker_text = ''
         priv_text = conf_priv(obj)
-        change_text = ' change="%d"' % obj.get_change_time()
-        handle_id_text = ' id="%s" handle="_%s"' % (escxml(obj.gramps_id), obj.handle)
-        obj_text = '%s<%s' % (sp,tagname)
+        id_text = ' id="%s"' % escxml(obj.gramps_id)
 
-        self.g.write(obj_text + handle_id_text + priv_text + marker_text +
-                     change_text)
+        self.write_table_tag(tagname, obj, index, False)
+        self.g.write(id_text + priv_text)
+        if close:
+            self.g.write('>\n')
+
+    def write_table_tag(self, tagname, obj, index=1, close=True):
+        """
+        Write the tag attributes common to all table objects.
+        """
+        if not obj:
+            return
+        sp = "  " * index
+        try:
+            change_text = ' change="%d"' %  obj.get_change_time()
+        except:
+            change_text = ' change="%d"' %  0
+            
+        handle_text = ' handle="_%s"' % obj.get_handle()
+        
+        obj_text = '%s<%s' % (sp, tagname)
+        self.g.write(obj_text + handle_text + change_text)
         if close:
             self.g.write('>\n')
 
@@ -732,16 +802,26 @@ class GrampsXmlWriter(UpdateCallback):
             if rel != "":
                 self.g.write('  %s<rel type="%s"/>\n' % (sp,rel) )
 
-    def write_last(self, name,indent=1):
-        p = name.get_surname_prefix()
-        n = name.get_surname()
-        g = name.get_group_as()
-        self.g.write('%s<last' % ('  '*indent))
-        if p:
-            self.g.write(' prefix="%s"' % escxml(p))
-        if g:
-            self.g.write(' group="%s"' % escxml(g))
-        self.g.write('>%s</last>\n' % self.fix(n))
+    def write_surname(self, surname, indent=1):
+        """
+        Writes a surname of the name
+        """
+        pre = surname.get_prefix()
+        con = surname.get_connector()
+        nam = surname.get_surname()
+        der = surname.get_origintype().xml_str()
+        pri = surname.get_primary()
+        self.g.write('%s<surname' % ('  '*indent))
+        if pre:
+            self.g.write(' prefix="%s"' % escxml(pre))
+        if not pri:
+            self.g.write(' prim="0"')
+        if con:
+            self.g.write(' connector="%s"' % escxml(con))
+        if der:
+            self.g.write(' derivation="%s"' % escxml(der))
+            
+        self.g.write('>%s</surname>\n' % self.fix(nam))
 
     def write_line(self,tagname,value,indent=1):
         if value:
@@ -804,9 +884,9 @@ class GrampsXmlWriter(UpdateCallback):
         else:
             dualdated_str = ''
 
-        newyear = date.get_new_year()
-        if newyear != gen.lib.Date.NEWYEAR_JAN1:
-            newyear_str = ' newyear="%d"' % newyear
+        newyear = date.newyear_to_str()
+        if newyear:
+            newyear_str = ' newyear="%s"' % newyear
         else:
             newyear_str = ''
 
@@ -864,12 +944,15 @@ class GrampsXmlWriter(UpdateCallback):
         if name.get_display_as() != 0:
             self.g.write(' display="%d"' % name.get_display_as())
         self.g.write('>\n')
-        self.write_line("first", name.get_first_name(),index+1)
-        self.write_line("call", name.get_call_name(),index+1)
-        self.write_last(name,index+1)
-        self.write_line("suffix", name.get_suffix(),index+1)
-        self.write_line("patronymic", name.get_patronymic(),index+1)
-        self.write_line("title", name.get_title(),index+1)
+        self.write_line("first", name.get_first_name(), index+1)
+        self.write_line("call", name.get_call_name(), index+1)
+        for surname in name.get_surname_list():
+            self.write_surname(surname,index+1)
+        self.write_line("suffix", name.get_suffix(), index+1)
+        self.write_line("title", name.get_title(), index+1)
+        self.write_line("nick", name.get_nick_name(), index+1)
+        self.write_line("familynick", name.get_family_nick_name(), index+1)
+        self.write_line("group", name.get_group_as(), index+1)
         if name.date:
             self.write_date(name.date,4)
         self.write_note_list(name.get_note_list(),index+1)
@@ -886,45 +969,51 @@ class GrampsXmlWriter(UpdateCallback):
 
     def build_place_title(self,loc):
         "Builds a title from a location"
-        city = self.fix(loc.get_city())
         street = self.fix(loc.get_street())
+        locality = self.fix(loc.get_locality())
+        city = self.fix(loc.get_city())
         parish = self.fix(loc.get_parish())
+        county = self.fix(loc.get_county())
         state = self.fix(loc.get_state())
         country = self.fix(loc.get_country())
-        county = self.fix(loc.get_county())
 
         value = ""
 
         if street:
             value = street
+        if locality:
+            value = self.append_value(value, locality)
         if city:
-            value = self.append_value(value,city)
+            value = self.append_value(value, city)
         if parish:
-            value = self.append_value(value,parish)
+            value = self.append_value(value, parish)
         if county:
-            value = self.append_value(value,county)
+            value = self.append_value(value, county)
         if state:
-            value = self.append_value(value,state)
+            value = self.append_value(value, state)
         if country:
-            value = self.append_value(value,country)
+            value = self.append_value(value, country)
         return value
 
     def dump_location(self,loc):
         "Writes the location information to the output file"
         if loc.is_empty():
             return
+        street = self.fix(loc.get_street())
+        locality = self.fix(loc.get_locality())
         city = self.fix(loc.get_city())
         parish = self.fix(loc.get_parish())
+        county = self.fix(loc.get_county())
         state = self.fix(loc.get_state())
         country = self.fix(loc.get_country())
-        county = self.fix(loc.get_county())
         zip_code = self.fix(loc.get_postal_code())
         phone = self.fix(loc.get_phone())
-        street = self.fix(loc.get_street())
         
         self.g.write('      <location')
         if street:
             self.g.write(' street="%s"' % street)
+        if locality:
+            self.g.write(' locality="%s"' % locality)
         if city:
             self.g.write(' city="%s"' % city)
         if parish:
@@ -944,9 +1033,10 @@ class GrampsXmlWriter(UpdateCallback):
     def write_attribute_list(self, list, indent=3):
         sp = '  ' * indent
         for attr in list:
-            self.g.write('%s<attribute%s type="%s" value="%s"' % \
+            self.g.write('%s<attribute%s type="%s" value="%s"' %
                          (sp,conf_priv(attr),escxml(attr.get_type().xml_str()),
-                         self.fix(attr.get_value())))
+                         self.fix(attr.get_value()))
+                         )
             slist = attr.get_source_references()
             nlist = attr.get_note_list()
             if (len(nlist)+len(slist)) == 0:
@@ -979,19 +1069,25 @@ class GrampsXmlWriter(UpdateCallback):
                 if corner2_x is None : corner2_x = 100
                 if corner2_y is None : corner2_y = 100
                 #don't output not set rectangle
-                if (corner1_x == corner1_y == corner2_x == corner2_y == 0) or \
-                   (corner1_x == corner1_y == 0 and corner2_x == corner2_y == 100
-                   ):
+                if (corner1_x == corner1_y == corner2_x == corner2_y == 0 or
+                   corner1_x == corner1_y == 0 and
+                   corner2_x == corner2_y == 100):
                     rect = None
-            if (len(proplist) + len(nreflist) + len(refslist)) == 0 \
-                    and rect is None:
+            if (len(proplist) + len(nreflist) + len(refslist) == 0 and
+                rect is None):
                 self.g.write("/>\n")
             else:
                 self.g.write(">\n")
                 if rect is not None :
                     self.g.write(' %s<region corner1_x="%d" corner1_y="%d" '
-                                 'corner2_x="%d" corner2_y="%d"/>\n' %
-                            (sp,corner1_x,corner1_y,corner2_x,corner2_y))
+                                 'corner2_x="%d" corner2_y="%d"/>\n' % (
+                                    sp,
+                                    corner1_x,
+                                    corner1_y,
+                                    corner2_x,
+                                    corner2_y
+                                    )
+                                )
                 self.write_attribute_list(proplist,indent+1)
                 for ref in refslist:
                     self.dump_source_ref(ref, indent+1)
@@ -1054,8 +1150,14 @@ class GrampsXmlWriter(UpdateCallback):
             else:
                 desc_text = ''
             path_text = '  href="%s"' % self.fix(url.get_path())
-            self.g.write('%s<url%s%s%s%s/>\n' % \
-                         (sp, priv_text, path_text, type_text, desc_text))
+            self.g.write('%s<url%s%s%s%s/>\n' % (
+                            sp,
+                            priv_text,
+                            path_text,
+                            type_text,
+                            desc_text
+                            )
+                        )
 
     def write_place_obj(self, place, index=1):
         self.write_primary_tag("placeobj", place, index)
@@ -1064,10 +1166,11 @@ class GrampsXmlWriter(UpdateCallback):
         longitude = self.fix(place.get_longitude())
         lat = self.fix(place.get_latitude())
         main_loc = place.get_main_location()
-        llen = len(place.get_alternate_locations()) \
-               + len(place.get_url_list()) + \
-               len(place.get_media_list()) + \
-               len(place.get_source_references())
+        llen = (len(place.get_alternate_locations()) +
+                len(place.get_url_list()) +
+                len(place.get_media_list()) +
+                len(place.get_source_references())
+               )
                                                       
         ml_empty = main_loc.is_empty()
 
@@ -1115,6 +1218,10 @@ class GrampsXmlWriter(UpdateCallback):
             self.write_date(dval, index+1)
         for s in obj.get_source_references():
             self.dump_source_ref(s, index+1)
+
+        for tag_handle in obj.get_tag_list():
+            self.write_ref("tagref", tag_handle, index+1)
+
         self.g.write("%s</object>\n" % ("  "*index))
 
 #-------------------------------------------------------------------------
@@ -1147,7 +1254,7 @@ def conf_priv(obj):
 # export_data
 #
 #-------------------------------------------------------------------------
-def export_data(database, filename, option_box=None, callback=None):
+def export_data(database, filename, msg_callback, option_box=None, callback=None):
     """
     Call the XML writer with the syntax expected by the export plugin.
     """
@@ -1162,29 +1269,9 @@ def export_data(database, filename, option_box=None, callback=None):
 
     if option_box:
         option_box.parse_options()
-    
-        if option_box.private:
-            database = gen.proxy.PrivateProxyDb(database)
-    
-        if option_box.restrict:
-            database = gen.proxy.LivingProxyDb(
-                database, gen.proxy.LivingProxyDb.MODE_INCLUDE_LAST_NAME_ONLY)
-    
-        # Apply the Person Filter
-        if not option_box.cfilter.is_empty():
-            database = gen.proxy.FilterProxyDb(database, option_box.cfilter)
-    
-        # Apply the Note Filter
-        if not option_box.nfilter.is_empty():
-            database = gen.proxy.FilterProxyDb(
-                database, note_filter=option_box.nfilter)
-        
-        # Apply the ReferencedProxyDb to remove any objects not referenced
-        # after any of the other proxies have been applied
-        if option_box.unlinked:
-            database = gen.proxy.ReferencedProxyDb(database)
+        database = option_box.get_filtered_database(database)
 
-    g = XmlWriter(database, callback, 0, compress)
+    g = XmlWriter(database, msg_callback, callback, 0, compress)
     return g.write(filename)
 
 #-------------------------------------------------------------------------
@@ -1197,9 +1284,10 @@ class XmlWriter(GrampsXmlWriter):
     Writes a database to the XML file.
     """
 
-    def __init__(self, dbase, callback, strip_photos, compress=1):
+    def __init__(self, dbase, msg_callback, callback, strip_photos, compress=1):
         GrampsXmlWriter.__init__(
             self, dbase, strip_photos, compress, const.VERSION, callback)
+        self.msg_callback = msg_callback
         
     def write(self, filename):
         """
@@ -1210,5 +1298,5 @@ class XmlWriter(GrampsXmlWriter):
             ret = GrampsXmlWriter.write(self, filename)
         except DbWriteFailure, msg:
             (m1,m2) = msg.messages()
-            ErrorDialog(m1, m2)
+            self.msg_callback(m1, m2)
         return ret
