@@ -18,7 +18,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: export.gpr.py 14888 2010-03-21 10:32:36Z bmcage $
+# $Id: export.gpr.py 16543 2011-02-02 18:33:16Z romjerome $
 
 #------------------------------------------------------------------------
 #
@@ -32,7 +32,7 @@ plg.name  = _("Comma Separated Values Spreadsheet (CSV)")
 plg.name_accell  = _("Comma _Separated Values Spreadsheet (CSV)")
 plg.description =  _("CSV is a common spreadsheet format.")
 plg.version = '1.0'
-plg.gramps_target_version = '3.2'
+plg.gramps_target_version = '3.3'
 plg.status = STABLE
 plg.fname = 'ExportCsv.py'
 plg.ptype = EXPORT
@@ -53,12 +53,12 @@ plg.name  = _('Web Family Tree')
 plg.name_accell  = _('_Web Family Tree')
 plg.description =  _("Web Family Tree format")
 plg.version = '1.0'
-plg.gramps_target_version = '3.2'
+plg.gramps_target_version = '3.3'
 plg.status = STABLE
 plg.fname = 'ExportFtree.py'
 plg.ptype = EXPORT
 plg.export_function = 'writeData'
-plg.export_options = 'FtreeWriterOptionBox'
+plg.export_options = 'WriterOptionBox'
 plg.export_options_title = ('Web Family Tree export options')
 plg.extension = "wft"
 
@@ -75,7 +75,7 @@ plg.name_accell  = _('GE_DCOM')
 plg.description =  _('GEDCOM is used to transfer data between genealogy programs. '
                 'Most genealogy software will accept a GEDCOM file as input.')
 plg.version = '1.0'
-plg.gramps_target_version = '3.2'
+plg.gramps_target_version = '3.3'
 plg.status = STABLE
 plg.fname = 'ExportGedcom.py'
 plg.ptype = EXPORT
@@ -96,12 +96,12 @@ plg.name  = _('GeneWeb')
 plg.name_accell  = _('_GeneWeb')
 plg.description =  _('GeneWeb is a web based genealogy program.')
 plg.version = '1.0'
-plg.gramps_target_version = '3.2'
+plg.gramps_target_version = '3.3'
 plg.status = STABLE
 plg.fname = 'ExportGeneWeb.py'
 plg.ptype = EXPORT
 plg.export_function = 'exportData'
-plg.export_options = 'GeneWebWriterOptionBox'
+plg.export_options = 'WriterOptionBox'
 plg.export_options_title = ('GeneWeb export options')
 plg.extension = "gw"
 
@@ -118,7 +118,7 @@ plg.name_accell  = _('Gra_mps XML Package (family tree and media)')
 plg.description =  _('Gramps package is an archived XML family tree together '
                  'with the media object files.')
 plg.version = '1.0'
-plg.gramps_target_version = '3.2'
+plg.gramps_target_version = '3.3'
 plg.status = STABLE
 plg.fname = 'ExportPkg.py'
 plg.ptype = EXPORT
@@ -141,7 +141,7 @@ plg.description =  _('Gramps XML export is a complete archived XML backup of a'
                  ' Gramps family tree without the media object files.'
                  ' Suitable for backup purposes.')
 plg.version = '1.0'
-plg.gramps_target_version = '3.2'
+plg.gramps_target_version = '3.3'
 plg.status = STABLE
 plg.fname = 'ExportXml.py'
 plg.ptype = EXPORT
@@ -160,14 +160,14 @@ plg = newplugin()
 plg.id    = 'ex_vcal'
 plg.name  = _('vCalendar')
 plg.name_accell  = _('vC_alendar')
-plg.description =  _('vCalendar is used in many calendaring and pim applications.')
+plg.description =  _('vCalendar is used in many calendaring and PIM applications.')
 plg.version = '1.0'
-plg.gramps_target_version = '3.2'
+plg.gramps_target_version = '3.3'
 plg.status = STABLE
 plg.fname = 'ExportVCalendar.py'
 plg.ptype = EXPORT
 plg.export_function = 'exportData'
-plg.export_options = 'CalendarWriterOptionBox'
+plg.export_options = 'WriterOptionBox'
 plg.export_options_title = ('vCalendar export options')
 plg.extension = "vcs"
 
@@ -183,11 +183,11 @@ plg.name  = _('vCard')
 plg.name_accell  = _('_vCard')
 plg.description =  _('vCard is used in many addressbook and pim applications.')
 plg.version = '1.0'
-plg.gramps_target_version = '3.2'
+plg.gramps_target_version = '3.3'
 plg.status = STABLE
 plg.fname = 'ExportVCard.py'
 plg.ptype = EXPORT
 plg.export_function = 'exportData'
-plg.export_options = 'CardWriterOptionBox'
+plg.export_options = 'WriterOptionBox'
 plg.export_options_title = ('vCard export options')
 plg.extension = "vcf"
