@@ -4,6 +4,7 @@
 # Copyright (C) 2000-2007  Donald N. Allingham
 # Copyright (C) 2007-2008  Brian G. Matherly
 # Copyright (C) 2008       Douglas S. Blank
+# Copyright (C) 2010       Jakim Friant
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,7 +21,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: DescendGramplet.py 14091 2010-01-18 04:42:17Z pez4brian $
+# $Id: DescendGramplet.py 16979 2011-03-30 01:50:08Z pez4brian $
 
 """Descendant Gramplet"""
 
@@ -37,7 +38,7 @@ from gen.ggettext import gettext as _
 #
 #------------------------------------------------------------------------
 from gen.plug import Gramplet
-from ReportBase import ReportUtils
+from gen.plug.report import utils as ReportUtils
 from gen.display.name import displayer as name_displayer
 import DateHandler
 from gen.utils import get_birth_or_fallback, get_death_or_fallback
@@ -63,6 +64,8 @@ class DescendantGramplet(Gramplet):
             return
         self.set_text("")
         self.center_person = self.dbstate.db.get_person_from_handle(active_handle)
+        if self.center_person is None:
+            return
         name = name_displayer.display(self.center_person)
         title = _("Descendants of %s") % name
         self.append_text(title)
