@@ -20,7 +20,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-# $Id: EditExifMetadata.py 17749 2011-06-12 02:58:47Z robhealey1 $
+# $Id: EditExifMetadata.py 17877 2011-07-01 02:17:20Z robhealey1 $
 
 # ************************************************************************
 # Python Modules
@@ -28,7 +28,7 @@
 import os
 from datetime import datetime, date
 import calendar, time
-
+import subprocess
 
 # abilty to escape certain characters from output...
 from xml.sax.saxutils import escape as _html_escape
@@ -39,7 +39,10 @@ from decimal import Decimal, getcontext
 getcontext().prec = 4
 from fractions import Fraction
 
-import subprocess
+try:
+    from PIL import Image
+except ImportError:
+    from python.PIL import Image
 
 # ------------------------------------------------------------------------
 # GTK modules
@@ -62,7 +65,7 @@ import gen.mime
 import Utils
 from PlaceUtils import conv_lat_lon
 
-from QuestionDialog import WarningDialog, QuestionDialog
+from QuestionDialog import WarningDialog, QuestionDialog, OptionDialog
 
 from gen.ggettext import gettext as _
 
@@ -107,59 +110,19 @@ if (software_version and (software_version < Min_VERSION)):
     WarningDialog(msg)
     raise Exception(msg)
 
-# *******************************************************************
-#         Determine if we have access to outside Programs
-#
-# The programs are ImageMagick, and jhead
-# * ImageMagick -- Convert and Delete all Exif metadata...
-# * jhead       -- re-initialize a jpeg, and other features...
-#********************************************************************
-# Windows 32bit systems
-system_platform = os.sys.platform
-MAGICK_FOUND_ = False
-JHEAD_FOUND_ = False
-if system_platform == "win32":
-    if Utils.search_for("convert.exe"):
-        MAGICK_FOUND_ = "convert.exe"
-
-    if Utils.search_for("jhead.exe"):
-        _JHEAD_FOUND_ = "jhead.exe"
-
-elif system_platform == "linux2":
-    if Utils.search_for("convert"):
-        MAGICK_FOUND_ = "convert"
-
-    if Utils.search_for("jhead"):
-        JHEAD_FOUND_ = "jhead"
-
-else:
-    if Utils.search_for("convert"):
-        MAGICK_FOUND_ = "convert"
-
-    if Utils.search_for("jhead"):
-        JHEAD_FOUND_ = "jhead"
-
-# if external programs are not found, let the user know about 
-# the missing functionality?
-if not MAGICK_FOUND_:
-    print(_("ImageMagick's convert program was not found "
-        "on this computer.\n  You may download it from "
-        "here: %s...") % ("http://www.imagemagick.org/script/index.php"))
-
-if not JHEAD_FOUND_:
-    print(_("Jhead program was not found on this computer.\n"
-        "You may download it from: %s...") % (
-            "http://www.sentex.net/~mwandel/jhead/"))
-
 # ------------------------------------------------------------------------
 # Constants
 # ------------------------------------------------------------------------
 # available image types for exiv2 and pyexiv2
 # ["jpeg", "jpg", "exv", "tiff", "dng", "nef", "pef", "pgf", "png", "psd", "jp2"]
 
+if os.sys.platform == "win32":
+    EXIV2_FOUND_ = "exiv2.exe" if Utils.search_for("exiv2.exe") else False
+else:
+    EXIV2_FOUND_ = "exiv2" if Utils.search_for("exiv2") else False
+
 # define tooltips for all entries
 _TOOLTIPS = {
-
     # Description...
     "Description"       : _("Provide a short descripion for this image."),
 
@@ -172,29 +135,18 @@ _TOOLTIPS = {
     "Copyright" : _("Enter the copyright information for this image. \n"
         "Example: (C) 2010 Smith and Wesson"),
 
-    # Calendar date select...
-    "Date:Select" : _("Allows you to select a date from a pop-up window "
-        "calendar. \n  Warning:  You will still need to edit the time..."),
-
     # Original Date/ Time... 
     "DateTime" : _("Original Date/ Time of this image.\n"
         "Example: 1826-Apr-12 14:30:00, 1826-April-12, 1998-01-31 13:30:00"),
 
-    # Convert to decimal button...
-    "GPSFormat:Decimal" : _("Converts Degree, Minutes, Seconds GPS "
-        "coordinates to a Decimal representation."),
-
-    # convert to degrees, minutes, seconds button...
-    "GPSFormat:DMS" : _("Converts Decimal GPS coordinates "
-        "to a Degrees, Minutes, Seconds representation."),
-
     # GPS Latitude...
     "Latitude" : _(u"Enter the GPS Latitude coordinates for your image,\n"
         u"Example: 43.722965, 43 43 22 N, 38° 38′ 03″ N, 38 38 3"),
 
     # GPS Longitude...
     "Longitude" : _(u"Enter the GPS Longitude coordinates for your image,\n"
-        u"Example: 10.396378, 10 23 46 E, 105° 6′ 6″ W, -105 6 6") }.items()
+        u"Example: 10.396378, 10 23 46 E, 105° 6′ 6″ W, -105 6 6") }
+_TOOLTIPS = dict( [widget, tooltip] for widget, tooltip in _TOOLTIPS.items() )
 
 # set up Exif keys for Image.exif_keys
 _DATAMAP = {
@@ -225,19 +177,15 @@ _BUTTONTIPS = {
 
     # Save Exif Metadata button...
     "Save" : _("Saves/ writes the Exif metadata to this image.\n"
-        "WARNING: Exif metadata will be erased if you save a blank entry field...") }
-
-# if ImageMagick is installed on this computer then, add button 
-# tooltips for these two buttons...
-if MAGICK_FOUND_:
-    _BUTTONTIPS.update( {
-
-        # Convert to .Jpeg button...
-        "Convert" : _("If your image is not a jpeg image, convert it to jpeg?"),
+        "WARNING: Exif metadata will be erased if you save a blank entry field..."),
 
         # Delete/ Erase/ Wipe Exif metadata button...
         "Delete" : _("WARNING:  This will completely erase all Exif metadata "
-            "from this image!  Are you sure that you want to do this?") } )
+            "from this image!  Are you sure that you want to do this?"),
+
+        # Convert to .Jpeg button...
+        "Convert" : _("If your image is not an exiv2 compatible image, convert it?") }
+_BUTTONTIPS = dict( [widget, tooltip] for widget, tooltip in _BUTTONTIPS.items() )
 
 # ------------------------------------------------------------------------
 #
@@ -245,7 +193,13 @@ if MAGICK_FOUND_:
 #
 # ------------------------------------------------------------------------
 class EditExifMetadata(Gramplet):
+    """
+    Special symbols...
 
+    degrees symbol = [Ctrl] [Shift] u \00b0
+    minutes symbol =                  \2032
+    seconds symbol =                  \2033
+    """
     def init(self):
 
         self.exif_column_width = 12
@@ -312,12 +266,9 @@ class EditExifMetadata(Gramplet):
         ccc_box.add( self.__create_button(
             "Clear", False, [self.clear_metadata], gtk.STOCK_CLEAR, False) )
 
-        # is ImageMagick installed?
-        if MAGICK_FOUND_:
-            # Convert button...
-            ccc_box.add( self.__create_button(
-                "Convert", False, [self.__convert_dialog],
-                    gtk.STOCK_CONVERT, False) )
+        # Convert button...
+        ccc_box.add( self.__create_button(
+            "Convert", False, [self.__convert_dialog], gtk.STOCK_CONVERT, False) )
 
         for items in [
 
@@ -368,10 +319,9 @@ class EditExifMetadata(Gramplet):
             "Save", False, [self.save_metadata, self.update, self.display_exif_tags, self.copyto],
                 gtk.STOCK_SAVE, False))
 
-        if MAGICK_FOUND_:
-            # Delete All Metadata button...
-            hsd_box.add(self.__create_button(
-                "Delete", False, [self.__delete_dialog], gtk.STOCK_DELETE, False))
+        # Delete All Metadata button...
+        hsd_box.add(self.__create_button(
+            "Delete", False, [self.__delete_dialog], gtk.STOCK_DELETE, False))
 
         # adds Exif Metadata Viewing Area
         vbox.pack_start(view, padding =10)
@@ -416,9 +366,9 @@ class EditExifMetadata(Gramplet):
 
         return False
 
-    def _help_page(self):
+    def _help_page(self, object):
         """
-        will bring up a Wiki help page.
+        will bring up a Wiki help page on your web browser.
         """
 
         GrampsDisplay.help(webpage = "Edit Image Exif Metadata")
@@ -444,15 +394,16 @@ class EditExifMetadata(Gramplet):
         """
         db = self.dbstate.db
 
+        # set all data fields to editable...
+        for widget in _TOOLTIPS.keys():
+            self.exif_widgets[widget].set_editable(True)
+
         # clear Display and Edit Areas
         self.clear_metadata(self.orig_image)
         self.model.clear()
 
         # De-activate the buttons except for Help...
-        self.deactivate_buttons(["CopyTo", "Clear", "Save"])
-
-        if MAGICK_FOUND_:
-            self.deactivate_buttons(["Convert", "Delete"])
+        self.deactivate_buttons(["CopyTo", "Clear", "Save", "Convert", "Delete"])
 
         # Re-post initial image message...
         self.exif_widgets["Message:Area"].set_text(_("Select an "
@@ -466,8 +417,10 @@ class EditExifMetadata(Gramplet):
         self.orig_image = db.get_object_from_handle(active_handle)
         self.image_path = Utils.media_path_full(db, self.orig_image.get_path() )
         if (not self.orig_image or not os.path.isfile(self.image_path)):
+
+            # set Message Area to Missing/ Delete...
             self.exif_widgets["Message:Area"].set_text(_("Image is either "
-            "missing or deleted,\n  Choose a different image..."))
+                "missing or deleted,\n  Choose a different image..."))
             return
 
         # check image read privileges...
@@ -483,6 +436,10 @@ class EditExifMetadata(Gramplet):
             self.exif_widgets["Message:Area"].set_text(_("Image is NOT writable,\n"
                 "You will NOT be able to save Exif metadata...."))
 
+        # display file description/ title...
+        self.exif_widgets["Media:Label"].set_text( _html_escape(
+            self.orig_image.get_description()) )
+
         # Mime type information...
         mime_type = self.orig_image.get_mime_type()
         _mtype = gen.mime.get_description(mime_type)
@@ -493,32 +450,31 @@ class EditExifMetadata(Gramplet):
             if mime_type.startswith("image"):
                 self.activate_buttons(["Save"])
 
-                # display file description/ title...
-                self.exif_widgets["Media:Label"].set_text( _html_escape(
-                    self.orig_image.get_description()))
-
                 # will create the image and read it...
                 self.setup_image(self.image_path, True)
 
-                # Checks to make sure that ImageMagick is installed on 
-                # this computer and the image is NOT a jpeg image...
-                if MAGICK_FOUND_:
-                    basename, extension = os.path.splitext(self.image_path)
-                    if extension not in [".jpeg", ".jpg", ".jfif"]:
-                        self.activate_buttons(["Convert"])
+                self.basename, self.extension = os.path.splitext(self.image_path)
+                if self.extension not in [".jpeg", ".jpg", ".jfif"]:
+                    self.activate_buttons(["Convert"])
 
-                # displays the imge Exif metadata
+                # displays the imge Exif metadata...
                 self.display_exif_tags()
+                imagefile = True
 
             else:
                 self.exif_widgets["Message:Area"].set_text(_("Choose a "
                     "different image..."))
-                return
-
+                imagefile = False
         else:
             self.exif_widgets["Message:Area"].set_text(_("Choose a "
                 "different image..."))
-            return
+            imagefile = False
+
+        if not imagefile:
+
+            # set all data fields to not editable and set a connect to the data fields...
+            for widget in _TOOLTIPS.keys():
+                self.exif_widgets[widget].set_editable(False)
 
     def __create_column(self, name, colnum, fixed =True):
         """
@@ -561,16 +517,6 @@ class EditExifMetadata(Gramplet):
 
         return button
 
-    def __convert_dialog(self, object):
-        """
-        Handles the Convert question Dialog...
-        """
-
-        # is ImageMagick installled?
-        if MAGICK_FOUND_:
-            QuestionDialog(_("Edit Image Exif Metadata"), _("Convert this "
-                "image to a .jpeg image?"), _("Convert"), self.convert2Jpeg)
-
     def __delete_dialog(self, object):
         """
         Handles the Delete Dialog...
@@ -580,6 +526,81 @@ class EditExifMetadata(Gramplet):
             "about to completely delete the Exif metadata from this image?"),
             _("Delete"), self.strip_metadata)
 
+    def __convert_dialog(self, object):
+        """
+        Handles the Convert question Dialog...
+        """
+
+        # Convert and delete original file...
+        OptionDialog(_("Edit Image Exif Metadata"), _("WARNING: You are "
+            "about to convert this image into an .tiff image.  Tiff "
+            "images are the industry standard for lossless compression.\n\n"
+            "Are you sure that you want to do this?"), _("Convert and Delete"),
+            self.__convert_delete, _("Convert"), self.__convert_file)
+
+    def __convert_file(self, full_path =None):
+        """
+        Will attempt to convert an image to jpeg if it is not?
+        """
+
+        if full_path is None:
+            full_path = self.image_path
+
+        basename, oldext = os.path.splitext(full_path)
+        filepath, basename = os.path.split(basename)
+        basename += ".tiff"
+
+        dest_file = os.path.join(filepath, basename)
+
+        # Convert the file based upon file suffix
+        im = Image.open(full_path)
+        im.save(dest_file)
+
+        if LesserVersion:  # prior to pyexiv2-0.2.0...
+            src_meta = pyexiv2.Image(full_path)
+
+            # Copy the image metadata...
+            dest_meta = pyexiv2.Image(dest_file)
+            dest_meta.readMetadata()
+            src_meta.readMetadata()
+            src_meta.copy(dest_meta, comment =False)
+            dest_meta.writeMetadata()
+
+        else:  # pyexiv2-0.2.0 and above...
+            src_meta = pyexiv2.ImageMetadata(full_path)
+
+            # Copy the image metadata...
+            dest_meta = pyexiv2.ImageMetadata(dest_file)
+            src_meta.read()
+            dest_meta.read()
+            src_meta.copy(dest_meta, comment =False)
+            dest_meta.write()
+
+    def __convert_delete(self, full_path =None):
+        """
+        will convert an image file and delete original non-jpeg image.
+        """
+        self.db = self.dbstate.db
+
+        if full_path is None:
+            full_path = self.image_path
+
+        # Convert image and copy over it's Exif metadata (if any?)
+        self.__convert_file(full_path)
+
+        # delete original file from this computer...
+        try:
+            os.remove(full_path)
+            delete_results = True
+        except (IOError, OSError):
+            delete_results = False
+
+        if delete_results:
+            filepath, __basename = os.path.split(self.basename)
+
+            self.exif_widgets["Message:Area"].set_text(_("Your image has been "
+                "converted and the original file has been deleted..."))
+
     def setup_image(self, full_path, createimage =False):
         """
         will return an image instance and read the Exif metadata.
@@ -703,23 +724,16 @@ class EditExifMetadata(Gramplet):
             all of the image Exif metadata...
         """
 
-        mediadatatags_ = _get_exif_keypairs(self.plugin_image)
-        if not mediadatatags_:
-            return
-
         self.model.clear()
 
-        # set has data flag...
-        self.set_has_data(len(mediadatatags_) > 0)
-
-        # Activate Clear and Save buttons...
-        self.activate_buttons(["Clear", "Save"])
-
-        # check to see if we got metadata from the media object?
+        mediadatatags_ = _get_exif_keypairs(self.plugin_image)
         if mediadatatags_:
 
-            # activate copyto button...
-            self.activate_buttons(["CopyTo"])
+            # set has data flag...
+            self.set_has_data(True)
+
+            # Activate Clear and Save buttons...
+            self.activate_buttons(["Clear", "Save", "CopyTo", "Delete"])
 
             # set Message Area to Display...
             self.exif_widgets["Message:Area"].set_text(_("Displaying image "
@@ -730,7 +744,6 @@ class EditExifMetadata(Gramplet):
                 if LesserVersion:
                     label = self.plugin_image.tagDetails(keytag)[0]
                     human_value = self.plugin_image.interpretedExifValue(keytag)
-
                 else:
                     try:
                         tag = self.plugin_image[keytag]
@@ -747,12 +760,21 @@ class EditExifMetadata(Gramplet):
                 if human_value is not False:
                     self.model.append((self.plugin_image, label, human_value))
 
+        # there is no Exif metadata for this image yet...
+        else:
+
+            self.set_has_data(False)
+
+            # set Message Area to None...
+            self.exif_widgets["Message:Area"].set_text(_("No Exif metadata for this image..."))
+
     def copyto(self, imagekeytags_ =None):
         """
         reads the image metadata after the pyexiv2.Image has been created
         """
 
         imagekeytags_ = _get_exif_keypairs(self.plugin_image)
+
         if imagekeytags_:
             imagekeytags_ = [keytag for keytag in imagekeytags_ if keytag in _DATAMAP]
 
@@ -815,9 +837,6 @@ class EditExifMetadata(Gramplet):
         # enable Save button after metadata has been "Copied to Edit Area"...
         self.activate_buttons(["Save"])
 
-        if MAGICK_FOUND_:
-            self.activate_buttons(["Delete"])
-
         # Clear the Message Area...
         self.exif_widgets["Message:Area"].set_text("")
 
@@ -843,28 +862,6 @@ class EditExifMetadata(Gramplet):
         else:
              self.exif_widgets["DateTime"].set_text("")
 
-    def convert2Jpeg(self):
-        """
-        Will attempt to convert an image to jpeg if it is not?
-        """
-
-        # if ImageMagick's convert is installed...
-        if MAGICK_FOUND_:
-
-            filepath, basename = os.path.split(self.image_path)
-            basename, oldext = os.path.splitext(self.image_path)
-            newextension = ".jpeg"
-
-            convert = subprocess.check_call(["convert", self.image_path, 
-                    os.path.join(filepath, basename + newextension) ] )
-            if str(convert):
-
-                # set Message Area to Convert...
-                self.exif_widgets["Message:Area"].set_text(_("Converting image,\n"
-                    "You will need to delete the original image file..."))
-
-                self.deactivate_buttons(["Convert"])
-
     def _set_exif_keytag(self, keytag, KeyValue):
         """
         sets the value for the metadata keytags
@@ -1056,8 +1053,9 @@ class EditExifMetadata(Gramplet):
                 self._set_exif_keytag(_DATAMAP["DateTime"], datetime_)
 
         # Latitude/ Longitude data fields...
-        self.__process_lat_long( self.exif_widgets["Latitude"].get_text(),
-                                self.exif_widgets["Longitude"].get_text() )
+        latitude  =  self.exif_widgets["Latitude"].get_text()
+        longitude = self.exif_widgets["Longitude"].get_text()
+        self.__process_lat_long(latitude, longitude)
 
         if datatags:
             # set Message Area to Saved...
@@ -1072,75 +1070,78 @@ class EditExifMetadata(Gramplet):
         # writes all Exif Metadata to image even if the fields are all empty.
         self.write_metadata(self.plugin_image)
 
-        # Activate Delete button...
-        if MAGICK_FOUND_:
-            self.activate_buttons(["Delete"])
+        self.activate_buttons(["Delete"])
 
     def __process_lat_long(self, latitude, longitude):
         """
         process the latitude/ longitude for saving...
         """
 
-        if not latitude and not longitude:
-            return False, False
+        if (latitude and longitude):
 
-        # complete some error checking to prevent crashes...
-        # if "?" character exist, remove it?
-        if "?" in latitude:
-            latitude = latitude.replace("?", "")
-        if "?" in longitude:
-            longitude = longitude.replace("?", "")
-
-        # if "," character exists, remove it?
-        if "," in latitude: 
-            latitude = latitude.replace(",", "")
-        if "," in longitude:
-            longitude = longitude.replace(",", "") 
-
-        # if there is no spaces then convert to DMS?
-        if latitude.find(" ") == longitude.find(" "):
-            if ((latitude.find(".") is not -1) and (longitude.find(".") is not -1)):
-                latitude, longitude = self.convert2dms(latitude, longitude)
-
-        # DMS is True...
-        if ((latitude.find(" ") is not -1) and (longitude.find(" ") is not -1)):
-
-            if latitude.find("N") > -1:
-                latituderef = "N"
-                latitude = latitude.replace("N", "")
-            elif latitude.find(_("S")) > -1:
-                latituderef = "S"
-                latitude = latitude.replace("S", "")
-            elif ((latitude.find(_("N")) == -1) and (latitude.find(_("S")) == -1)):
-                if latitude.find("-") == -1:
-                    latituderef = "N"
-                else:
-                    latituderef = "S"
-                    latitude = latitude.replace("-", "")
-
-            if longitude.find("E") > -1:
-                longituderef = "E"
-                longitude = longitude.replace("E", "")
-            elif longitude.find("W") > -1:
-                longituderef = "W"
-                longitude = longitude.replace("W", "")
-            elif ((longitude.find("E") == -1) and (longitude.find("W") == -1)):
-                if longitude.find("-") == -1:
-                    longituderef = "E"
-                else:
-                    longituderef = "W"
-                    longitude = longitude.replace("-", "")
-
-            # remove symbols before saving Latitude/ Longitude GPS coordinates
-            latitude, longitude = _removesymbolsb4saving(latitude, longitude) 
-
-            # remove leading and trailing whitespace
+            # remove leading and trailing whitespace...
             latitude  =  latitude.strip()
             longitude = longitude.strip()
- 
-            # convert to pyexiv2.Rational for saving...
-            latitude  =  coords_to_rational(latitude)
-            longitude = coords_to_rational(longitude)
+
+            for chrs in ["/", ","]:
+                if chrs in latitude:
+                    latitude = latitude.replace(chrs, "")
+                if chrs in longitude:
+                    longitude = longitude.replace(chrs, "")
+
+            # if there is no spaces then convert to DMS?
+            if (latitude.find(" ") == longitude.find(" ") == -1):
+                if ("." in latitude and "." in longitude):
+                    latitude, longitude = self.convert2dms(latitude, longitude)
+
+            # DMS is True...
+            if ((latitude.find(" ") is not -1) and (longitude.find(" ") is not -1)):
+
+                if latitude.find("N") > -1:
+                    latituderef = "N"
+                    latitude = latitude.replace("N", "")
+
+                elif latitude.find(_("S")) > -1:
+                    latituderef = "S"
+                    latitude = latitude.replace("S", "")
+
+                elif ((latitude.find(_("N")) == -1) and (latitude.find(_("S")) == -1)):
+                    if latitude.find("-") == -1:
+                        latituderef = "N"
+                    else:
+                        latituderef = "S"
+                        latitude = latitude.replace("-", "")
+
+                if longitude.find("E") > -1:
+                    longituderef = "E"
+                    longitude = longitude.replace("E", "")
+
+                elif longitude.find("W") > -1:
+                    longituderef = "W"
+                    longitude = longitude.replace("W", "")
+
+                elif ((longitude.find("E") == -1) and (longitude.find("W") == -1)):
+                    if longitude.find("-") == -1:
+                        longituderef = "E"
+                    else:
+                        longituderef = "W"
+                        longitude = longitude.replace("-", "")
+
+                # remove leading and trailing whitespace...
+                latitude  =  latitude.strip()
+                longitude = longitude.strip()
+
+                # remove symbols before saving Latitude/ Longitude GPS coordinates
+                latitude, longitude = _removesymbolsb4saving(latitude, longitude) 
+
+                # convert to pyexiv2.Rational for saving...
+                latitude  =  coords_to_rational(latitude)
+                longitude = coords_to_rational(longitude)
+
+            # save empty strings to delete Latitude, Longitude...
+            else:
+                latitude, latituderef = "", ""
+                longitude, longituderef = "", ""
 
             # save Latitude and LatitudeRef
             self._set_exif_keytag(_DATAMAP["Latitude"], latitude)
@@ -1150,51 +1151,44 @@ class EditExifMetadata(Gramplet):
             self._set_exif_keytag(_DATAMAP["Longitude"], longitude)
             self._set_exif_keytag(_DATAMAP["LongitudeRef"], longituderef)
 
-    def strip_metadata(self, erase_results =None):
+    def strip_metadata(self, mediadatatags =None):
         """
         Will completely and irrevocably erase all Exif metadata from this image.
         """
 
-        if MAGICK_FOUND_:
-            erase = subprocess.check_call( ["convert", self.image_path,
-                "-strip", self.image_path] )
-            erase_results = str(erase)
+        # make sure the image has Exif metadata...
+        mediadatatags_ = _get_exif_keypairs(self.plugin_image)
+        if not mediadatatags_:
+            return
 
-        else:
-            mediadatatags_ = _get_exif_keypairs(self.plugin_image)
-            if mediadatatags_: 
-                for keytag in mediadatatags_:
-                    del self.plugin_image[keytag]
-                erase_results = True
+        try:
+            erase = subprocess.check_call( [EXIV2_FOUND_, "delete", self.image_path] )
+            erase_results = str(erase) 
+        except subprocess.CalledProcessError:
+            erase_results = False
 
-                # write wiped metadata to image...
-                self.write_metadata(self.plugin_image)
+        if not erase_results:
+            for keytag_ in mediadatatags_:
+                del self.plugin_image[keytag_]
+            erase_results = True
+
+            # write wiped metadata to image...
+            self.write_metadata(self.plugin_image)
 
         if erase_results:
+            for widget in ["Media:Label", "Mime:Type", "Message:Area"]:
+                self.exif_widgets[widget].set_text("") 
 
-            # set Message Area for deleting...
-            self.exif_widgets["Message:Area"].set_text(_("Deleting all Exif metadata..."))
-
-            # Clear the Display and Edit Areas
-            self.clear_metadata(self.plugin_image)
+            # Clear the Viewing Area...
             self.model.clear()
 
-            # Notify the User...
-            self.exif_widgets["Message:Area"].set_text(_("All Exif metadata has been "
-                    "deleted from this image..."))
+            self.exif_widgets["Message:Area"].set_text(_("All Exif metadata "
+                "has been deleted from this image..."))
             self.update()
+        else:
+            self.exif_widgets["Message:Area"].set_text(_("There was an error "
+                "in stripping the Exif metadata from this image..."))
 
-            # re- initialize the image...
-            if JHEAD_FOUND_:
-                reinit = subprocess.check_call( ["jhead", "-purejpg", self.image_path] )
-                reinitialize = str(reinit)
-                if reinitialize:
-                    self.exif_widgets["Message:Area"].set_text(_("Image has "
-                    "be re- initialized for Exif metadata..."))
-
-# -----------------------------------------------
-#              Date Calendar functions
-# -----------------------------------------------
     def select_date(self, object):
         """
         will allow you to choose a date from the calendar widget
@@ -1282,24 +1276,26 @@ def _removesymbolsb4saving(latitude, longitude):
     """
     will recieve a DMS with symbols and return it without them
 
-    @param: latitude -- Latitude GPS coordinates
+    @param: latitude  --  Latitude GPS coordinates
     @param: longitude -- GPS Longitude coordinates
     """
 
     # check to see if latitude/ longitude exist?
     if (latitude and longitude):
 
-        # remove degrees symbol if it exist?
-        latitude = latitude.replace("°", "")
-        longitude = longitude.replace("°", "")
+        # remove degrees, minutes, seconds symbols if they exist in 
+        # Latitude/ Longitude...
+        for symbol in ["°", "#", "먊", "′", "'", '″', '"']:
 
-        # remove minutes symbol if it exist?
-        latitude = latitude.replace("′", "")
-        longitude = longitude.replace("′", "")
+            if symbol in latitude:
+                latitude = latitude.replace(symbol, "")
 
-        # remove seconds symbol if it exist?
-        latitude = latitude.replace('″', "")
-        longitude = longitude.replace('″', "")
+            if symbol in longitude:
+                longitude = longitude.replace(symbol, "")
+
+    # remove leading and trailing whitespace...
+    latitude  =  latitude.strip()
+    longitude = longitude.strip()
 
     return latitude, longitude
 
@@ -1415,9 +1411,9 @@ def _create_datetime(date_elements):
 
     if pyear < 1900:
         try:
-            tmpdate = "%04d-%s-%02d %02d:%02d:%02d" % (
-                    pyear, _dd.long_months[pmonth], day,
-                    hour, minutes, seconds)
+            tmpdate = "%(yr)04d:%(mon)02d:%(dy)02d %(hr)02d:%(mins)02d:%(secs)02d" % {
+                        'yr' : pyear, 'mon' : pmonth, 'dy' : day,
+                        'hr' : hour, 'mins' : minutes, 'secs' : seconds }
 
         except ValueError:
             tmpdate = False
@@ -1500,7 +1496,7 @@ def _setup_widget_tooltips(exif_widgets):
     """
 
     # add tooltips for the data entry fields...
-    for widget, tooltip in _TOOLTIPS:
+    for widget, tooltip in _TOOLTIPS.items():
         exif_widgets[widget].set_tooltip_text(tooltip)
 
     # add tooltips for the buttons...
