@@ -16,7 +16,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-# $Id: QuickViewGramplet.py 14355 2010-02-13 17:45:55Z dsblank $
+# $Id: QuickViewGramplet.py 17144 2011-04-18 13:26:59Z dsblank $
 
 #------------------------------------------------------------------------
 #
@@ -34,7 +34,7 @@ from gen.plug import Gramplet
 from gen.ggettext import sgettext as _
 from QuickReports import run_quick_report_by_name, get_quick_report_list
 from gen.plug  import (CATEGORY_QR_PERSON, CATEGORY_QR_FAMILY,
-                         CATEGORY_QR_EVENT, CATEGORY_QR_SOURCE, 
+                         CATEGORY_QR_EVENT, CATEGORY_QR_SOURCE, CATEGORY_QR_NOTE,
                          CATEGORY_QR_MISC, CATEGORY_QR_PLACE, CATEGORY_QR_MEDIA,
                          CATEGORY_QR_REPOSITORY)
 
@@ -56,32 +56,45 @@ class QuickViewGramplet(Gramplet):
         self.connect_signal('Media', self._active_changed)
         self.connect_signal('Note', self._active_changed)
 
+    def clear_text(self):
+        self.gui.textview.get_buffer().set_text("")
+
     def on_load(self):
         if len(self.gui.data) != 2:
             self.gui.data[:] = ["Person", None]
 
-    def on_save(self):
+    def save_update_options(self, widget=None):
         qv_type = self.get_option(_("View Type"))
         quick_type = qv_type.get_value()
         qv_option = self.get_option(_("Quick Views"))
         quick_view = qv_option.get_value()
         self.gui.data[:] = [quick_type, quick_view]
+        self.update()
 
     def main(self):
-        qv_type = self.get_option(_("View Type"))
-        quick_type = qv_type.get_value()
+        quick_type = self.gui.data[0]
         qv_option = self.get_option(_("Quick Views"))
-        quick_view = qv_option.get_value()
+        quick_view = self.gui.data[1] or qv_option.get_value()
         try:
             active_handle = self.get_active(quick_type)
         except:
             active_handle = None
         if active_handle:
-            run_quick_report_by_name(self.gui.dbstate, 
-                                     self.gui.uistate, 
-                                     quick_view,
-                                     active_handle,
-                                     container=self.gui.textview)
+            docman = run_quick_report_by_name(self.gui.dbstate, 
+                                              self.gui.uistate, 
+                                              quick_view,
+                                              active_handle,
+                                              container=self.gui.textview)
+            self.set_has_data(docman.document.has_data)
+        else:
+            self.clear_text()
+            self.set_has_data(False)
+
+    def update_has_data(self):
+        """
+        Update the has_data indicator when gramplet is not visible.
+        """
+        self.main()
 
     def build_options(self):
         from gen.plug.menu import EnumeratedListOption
@@ -91,6 +104,7 @@ class QuickViewGramplet(Gramplet):
                      ("Event", _("Event")), 
                      ("Family", _("Family")), 
                      ("Media", _("Media")), 
+                     ("Note", _("Note")), 
                      ("Place", _("Place")), 
                      ("Repository", _("Repository")),
                      ("Source", _("Source")), 
@@ -108,6 +122,7 @@ class QuickViewGramplet(Gramplet):
         self.add_option(list_option)
         type_widget = self.get_option_widget(_("View Type"))
         type_widget.value_changed = self.rebuild_option_list
+        self.rebuild_option_list() # call for initial setting
 
     def rebuild_option_list(self):
         code_map = {"Person": CATEGORY_QR_PERSON, 
@@ -116,6 +131,7 @@ class QuickViewGramplet(Gramplet):
                     "Source": CATEGORY_QR_SOURCE, 
                     "Place": CATEGORY_QR_PLACE, 
                     "Media": CATEGORY_QR_MEDIA,
+                    "Note": CATEGORY_QR_NOTE,
                     "Repository": CATEGORY_QR_REPOSITORY}
         qv_option = self.get_option(_("View Type"))
         list_option = self.get_option(_("Quick Views"))
@@ -123,3 +139,4 @@ class QuickViewGramplet(Gramplet):
         qv_list = get_quick_report_list(code_map[qv_option.get_value()])
         for pdata in qv_list:
             list_option.add_item(pdata.id, pdata.name)
+
