@@ -4,6 +4,7 @@
 # Copyright (C) 2007-2008 Brian G. Matherly
 # Copyright (C) 2008      Stephane Charette <stephanecharette@gmail.com>
 # Contribution 2009 by    Bob Ham <rah@bash.sh>
+# Copyright (C) 2010       Jakim Friant
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,7 +20,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
-# $Id: GVHourGlass.py 14091 2010-01-18 04:42:17Z pez4brian $
+# $Id: GVHourGlass.py 16979 2011-03-30 01:50:08Z pez4brian $
 
 """
 Generate an hourglass graph using the GraphViz generator.
@@ -41,7 +42,9 @@ from gen.display.name import displayer as name_displayer
 from Errors import ReportError
 from gen.plug.menu import (PersonOption, BooleanOption, NumberOption, 
                           EnumeratedListOption)
-from ReportBase import Report, ReportUtils, MenuReportOptions
+from gen.plug.report import Report
+from gen.plug.report import utils as ReportUtils
+from gui.plug.report import MenuReportOptions
 import DateHandler
 from gen.utils import get_birth_or_fallback, get_death_or_fallback
 
@@ -260,12 +263,12 @@ class HourGlassOptions(MenuReportOptions):
         menu.add_option(category_name, "pid", pid)
         
         max_gen = NumberOption(_('Max Descendant Generations'), 10, 1, 15)
-        max_gen.set_help(_("The number of generations of descendants to " \
+        max_gen.set_help(_("The number of generations of descendants to "
                            "include in the graph"))
         menu.add_option(category_name, "maxdescend", max_gen)
         
         max_gen = NumberOption(_('Max Ancestor Generations'), 10, 1, 15)
-        max_gen.set_help(_("The number of generations of ancestors to " \
+        max_gen.set_help(_("The number of generations of ancestors to "
                            "include in the graph"))
         menu.add_option(category_name, "maxascend", max_gen)
 
