@@ -19,7 +19,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: ImportGeneWeb.py 12560 2009-05-21 17:58:31Z gbritton $
+# $Id: ImportGeneWeb.py 17111 2011-04-12 16:33:31Z romjerome $
 
 "Import from GeneWeb"
 
@@ -30,8 +30,8 @@
 #-------------------------------------------------------------------------
 import re
 import time
-from gettext import gettext as _
-from gettext import ngettext
+from gen.ggettext import gettext as _
+from gen.ggettext import ngettext
 
 #------------------------------------------------------------------------
 #
@@ -39,7 +39,7 @@ from gettext import ngettext
 #
 #------------------------------------------------------------------------
 import logging
-log = logging.getLogger(".ImportGeneWeb")
+LOG = logging.getLogger(".ImportGeneWeb")
 
 #-------------------------------------------------------------------------
 #
@@ -48,8 +48,8 @@ log = logging.getLogger(".ImportGeneWeb")
 #-------------------------------------------------------------------------
 import Errors
 import gen.lib
+from gen.db import DbTxn
 from QuestionDialog import ErrorDialog
-from gen.plug import PluginManager, ImportPlugin
 from htmlentitydefs import name2codepoint
 
 _date_parse = re.compile('([kmes~?<>]+)?([0-9/]+)([J|H|F])?(\.\.)?([0-9/]+)?([J|H|F])?')
@@ -67,8 +67,6 @@ _cal_map = {
     'F' : gen.lib.Date.CAL_FRENCH,
     }
 
-enable_debug = False
-
 #-------------------------------------------------------------------------
 #
 #
@@ -115,78 +113,78 @@ class GeneWebParser(object):
         return line
         
     def parse_geneweb_file(self):
-        self.trans = self.db.transaction_begin("",batch=True)
-        self.db.disable_signals()
-        t = time.time()
-        self.lineno = 0
-        self.index = 0
-        self.fam_count = 0
-        self.indi_count = 0
+        with DbTxn(_("GeneWeb import"), self.db, batch=True) as self.trans:
+            self.db.disable_signals()
+            t = time.time()
+            self.lineno = 0
+            self.index = 0
+            self.fam_count = 0
+            self.indi_count = 0
         
-        self.fkeys = []
-        self.ikeys = {}
-        self.pkeys = {}
-        self.skeys = {}
+            self.fkeys = []
+            self.ikeys = {}
+            self.pkeys = {}
+            self.skeys = {}
         
-        self.current_mode = None
-        self.current_family = None
-        self.current_husband_handle = None
-        self.current_child_birthplace_handle = None
-        self.current_child_source_handle = None
-        try:
-            while 1:
-                line = self.get_next_line()
-                if line is None:
-                    break
-                if line == "":
-                    continue
+            self.current_mode = None
+            self.current_family = None
+            self.current_husband_handle = None
+            self.current_child_birthplace_handle = None
+            self.current_child_source_handle = None
+            try:
+                while 1:
+                    line = self.get_next_line()
+                    if line is None:
+                        break
+                    if line == "":
+                        continue
                 
-                fields = line.split(" ")
+                    fields = line.split(" ")
             
-                self.debug("LINE: %s" %line)
-                if fields[0] == "fam":
-                    self.current_mode = "fam"
-                    self.read_family_line(line,fields)
-                elif fields[0] == "rel":
-                    self.current_mode = "rel"
-                    self.read_relationship_person(line,fields)
-                elif fields[0] == "src":
-                    self.read_source_line(line,fields)
-                elif fields[0] in ("wit", "wit:"):
-                    self.read_witness_line(line,fields)
-                elif fields[0] == "cbp":
-                    self.read_children_birthplace_line(line,fields)
-                elif fields[0] == "csrc":
-                    self.read_children_source_line(line,fields)
-                elif fields[0] == "beg" and self.current_mode == "fam":
-                    self.read_children_lines()
-                elif fields[0] == "beg" and self.current_mode == "rel":
-                    self.read_relation_lines()
-                elif fields[0] == "comm":
-                    self.read_family_comment(line,fields)
-                elif fields[0] == "notes":
-                    self.read_person_notes_lines(line,fields)
-                elif fields[0] == "notes-db":
-                    self.read_database_notes_lines(line,fields)
-                elif fields[0] == "end":
-                    self.current_mode = None
-                elif fields[0] == "encoding:":
-                    self.encoding = fields[1]
-                else:
-                    print "parse_geneweb_file(): Token >%s< unknown. line %d skipped: %s" % (fields[0],self.lineno,line)
-        except Errors.GedcomError, err:
-            self.errmsg(str(err))
-            
-        t = time.time() - t
-        msg = ngettext('Import Complete: %d second','Import Complete: %d seconds', t ) % t
-
-        self.db.transaction_commit(self.trans,_("GeneWeb import"))
+                    LOG.debug("LINE: %s" %line)
+                    if fields[0] == "fam":
+                        self.current_mode = "fam"
+                        self.read_family_line(line,fields)
+                    elif fields[0] == "rel":
+                        self.current_mode = "rel"
+                        self.read_relationship_person(line,fields)
+                    elif fields[0] == "src":
+                        self.read_source_line(line,fields)
+                    elif fields[0] in ("wit", "wit:"):
+                        self.read_witness_line(line,fields)
+                    elif fields[0] == "cbp":
+                        self.read_children_birthplace_line(line,fields)
+                    elif fields[0] == "csrc":
+                        self.read_children_source_line(line,fields)
+                    elif fields[0] == "beg" and self.current_mode == "fam":
+                        self.read_children_lines()
+                    elif fields[0] == "beg" and self.current_mode == "rel":
+                        self.read_relation_lines()
+                    elif fields[0] == "comm":
+                        self.read_family_comment(line,fields)
+                    elif fields[0] == "notes":
+                        self.read_person_notes_lines(line,fields)
+                    elif fields[0] == "notes-db":
+                        self.read_database_notes_lines(line,fields)
+                    elif fields[0] == "end":
+                        self.current_mode = None
+                    elif fields[0] == "encoding:":
+                        self.encoding = fields[1]
+                    else:
+                        LOG.warn("parse_geneweb_file(): Token >%s< unknown. line %d skipped: %s" % 
+                                 (fields[0],self.lineno,line))
+            except Errors.GedcomError, err:
+                self.errmsg(str(err))
+                
+            t = time.time() - t
+            msg = ngettext('Import Complete: %d second','Import Complete: %d seconds', t ) % t
+    
         self.db.enable_signals()
         self.db.request_rebuild()
         
-        print msg
-        print "Families: %d" % len(self.fkeys)
-        print "Individuals: %d" % len(self.ikeys)
+        LOG.debug(msg)
+        LOG.debug("Families: %d" % len(self.fkeys))
+        LOG.debug("Individuals: %d" % len(self.ikeys))
         return None
 
     def read_family_line(self,line,fields):
@@ -199,7 +197,7 @@ class GeneWebParser(object):
         self.fkeys.append(self.current_family.get_handle())
         idx = 1;
         
-        self.debug("\nHusband:")
+        LOG.debug("\nHusband:")
         (idx, husband) = self.parse_person(fields,idx,gen.lib.Person.MALE,None)
         if husband:
             self.current_husband_handle = husband.get_handle()
@@ -207,9 +205,9 @@ class GeneWebParser(object):
             self.db.commit_family(self.current_family,self.trans)
             husband.add_family_handle(self.current_family.get_handle())
             self.db.commit_person(husband,self.trans)
-        self.debug("Marriage:")
+        LOG.debug("Marriage:")
         idx = self.parse_marriage(fields,idx)
-        self.debug("Wife:")
+        LOG.debug("Wife:")
         (idx,wife) = self.parse_person(fields,idx,gen.lib.Person.FEMALE,None)
         if wife:
             self.current_family.set_mother_handle(wife.get_handle())
@@ -219,14 +217,14 @@ class GeneWebParser(object):
         return None
 
     def read_relationship_person(self,line,fields):
-        self.debug("\Relationships:")
+        LOG.debug("\Relationships:")
         (idx,person) = self.parse_person(fields,1,gen.lib.Person.UNKNOWN,None)
         if person:
             self.current_relationship_person_handle = person.get_handle()
 
     def read_relation_lines(self):
         if not self.current_relationship_person_handle:
-            print "Unknown person for relationship in line %d!" % self.lineno
+            LOG.warn("Unknown person for relationship in line %d!" % self.lineno)
             return None
         rel_person = self.db.get_person_from_handle(self.current_relationship_person_handle)
         while 1:
@@ -246,14 +244,14 @@ class GeneWebParser(object):
                     (idx,asso_p) = self.parse_person(fields,0,gen.lib.Person.UNKNOWN,None)
                     pref = gen.lib.PersonRef()
                     pref.set_relation(matches.groups()[0])
-                    print("TODO: Handle association types properly")
+                    LOG.warn("TODO: Handle association types properly")
                     pref.set_reference_handle(asso_p.get_handle())
                     rel_person.add_person_ref(pref)
                     self.db.commit_person(rel_person,self.trans)
                 else:
-                    print "Invalid name of person in line %d" % self.lineno
+                    LOG.warn("Invalid name of person in line %d" % self.lineno)
             else:
-                print "Invalid relationship in line %d" % self.lineno
+                LOG.warn("Invalid relationship in line %d" % self.lineno)
                 break
         self.current_mode = None
         return None
@@ -262,7 +260,7 @@ class GeneWebParser(object):
 
     def read_source_line(self,line,fields):
         if not self.current_family:
-            print "Unknown family of child in line %d!" % self.lineno
+            LOG.warn("Unknown family of child in line %d!" % self.lineno)
             return None
         source = self.get_or_create_source(self.decode(fields[1]))
         self.current_family.add_source_reference(source)
@@ -270,7 +268,7 @@ class GeneWebParser(object):
         return None
     
     def read_witness_line(self,line,fields):
-        self.debug("Witness:")
+        LOG.debug("Witness:")
         if fields[1] == "m:":
             (idx,wit_p) = self.parse_person(fields,2,gen.lib.Person.MALE,None)
         elif fields[1] == "f:":
@@ -299,12 +297,12 @@ class GeneWebParser(object):
     def read_children_lines(self):
         father_surname = "Dummy"
         if not self.current_husband_handle:
-            print "Unknown father for child in line %d!" % self.lineno
+            LOG.warn("Unknown father for child in line %d!" % self.lineno)
             return None
         husb = self.db.get_person_from_handle(self.current_husband_handle)
         father_surname = husb.get_primary_name().get_surname()
         if not self.current_family:
-            print "Unknown family of child in line %d!" % self.lineno
+            LOG.warn("Unknown family of child in line %d!" % self.lineno)
             return None
         while 1:
             line = self.get_next_line()
@@ -315,7 +313,7 @@ class GeneWebParser(object):
 
             fields = line.split(" ")
             if fields[0] == "-":
-                self.debug("Child:")
+                LOG.debug("Child:")
                 child = None
                 if fields[1] == "h":
                     (idx,child) = self.parse_person(fields,2,gen.lib.Person.MALE,father_surname)
@@ -364,7 +362,7 @@ class GeneWebParser(object):
 
     def read_family_comment(self,line,fields):
         if not self.current_family:
-            print "Unknown family of child in line %d!" % self.lineno
+            LOG.warn("Unknown family of child in line %d!" % self.lineno)
             return None
         n = gen.lib.Note()
         n.set(line)
@@ -425,68 +423,74 @@ class GeneWebParser(object):
         #while idx < len(fields) and not fields[idx][0] == "+":
         while idx < len(fields) and not (fields[idx] and fields[idx][0] == "+"):
             if fields[idx]:
-                print "parse_marriage(): Unknown field: '%s' in line %d!" %(fields[idx],self.lineno)
-            idx = idx + 1
+                LOG.warn(("parse_marriage(): Unknown field: " +
+                          "'%s' in line %d!") % (fields[idx], self.lineno))
+            idx += 1
 
         while idx < len(fields) and mariageDataRe.match(fields[idx]):
-            if fields[idx][0] == "+":
-                mar_date = self.parse_date(self.decode(fields[idx]))
-                self.debug(" Married at: %s" % fields[idx])
-                idx = idx + 1
-            elif fields[idx][0] == "-":
-                div_date = self.parse_date(self.decode(fields[idx]))
-                self.debug(" Div at: %s" % fields[idx])
-                idx = idx + 1
-            elif fields[idx] == "#mp":
-                idx = idx + 1
+            field = fields[idx]
+            idx += 1
+            if field.startswith("+"):
+                field = field[1:]
+                mar_date = self.parse_date(self.decode(field))
+                LOG.debug(" Married at: %s" % field)
+            elif field.startswith("-"):
+                field = field[1:]
+                div_date = self.parse_date(self.decode(field))
+                LOG.debug(" Div at: %s" % field)
+            elif field == "#mp" and idx < len(fields):
                 mar_place = self.get_or_create_place(self.decode(fields[idx]))
-                self.debug(" Marriage place: %s" % fields[idx])
-                idx = idx + 1
-            elif fields[idx] == "#ms":
-                idx = idx + 1
+                LOG.debug(" Marriage place: %s" % fields[idx])
+                idx += 1
+            elif field == "#ms" and idx < len(fields):
                 mar_source = self.get_or_create_source(self.decode(fields[idx]))
-                self.debug(" Marriage source: %s" % fields[idx])
-                idx = idx + 1
-            elif fields[idx] == "#sep":
-                idx = idx + 1
+                LOG.debug(" Marriage source: %s" % fields[idx])
+                idx += 1
+            elif field == "#sep" and idx < len(fields):
                 sep_date = self.parse_date(self.decode(fields[idx]))
-                self.debug(" Seperated since: %s" % fields[idx])
-                idx = idx + 1
-            elif fields[idx] == "#nm":
-                self.debug(" Are not married.")
+                LOG.debug(" Seperated since: %s" % fields[idx])
+                idx += 1
+            elif field == "#nm":
+                LOG.debug(" Are not married.")
                 married = 0
-                idx = idx + 1
-            elif fields[idx] == "#noment":
-                self.debug(" Not mentioned.")
-                idx = idx + 1
-            elif fields[idx] == "#eng":
-                self.debug(" Are engaged.")
+            elif field == "#noment":
+                LOG.debug(" Not mentioned.")
+            elif field == "#eng":
+                LOG.debug(" Are engaged.")
                 engaged = 1
-                idx = idx + 1
             else:
-                print "parse_marriage(): Unknown field '%s'for mariage in line %d!" % (fields[idx],self.lineno)
-                idx = idx + 1
+                LOG.warn(("parse_marriage(): Unknown field " +
+                          "'%s'for mariage in line %d!") % (field, self.lineno))
 
         if mar_date or mar_place or mar_source:
-            mar = self.create_event(gen.lib.EventType.MARRIAGE, None, mar_date, mar_place, mar_source)
+            mar = self.create_event(
+                gen.lib.EventType.MARRIAGE, None, mar_date, mar_place, mar_source)
             mar_ref = gen.lib.EventRef()
             mar_ref.set_reference_handle(mar.get_handle())
+            mar_ref.set_role(gen.lib.EventRoleType.FAMILY)
             self.current_family.add_event_ref(mar_ref)
+            self.current_family.set_relationship(
+                gen.lib.FamilyRelType(gen.lib.FamilyRelType.MARRIED))
 
         if div_date:
-            div = self.create_event(gen.lib.EventType.DIVORCE, None, div_date, None, None)
+            div = self.create_event(
+                gen.lib.EventType.DIVORCE, None, div_date, None, None)
             div_ref = gen.lib.EventRef()
             div_ref.set_reference_handle(div.get_handle())
+            div_ref.set_role(gen.lib.EventRoleType.FAMILY)
             self.current_family.add_event_ref(div_ref)
 
         if sep_date or engaged:
-            sep = self.create_event(gen.lib.EventType.ENGAGEMENT, None, sep_date, None, None)
+            sep = self.create_event(
+                gen.lib.EventType.ENGAGEMENT, None, sep_date, None, None)
             sep_ref = gen.lib.EventRef()
             sep_ref.set_reference_handle(sep.get_handle())
+            sep_ref.set_role(gen.lib.EventRoleType.FAMILY)
             self.current_family.add_event_ref(sep_ref)
 
         if not married:
-            self.current_family.set_relationship(gen.lib.FamilyRelType(gen.lib.FamilyRelType.UNMARRIED))
+            self.current_family.set_relationship(
+                gen.lib.FamilyRelType(gen.lib.FamilyRelType.UNMARRIED))
             
         self.db.commit_family(self.current_family,self.trans)
         return idx
@@ -495,32 +499,33 @@ class GeneWebParser(object):
         
         if not father_surname:
             if not idx < len(fields):
-                print "Missing surname of person in line %d!" % self.lineno
+                LOG.warn("Missing surname of person in line %d!" % self.lineno)
                 surname =""
             else:
                 surname = self.decode(fields[idx])
-            idx = idx + 1
+            idx += 1
         else:
             surname = father_surname
         
         if not idx < len(fields):
-            print "Missing firstname of person in line %d!" % self.lineno
+            LOG.warn("Missing firstname of person in line %d!" % self.lineno)
             firstname = ""
         else:
             firstname = self.decode(fields[idx])
-        idx = idx + 1
+        idx += 1
         if idx < len(fields) and father_surname:
             noSurnameRe = re.compile("^[({\[~><?0-9#].*$")
             if not noSurnameRe.match(fields[idx]):
                 surname = self.decode(fields[idx])
-                idx = idx + 1
+                idx += 1
 
-        self.debug("Person: %s %s" % (firstname, surname))
+        LOG.debug("Person: %s %s" % (firstname, surname))
         person = self.get_or_create_person(firstname,surname)
         name = gen.lib.Name()
         name.set_type( gen.lib.NameType(gen.lib.NameType.BIRTH))
         name.set_first_name(firstname)
-        name.set_surname(surname)
+        surname_obj = name.get_primary_surname()
+        surname_obj.set_surname(surname)
         person.set_primary_name(name)
         if person.get_gender() == gen.lib.Person.UNKNOWN and gender is not None:
             person.set_gender(gender)
@@ -555,22 +560,22 @@ class GeneWebParser(object):
         surname_aliases = []
         
         while idx < len(fields) and personDataRe.match(fields[idx]):
-            if fields[idx][0] == '(':
-                self.debug("Public Name: %s" % fields[idx])
-                public_name = self.decode(fields[idx][1:-1])
-                idx += 1
-            elif fields[idx][0] == '{':
-                self.debug("Firstsname Alias: %s" % fields[idx])
-                firstname_aliases.append(self.decode(fields[idx][1:-1]))
-                idx += 1
-            elif fields[idx][0] == '[':
-                self.debug("Title: %s" % fields[idx])
-                titleparts = self.decode(fields[idx][1:-1]).split(":")
+            field = fields[idx]
+            idx += 1
+            if field.startswith('('):
+                LOG.debug("Public Name: %s" % field)
+                public_name = self.decode(field[1:-1])
+            elif field.startswith('{'):
+                LOG.debug("Firstsname Alias: %s" % field)
+                firstname_aliases.append(self.decode(field[1:-1]))
+            elif field.startswith('['):
+                LOG.debug("Title: %s" % field)
+                titleparts = self.decode(field[1:-1]).split(":")
                 tname = ttitle = tplace = tstart = tend = tnth = None
                 try:
                     tname =  titleparts[0]
                     ttitle = titleparts[1]
-                    if( titleparts[2]):
+                    if titleparts[2]:
                         tplace = self.get_or_create_place(titleparts[2])
                     tstart = self.parse_date(titleparts[3])
                     tend =   self.parse_date(titleparts[4])
@@ -578,8 +583,9 @@ class GeneWebParser(object):
                 except IndexError:  # not all parts are written all the time
                     pass
                 if tnth:    # Append title numer to title
-                    ttitle = "%s, %s" % (ttitle, tnth)
-                title = self.create_event(gen.lib.EventType.NOB_TITLE,ttitle,tstart,tplace)
+                    ttitle += ", " + tnth
+                title = self.create_event(
+                           gen.lib.EventType.NOB_TITLE, ttitle, tstart, tplace)
                 # TODO: Geneweb has a start date and an end date, and therefore
                 # supprts stuff like: FROM about 1955 TO between 1998 and 1999
                 # gramps only supports one single date ore range.
@@ -591,131 +597,108 @@ class GeneWebParser(object):
                 title_ref = gen.lib.EventRef()
                 title_ref.set_reference_handle(title.get_handle())
                 person.add_event_ref(title_ref)
-                idx += 1
-            elif fields[idx] == '#nick':
-                idx += 1
-                self.debug("Nick Name: %s" % fields[idx])
+            elif field == '#nick' and idx < len(fields):
+                LOG.debug("Nick Name: %s" % fields[idx])
                 nick_names.append(self.decode(fields[idx]))
                 idx += 1
-            elif fields[idx] == '#occu':
-                idx += 1
-                self.debug("Occupation: %s" % fields[idx])
-                occu = self.create_event(gen.lib.EventType.OCCUPATION,self.decode(fields[idx]))
+            elif field == '#occu' and idx < len(fields):
+                LOG.debug("Occupation: %s" % fields[idx])
+                occu = self.create_event(
+                        gen.lib.EventType.OCCUPATION, self.decode(fields[idx]))
                 occu_ref = gen.lib.EventRef()
                 occu_ref.set_reference_handle(occu.get_handle())
                 person.add_event_ref(occu_ref)
                 idx += 1
-            elif fields[idx] == '#alias':
-                idx += 1
-                self.debug("Name Alias: %s" % fields[idx])
+            elif field == '#alias' and idx < len(fields):
+                LOG.debug("Name Alias: %s" % fields[idx])
                 name_aliases.append(self.decode(fields[idx]))
                 idx += 1
-            elif fields[idx] == '#salias':
-                idx += 1
-                self.debug("Surname Alias: %s" % fields[idx])
+            elif field == '#salias' and idx < len(fields):
+                LOG.debug("Surname Alias: %s" % fields[idx])
                 surname_aliases.append(self.decode(fields[idx]))
                 idx += 1
-            elif fields[idx] == '#image':
+            elif field == '#image' and idx < len(fields):
+                LOG.debug("Image: %s" % fields[idx])
                 idx += 1
-                self.debug("Image: %s" % fields[idx])
-                idx += 1
-            elif fields[idx] == '#src':
-                idx += 1
-                self.debug("Source: %s" % fields[idx])
+            elif field == '#src' and idx < len(fields):
+                LOG.debug("Source: %s" % fields[idx])
                 source = self.get_or_create_source(self.decode(fields[idx]))
                 idx += 1
-            elif fields[idx] == '#bs':
-                idx += 1
-                self.debug("Birth Source: %s" % fields[idx])
+            elif field == '#bs' and idx < len(fields):
+                LOG.debug("Birth Source: %s" % fields[idx])
                 birth_source = self.get_or_create_source(self.decode(fields[idx]))
                 idx += 1
-            elif fields[idx][0] == '!':
-                self.debug("Baptize at: %s" % fields[idx])
+            elif field[0] == '!':
+                LOG.debug("Baptize at: %s" % fields[idx])
                 bapt_date = self.parse_date(self.decode(fields[idx][1:]))
                 idx += 1
-            elif fields[idx] == '#bp':
-                idx += 1
-                self.debug("Birth Place: %s" % fields[idx])
+            elif field == '#bp' and idx < len(fields):
+                LOG.debug("Birth Place: %s" % fields[idx])
                 birth_place = self.get_or_create_place(self.decode(fields[idx]))
                 idx += 1
-            elif fields[idx] == '#pp':
-                idx += 1
-                self.debug("Baptize Place: %s" % fields[idx])
+            elif field == '#pp' and idx < len(fields):
+                LOG.debug("Baptize Place: %s" % fields[idx])
                 bapt_place = self.get_or_create_place(self.decode(fields[idx]))
                 idx += 1
-            elif fields[idx] == '#ps':
-                idx += 1
-                self.debug("Baptize Source: %s" % fields[idx])
+            elif field == '#ps' and idx < len(fields):
+                LOG.debug("Baptize Source: %s" % fields[idx])
                 bapt_source = self.get_or_create_source(self.decode(fields[idx]))
                 idx += 1
-            elif fields[idx] == '#dp':
-                idx += 1
-                self.debug("Death Place: %s" % fields[idx])
+            elif field == '#dp' and idx < len(fields):
+                LOG.debug("Death Place: %s" % fields[idx])
                 death_place = self.get_or_create_place(self.decode(fields[idx]))
                 idx += 1
-            elif fields[idx] == '#ds':
-                idx += 1
-                self.debug("Death Source: %s" % fields[idx])
+            elif field == '#ds' and idx < len(fields):
+                LOG.debug("Death Source: %s" % fields[idx])
                 death_source = self.get_or_create_source(self.decode(fields[idx]))
                 idx += 1
-            elif fields[idx] == '#buri':
+            elif field == '#buri' and idx < len(fields):
+                LOG.debug("Burial Date: %s" % fields[idx])
+                bur_date = self.parse_date(self.decode(fields[idx]))
                 idx += 1
-                self.debug("Burial Date: %s" % fields[idx])
-                try:
-                    bur_date = self.parse_date(self.decode(fields[idx]))
-                except IndexError:
-                    pass
-                idx += 1
-            elif fields[idx] == '#crem':
-                idx += 1
-                self.debug("Cremention Date: %s" % fields[idx])
+            elif field == '#crem' and idx < len(fields):
+                LOG.debug("Cremention Date: %s" % fields[idx])
                 crem_date = self.parse_date(self.decode(fields[idx]))
                 idx += 1
-            elif fields[idx] == '#rp':
-                idx += 1
-                self.debug("Burial Place: %s" % fields[idx])
+            elif field == '#rp' and idx < len(fields):
+                LOG.debug("Burial Place: %s" % fields[idx])
                 bur_place = self.get_or_create_place(self.decode(fields[idx]))
                 idx += 1
-            elif fields[idx] == '#rs':
-                idx += 1
-                self.debug("Burial Source: %s" % fields[idx])
+            elif field == '#rs' and idx < len(fields):
+                LOG.debug("Burial Source: %s" % fields[idx])
                 bur_source = self.get_or_create_source(self.decode(fields[idx]))
                 idx += 1
-            elif fields[idx] == '#apubl':
-                self.debug("This is a public record")
-                idx += 1
-            elif fields[idx] == '#apriv':
-                self.debug("This is a private record")
+            elif field == '#apubl':
+                LOG.debug("This is a public record")
+            elif field == '#apriv':
+                LOG.debug("This is a private record")
                 person.set_privacy(True)
-                idx += 1
-            elif fields[idx] == '#h':
-                self.debug("This is a restricted record")
+            elif field == '#h':
+                LOG.debug("This is a restricted record")
                 #TODO: Gramps does currently not feature this level
                 person.set_privacy(True)
-                idx += 1
-            elif dateRe.match( fields[idx]):
+            elif dateRe.match(field):
                 if not birth_parsed:
-                    self.debug("Birth Date: %s" % fields[idx])
-                    birth_date = self.parse_date(self.decode(fields[idx]))
+                    LOG.debug("Birth Date: %s" % field)
+                    birth_date = self.parse_date(self.decode(field))
                     birth_parsed = True
                 else:
-                    self.debug("Death Date: %s" % fields[idx])
-                    death_date = self.parse_date(self.decode(fields[idx]))
-                    if fields[idx] == "mj":
+                    LOG.debug("Death Date: %s" % field)
+                    death_date = self.parse_date(self.decode(field))
+                    if field == "mj":
                         death_cause = "Died joung"
-                    elif fields[idx][0] == "k":
+                    elif field.startswith("k"):
                         death_cause = "Killed"
-                    elif fields[idx][0] == "m":
+                    elif field.startswith("m"):
                         death_cause = "Murdered"
-                    elif fields[idx][0] == "e":
+                    elif field.startswith("e"):
                         death_cause = "Executed"
-                    elif fields[idx][0] == "d":
+                    elif field.startswith("d"):
                         death_cause = "Disappeared"
                     #TODO: Set special death types more properly
-                idx += 1
             else:
-                print "parse_person(): Unknown field '%s' for person in line %d!" % (fields[idx],self.lineno)
-                idx += 1
+                LOG.warn(("parse_person(): Unknown field " +
+                          "'%s' for person in line %d!") % (field, self.lineno))
         
         if public_name:
             name = person.get_primary_name()
@@ -724,7 +707,8 @@ class GeneWebParser(object):
             name = gen.lib.Name()
             name.set_type(gen.lib.NameType(gen.lib.NameType.AKA))
             name.set_first_name(public_name)
-            name.set_surname(surname)
+            surname_obj = name.get_primary_surname()
+            surname_obj.set_surname(surname)
             person.set_primary_name(name)
         
         for aka in nick_names:
@@ -737,14 +721,16 @@ class GeneWebParser(object):
             name = gen.lib.Name()
             name.set_type(gen.lib.NameType(gen.lib.NameType.AKA))
             name.set_first_name(aka)
-            name.set_surname(surname)
+            surname_obj = name.get_primary_surname()
+            surname_obj.set_surname(surname)
             person.add_alternate_name(name)
 
         for aka in name_aliases:
             name = gen.lib.Name()
             name.set_type(gen.lib.NameType(gen.lib.NameType.AKA))
             name.set_first_name(aka)
-            name.set_surname(surname)
+            surname_obj = name.get_primary_surname()
+            surname_obj.set_surname(surname)
             person.add_alternate_name(name)
 
         for aka in surname_aliases:
@@ -754,7 +740,8 @@ class GeneWebParser(object):
                 name.set_first_name(public_name)
             else:
                 name.set_first_name(firstname)
-            name.set_surname(aka)
+            surname_obj = name.get_primary_surname()
+            surname_obj.set_surname(aka)
             person.add_alternate_name(name)
 
         if source:
@@ -821,7 +808,7 @@ class GeneWebParser(object):
             cal1 = _cal_map.get(groups[2],gen.lib.Date.CAL_GREGORIAN)
             sub1 = self.sub_date(groups[1])
             date.set(gen.lib.Date.QUAL_NONE,mod, cal1,
-                     (sub1[0],sub1[1],sub1[2],None,sub2[0],sub2[1],sub2[2],None))
+                     (sub1[0],sub1[1],sub1[2],0,sub2[0],sub2[1],sub2[2],0))
             return date
         else:
             return None
@@ -917,14 +904,3 @@ class GeneWebParser(object):
     def debug( self, txt):
         if enable_debug:
             print txt
-#-------------------------------------------------------------------------
-#
-# Register with the plugin system
-#
-#-------------------------------------------------------------------------
-pmgr = PluginManager.get_instance()
-plugin = ImportPlugin(name            = _('GeneWeb'), 
-                      description     =  _("Import data from GeneWeb files"),
-                      import_function = importData,
-                      extension       = "gw")
-pmgr.register_plugin(plugin)
