@@ -5,6 +5,7 @@
 # Copyright (C) 2009  Benny Malengier
 # Copyright (C) 2009  Brian Matherly
 # Copyright (C) 2010  Peter Landgren
+# Copyright (C) 2010       Jakim Friant
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -21,7 +22,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: libcairodoc.py 15168 2010-04-16 20:58:51Z bmcage $
+# $Id: libcairodoc.py 16979 2011-03-30 01:50:08Z pez4brian $
 
 """Report output generator based on Cairo.
 """
@@ -40,9 +41,9 @@ from math import radians
 #
 #------------------------------------------------------------------------
 from gen.plug.docgen import (BaseDoc, TextDoc, DrawDoc, ParagraphStyle,
-                        TableCellStyle, FONT_SANS_SERIF, FONT_SERIF, 
+                        TableCellStyle, SOLID, FONT_SANS_SERIF, FONT_SERIF,
                         FONT_MONOSPACE, PARA_ALIGN_CENTER, PARA_ALIGN_LEFT)
-from ReportBase import ReportUtils
+from gen.plug.report import utils as ReportUtils
 from Errors import PluginError
 from gen.plug.docbackend import CairoBackend
 
@@ -393,7 +394,7 @@ class GtkDocParagraph(GtkDocBaseElement):
     _allowed_children = []
     
     # line spacing is not defined in ParagraphStyle
-    spacing = 2
+    spacingfractionfont = 0.2
     
     def __init__(self, style, leader=None):
         GtkDocBaseElement.__init__(self, style)
@@ -453,8 +454,7 @@ class GtkDocParagraph(GtkDocBaseElement):
         layout.set_width(int(text_width * pango.SCALE))
         
         # set paragraph properties
-        layout.set_wrap(pango.WRAP_WORD_CHAR)
-        layout.set_spacing(self.spacing * pango.SCALE)
+        layout.set_wrap(pango.WRAP_WORD_CHAR)        
         layout.set_indent(int(f_indent * pango.SCALE))
         layout.set_tabs(tabstops_to_tabarray(self._style.get_tabs(), dpi_x))
         #
@@ -472,6 +472,10 @@ class GtkDocParagraph(GtkDocBaseElement):
         #
         font_style = self._style.get_font()
         layout.set_font_description(fontstyle_to_fontdescription(font_style))
+        #set line spacing based on font:
+        spacing = font_style.get_size() * self.spacingfractionfont
+        layout.set_spacing(int(round(spacing * pango.SCALE)))
+        
         text_height = height - t_margin - 2 * v_padding
         
         # calculate where to cut the paragraph
@@ -483,7 +487,7 @@ class GtkDocParagraph(GtkDocBaseElement):
         
         # if all paragraph fits we don't need to cut
         if layout_height - spacing <= text_height:
-            paragraph_height = layout_height + t_margin + (2 * v_padding)
+            paragraph_height = layout_height + spacing +t_margin + (2 * v_padding)
             if height - paragraph_height > b_margin:
                 paragraph_height += b_margin
             return (self, None), paragraph_height
@@ -558,7 +562,7 @@ class GtkDocParagraph(GtkDocBaseElement):
         self.__set_plaintext(self._plaintext.encode('utf-8')[:index])
         self._style.set_bottom_margin(0)
         
-        paragraph_height = endheight - startheight + t_margin + 2 * v_padding
+        paragraph_height = endheight - startheight + spacing + t_margin + 2 * v_padding
         return (self, new_paragraph), paragraph_height
 
     def filterattr(self, attr, index):
@@ -588,7 +592,6 @@ class GtkDocParagraph(GtkDocBaseElement):
         
         # set paragraph properties
         layout.set_wrap(pango.WRAP_WORD_CHAR)
-        layout.set_spacing(self.spacing * pango.SCALE)
         layout.set_indent(int(f_indent * pango.SCALE))
         layout.set_tabs(tabstops_to_tabarray(self._style.get_tabs(), dpi_x))
         #
@@ -604,6 +607,9 @@ class GtkDocParagraph(GtkDocBaseElement):
         #
         font_style = self._style.get_font()
         layout.set_font_description(fontstyle_to_fontdescription(font_style))
+        #set line spacing based on font:
+        spacing = font_style.get_size() * self.spacingfractionfont
+        layout.set_spacing(int(round(spacing * pango.SCALE)))
 
         # layout the text
         layout.set_text(self._plaintext)
@@ -614,12 +620,13 @@ class GtkDocParagraph(GtkDocBaseElement):
         x = l_margin + h_padding
         if f_indent < 0:
             x += f_indent
-        cr.move_to(x, t_margin + v_padding)
+        # 3/4 of the spacing is added above the text, 1/4 is added below
+        cr.move_to(x, t_margin + v_padding + spacing * 0.75)
         cr.set_source_rgb(*ReportUtils.rgb_color(font_style.get_color()))
         cr.show_layout(layout)
         
         # calculate the full paragraph height
-        height = layout_height + t_margin + 2*v_padding + b_margin
+        height = layout_height + spacing + t_margin + 2*v_padding + b_margin
 
         # draw the borders
         if self._style.get_top_border():
@@ -1025,6 +1032,9 @@ class GtkDocLine(GtkDocBaseElement):
         cr.set_source_rgb(*line_color)
         cr.set_line_width(self._style.get_line_width())
         # TODO line style
+        line_style = self._style.get_line_style()
+        if line_style != SOLID:
+            cr.set_dash(self._style.get_dash_style(line_style), 0)
         cr.move_to(*start)
         cr.line_to(*end)
         cr.stroke()
@@ -1058,6 +1068,9 @@ class GtkDocPolygon(GtkDocBaseElement):
         cr.set_source_rgb(*path_stroke_color)
         cr.set_line_width(self._style.get_line_width())
         # TODO line style
+        line_style = self._style.get_line_style()
+        if line_style != SOLID:
+            cr.set_dash(self._style.get_dash_style(line_style), 0)
         cr.stroke()
         cr.restore()
         
@@ -1090,6 +1103,9 @@ class GtkDocBox(GtkDocBaseElement):
         
         cr.set_line_width(self._style.get_line_width())
         # TODO line style
+        line_style = self._style.get_line_style()
+        if line_style != SOLID:
+            cr.set_dash(self._style.get_dash_style(line_style), 0)
 
         if self._style.get_shadow():
             shadow_x = box_x + self._style.get_shadow_space() * dpi_x / 2.54
@@ -1116,7 +1132,7 @@ class GtkDocText(GtkDocBaseElement):
     _allowed_children = []
 
     # line spacing is not defined in ParagraphStyle
-    spacing = 0
+    spacingfractionfont = 0.2
     
     def __init__(self, style, vertical_alignment, text, x, y, angle=0):
         GtkDocBaseElement.__init__(self, style)
@@ -1134,8 +1150,6 @@ class GtkDocText(GtkDocBaseElement):
         layout.set_width(-1)
         
         # set paragraph properties
-        layout.set_spacing(self.spacing * pango.SCALE)
-        #
         align = self._style.get_alignment_text()
         if align == 'left':
             layout.set_alignment(pango.ALIGN_LEFT)
@@ -1150,6 +1164,9 @@ class GtkDocText(GtkDocBaseElement):
         #
         font_style = self._style.get_font()
         layout.set_font_description(fontstyle_to_fontdescription(font_style))
+        #set line spacing based on font:
+        spacing = font_style.get_size() * self.spacingfractionfont
+        layout.set_spacing(int(round(spacing * pango.SCALE)))
 
         # layout the text
         layout.set_markup(self._text)
@@ -1318,10 +1335,12 @@ class CairoDoc(BaseDoc, TextDoc, DrawDoc):
             #FIXME: following split should be regex to match \n\s*\n instead?
             for line in markuptext.split('\n\n'):
                 self.start_paragraph(style_name)
-                #flowed, make normal whitespace go away
-                line = line.replace('\n',' ')
-                line = ' '.join(line.split())
-                self.__write_text(line, markup=True)
+                #flowed, normal whitespace goes away, but we keep linebreak
+                lines = line.split('\n')
+                newlines = []
+                for singleline in lines:
+                    newlines.append(' '.join(singleline.split()))
+                self.__write_text('\n'.join(newlines), markup=True)
                 self.end_paragraph()
 
     def __write_text(self, text, mark=None, markup=False):
