@@ -1,3 +1,5 @@
+#!/usr/bin/python
+# -*- coding: utf-8 -*-
 #
 # Gramps - a GTK+/GNOME based genealogy program
 #
@@ -18,7 +20,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: libhtml.py 16000 2010-10-16 07:47:05Z robhealey1 $
+# $Id: libhtml.py 16639 2011-02-17 07:42:37Z robhealey1 $
 
 #------------------------------------------------------------------------
 #
@@ -26,6 +28,13 @@
 #
 #------------------------------------------------------------------------
 
+from __future__ import print_function
+
+#------------------------------------------------------------------------
+# Python modules
+#------------------------------------------------------------------------
+import re
+
 """
 HTML operations.
 
@@ -95,18 +104,6 @@ _START_CLOSE = set([
     'param'
     ])
 
-#------------------------------------------------------------------------
-#
-# Helper functions.
-#
-#------------------------------------------------------------------------
-
-def print_(line):
-    """
-    Print function
-    """
-    print line
-
 #------------------------------------------------------------------------
 #
 # Html class.
@@ -280,7 +277,7 @@ class Html(list):
         for keyw, arg in keywargs.iteritems():
             if (keyw in ['indent', 'close', 'inline'] and
                arg in [True, False, None]):
-                setattr(self,keyw, arg)
+                setattr(self, keyw, arg)
             elif keyw == 'attr':                        # pass attributes along
                 attr += ' ' + arg
             elif keyw[-1] == '_':                       # avoid Python conflicts
@@ -383,7 +380,7 @@ class Html(list):
 #
     iterkeys = itervalues = iteritems = __iter__
 #
-    def write(self, method=print_, indent='\t', tabs=''):
+    def write(self, method=print, indent='\t', tabs=''):
         """
         Output function: performs an insertion-order tree traversal
         and calls supplied method for each item found.
@@ -471,8 +468,14 @@ class Html(list):
         :param name: new HTML tag
         """
         curtag = self.tag
+        
+        # Replace closing tag, if any
+        
         if self[-1] == '</%s>' % curtag:
             self[-1] = '</%s>' % newtag
+
+        # Replace opening tag
+
         self[0] = self[0].replace('<' + curtag, '<' + newtag)
     tag = property(__gettag, __settag)
 #
@@ -483,7 +486,7 @@ class Html(list):
         :rtype:  string
         :returns: HTML attributes
         """
-        attr = self[0].strip('<!?>').split(None, 1)
+        attr = self[0].strip('<!?/>').split(None, 1)
         return attr[1] if len(attr) > 1 else ''
 #
     def __setattr(self, value):
@@ -493,13 +496,24 @@ class Html(list):
         :type  name: string
         :param name: new HTML attributes
         """
-        self[0] = self[0][:len(self.tag)+1] + ' ' + value + self[0][-1:]
+        beg = len(self.tag) + 1
+
+        # See if self-closed or normal
+        
+        end = -2 if self.close is False else -1
+        self[0] = self[0][:beg] + ' ' + value + self[0][end:]        
 #
     def __delattr(self):
         """
         Removes HTML attributes for this object
         """
-        self[0] = '<' + self.tag + '>'
+        self[0] = '<' + self.tag + (
+
+                # Set correct closing delimiter(s)
+        
+                ' />' if self.close is False else '>'
+                )
+#
     attr = property(__getattr,  __setattr, __delattr)
 #
     def __getinside(self):
