@@ -2,7 +2,7 @@
 #
 # Copyright (C) 2000-2007  Donald N. Allingham
 # Copyright (C) 2008       Gary Burton
-# Copyright (C) 2009       Nick Hall
+# Copyright (C) 2009-2010  Nick Hall
 # Copyright (C) 2010       Benny Malengier
 #
 # This program is free software; you can redistribute it and/or modify
@@ -20,7 +20,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  021111307  USA
 #
 
-# $Id: libpersonview.py 14950 2010-03-27 22:59:41Z nick-h $
+# $Id: libpersonview.py 17446 2011-05-07 18:41:14Z nick-h $
 
 """
 Provide the base for a list person view.
@@ -47,6 +47,7 @@ _LOG = logging.getLogger(".gui.personview")
 #
 #-------------------------------------------------------------------------
 import gen.lib
+from gen.db import DbTxn
 from gui.views.listview import ListView
 import Utils
 from gen.display.name import displayer as name_displayer
@@ -61,7 +62,7 @@ from gen.plug import CATEGORY_QR_PERSON
 
 #-------------------------------------------------------------------------
 #
-# internationalization
+# Python modules
 #
 #-------------------------------------------------------------------------
 from gen.ggettext import sgettext as _
@@ -83,7 +84,8 @@ class BasePersonView(ListView):
     COL_DDAT = 5
     COL_DPLAC = 6
     COL_SPOUSE = 7
-    COL_CHAN = 8
+    COL_TAGS = 8
+    COL_CHAN = 9
     #name of the columns
     COLUMN_NAMES = [
         _('Name'),
@@ -94,6 +96,7 @@ class BasePersonView(ListView):
         _('Death Date'),
         _('Death Place'),
         _('Spouse'),
+        _('Tags'),
         _('Last Changed'),
         ]
     # columns that contain markup
@@ -102,16 +105,18 @@ class BasePersonView(ListView):
     CONFIGSETTINGS = (
         ('columns.visible', [COL_NAME, COL_ID, COL_GEN, COL_BDAT, COL_DDAT]),
         ('columns.rank', [COL_NAME, COL_ID, COL_GEN, COL_BDAT, COL_BPLAC,
-                           COL_DDAT, COL_DPLAC, COL_SPOUSE, COL_CHAN]),
-        ('columns.size', [250, 75, 75, 100, 175, 100, 175, 100, 100])
+                           COL_DDAT, COL_DPLAC, COL_SPOUSE, COL_TAGS,
+                           COL_CHAN]),
+        ('columns.size', [250, 75, 75, 100, 175, 100, 175, 100, 100, 100])
         )  
     ADD_MSG     = _("Add a new person")
     EDIT_MSG    = _("Edit the selected person")
-    DEL_MSG     = _("Delete the selected person")
+    DEL_MSG     = _("Remove the selected person")
+    MERGE_MSG   = _("Merge the selected persons")
     FILTER_TYPE = "Person"
     QR_CATEGORY = CATEGORY_QR_PERSON
 
-    def __init__(self, dbstate, uistate, title, model, nav_group=0):
+    def __init__(self, pdata, dbstate, uistate, title, model, nav_group=0):
         """
         Create the Person View
         """
@@ -121,10 +126,11 @@ class BasePersonView(ListView):
             'person-delete'  : self.row_delete,
             'person-rebuild' : self.object_build,
             'person-groupname-rebuild' : self.object_build,
+            'tag-update' : self.tag_updated
             }
  
         ListView.__init__(
-            self, title, dbstate, uistate,
+            self, title, pdata, dbstate, uistate,
             BasePersonView.COLUMN_NAMES, len(BasePersonView.COLUMN_NAMES), 
             model, signal_map, dbstate.db.get_bookmarks(),
             Bookmarks.PersonBookmarks, nav_group,
@@ -132,15 +138,19 @@ class BasePersonView(ListView):
             filter_class=PersonSidebarFilter,
             markup=BasePersonView.MARKUP_COLS)
             
-        self.func_list = {
+        self.func_list.update({
             '<CONTROL>J' : self.jump,
             '<CONTROL>BackSpace' : self.key_delete,
-            }
+            })
 
-        config.connect("interface.filter", self.filter_toggle)
         uistate.connect('nameformat-changed', self.build_tree)
 
+        self.additional_uis.append(self.additional_ui())
+
     def navigation_type(self):
+        """
+        Return the navigation type of the view.
+        """
         return 'Person'
 
     def get_bookmarks(self):
@@ -155,17 +165,12 @@ class BasePersonView(ListView):
         """
         return DdTargets.PERSON_LINK
         
-    def drag_list_info(self):
-        """
-        Specify the drag type for a multiple selected rows
-        """
-        return DdTargets.PERSON_LINK_LIST
-
     def exact_search(self):
         """
         Returns a tuple indicating columns requiring an exact search
+        'female' contains the string 'male' so we need an exact search
         """
-        return (BasePersonView.COL_GEN,) # Gender ('female' contains the string 'male')
+        return (BasePersonView.COL_GEN,)
 
     def get_stock(self):
         """
@@ -173,7 +178,7 @@ class BasePersonView(ListView):
         """
         return 'gramps-person'
 
-    def ui_definition(self):
+    def additional_ui(self):
         """
         Defines the UI string for UIManager
         """
@@ -204,13 +209,10 @@ class BasePersonView(ListView):
                 <menuitem action="Add"/>
                 <menuitem action="Edit"/>
                 <menuitem action="Remove"/>
+                <menuitem action="Merge"/>
               </placeholder>
               <menuitem action="SetActive"/>
               <menuitem action="FilterEdit"/>
-              <placeholder name="Merge">
-                <menuitem action="CmpMerge"/>
-                <menuitem action="FastMerge"/>
-              </placeholder>
             </menu>
           </menubar>
           <toolbar name="ToolBar">
@@ -223,6 +225,7 @@ class BasePersonView(ListView):
               <toolitem action="Add"/>
               <toolitem action="Edit"/>
               <toolitem action="Remove"/>
+              <toolitem action="Merge"/>
             </placeholder>
           </toolbar>
           <popup name="Popup">
@@ -233,14 +236,21 @@ class BasePersonView(ListView):
             <menuitem action="Add"/>
             <menuitem action="Edit"/>
             <menuitem action="Remove"/>
+            <menuitem action="Merge"/>
             <separator/>
             <menu name="QuickReport" action="QuickReport">
               <menuitem action="Dummy"/>
             </menu>
+            <menu name="WebConnect" action="WebConnect">
+              <menuitem action="Dummy"/>
+            </menu>
           </popup>
         </ui>'''
 
     def get_handle_from_gramps_id(self, gid):
+        """
+        Return the handle of the person having the given Gramps ID. 
+        """
         obj = self.dbstate.db.get_person_from_gramps_id(gid)
         if obj:
             return obj.get_handle()
@@ -248,14 +258,23 @@ class BasePersonView(ListView):
             return None
 
     def add(self, obj):
+        """
+        Add a new person to the database.
+        """
         person = gen.lib.Person()
+        #the editor requires a surname
+        person.primary_name.add_surname(gen.lib.Surname())
+        person.primary_name.set_primary_surname(0)
         
         try:
-            EditPerson(self.dbstate, self.uistate, [], gen.lib.Person())
+            EditPerson(self.dbstate, self.uistate, [], person)
         except Errors.WindowActiveError:
             pass
  
     def edit(self, obj):
+        """
+        Edit an existing person in the database.
+        """
         for handle in self.selected_handles():
             person = self.dbstate.db.get_person_from_handle(handle)
             try:
@@ -264,6 +283,9 @@ class BasePersonView(ListView):
                 pass
 
     def remove(self, obj):
+        """
+        Remove a person from the database.
+        """
         for sel in self.selected_handles():
             person = self.dbstate.db.get_person_from_handle(sel)
             self.active_person = person
@@ -285,19 +307,17 @@ class BasePersonView(ListView):
         self.uistate.set_busy_cursor(True)
 
         # create the transaction
-        trans = self.dbstate.db.transaction_begin()
+        with DbTxn('', self.dbstate.db) as trans:
         
-        # create name to save
-        person = self.active_person
-        active_name = _("Delete Person (%s)") % name_displayer.display(person)
+            # create name to save
+            person = self.active_person
+            active_name = _("Delete Person (%s)") % name_displayer.display(person)
 
-        # delete the person from the database
-        # Above will emit person-delete, which removes the person via 
-        # callback to the model, so row delete is signaled
-        self.dbstate.db.delete_person_from_database(person, trans)
-
-        # commit the transaction
-        self.dbstate.db.transaction_commit(trans, active_name)
+            # delete the person from the database
+            # Above will emit person-delete, which removes the person via 
+            # callback to the model, so row delete is signaled
+            self.dbstate.db.delete_person_from_database(person, trans)
+            trans.set_description(active_name)
 
         self.uistate.set_busy_cursor(False)
 
@@ -330,9 +350,10 @@ class BasePersonView(ListView):
         self.all_action.add_actions([
                 ('FilterEdit', None, _('Person Filter Editor'), None, None,
                 self.filter_editor),
-                ('Edit', gtk.STOCK_EDIT, _("action|_Edit..."), "<control>Return", 
-                 _("Edit the selected person"), self.edit), 
+                ('Edit', gtk.STOCK_EDIT, _("action|_Edit..."),
+                "<control>Return", self.EDIT_MSG, self.edit), 
                 ('QuickReport', None, _("Quick View"), None, None, None), 
+                ('WebConnect', None, _("Web Connection"), None, None, None), 
                 ('Dummy', None, '  ', None, None, self.dummy_report), 
                 ])
 
@@ -340,56 +361,54 @@ class BasePersonView(ListView):
         self.edit_action.add_actions(
             [
                 ('Add', gtk.STOCK_ADD, _("_Add..."), "<control>Insert", 
-                _("Add a new person"), self.add), 
+                 self.ADD_MSG, self.add), 
                 ('Remove', gtk.STOCK_REMOVE, _("_Remove"), "<control>Delete", 
-                 _("Remove the Selected Person"), self.remove),
-                ('CmpMerge', None, _('Compare and _Merge...'), None, None, 
-                 self.cmp_merge), 
-                ('FastMerge', None, _('_Fast Merge...'), None, None, 
-                 self.fast_merge), 
-                ('ExportTab', None, _('Export View...'), None, None, self.export), 
+                 self.DEL_MSG, self.remove),
+                ('Merge', 'gramps-merge', _('_Merge...'), None,
+                 self.MERGE_MSG, self.merge),
+                ('ExportTab', None, _('Export View...'), None, None,
+                 self.export), 
                 ])
 
         self._add_action_group(self.edit_action)
         self._add_action_group(self.all_action)
 
     def enable_action_group(self, obj):
+        """
+        Turns on the visibility of the View's action group.
+        """
         ListView.enable_action_group(self, obj)
         self.all_action.set_visible(True)
-        self.edit_action.set_visible(False)
+        self.edit_action.set_visible(True)
         self.edit_action.set_sensitive(not self.dbstate.db.readonly)
         
     def disable_action_group(self):
+        """
+        Turns off the visibility of the View's action group.
+        """
         ListView.disable_action_group(self)
 
         self.all_action.set_visible(False)
         self.edit_action.set_visible(False)
 
-    def cmp_merge(self, obj):
-        mlist = self.selected_handles()
+    def set_active(self):
+        """
+        Called when the page is displayed.
+        """
+        ListView.set_active(self)
+        self.uistate.viewmanager.tags.tag_enable()
 
-        if len(mlist) != 2:
-            ErrorDialog(
-        _("Cannot merge people"), 
-        _("Exactly two people must be selected to perform a merge. "
-          "A second person can be selected by holding down the "
-          "control key while clicking on the desired person."))
-        else:
-            import Merge 
-            person1 = self.dbstate.db.get_person_from_handle(mlist[0])
-            person2 = self.dbstate.db.get_person_from_handle(mlist[1])
-            if person1 and person2:
-                Merge.PersonCompare(self.dbstate, self.uistate, person1, 
-                                    person2, self.build_tree)
-            else:
-                ErrorDialog(
-                    _("Cannot merge people"), 
-                    _("Exactly two people must be selected to perform a "
-                      "merge. A second person can be selected by holding "
-                      "down the control key while clicking on the desired "
-                      "person."))
+    def set_inactive(self):
+        """
+        Called when the page is no longer displayed.
+        """
+        ListView.set_inactive(self)
+        self.uistate.viewmanager.tags.tag_disable()
 
-    def fast_merge(self, obj):
+    def merge(self, obj):
+        """
+        Merge the selected people.
+        """
         mlist = self.selected_handles()
 
         if len(mlist) != 2:
@@ -400,15 +419,38 @@ class BasePersonView(ListView):
           "control key while clicking on the desired person."))
         else:
             import Merge
-            
-            person1 = self.dbstate.db.get_person_from_handle(mlist[0])
-            person2 = self.dbstate.db.get_person_from_handle(mlist[1])
-            if person1 and person2:
-                Merge.MergePeopleUI(self.dbstate, self.uistate, person1, 
-                                    person2, self.build_tree)
-            else:
-                ErrorDialog(
-            _("Cannot merge people"), 
-            _("Exactly two people must be selected to perform a merge. "
-              "A second person can be selected by holding down the "
-              "control key while clicking on the desired person."))
+            Merge.MergePeople(self.dbstate, self.uistate, mlist[0], mlist[1])
+
+    def tag_updated(self, handle_list):
+        """
+        Update tagged rows when a tag color changes.
+        """
+        all_links = set([])
+        for tag_handle in handle_list:
+            links = set([link[1] for link in
+                         self.dbstate.db.find_backlink_handles(tag_handle,
+                                                    include_classes='Person')])
+            all_links = all_links.union(links)
+        self.row_update(list(all_links))
+
+    def add_tag(self, transaction, person_handle, tag_handle):
+        """
+        Add the given tag to the given person.
+        """
+        person = self.dbstate.db.get_person_from_handle(person_handle)
+        person.add_tag(tag_handle)
+        self.dbstate.db.commit_person(person, transaction)
+        
+    def get_default_gramplets(self):
+        """
+        Define the default gramplets for the sidebar and bottombar.
+        """
+        return (("Person Filter",),
+                ("Person Details",
+                 "Person Gallery",
+                 "Person Events",
+                 "Person Children",
+                 "Person Sources",
+                 "Person Notes",
+                 "Person Attributes",
+                 "Person Backlinks"))
