@@ -2,6 +2,7 @@
 #
 # Copyright (C) 2001-2006  Donald N. Allingham
 # Copyright (C) 2008       Gary Burton
+# Copyright (C) 2010       Nick Hall
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,7 +19,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: libplaceview.py 14971 2010-03-29 22:43:03Z nick-h $
+# $Id: libplaceview.py 17446 2011-05-07 18:41:14Z nick-h $
 
 """
 Base view for Place Views
@@ -74,50 +75,53 @@ class PlaceBaseView(ListView):
     """
     COL_NAME = 0
     COL_ID = 1
-    COL_PARISH = 2
-    COL_ZIP = 3
+    COL_STREET = 2
+    COL_LOCALITY = 3
     COL_CITY = 4
     COL_COUNTY = 5
     COL_STATE = 6
     COL_COUNTRY = 7
-    COL_LAT = 8
-    COL_LON = 9
-    COL_CHAN = 10
-    COL_STREET = 11
+    COL_ZIP = 8
+    COL_PARISH = 9
+    COL_LAT = 10
+    COL_LON = 11
+    COL_CHAN = 12
     # name of the columns
     COLUMN_NAMES = [
         _('Place Name'),
         _('ID'),
-        _('Church Parish'),
-        _('ZIP/Postal Code'),
+        _('Street'),
+        _('Locality'),
         _('City'),
         _('County'),
         _('State'),
         _('Country'),
+        _('ZIP/Postal Code'),
+        _('Church Parish'),
         _('Latitude'),
         _('Longitude'),
         _('Last Changed'),
-        _('Street'),
         ]
     # columns that contain markup
     MARKUP_COLS = [COL_NAME]
     # default setting with visible columns, order of the col, and their size
     CONFIGSETTINGS = (
-        ('columns.visible', [COL_NAME, COL_ID, COL_STREET, COL_CITY, COL_STATE
-                             ]),
-        ('columns.rank', [COL_NAME, COL_ID, COL_STREET, COL_ZIP, COL_CITY, 
-                           COL_COUNTY, COL_STATE, COL_COUNTRY, COL_LAT,
-                           COL_LON, COL_PARISH, COL_CHAN]),
-        ('columns.size', [250, 75, 100, 100, 100, 100, 150, 150, 150, 
-                             150, 150, 100])
+        ('columns.visible', [COL_NAME, COL_ID, COL_STREET, COL_LOCALITY,
+                             COL_CITY, COL_COUNTY, COL_STATE]),
+        ('columns.rank', [COL_NAME, COL_ID, COL_STREET, COL_LOCALITY, COL_CITY,
+                           COL_COUNTY, COL_STATE, COL_COUNTRY, COL_ZIP,
+                           COL_PARISH, COL_LAT, COL_LON, COL_CHAN]),
+        ('columns.size', [250, 75, 150, 150, 150, 150, 100, 100, 100, 
+                             100, 150, 150, 100])
         )    
     ADD_MSG     = _("Add a new place")
     EDIT_MSG    = _("Edit the selected place")
     DEL_MSG     = _("Delete the selected place")
+    MERGE_MSG   = _("Merge the selected places")
     FILTER_TYPE = "Place"
     QR_CATEGORY = CATEGORY_QR_PLACE
 
-    def __init__(self, dbstate, uistate, title, model, nav_group,
+    def __init__(self, pdata, dbstate, uistate, title, model, nav_group,
                  markup=None):
 
         signal_map = {
@@ -127,25 +131,24 @@ class PlaceBaseView(ListView):
             'place-rebuild' : self.object_build,
             }
 
-        self.func_list = {
-            '<CONTROL>J' : self.jump,
-            '<CONTROL>BackSpace' : self.key_delete,
-            }
-
         self.mapservice = config.get('interface.mapservice')
         self.mapservicedata = {}
 
         ListView.__init__(
-            self, title, dbstate, uistate,
-            self.COLUMN_NAMES, 13, 
+            self, title, pdata, dbstate, uistate,
+            self.COLUMN_NAMES, 14, 
             model, signal_map,
             dbstate.db.get_place_bookmarks(),
             Bookmarks.PlaceBookmarks, nav_group,
             multiple=True,
             filter_class=PlaceSidebarFilter, markup=markup)
 
-        config.connect("interface.filter",
-                          self.filter_toggle)
+        self.func_list.update({
+            '<CONTROL>J' : self.jump,
+            '<CONTROL>BackSpace' : self.key_delete,
+            })
+
+        self.additional_uis.append(self.additional_ui())
 
     def navigation_type(self):
         return 'Place'
@@ -155,8 +158,6 @@ class PlaceBaseView(ListView):
 
     def define_actions(self):
         ListView.define_actions(self)
-        self._add_action('FastMerge', None, _('_Merge...'),
-                         callback=self.fast_merge)
         self._add_toolmenu_action('MapsList', _('Loading...'),
                         _("Attempt to see selected locations with a Map "
                                 "Service (OpenstreetMap, Google Maps, ...)"),
@@ -291,7 +292,7 @@ class PlaceBaseView(ListView):
     def get_stock(self):
         return 'gramps-place'
 
-    def ui_definition(self):
+    def additional_ui(self):
         return '''<ui>
           <menubar name="MenuBar">
             <menu action="FileMenu">
@@ -317,11 +318,9 @@ class PlaceBaseView(ListView):
                 <menuitem action="Add"/>
                 <menuitem action="Edit"/>
                 <menuitem action="Remove"/>
+                <menuitem action="Merge"/>
               </placeholder>
               <menuitem action="FilterEdit"/>
-              <placeholder name="Merge">
-                <menuitem action="FastMerge"/>
-              </placeholder>
             </menu>
           </menubar>
           <toolbar name="ToolBar">
@@ -333,6 +332,7 @@ class PlaceBaseView(ListView):
               <toolitem action="Add"/>
               <toolitem action="Edit"/>
               <toolitem action="Remove"/>
+              <toolitem action="Merge"/>
               <separator/>
               <toolitem action="MapsList"/>
             </placeholder>
@@ -344,6 +344,7 @@ class PlaceBaseView(ListView):
             <menuitem action="Add"/>
             <menuitem action="Edit"/>
             <menuitem action="Remove"/>
+            <menuitem action="Merge"/>
             <separator/>
             <menu name="QuickReport" action="QuickReport">
               <menuitem action="Dummy"/>
@@ -397,7 +398,10 @@ class PlaceBaseView(ListView):
             except Errors.WindowActiveError:
                 pass
 
-    def fast_merge(self, obj):
+    def merge(self, obj):
+        """
+        Merge the selected places.
+        """
         mlist = self.selected_handles()
         
         if len(mlist) != 2:
@@ -417,5 +421,16 @@ class PlaceBaseView(ListView):
         else:
             return None
 
+    def get_default_gramplets(self):
+        """
+        Define the default gramplets for the sidebar and bottombar.
+        """
+        return (("Place Filter",),
+                ("Place Details",
+                 "Place Gallery",
+                 "Place Sources",
+                 "Place Notes",
+                 "Place Backlinks"))
+
 def make_callback(func, val):
     return lambda x: func(val)
