@@ -20,8 +20,10 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: rel_nl.py 12362 2009-03-19 04:25:37Z pez4brian $
-
+# $Id: rel_nl.py 16979 2011-03-30 01:50:08Z pez4brian $
+"""
+Specific classes for relationships.
+"""
 #-------------------------------------------------------------------------
 #
 # GRAMPS modules
@@ -30,7 +32,6 @@
 
 import gen.lib
 import Relationship
-from gen.plug import PluginManager
 
 #-------------------------------------------------------------------------
 #
@@ -243,12 +244,15 @@ _uncle_level = [ "",
 #
 #-------------------------------------------------------------------------
 class RelationshipCalculator(Relationship.RelationshipCalculator):
+    """
+    RelationshipCalculator Class
+    """
     
     #sibling strings
-    STEP= 'stief'
+    STEP = 'stief'
     HALF = 'half'
     
-    INLAW='aangetrouwde '
+    INLAW = 'aangetrouwde '
 
     def __init__(self):
         Relationship.RelationshipCalculator.__init__(self)
@@ -276,7 +280,7 @@ class RelationshipCalculator(Relationship.RelationshipCalculator):
             return "verre %s%sachterkleinzoon (%d generaties)" % (inlaw,
                                                                   step, level)
 
-    def _get_mother(self,level, step='', inlaw=''):
+    def _get_mother(self, level, step='', inlaw=''):
         """Internal Dutch method to create relation string
         """
         if level > len(_mother_level)-1:
@@ -393,7 +397,7 @@ class RelationshipCalculator(Relationship.RelationshipCalculator):
             going sideways in a branch as the nieces/newphews going downward
             from your brother/sisters. This used to be called "kozijn"
         """
-        removed = removed - 1
+        removed -= 1
         if removed > len(_removed_level)-1:
             return "verre %s%sneef (kozijn, %d graden)" % (inlaw, step, 
                                                            removed)
@@ -408,7 +412,7 @@ class RelationshipCalculator(Relationship.RelationshipCalculator):
             going sideways in a branch as the nieces/newphews going downward
             from your brother/sisters.  This used to be called "kozijn"
         """
-        removed = removed - 1
+        removed -= 1
         if removed > len(_removed_level)-1:
             return "verre %s%snicht (kozijn, %d graden)" % (inlaw, step, 
                                                            removed)
@@ -538,28 +542,17 @@ class RelationshipCalculator(Relationship.RelationshipCalculator):
                 rel_str = self._get_female_cousin(1, typestr, inlaw)
         return rel_str
 
-#-------------------------------------------------------------------------
-#
-# Register this class with the Plugins system 
-#
-#-------------------------------------------------------------------------
-pmgr = PluginManager.get_instance()
-pmgr.register_relcalc(RelationshipCalculator,
-    ["nl", "NL", "nl_NL", "nl_BE", "nederlands", "Nederlands", "nl_NL.UTF8",
-     "nl_BE.UTF8","nl_NL@euro", "nl_NL.UTF8@euro","nl_BE@euro",
-     "dutch","Dutch", "nl_NL.UTF-8", "nl_BE.UTF-8","nl_NL.utf-8",
-     "nl_BE.utf-8","nl_NL.utf8", "nl_BE.UTF-8", "nl_BE.UTF8@euro"])
 
 if __name__ == "__main__":
     # Test function. Call it as follows from the command line (so as to find
     #        imported modules):
     #    export PYTHONPATH=/path/to/gramps/src
-    #    python src/plugins/rel_nl.py
+    #    python src/plugins/rel/rel_nl.py
     
     """TRANSLATORS, copy this if statement at the bottom of your 
         rel_xx.py module, and test your work with:
-        python src/plugins/rel_xx.py
+        python src/plugins/rel/rel_xx.py
     """
     from Relationship import test
-    rc = RelationshipCalculator()
-    test(rc, True)
+    RC = RelationshipCalculator()
+    test(RC, True)
