@@ -21,12 +21,15 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: rel_sv.py 12362 2009-03-19 04:25:37Z pez4brian $
+# $Id: rel_sv.py 16979 2011-03-30 01:50:08Z pez4brian $
 
 # Written by Alex Roitman, largely based on Relationship.py by Don Allingham
 # and on valuable input from Jens Arvidsson
 # Updated to 3.0 by Peter Landgren 2007-12-30.
-
+#
+"""
+Swedish-specific definitions of relationships
+"""
 #-------------------------------------------------------------------------
 #
 # GRAMPS modules
@@ -35,12 +38,7 @@
 
 import gen.lib
 import Relationship
-from gen.plug import PluginManager
 
-#-------------------------------------------------------------------------
-#
-# Swedish-specific definitions of relationships
-#
 #-------------------------------------------------------------------------
 
 _cousin_level = [ "", "kusin", 
@@ -70,6 +68,9 @@ u"artonde", u"nittonde", u"tjugonde" ]
 #
 #-------------------------------------------------------------------------
 class RelationshipCalculator(Relationship.RelationshipCalculator):
+    """
+    RelationshipCalculator Class
+    """
 
     #sibling strings
     STEP = 'styv'
@@ -186,7 +187,7 @@ class RelationshipCalculator(Relationship.RelationshipCalculator):
             else:
                 result.append('mor')
         if inlaw != '' :
-            inlaw = 'ingifta '
+            inlaw = 's ingifta '
         result.append(self._get_cousin(level, step, inlaw))
         if step != '' and len(result)==1:
             result[0] = self.STEP + result[0]
@@ -310,7 +311,10 @@ class RelationshipCalculator(Relationship.RelationshipCalculator):
         return rel_str
 
 
-    def get_plural_relationship_string(self, Ga, Gb):
+    def get_plural_relationship_string(self, Ga, Gb,
+                                       reltocommon_a='', reltocommon_b='',
+                                       only_birth=True,
+                                       in_law_a=False, in_law_b=False):
         """
         Provide a string that describes the relationsip between a person, and
         a group of people with the same relationship. E.g. "grandparents" or
@@ -393,6 +397,10 @@ class RelationshipCalculator(Relationship.RelationshipCalculator):
                 rel_str = self.pair_up(result,'')                
             else:
                 rel_str =  u"avlägsna kusiner"
+                
+        if in_law_b == True:
+            rel_str = "makar till %s" % rel_str
+                
         return rel_str
 
     def get_single_relationship_string(self, Ga, Gb, gender_a, gender_b,
@@ -520,27 +528,18 @@ class RelationshipCalculator(Relationship.RelationshipCalculator):
             rel_str = self._get_cousins_descendant(gender_b, reltocommon_b, reltocommon_a, step, inlaw)
         return rel_str
 
-#-------------------------------------------------------------------------
-#
-# Register this class with the Plugins system 
-#
-#-------------------------------------------------------------------------
-pmgr = PluginManager.get_instance()
-pmgr.register_relcalc(RelationshipCalculator,
-    ["sv","SV","sv_SE","swedish","Swedish","sv_SE.UTF8","sv_SE@euro","sv_SE.UTF8@euro",
-            "svenska","Svenska", "sv_SE.UTF-8", "sv_SE.utf-8", "sv_SE.utf8"])
 
 if __name__ == "__main__":
     # Test function. Call it as follows from the command line (so as to find
     #        imported modules):
     #    export PYTHONPATH=/path/to/gramps/src 
-    # python src/plugins/rel_fr.py 
+    # python src/plugins/rel/rel_sv.py 
     # (Above not needed here)
     
     """TRANSLATORS, copy this if statement at the bottom of your 
         rel_xx.py module, and test your work with:
-        python src/plugins/rel_xx.py
+        python src/plugins/rel/rel_xx.py
     """
     from Relationship import test
-    rc = RelationshipCalculator()
-    test(rc, True)
+    RC = RelationshipCalculator()
+    test(RC, True)
