@@ -19,7 +19,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
-# $Id: BirthdayReport.py 16462 2011-01-24 20:14:35Z gbritton $
+# $Id: BirthdayReport.py 17839 2011-06-27 15:21:40Z m_d_n $
 #
 
 #------------------------------------------------------------------------
@@ -108,7 +108,8 @@ class CalendarReport(Report):
                 name = Name(married_name)
             else:
                 name = Name(primary_name)
-                name.set_surname(maiden_name)
+                surname_obj = name.get_primary_surname()
+                surname_obj.set_surname(maiden_name)
         else:
             name = Name(primary_name)
         name.set_display_as(self.name_format)
