@@ -2,6 +2,7 @@
 #
 # Copyright (C) 2003-2006  Donald N. Allingham
 # Copyright (C) 2008       Brian G. Matherly
+# Copyright (C) 2010       Jakim Friant
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -21,7 +22,7 @@
 # Written by Alex Roitman, 
 # largely based on the SimpleBookTitle.py by Don Allingham
 
-# $Id: CustomBookText.py 14091 2010-01-18 04:42:17Z pez4brian $
+# $Id: CustomBookText.py 16979 2011-03-30 01:50:08Z pez4brian $
 
 #------------------------------------------------------------------------
 #
@@ -42,7 +43,8 @@ from gen.ggettext import gettext as _
 #
 #------------------------------------------------------------------------
 from gen.plug.menu import TextOption
-from ReportBase import Report, MenuReportOptions
+from gen.plug.report import Report
+from gui.plug.report import MenuReportOptions
 from gen.plug.docgen import (FontStyle, ParagraphStyle, FONT_SANS_SERIF, 
                              PARA_ALIGN_CENTER)
 
