@@ -6,6 +6,8 @@
 # Copyright (C) 2007-2009 Brian G. Matherly
 # Copyright (C) 2008      James Friedmann <jfriedmannj@gmail.com>
 # Copyright (C) 2009      Benny Malengier <benny.malengier@gramps-project.org>
+# Copyright (C) 2010      Jakim Friant
+# Copyright (C) 2010      Vlada Peri\u0107
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -22,7 +24,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: DetAncestralReport.py 15845 2010-09-01 16:17:36Z ldnp $
+# $Id: DetAncestralReport.py 17729 2011-06-09 13:33:49Z ldnp $
 
 """Reports/Text Reports/Detailed Ancestral Report"""
 
@@ -44,12 +46,17 @@ from gen.lib import EventType, FamilyRelType, Person, NoteType
 from gen.plug.docgen import (IndexMark, FontStyle, ParagraphStyle,
                             FONT_SANS_SERIF, FONT_SERIF, 
                             INDEX_TYPE_TOC, PARA_ALIGN_CENTER)
-from gen.plug.menu import BooleanOption, NumberOption, PersonOption
-from ReportBase import (Report, ReportUtils, MenuReportOptions,
-                        Bibliography, Endnotes)
+from gen.plug.menu import BooleanOption, NumberOption, PersonOption, EnumeratedListOption
+from gen.plug.report import ( Report, Bibliography )
+from gen.plug.report import endnotes
+from gen.plug.report import utils as ReportUtils
+from gui.plug.report import MenuReportOptions
+                        
 import DateHandler
 
 from libnarrate import Narrator
+import TransUtils
+from libtranslate import Translator, get_language_string
 
 #------------------------------------------------------------------------
 #
@@ -100,28 +107,31 @@ class DetAncestorReport(Report):
         self.map = {}
 
         menu = options_class.menu
-        self.max_generations = menu.get_option_by_name('gen').get_value()
-        self.pgbrk         = menu.get_option_by_name('pagebbg').get_value()
-        self.pgbrkenotes   = menu.get_option_by_name('pageben').get_value()
-        self.fulldate      = menu.get_option_by_name('fulldates').get_value()
-        use_fulldate       = self.fulldate
-        self.listchildren  = menu.get_option_by_name('listc').get_value()
-        self.includenotes  = menu.get_option_by_name('incnotes').get_value()
-        use_call           = menu.get_option_by_name('usecall').get_value()
-        blankplace         = menu.get_option_by_name('repplace').get_value()
-        blankdate          = menu.get_option_by_name('repdate').get_value()
-        self.calcageflag   = menu.get_option_by_name('computeage').get_value()
-        self.dupperson     = menu.get_option_by_name('omitda').get_value()
-        self.verbose       = menu.get_option_by_name('verbose').get_value()
-        self.childref      = menu.get_option_by_name('desref').get_value()
-        self.addimages     = menu.get_option_by_name('incphotos').get_value()
-        self.inc_names     = menu.get_option_by_name('incnames').get_value()
-        self.inc_events    = menu.get_option_by_name('incevents').get_value()
-        self.inc_addr      = menu.get_option_by_name('incaddresses').get_value()
-        self.inc_sources   = menu.get_option_by_name('incsources').get_value()
-        self.inc_srcnotes  = menu.get_option_by_name('incsrcnotes').get_value()
-        self.inc_attrs     = menu.get_option_by_name('incattrs').get_value()
-        pid                = menu.get_option_by_name('pid').get_value()
+        get_option_by_name = menu.get_option_by_name
+        get_value = lambda name: get_option_by_name(name).get_value()
+
+        self.max_generations = get_value('gen')
+        self.pgbrk         = get_value('pagebbg')
+        self.pgbrkenotes   = get_value('pageben')
+        self.fulldate      = get_value('fulldates')
+        use_fulldate     = self.fulldate
+        self.listchildren  = get_value('listc')
+        self.includenotes  = get_value('incnotes')
+        use_call           = get_value('usecall')
+        blankplace         = get_value('repplace')
+        blankdate          = get_value('repdate')
+        self.calcageflag   = get_value('computeage')
+        self.dupperson     = get_value('omitda')
+        self.verbose       = get_value('verbose')
+        self.childref      = get_value('desref')
+        self.addimages     = get_value('incphotos')
+        self.inc_names     = get_value('incnames')
+        self.inc_events    = get_value('incevents')
+        self.inc_addr      = get_value('incaddresses')
+        self.inc_sources   = get_value('incsources')
+        self.inc_srcnotes  = get_value('incsrcnotes')
+        self.inc_attrs     = get_value('incattrs')
+        pid                = get_value('pid')
         self.center_person = database.get_person_from_gramps_id(pid)
         if (self.center_person == None) :
             raise ReportError(_("Person %s is not in the Database") % pid )
@@ -138,13 +148,21 @@ class DetAncestorReport(Report):
             empty_place = EMPTY_ENTRY
         else:
             empty_place = ""
-            
-        self.__narrator = Narrator(self.database, self.verbose, 
-                                   use_call, use_fulldate, 
-                                   empty_date, empty_place, 
+
+        language = get_value('trans')
+        translator = Translator(language)
+        self._ = translator.gettext
+
+        self.__narrator = Narrator(self.database, self.verbose, use_call,use_fulldate , 
+                                   empty_date, empty_place,
+                                   translator=translator,
                                    get_endnote_numbers=self.endnotes)
 
-        self.bibli = Bibliography(Bibliography.MODE_PAGE)
+        self.__get_date = translator.get_date
+
+        self.__get_type = translator.get_type
+
+        self.bibli = Bibliography(Bibliography.MODE_DATE|Bibliography.MODE_PAGE)
 
     def apply_filter(self, person_handle, index):
         if (not person_handle) or (index >= 2**self.max_generations):
@@ -163,7 +181,7 @@ class DetAncestorReport(Report):
 
         name = _nd.display_name(self.center_person.get_primary_name())
         self.doc.start_paragraph("DAR-Title")
-        title = _("Ancestral Report for %s") % name
+        title = self._("Ancestral Report for %s") % name
         mark = IndexMark(title, INDEX_TYPE_TOC, 1)
         self.doc.write_text(title, mark)
         self.doc.end_paragraph()
@@ -175,7 +193,7 @@ class DetAncestorReport(Report):
                 if self.pgbrk and generation > 0:
                     self.doc.page_break()
                 self.doc.start_paragraph("DAR-Generation")
-                text = _("Generation %d") % (generation+1)
+                text = self._("Generation %d") % (generation+1)
                 mark = IndexMark(text, INDEX_TYPE_TOC, 2)
                 self.doc.write_text(text, mark)
                 self.doc.end_paragraph()
@@ -206,7 +224,8 @@ class DetAncestorReport(Report):
         if self.inc_sources:
             if self.pgbrkenotes:
                 self.doc.page_break()
-            Endnotes.write_endnotes(self.bibli, self.database, self.doc,
+            # it ignores language set for Note type (use locale)
+            endnotes.write_endnotes(self.bibli, self.database, self.doc,
                                     printnotes=self.inc_srcnotes)
 
     def write_person(self, key):
@@ -241,7 +260,7 @@ class DetAncestorReport(Report):
                     break
                 if self.map[key] == self.map[dkey]:
                     self.doc.write_text(
-                        _("%(name)s is the same person as [%(id_str)s].") % 
+                        self._("%(name)s is the same person as [%(id_str)s].") % 
                         { 'name' : '', 'id_str' : str(dkey) })
                     self.doc.end_paragraph()
                     return 1    # Duplicate person
@@ -282,32 +301,34 @@ class DetAncestorReport(Report):
         notelist = person.get_note_list()
         if len(notelist) > 0 and self.includenotes:
             self.doc.start_paragraph("DAR-NoteHeader")
-            self.doc.write_text(_("Notes for %s") % name)
+            self.doc.write_text(self._("Notes for %s") % name)
             self.doc.end_paragraph()
             for notehandle in notelist:
                 note = self.database.get_note_from_handle(notehandle)
                 self.doc.write_styled_note(note.get_styledtext(), 
                                            note.get_format(), "DAR-Entry",
-                                           contains_html= note.get_type() \
-                                                        == NoteType.HTML_CODE)
+                                           contains_html = note.get_type()
+                                                        == NoteType.HTML_CODE
+                                          )
 
         first = True
         if self.inc_names:
             for alt_name in person.get_alternate_names():
                 if first:
                     self.doc.start_paragraph('DAR-MoreHeader')
-                    self.doc.write_text(_('More about %(person_name)s:') % { 
-                        'person_name' : name })
+                    self.doc.write_text(self._('More about %(person_name)s:')
+                                                % {'person_name': name})
                     self.doc.end_paragraph()
                     first = False
                 self.doc.start_paragraph('DAR-MoreDetails')
-                atype = str( alt_name.get_type() )
+                atype = self.__get_type(alt_name.get_type())
                 self.doc.write_text_citation(
-                    _('%(name_kind)s: %(name)s%(endnotes)s') % {
-                    'name_kind' : atype,
-                    'name' : alt_name.get_regular_name(),
-                    'endnotes' : self.endnotes(alt_name),
-                    })
+                    self._('%(name_kind)s: %(name)s%(endnotes)s') % {
+                                'name_kind' : self._(atype),
+                                'name' : alt_name.get_regular_name(),
+                                'endnotes' : self.endnotes(alt_name),
+                                }
+                          )
                 self.doc.end_paragraph()
 
         if self.inc_events:
@@ -319,7 +340,7 @@ class DetAncestorReport(Report):
                 
                 if first:
                     self.doc.start_paragraph('DAR-MoreHeader')
-                    self.doc.write_text(_('More about %(person_name)s:') % { 
+                    self.doc.write_text(self._('More about %(person_name)s:') % { 
                         'person_name' : _nd.display(person) })
                     self.doc.end_paragraph()
                     first = 0
@@ -330,17 +351,17 @@ class DetAncestorReport(Report):
             for addr in person.get_address_list():
                 if first:
                     self.doc.start_paragraph('DAR-MoreHeader')
-                    self.doc.write_text(_('More about %(person_name)s:') % { 
+                    self.doc.write_text(self._('More about %(person_name)s:') % { 
                         'person_name' : name })
                     self.doc.end_paragraph()
                     first = False
                 self.doc.start_paragraph('DAR-MoreDetails')
-
-                text = ReportUtils.get_address_str(addr)
-                self.doc.write_text(_('Address: '))
                 
+                text = ReportUtils.get_address_str(addr)
+                self.doc.write_text(self._('Address: '))
+
                 if self.fulldate:
-                    date = DateHandler.get_date(addr)
+                    date = self.__get_date(addr.get_date_object())
                 else:
                     date = addr.get_date_object().get_year()
 
@@ -354,15 +375,16 @@ class DetAncestorReport(Report):
             attrs = person.get_attribute_list()
             if first and attrs:
                 self.doc.start_paragraph('DAR-MoreHeader')
-                self.doc.write_text(_('More about %(person_name)s:') % { 
+                self.doc.write_text(self._('More about %(person_name)s:') % { 
                     'person_name' : name })
                 self.doc.end_paragraph()
                 first = False
 
             for attr in attrs:
                 self.doc.start_paragraph('DAR-MoreDetails')
-                text = _("%(type)s: %(value)s%(endnotes)s") % {
-                    'type'     : attr.get_type(),
+                attrName = self.__get_type(attr.get_type())
+                text = self._("%(type)s: %(value)s%(endnotes)s") % {
+                    'type'     : self._(attrName),
                     'value'    : attr.get_value(),
                     'endnotes' : self.endnotes(attr) }
                 self.doc.write_text_citation( text )
@@ -373,12 +395,12 @@ class DetAncestorReport(Report):
     def write_event(self, event_ref):
         text = ""
         event = self.database.get_event_from_handle(event_ref.ref)
-        
+
         if self.fulldate:
-            date = DateHandler.get_date(event)
+            date = self.__get_date(event.get_date_object())
         else:
             date = event.get_date_object().get_year()
-            
+
         ph = event.get_place_handle()
         if ph:
             place = self.database.get_place_from_handle(ph).get_title()
@@ -386,14 +408,14 @@ class DetAncestorReport(Report):
             place = u''
 
         self.doc.start_paragraph('DAR-MoreDetails')
-        evtName = str( event.get_type() )
+        evtName = self.__get_type(event.get_type())
         if date and place:
-            text +=  _('%(date)s, %(place)s') % { 
+            text +=  self._('%(date)s, %(place)s') % { 
                        'date' : date, 'place' : place }
         elif date:
-            text += _('%(date)s') % {'date' : date}
+            text += self._('%(date)s') % {'date' : date}
         elif place:
-            text += _('%(place)s') % { 'place' : place }
+            text += self._('%(place)s') % { 'place' : place }
 
         if event.get_description():
             if text:
@@ -405,8 +427,8 @@ class DetAncestorReport(Report):
         if text:
             text += ". "
             
-        text = _('%(event_name)s: %(event_text)s') % {
-                 'event_name' : _(evtName),
+        text = self._('%(event_name)s: %(event_text)s') % {
+                 'event_name' : self._(evtName),
                  'event_text' : text }
         
         self.doc.write_text_citation(text)
@@ -418,8 +440,9 @@ class DetAncestorReport(Report):
             for attr in attr_list:
                 if text:
                     text += "; "
-                text += _("%(type)s: %(value)s%(endnotes)s") % {
-                    'type'     : attr.get_type(),
+                attrName = self.__get_type(attr.get_type())
+                text += self._("%(type)s: %(value)s%(endnotes)s") % {
+                    'type'     : self._(attrName),
                     'value'    : attr.get_value(),
                     'endnotes' : self.endnotes(attr) }
             text = " " + text
@@ -436,8 +459,9 @@ class DetAncestorReport(Report):
                 note = self.database.get_note_from_handle(notehandle)
                 self.doc.write_styled_note(note.get_styledtext(), 
                                            note.get_format(),"DAR-MoreDetails",
-                                           contains_html= note.get_type() \
+                                           contains_html = (note.get_type()
                                                         == NoteType.HTML_CODE)
+                                          )
                 
     def write_parents(self, person):
         family_handle = person.get_main_parents_family_handle()
@@ -498,18 +522,18 @@ class DetAncestorReport(Report):
             mother = self.database.get_person_from_handle(mother_handle)
             mother_name = _nd.display(mother)
         else:
-            mother_name = _("unknown")
+            mother_name = self._("unknown")
 
         father_handle = family.get_father_handle()
         if father_handle:
             father = self.database.get_person_from_handle(father_handle)
             father_name = _nd.display(father)
         else:
-            father_name = _("unknown")
+            father_name = self._("unknown")
 
         self.doc.start_paragraph("DAR-ChildTitle")
         self.doc.write_text(
-                        _("Children of %(mother_name)s and %(father_name)s") % 
+                        self._("Children of %(mother_name)s and %(father_name)s") % 
                             {'father_name': father_name,
                              'mother_name': mother_name} )
         self.doc.end_paragraph()
@@ -548,21 +572,21 @@ class DetAncestorReport(Report):
             mother = self.database.get_person_from_handle(mother_handle)
             mother_name = _nd.display(mother)
         else:
-            mother_name = _("unknown")
+            mother_name = self._("unknown")
 
         father_handle = family.get_father_handle()
         if father_handle:
             father = self.database.get_person_from_handle(father_handle)
             father_name = _nd.display(father)
         else:
-            father_name = _("unknown")
+            father_name = self._("unknown")
 
         first = 1
         for event_ref in family.get_event_ref_list():
             if first:
                 self.doc.start_paragraph('DAR-MoreHeader')
                 self.doc.write_text(
-                    _('More about %(mother_name)s and %(father_name)s:') % { 
+                    self._('More about %(mother_name)s and %(father_name)s:') % { 
                     'mother_name' : mother_name,
                     'father_name' : father_name })
                 self.doc.end_paragraph()
@@ -587,12 +611,12 @@ class DetAncestorReport(Report):
                     event = self.database.get_event_from_handle(event_ref.ref)
                     if event:
                         etype = event.get_type()
-                        if etype == EventType.BAPTISM or \
-                           etype == EventType.BURIAL or \
-                           etype == EventType.BIRTH  or \
-                           etype == EventType.DEATH     :
-                            has_info = True
-                            break   
+                        if (etype == EventType.BAPTISM or
+                            etype == EventType.BURIAL or
+                            etype == EventType.BIRTH  or
+                            etype == EventType.DEATH):
+                                has_info = True
+                                break   
                 if not has_info:
                     family_handle = ind.get_main_parents_family_handle()
                     if family_handle:
@@ -614,9 +638,9 @@ class DetAncestorReport(Report):
                 mark = ReportUtils.get_person_mark(self.database, ind)
         
                 if family.get_relationship() == FamilyRelType.MARRIED:
-                    self.doc.write_text(_("Spouse: %s") % name, mark)
+                    self.doc.write_text(self._("Spouse: %s") % name, mark)
                 else:
-                    self.doc.write_text(_("Relationship with: %s") % name, mark)
+                    self.doc.write_text(self._("Relationship with: %s") % name, mark)
                 if name[-1:] != '.':
                     self.doc.write_text(".")
                 self.doc.write_text_citation(self.endnotes(ind))
@@ -654,7 +678,7 @@ class DetAncestorReport(Report):
         if not obj or not self.inc_sources:
             return ""
         
-        txt = Endnotes.cite_source(self.bibli, obj)
+        txt = endnotes.cite_source(self.bibli, obj)
         if txt:
             txt = '<super>' + txt + '</super>'
         return txt
@@ -674,103 +698,121 @@ class DetAncestorOptions(MenuReportOptions):
         MenuReportOptions.__init__(self, name, dbase)
         
     def add_menu_options(self, menu):
-        category_name = _("Report Options")
+        from functools import partial
+
+        # Report Options
+
+        addopt = partial(menu.add_option, _("Report Options"))
         
         pid = PersonOption(_("Center Person"))
         pid.set_help(_("The center person for the report"))
-        menu.add_option(category_name, "pid", pid)
+        addopt("pid", pid)
         
         gen = NumberOption(_("Generations"),10,1,100)
         gen.set_help(_("The number of generations to include in the report"))
-        menu.add_option(category_name,"gen",gen)
+        addopt("gen", gen)
         
         pagebbg = BooleanOption(_("Page break between generations"),False)
         pagebbg.set_help(
                      _("Whether to start a new page after each generation."))
-        menu.add_option(category_name,"pagebbg",pagebbg)
+        addopt("pagebbg", pagebbg)
 
         pageben = BooleanOption(_("Page break before end notes"),False)
         pageben.set_help(
                      _("Whether to start a new page before the end notes."))
-        menu.add_option(category_name,"pageben",pageben)
+        addopt("pageben", pageben)
         
-        category_name = _("Content")
+        trans = EnumeratedListOption(_("Translation"),
+                                      Translator.DEFAULT_TRANSLATION_STR)
+        trans.add_item(Translator.DEFAULT_TRANSLATION_STR, _("Default"))
+        for language in TransUtils.get_available_translations():
+            trans.add_item(language, get_language_string(language))
+        trans.set_help(_("The translation to be used for the report."))
+        addopt("trans", trans)
+
+        # Content options
+
+        addopt = partial(menu.add_option, _("Content"))
 
         usecall = BooleanOption(_("Use callname for common name"),False)
         usecall.set_help(_("Whether to use the call name as the first name."))
-        menu.add_option(category_name,"usecall",usecall)
+        addopt("usecall", usecall)
         
         fulldates = BooleanOption(
                               _("Use full dates instead of only the year"),True)
         fulldates.set_help(_("Whether to use full dates instead of just year."))
-        menu.add_option(category_name,"fulldates",fulldates)
+        addopt("fulldates", fulldates)
         
         listc = BooleanOption(_("List children"),True)
         listc.set_help(_("Whether to list children."))
-        menu.add_option(category_name,"listc",listc)
+        addopt("listc", listc)
         
         computeage = BooleanOption(_("Compute death age"),True)
         computeage.set_help(_("Whether to compute a person's age at death."))
-        menu.add_option(category_name,"computeage",computeage)
+        addopt("computeage", computeage)
         
         omitda = BooleanOption(_("Omit duplicate ancestors"),True)
         omitda.set_help(_("Whether to omit duplicate ancestors."))
-        menu.add_option(category_name,"omitda", omitda)
+        addopt("omitda", omitda)
         
         verbose = BooleanOption(_("Use Complete Sentences"),True)
         verbose.set_help(
                  _("Whether to use complete sentences or succinct language."))
-        menu.add_option(category_name,"verbose",verbose)
+        addopt("verbose", verbose)
 
         desref = BooleanOption(_("Add descendant reference in child list"),True)
         desref.set_help(
                     _("Whether to add descendant references in child list."))
-        menu.add_option(category_name,"desref",desref)
+        addopt("desref", desref)
 
-        category_name = _("Include")
+        # What to include
+
+        addopt = partial(menu.add_option, _("Include"))
         
         incnotes = BooleanOption(_("Include notes"),True)
         incnotes.set_help(_("Whether to include notes."))
-        menu.add_option(category_name,"incnotes",incnotes)
+        addopt("incnotes", incnotes)
 
         incattrs = BooleanOption(_("Include attributes"),False)
         incattrs.set_help(_("Whether to include attributes."))
-        menu.add_option(category_name,"incattrs",incattrs)
+        addopt("incattrs", incattrs)
         
         incphotos = BooleanOption(_("Include Photo/Images from Gallery"),False)
         incphotos.set_help(_("Whether to include images."))
-        menu.add_option(category_name,"incphotos",incphotos)
+        addopt("incphotos", incphotos)
 
         incnames = BooleanOption(_("Include alternative names"),False)
         incnames.set_help(_("Whether to include other names."))
-        menu.add_option(category_name,"incnames",incnames)
+        addopt("incnames", incnames)
 
         incevents = BooleanOption(_("Include events"),False)
         incevents.set_help(_("Whether to include events."))
-        menu.add_option(category_name,"incevents",incevents)
+        addopt("incevents", incevents)
 
         incaddresses = BooleanOption(_("Include addresses"),False)
         incaddresses.set_help(_("Whether to include addresses."))
-        menu.add_option(category_name,"incaddresses",incaddresses)
+        addopt("incaddresses", incaddresses)
 
         incsources = BooleanOption(_("Include sources"),False)
         incsources.set_help(_("Whether to include source references."))
-        menu.add_option(category_name,"incsources",incsources)
+        addopt("incsources", incsources)
 
         incsrcnotes = BooleanOption(_("Include sources notes"), False)
         incsrcnotes.set_help(_("Whether to include source notes in the "
             "Endnotes section. Only works if Include sources is selected."))
-        menu.add_option(category_name, "incsrcnotes", incsrcnotes)
+        addopt("incsrcnotes", incsrcnotes)
 
-        category_name = _("Missing information")        
+        # How to handle missing information
+
+        addopt = partial(menu.add_option, _("Missing information"))
 
         repplace = BooleanOption(_("Replace missing places with ______"),False)
         repplace.set_help(_("Whether to replace missing Places with blanks."))
-        menu.add_option(category_name,"repplace",repplace)
+        addopt("repplace", repplace)
 
         repdate = BooleanOption(_("Replace missing dates with ______"),False)
         repdate.set_help(_("Whether to replace missing Dates with blanks."))
-        menu.add_option(category_name,"repdate",repdate)
+        addopt("repdate", repdate)
 
     def make_default_style(self,default_style):
         """Make the default output style for the Detailed Ancestral Report"""
@@ -858,4 +900,4 @@ class DetAncestorOptions(MenuReportOptions):
         para.set_description(_('The style used for additional detail data.'))
         default_style.add_paragraph_style("DAR-MoreDetails",para)
 
-        Endnotes.add_endnote_styles(default_style)
+        endnotes.add_endnote_styles(default_style)
