@@ -7,6 +7,8 @@
 # Copyright (C) 2007      Robert Cawley  <rjc@cawley.id.au>
 # Copyright (C) 2008-2009 James Friedmann <jfriedmannj@gmail.com>
 # Copyright (C) 2009      Benny Malengier <benny.malengier@gramps-project.org>
+# Copyright (C) 2010      Jakim Friant
+# Copyright (C) 2010      Vlada Peri\u0107
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -23,7 +25,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: DetDescendantReport.py 15168 2010-04-16 20:58:51Z bmcage $
+# $Id: DetDescendantReport.py 17729 2011-06-09 13:33:49Z ldnp $
 
 """Reports/Text Reports/Detailed Descendant Report"""
 
@@ -33,6 +35,7 @@
 #
 #------------------------------------------------------------------------
 from gen.ggettext import gettext as _
+from functools import partial
 
 #------------------------------------------------------------------------
 #
@@ -47,11 +50,16 @@ from gen.plug.menu import (BooleanOption, NumberOption, PersonOption,
 from gen.plug.docgen import (IndexMark, FontStyle, ParagraphStyle, 
                              FONT_SANS_SERIF, FONT_SERIF, 
                              INDEX_TYPE_TOC, PARA_ALIGN_CENTER)
-from ReportBase import (Report, ReportUtils, MenuReportOptions,
-                        Bibliography, Endnotes)
+from gen.plug.report import (Report, Bibliography)
+from gen.plug.report import endnotes
+from gen.plug.report import utils as ReportUtils
+from gui.plug.report import MenuReportOptions
+                        
 import DateHandler
 
 from libnarrate import Narrator
+import TransUtils
+from libtranslate import Translator, get_language_string
 
 #------------------------------------------------------------------------
 #
@@ -83,20 +91,27 @@ class DetDescendantReport(Report):
         
         gen           - Maximum number of generations to include.
         pagebgg       - Whether to include page breaks between generations.
-        firstName     - Whether to use first names instead of pronouns.
-        fulldate      - Whether to use full dates instead of just year.
-        listchildren  - Whether to list children.
-        inc_mates     - Whether to include information about spouses
-        inc_notes     - Whether to include notes.
-        inc_attrs     - Whether to include attributes
-        blankPlace    - Whether to replace missing Places with ___________.
-        blankDate     - Whether to replace missing Dates with ___________.
-        calcageflag   - Whether to compute age.
-        dubperson     - Whether to omit duplicate ancestors (e.g. when distant cousins mary).
+        pageben       - Whether to include page break before End Notes.
+        fulldates     - Whether to use full dates instead of just year.
+        listc         - Whether to list children.
+        incnotes      - Whether to include notes.
+        usecall       - Whether to use the call name as the first name.
+        repplace      - Whether to replace missing Places with ___________.
+        repdate       - Whether to replace missing Dates with ___________.
+        computeage    - Whether to compute age.
+        omitda        - Whether to omit duplicate ancestors (e.g. when distant cousins marry).
         verbose       - Whether to use complete sentences.
-        numbering     - The descendency numbering system to be utilized.
-        childref      - Whether to add descendant references in child list.
-        addimages     - Whether to include images.
+        numbering     - The descendancy numbering system to be utilized.
+        desref        - Whether to add descendant references in child list.
+        incphotos     - Whether to include images.
+        incnames      - Whether to include other names.
+        incevents     - Whether to include events.
+        incaddresses  - Whether to include addresses.
+        incsrcnotes   - Whether to include source notes in the Endnotes section. Only works if Include sources is selected.
+        incmates      - Whether to include information about spouses
+        incattrs      - Whether to include attributes
+        incpaths      - Whether to include the path of descendancy from the start-person to each descendant.
+        incssign      - Whether to include a sign ('+') before the descendant number in the child-list to indicate a child has succession.
         pid           - The Gramps ID of the center person for the report.
         """
         Report.__init__(self, database, options_class)
@@ -104,30 +119,34 @@ class DetDescendantReport(Report):
         self.map = {}
 
         menu = options_class.menu
-        self.max_generations = menu.get_option_by_name('gen').get_value()
-        self.pgbrk         = menu.get_option_by_name('pagebbg').get_value()
-        self.fulldate      = menu.get_option_by_name('fulldates').get_value()
-        self.listchildren  = menu.get_option_by_name('listc').get_value()
-        self.inc_notes     = menu.get_option_by_name('incnotes').get_value()
-        use_call           = menu.get_option_by_name('usecall').get_value()
-        blankplace         = menu.get_option_by_name('repplace').get_value()
-        blankdate          = menu.get_option_by_name('repdate').get_value()
-        self.calcageflag   = menu.get_option_by_name('computeage').get_value()
-        self.dubperson     = menu.get_option_by_name('omitda').get_value()
-        self.verbose       = menu.get_option_by_name('verbose').get_value()
-        self.numbering     = menu.get_option_by_name('numbering').get_value()
-        self.childref      = menu.get_option_by_name('desref').get_value()
-        self.addimages     = menu.get_option_by_name('incphotos').get_value()
-        self.inc_names     = menu.get_option_by_name('incnames').get_value()
-        self.inc_events    = menu.get_option_by_name('incevents').get_value()
-        self.inc_addr      = menu.get_option_by_name('incaddresses').get_value()
-        self.inc_sources   = menu.get_option_by_name('incsources').get_value()
-        self.inc_srcnotes  = menu.get_option_by_name('incsrcnotes').get_value()
-        self.inc_mates     = menu.get_option_by_name('incmates').get_value()
-        self.inc_attrs     = menu.get_option_by_name('incattrs').get_value()
-        self.inc_paths     = menu.get_option_by_name('incpaths').get_value()
-        self.inc_ssign     = menu.get_option_by_name('incssign').get_value()
-        pid                = menu.get_option_by_name('pid').get_value()
+        get_option_by_name = menu.get_option_by_name
+        get_value = lambda name: get_option_by_name(name).get_value()
+        self.max_generations = get_value('gen')
+        self.pgbrk         = get_value('pagebbg')
+        self.pgbrkenotes   = get_value('pageben')
+        self.fulldate      = get_value('fulldates')
+        use_fulldate     = self.fulldate
+        self.listchildren  = get_value('listc')
+        self.inc_notes     = get_value('incnotes')
+        use_call           = get_value('usecall')
+        blankplace         = get_value('repplace')
+        blankdate          = get_value('repdate')
+        self.calcageflag   = get_value('computeage')
+        self.dubperson     = get_value('omitda')
+        self.verbose       = get_value('verbose')
+        self.numbering     = get_value('numbering')
+        self.childref      = get_value('desref')
+        self.addimages     = get_value('incphotos')
+        self.inc_names     = get_value('incnames')
+        self.inc_events    = get_value('incevents')
+        self.inc_addr      = get_value('incaddresses')
+        self.inc_sources   = get_value('incsources')
+        self.inc_srcnotes  = get_value('incsrcnotes')
+        self.inc_mates     = get_value('incmates')
+        self.inc_attrs     = get_value('incattrs')
+        self.inc_paths     = get_value('incpaths')
+        self.inc_ssign     = get_value('incssign')
+        pid                = get_value('pid')
         self.center_person = database.get_person_from_gramps_id(pid)
         if (self.center_person == None) :
             raise ReportError(_("Person %s is not in the Database") % pid )
@@ -146,12 +165,22 @@ class DetDescendantReport(Report):
             empty_place = EMPTY_ENTRY
         else:
             empty_place = ""
-            
-        self.__narrator = Narrator(self.database, self.verbose, use_call,
-                                   empty_date, empty_place, 
+
+        language = get_value('trans')
+        translator = Translator(language)
+        self._ = translator.gettext
+
+        self.__narrator = Narrator(self.database, self.verbose,
+                                   use_call, use_fulldate, 
+                                   empty_date, empty_place,
+                                   translator=translator,
                                    get_endnote_numbers=self.endnotes)
 
-        self.bibli = Bibliography(Bibliography.MODE_PAGE)
+        self.__get_date = translator.get_date
+
+        self.__get_type = translator.get_type
+
+        self.bibli = Bibliography(Bibliography.MODE_DATE|Bibliography.MODE_PAGE)
 
     def apply_henry_filter(self,person_handle, index, pid, cur_gen=1):
         if (not person_handle) or (cur_gen > self.max_generations):
@@ -241,7 +270,7 @@ class DetDescendantReport(Report):
 
         self.doc.start_paragraph("DDR-Title")
 
-        title = _("Descendant Report for %(person_name)s") % {
+        title = self._("Descendant Report for %(person_name)s") % {
                     'person_name' : name }
         mark = IndexMark(title, INDEX_TYPE_TOC, 1)
         self.doc.write_text(title, mark)
@@ -253,7 +282,7 @@ class DetDescendantReport(Report):
             if self.pgbrk and generation > 0:
                 self.doc.page_break()
             self.doc.start_paragraph("DDR-Generation")
-            text = _("Generation %d") % (generation+1)
+            text = self._("Generation %d") % (generation+1)
             mark = IndexMark(text, INDEX_TYPE_TOC, 2)
             self.doc.write_text(text, mark)
             self.doc.end_paragraph()
@@ -267,7 +296,10 @@ class DetDescendantReport(Report):
                 self.write_person(key)
 
         if self.inc_sources:
-            Endnotes.write_endnotes(self.bibli, self.database, self.doc,
+            if self.pgbrkenotes:
+                self.doc.page_break()
+            # it ignores language set for Note type (use locale)
+            endnotes.write_endnotes(self.bibli, self.database, self.doc,
                                     printnotes=self.inc_srcnotes)
 
     def write_path(self, person):
@@ -333,8 +365,12 @@ class DetDescendantReport(Report):
                 if dkey >= key: 
                     break
                 if self.map[key] == self.map[dkey]:
-                    self.doc.write_text(_("%(name)s is the same person as [%(id_str)s].") % 
-                                        { 'name' : '', 'id_str' : str(dkey) })
+                    self.doc.write_text(self._(
+                        "%(name)s is the same person as [%(id_str)s].") % {
+                            'name' :'',
+                            'id_str': str(dkey),
+                            }
+                        )
                     self.doc.end_paragraph()
                     return
 
@@ -342,20 +378,31 @@ class DetDescendantReport(Report):
        
         self.write_person_info(person)
 
-        if self.listchildren or self.inc_events or self.inc_mates:
+        if (self.inc_mates or self.listchildren or self.inc_notes or
+            self.inc_events or self.inc_attrs):
             for family_handle in person.get_family_handle_list():
                 family = self.database.get_family_from_handle(family_handle)
                 if self.inc_mates:
                     self.__write_mate(person, family)
                 if self.listchildren:
                     self.__write_children(family)
+                if self.inc_notes:
+                    self.__write_family_notes(family)
+                first = True
                 if self.inc_events:
-                    self.__write_family_events(family)
+                    first = self.__write_family_events(family)
+                if self.inc_attrs:
+                    self.__write_family_attrs(family, first)
 
     def write_event(self, event_ref):
         text = ""
         event = self.database.get_event_from_handle(event_ref.ref)
-        date = DateHandler.get_date(event)
+
+        if self.fulldate:
+            date = self.__get_date(event.get_date_object())
+        else:
+            date = event.get_date_object().get_year()
+
         ph = event.get_place_handle()
         if ph:
             place = self.database.get_place_from_handle(ph).get_title()
@@ -363,14 +410,14 @@ class DetDescendantReport(Report):
             place = u''
 
         self.doc.start_paragraph('DDR-MoreDetails')
-        event_name = str( event.get_type() )
+        event_name = self.__get_type(event.get_type())
         if date and place:
-            text +=  _('%(date)s, %(place)s') % { 
+            text +=  self._('%(date)s, %(place)s') % { 
                        'date' : date, 'place' : place }
         elif date:
-            text += _('%(date)s') % {'date' : date}
+            text += self._('%(date)s') % {'date' : date}
         elif place:
-            text += _('%(place)s') % { 'place' : place }
+            text += self._('%(place)s') % { 'place' : place }
 
         if event.get_description():
             if text:
@@ -382,8 +429,8 @@ class DetDescendantReport(Report):
         if text:
             text += ". "
             
-        text = _('%(event_name)s: %(event_text)s') % {
-                 'event_name' : _(event_name),
+        text = self._('%(event_name)s: %(event_text)s') % {
+                 'event_name' : self._(event_name),
                  'event_text' : text }
         
         self.doc.write_text_citation(text)
@@ -395,8 +442,9 @@ class DetDescendantReport(Report):
             for attr in attr_list:
                 if text:
                     text += "; "
-                text += _("%(type)s: %(value)s%(endnotes)s") % {
-                    'type'     : attr.get_type(),
+                attrName = self.__get_type(attr.get_type())
+                text += self._("%(type)s: %(value)s%(endnotes)s") % {
+                    'type'     : self._(attrName),
                     'value'    : attr.get_value(),
                     'endnotes' : self.endnotes(attr) }
             text = " " + text
@@ -477,9 +525,9 @@ class DetDescendantReport(Report):
             name = _nd.display_formal(mate)
             mark = ReportUtils.get_person_mark(self.database, mate)
             if family.get_relationship() == FamilyRelType.MARRIED:
-                self.doc.write_text(_("Spouse: %s") % name, mark)
+                self.doc.write_text(self._("Spouse: %s") % name, mark)
             else:
-                self.doc.write_text(_("Relationship with: %s") % name, mark)
+                self.doc.write_text(self._("Relationship with: %s") % name, mark)
             if name[-1:] != '.':
                 self.doc.write_text(".")
             self.doc.write_text_citation(self.endnotes(mate))
@@ -487,6 +535,23 @@ class DetDescendantReport(Report):
 
             self.write_person_info(mate)
 
+    def __get_mate_names(self, family):
+        mother_handle = family.get_mother_handle()
+        if mother_handle:
+            mother = self.database.get_person_from_handle(mother_handle)
+            mother_name = _nd.display(mother)
+        else:
+            mother_name = self._("unknown")
+
+        father_handle = family.get_father_handle()
+        if father_handle:
+            father = self.database.get_person_from_handle(father_handle)
+            father_name = _nd.display(father)
+        else:
+            father_name = self._("unknown")
+
+        return mother_name, father_name
+
     def __write_children(self, family):
         """ 
         List the children for the given family.
@@ -494,23 +559,11 @@ class DetDescendantReport(Report):
         if not family.get_child_ref_list():
             return
 
-        mother_handle = family.get_mother_handle()
-        if mother_handle:
-            mother = self.database.get_person_from_handle(mother_handle)
-            mother_name = _nd.display(mother)
-        else:
-            mother_name = _("unknown")
-
-        father_handle = family.get_father_handle()
-        if father_handle:
-            father = self.database.get_person_from_handle(father_handle)
-            father_name = _nd.display(father)
-        else:
-            father_name = _("unknown")
+        mother_name, father_name = self.__get_mate_names(family)
 
         self.doc.start_paragraph("DDR-ChildTitle")
         self.doc.write_text(
-                        _("Children of %(mother_name)s and %(father_name)s") % 
+                        self._("Children of %(mother_name)s and %(father_name)s") % 
                             {'father_name': father_name,
                              'mother_name': mother_name
                              } )
@@ -558,6 +611,25 @@ class DetDescendantReport(Report):
                                          self.__narrator.get_buried_string())
             self.doc.end_paragraph()
 
+    def __write_family_notes(self, family):
+        """ 
+        Write the notes for the given family.
+        """
+        notelist = family.get_note_list()
+        if len(notelist) > 0:
+            mother_name, father_name = self.__get_mate_names(family)
+
+            self.doc.start_paragraph("DDR-NoteHeader")
+            self.doc.write_text(
+                self._('Notes for %(mother_name)s and %(father_name)s:') % { 
+                'mother_name' : mother_name,
+                'father_name' : father_name })
+            self.doc.end_paragraph()
+            for notehandle in notelist:
+                note = self.database.get_note_from_handle(notehandle)
+                self.doc.write_styled_note(note.get_styledtext(), 
+                                           note.get_format(),"DDR-Entry")
+
     def __write_family_events(self, family):
         """ 
         List the events for the given family.
@@ -565,31 +637,56 @@ class DetDescendantReport(Report):
         if not family.get_event_ref_list():
             return
 
-        mother_handle = family.get_mother_handle()
-        if mother_handle:
-            mother = self.database.get_person_from_handle(mother_handle)
-            mother_name = _nd.display(mother)
-        else:
-            mother_name = _("unknown")
-
-        father_handle = family.get_father_handle()
-        if father_handle:
-            father = self.database.get_person_from_handle(father_handle)
-            father_name = _nd.display(father)
-        else:
-            father_name = _("unknown")
+        mother_name, father_name = self.__get_mate_names(family)
 
         first = 1
         for event_ref in family.get_event_ref_list():
             if first:
                 self.doc.start_paragraph('DDR-MoreHeader')
                 self.doc.write_text(
-                    _('More about %(mother_name)s and %(father_name)s:') % { 
+                    self._('More about %(mother_name)s and %(father_name)s:') % { 
                     'mother_name' : mother_name,
                     'father_name' : father_name })
                 self.doc.end_paragraph()
                 first = 0
             self.write_event(event_ref)
+            return first
+
+    def __write_family_attrs(self, family, first):
+        """ 
+        List the attributes for the given family.
+        """
+        attrs = family.get_attribute_list()
+
+        if first and attrs:
+            mother_name, father_name = self.__get_mate_names(family)
+
+            self.doc.start_paragraph('DDR-MoreHeader')
+            self.doc.write_text(
+                self._('More about %(mother_name)s and %(father_name)s:') % { 
+                'mother_name' : mother_name,
+                'father_name' : father_name })
+            self.doc.end_paragraph()
+
+        for attr in attrs:
+            self.doc.start_paragraph('DDR-MoreDetails')
+            attrName = self.__get_type(attr.get_type())
+            text = self._("%(type)s: %(value)s%(endnotes)s") % {
+                'type'     : self._(attrName),
+                'value'    : attr.get_value(),
+                'endnotes' : self.endnotes(attr) }
+            self.doc.write_text_citation( text )
+            self.doc.end_paragraph()
+
+            if self.inc_notes:
+                # if the attr or attr reference has a note attached to it,
+                # get the text and format it correctly
+                notelist = attr.get_note_list()
+                for notehandle in notelist:
+                    note = self.database.get_note_from_handle(notehandle)
+                    self.doc.write_styled_note(note.get_styledtext(), 
+                                               note.get_format(),"DDR-MoreDetails")
+
 
     def write_person_info(self, person):
         name = _nd.display_formal(person)
@@ -633,7 +730,7 @@ class DetDescendantReport(Report):
         notelist = person.get_note_list()
         if len(notelist) > 0 and self.inc_notes:
             self.doc.start_paragraph("DDR-NoteHeader")
-            self.doc.write_text(_("Notes for %s") % name)
+            self.doc.write_text(self._("Notes for %s") % name)
             self.doc.end_paragraph()
             for notehandle in notelist:
                 note = self.database.get_note_from_handle(notehandle)
@@ -646,15 +743,15 @@ class DetDescendantReport(Report):
             for alt_name in person.get_alternate_names():
                 if first:
                     self.doc.start_paragraph('DDR-MoreHeader')
-                    self.doc.write_text(_('More about %(person_name)s:') % { 
+                    self.doc.write_text(self._('More about %(person_name)s:') % { 
                         'person_name' : name })
                     self.doc.end_paragraph()
                     first = False
                 self.doc.start_paragraph('DDR-MoreDetails')
-                atype = str( alt_name.get_type() )
+                atype = self.__get_type(alt_name.get_type())
                 aname = alt_name.get_regular_name()
-                self.doc.write_text_citation(_('%(name_kind)s: %(name)s%(endnotes)s') % {
-                    'name_kind' : atype,
+                self.doc.write_text_citation(self._('%(name_kind)s: %(name)s%(endnotes)s') % {
+                    'name_kind' : self._(atype),
                     'name' : aname,
                     'endnotes' : self.endnotes(alt_name),
                     })
@@ -664,7 +761,7 @@ class DetDescendantReport(Report):
             for event_ref in person.get_primary_event_ref_list():
                 if first:
                     self.doc.start_paragraph('DDR-MoreHeader')
-                    self.doc.write_text(_('More about %(person_name)s:') % { 
+                    self.doc.write_text(self._('More about %(person_name)s:') % { 
                         'person_name' : _nd.display(person) })
                     self.doc.end_paragraph()
                     first = 0
@@ -675,15 +772,20 @@ class DetDescendantReport(Report):
             for addr in person.get_address_list():
                 if first:
                     self.doc.start_paragraph('DDR-MoreHeader')
-                    self.doc.write_text(_('More about %(person_name)s:') % { 
+                    self.doc.write_text(self._('More about %(person_name)s:') % { 
                         'person_name' : name })
                     self.doc.end_paragraph()
                     first = False
                 self.doc.start_paragraph('DDR-MoreDetails')
                 
                 text = ReportUtils.get_address_str(addr)
-                date = DateHandler.get_date(addr)
-                self.doc.write_text(_('Address: '))
+
+                if self.fulldate:
+                    date = self.__get_date(addr.get_date_object())
+                else:
+                    date = addr.get_date_object().get_year()
+
+                self.doc.write_text(self._('Address: '))
                 if date:
                     self.doc.write_text( '%s, ' % date )
                 self.doc.write_text( text )
@@ -694,15 +796,16 @@ class DetDescendantReport(Report):
             attrs = person.get_attribute_list()
             if first and attrs:
                 self.doc.start_paragraph('DDR-MoreHeader')
-                self.doc.write_text(_('More about %(person_name)s:') % { 
+                self.doc.write_text(self._('More about %(person_name)s:') % { 
                     'person_name' : name })
                 self.doc.end_paragraph()
                 first = False
 
             for attr in attrs:
                 self.doc.start_paragraph('DDR-MoreDetails')
-                text = _("%(type)s: %(value)s%(endnotes)s") % {
-                    'type'     : attr.get_type(),
+                attrName = self.__get_type(attr.get_type())
+                text = self._("%(type)s: %(value)s%(endnotes)s") % {
+                    'type'     : self._(attrName),
                     'value'    : attr.get_value(),
                     'endnotes' : self.endnotes(attr) }
                 self.doc.write_text_citation( text )
@@ -712,7 +815,7 @@ class DetDescendantReport(Report):
         if not obj or not self.inc_sources:
             return ""
         
-        txt = Endnotes.cite_source(self.bibli, obj)
+        txt = endnotes.cite_source(self.bibli, obj)
         if txt:
             txt = '<super>' + txt + '</super>'
         return txt
@@ -735,11 +838,14 @@ class DetDescendantOptions(MenuReportOptions):
         """
         Add options to the menu for the detailed descendant report.
         """
-        category_name = _("Report Options")
+
+        # Report Options
+        
+        add_option = partial(menu.add_option, _("Report Options"))
         
         pid = PersonOption(_("Center Person"))
         pid.set_help(_("The center person for the report"))
-        menu.add_option(category_name, "pid", pid)
+        add_option("pid", pid)
         
         numbering = EnumeratedListOption(_("Numbering system"), "Henry")
         numbering.set_items([
@@ -748,112 +854,131 @@ class DetDescendantOptions(MenuReportOptions):
                 ("Record (Modified Register)", 
                                _("Record (Modified Register) numbering"))])
         numbering.set_help(_("The numbering system to be used"))
-        menu.add_option(category_name, "numbering", numbering)
+        add_option("numbering", numbering)
         
         generations = NumberOption(_("Generations"), 10, 1, 100)
-        generations.set_help(_("The number of generations to include in the " \
-                               "report"))
-        menu.add_option(category_name, "gen", generations)
+        generations.set_help(
+            _("The number of generations to include in the report")
+            )
+        add_option("gen", generations)
         
         pagebbg = BooleanOption(_("Page break between generations"), False)
         pagebbg.set_help(
                      _("Whether to start a new page after each generation."))
-        menu.add_option(category_name, "pagebbg", pagebbg)
+        add_option("pagebbg", pagebbg)
 
-        category_name = _("Content")
+        pageben = BooleanOption(_("Page break before end notes"),False)
+        pageben.set_help(
+                     _("Whether to start a new page before the end notes."))
+        add_option("pageben", pageben)
+
+        trans = EnumeratedListOption(_("Translation"),
+                                      Translator.DEFAULT_TRANSLATION_STR)
+        trans.add_item(Translator.DEFAULT_TRANSLATION_STR, _("Default"))
+        for language in TransUtils.get_available_translations():
+            trans.add_item(language, get_language_string(language))
+        trans.set_help(_("The translation to be used for the report."))
+        add_option("trans", trans)
+
+        # Content
+        
+        add_option = partial(menu.add_option, _("Content"))
 
         usecall = BooleanOption(_("Use callname for common name"), False)
         usecall.set_help(_("Whether to use the call name as the first name."))
-        menu.add_option(category_name, "usecall", usecall)
+        add_option("usecall", usecall)
         
         fulldates = BooleanOption(_("Use full dates instead of only the year"),
                                   True)
         fulldates.set_help(_("Whether to use full dates instead of just year."))
-        menu.add_option(category_name, "fulldates", fulldates)
+        add_option("fulldates", fulldates)
         
         listc = BooleanOption(_("List children"), True)
         listc.set_help(_("Whether to list children."))
-        menu.add_option(category_name, "listc", listc)
+        add_option("listc", listc)
         
         computeage = BooleanOption(_("Compute death age"),True)
         computeage.set_help(_("Whether to compute a person's age at death."))
-        menu.add_option(category_name, "computeage", computeage)
+        add_option("computeage", computeage)
         
         omitda = BooleanOption(_("Omit duplicate ancestors"), True)
         omitda.set_help(_("Whether to omit duplicate ancestors."))
-        menu.add_option(category_name, "omitda", omitda)
+        add_option("omitda", omitda)
         
         verbose = BooleanOption(_("Use complete sentences"), True)
         verbose.set_help(
                  _("Whether to use complete sentences or succinct language."))
-        menu.add_option(category_name, "verbose", verbose)
+        add_option("verbose", verbose)
 
         desref = BooleanOption(_("Add descendant reference in child list"),
                                True)
         desref.set_help(
                     _("Whether to add descendant references in child list."))
-        menu.add_option(category_name, "desref", desref)
+        add_option("desref", desref)
 
         category_name = _("Include")
+        add_option = partial(menu.add_option, _("Include"))
         
         incnotes = BooleanOption(_("Include notes"), True)
         incnotes.set_help(_("Whether to include notes."))
-        menu.add_option(category_name, "incnotes", incnotes)
+        add_option("incnotes", incnotes)
 
         incattrs = BooleanOption(_("Include attributes"), False)
         incattrs.set_help(_("Whether to include attributes."))
-        menu.add_option(category_name, "incattrs", incattrs)
+        add_option("incattrs", incattrs)
         
         incphotos = BooleanOption(_("Include Photo/Images from Gallery"), False)
         incphotos.set_help(_("Whether to include images."))
-        menu.add_option(category_name, "incphotos", incphotos)
+        add_option("incphotos", incphotos)
 
         incnames = BooleanOption(_("Include alternative names"), False)
         incnames.set_help(_("Whether to include other names."))
-        menu.add_option(category_name, "incnames", incnames)
+        add_option("incnames", incnames)
 
         incevents = BooleanOption(_("Include events"), False)
         incevents.set_help(_("Whether to include events."))
-        menu.add_option(category_name, "incevents", incevents)
+        add_option("incevents", incevents)
 
         incaddresses = BooleanOption(_("Include addresses"), False)
         incaddresses.set_help(_("Whether to include addresses."))
-        menu.add_option(category_name, "incaddresses", incaddresses)
+        add_option("incaddresses", incaddresses)
 
         incsources = BooleanOption(_("Include sources"), False)
         incsources.set_help(_("Whether to include source references."))
-        menu.add_option(category_name, "incsources", incsources)
+        add_option("incsources", incsources)
         
         incsrcnotes = BooleanOption(_("Include sources notes"), False)
         incsrcnotes.set_help(_("Whether to include source notes in the "
             "Endnotes section. Only works if Include sources is selected."))
-        menu.add_option(category_name, "incsrcnotes", incsrcnotes)
+        add_option("incsrcnotes", incsrcnotes)
 
         incmates = BooleanOption(_("Include spouses"), False)
         incmates.set_help(_("Whether to include detailed spouse information."))
-        menu.add_option(category_name, "incmates", incmates)
+        add_option("incmates", incmates)
 
-        incssign = BooleanOption(_("Include sign of succession ('+')" \
+        incssign = BooleanOption(_("Include sign of succession ('+')"
                                    " in child-list"), True)
         incssign.set_help(_("Whether to include a sign ('+') before the"
                             " descendant number in the child-list to indicate"
                             " a child has succession."))
-        menu.add_option(category_name, "incssign", incssign)
+        add_option("incssign", incssign)
 
         incpaths = BooleanOption(_("Include path to start-person"), False)
-        incpaths.set_help(_("Whether to include the path of descendancy " \
+        incpaths.set_help(_("Whether to include the path of descendancy "
                             "from the start-person to each descendant."))
-        menu.add_option(category_name, "incpaths", incpaths)
+        add_option("incpaths", incpaths)
 
-        category_name = _("Missing information")        
+        # Missing information
+        
+        add_option = partial(menu.add_option, _("Missing information"))      
 
         repplace = BooleanOption(_("Replace missing places with ______"), False)
         repplace.set_help(_("Whether to replace missing Places with blanks."))
-        menu.add_option(category_name, "repplace", repplace)
+        add_option("repplace", repplace)
 
         repdate = BooleanOption(_("Replace missing dates with ______"), False)
         repdate.set_help(_("Whether to replace missing Dates with blanks."))
-        menu.add_option(category_name, "repdate", repdate)
+        add_option("repdate", repdate)
 
     def make_default_style(self, default_style):
         """Make the default output style for the Detailed Ancestral Report"""
@@ -928,7 +1053,8 @@ class DetDescendantOptions(MenuReportOptions):
         para.set(first_indent=0.0, lmargin=1.5)
         para.set_top_margin(0.25)
         para.set_bottom_margin(0.25)
-        para.set_description(_('The style used for the More About header.'))
+        para.set_description(_('The style used for the More About header and '
+            'for headers of mates.'))
         default_style.add_paragraph_style("DDR-MoreHeader", para)
 
         font = FontStyle()
@@ -941,4 +1067,4 @@ class DetDescendantOptions(MenuReportOptions):
         para.set_description(_('The style used for additional detail data.'))
         default_style.add_paragraph_style("DDR-MoreDetails", para)
 
-        Endnotes.add_endnote_styles(default_style)
+        endnotes.add_endnote_styles(default_style)
