@@ -4,6 +4,7 @@
 # Copyright (C) 2007-2008 Brian G. Matherly
 # Copyright (C) 2009      Gary Burton
 # Contribution  2009 by   Reinhard Mueller <reinhard.mueller@bytewise.at> 
+# Copyright (C) 2010       Jakim Friant
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,7 +21,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: KinshipReport.py 14135 2010-01-25 17:45:21Z gbritton $
+# $Id: KinshipReport.py 16979 2011-03-30 01:50:08Z pez4brian $
 
 """Reports/Text Reports/Kinship Report"""
 
@@ -42,7 +43,9 @@ import Relationship
 from gen.plug.docgen import (IndexMark, FontStyle, ParagraphStyle,
                     FONT_SANS_SERIF, INDEX_TYPE_TOC, PARA_ALIGN_CENTER)
 from gen.plug.menu import NumberOption, BooleanOption, PersonOption
-from ReportBase import Report, ReportUtils, MenuReportOptions
+from gen.plug.report import Report
+from gen.plug.report import utils as ReportUtils
+from gui.plug.report import MenuReportOptions
 import DateHandler
 from gen.utils import get_birth_or_fallback, get_death_or_fallback
 
@@ -133,9 +136,9 @@ class KinshipReport(Report):
                 title = get_rel_str(Ga, Gb, in_law_b=False)
                 self.write_people(title, self.kinship_map[Ga][Gb])
                 
-                if self.inc_spouses and \
-                   Ga in self.spouse_map and \
-                   Gb in self.spouse_map[Ga]:
+                if (self.inc_spouses and
+                   Ga in self.spouse_map and
+                   Gb in self.spouse_map[Ga]):
                     title = get_rel_str(Ga, Gb, in_law_b=True)
                     self.write_people(title, self.spouse_map[Ga][Gb])
 
