@@ -2,6 +2,7 @@
 #
 # Copyright (C) 2003-2006  Donald N. Allingham
 # Copyright (C) 2008       Brian G. Matherly
+# Copyright (C) 2010       Jakim Friant
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,7 +19,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: SimpleBookTitle.py 14466 2010-02-24 23:06:21Z robhealey1 $
+# $Id: SimpleBookTitle.py 16979 2011-03-30 01:50:08Z pez4brian $
 
 #------------------------------------------------------------------------
 #
@@ -35,7 +36,8 @@ from gen.ggettext import sgettext as _
 #------------------------------------------------------------------------
 from gen.plug.menu import StringOption, MediaOption, NumberOption
 from Utils import media_path_full
-from ReportBase import Report, MenuReportOptions
+from gen.plug.report import Report
+from gui.plug.report import MenuReportOptions
 from gen.plug.docgen import (FontStyle, ParagraphStyle,
                              FONT_SANS_SERIF, PARA_ALIGN_CENTER)
 
