@@ -3,6 +3,7 @@
 #
 # Copyright (C) 2000-2006  Donald N. Allingham
 # Copyright (C) 2008       Brian G. Matherly
+# Copyright (C) 2010       Jakim Friant
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,7 +20,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: Summary.py 14091 2010-01-18 04:42:17Z pez4brian $
+# $Id: Summary.py 17495 2011-05-14 13:58:44Z romjerome $
 
 """
 Reports/Text Reports/Database Summary Report.
@@ -39,7 +40,9 @@ from gen.ggettext import gettext as _
 #
 #------------------------------------------------------------------------
 import gen.lib
-from ReportBase import Report, ReportUtils, MenuReportOptions
+from gen.plug.report import Report
+from gen.plug.report import utils as ReportUtils
+from gui.plug.report import MenuReportOptions
 from gen.plug.docgen import (IndexMark, FontStyle, ParagraphStyle,
                     FONT_SANS_SERIF, INDEX_TYPE_TOC, PARA_ALIGN_CENTER)
 from Utils import media_path_full
@@ -114,8 +117,8 @@ class SummaryReport(Report):
                 incomp_names += 1
                 
             # Count people without families.
-            if (not person.get_main_parents_family_handle()) and \
-               (not len(person.get_family_handle_list())):
+            if (not person.get_main_parents_family_handle() and
+               not len(person.get_family_handle_list())):
                 disconnected += 1
             
             # Count missing birthdays.
@@ -203,11 +206,17 @@ class SummaryReport(Report):
         self.doc.end_paragraph()
         
         total_media = len(self.__db.get_media_object_handles())
+        mbytes = "0"
         for media_id in self.__db.get_media_object_handles():
             media = self.__db.get_object_from_handle(media_id)
             try:
                 size_in_bytes += posixpath.getsize(
                                  media_path_full(self.__db, media.get_path()))
+                length = len(str(size_in_bytes))
+                if size_in_bytes <= 999999:
+                    mbytes = _("less than 1")
+                else:
+                    mbytes = str(size_in_bytes)[:(length-6)]
             except:
                 notfound.append(media.get_path())
                 
@@ -217,7 +226,7 @@ class SummaryReport(Report):
         self.doc.end_paragraph()
         
         self.doc.start_paragraph("SR-Normal")
-        self.doc.write_text(_("Total size of media objects: %d bytes") % size_in_bytes)
+        self.doc.write_text(_("Total size of media objects: %s MB") % mbytes)
         self.doc.end_paragraph()
     
         if len(notfound) > 0:
