@@ -3,6 +3,7 @@
 #
 # Copyright (C) 2000-2007  Donald N. Allingham
 # Copyright (C) 2008       Brian G. Matherly
+# Copyright (C) 2010       Jakim Friant
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,17 +20,10 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: ChangeNames.py 12362 2009-03-19 04:25:37Z pez4brian $
+# $Id: ChangeNames.py 16979 2011-03-30 01:50:08Z pez4brian $
 
 """Tools/Database Processing/Fix Capitalization of Family Names"""
 
-#-------------------------------------------------------------------------
-#
-# python modules
-#
-#-------------------------------------------------------------------------
-import os
-
 #-------------------------------------------------------------------------
 #
 # gnome/gtk
@@ -37,22 +31,23 @@ import os
 #-------------------------------------------------------------------------
 import gobject
 import gtk
-from gtk import glade
 
 #-------------------------------------------------------------------------
 #
 # gramps modules
 #
 #-------------------------------------------------------------------------
+from gen.db import find_surname_name, DbTxn
 import const
-import Utils
+from gui.utils import ProgressMeter
 import GrampsDisplay
 import ManagedWindow
 
 from QuestionDialog import OkDialog
-from PluginUtils import Tool
-from gen.plug import PluginManager
-from TransUtils import sgettext as _
+from gui.plug import tool
+from gen.ggettext import sgettext as _
+from glade import Glade
+
 #-------------------------------------------------------------------------
 #
 # constants
@@ -74,7 +69,7 @@ WIKI_HELP_SEC = _('manual|Fix_Capitalization_of_Family_Names...')
 # ChangeNames
 #
 #-------------------------------------------------------------------------
-class ChangeNames(Tool.BatchTool, ManagedWindow.ManagedWindow):
+class ChangeNames(tool.BatchTool, ManagedWindow.ManagedWindow):
 
     def __init__(self, dbstate, uistate, options_class, name, callback=None):
         self.label = _('Capitalization changes')
@@ -83,11 +78,11 @@ class ChangeNames(Tool.BatchTool, ManagedWindow.ManagedWindow):
         ManagedWindow.ManagedWindow.__init__(self,uistate,[],self.__class__)
         self.set_window(gtk.Window(),gtk.Label(),'')
 
-        Tool.BatchTool.__init__(self, dbstate, options_class, name)
+        tool.BatchTool.__init__(self, dbstate, options_class, name)
         if self.fail:
             return
 
-        self.progress = Utils.ProgressMeter(_('Checking Family Names'),'')
+        self.progress = ProgressMeter(_('Checking Family Names'),'')
         self.progress.set_pass(_('Searching family names'),
                                len(self.db.get_surname_list()))
         self.name_list = []
@@ -113,7 +108,7 @@ class ChangeNames(Tool.BatchTool, ManagedWindow.ManagedWindow):
                 s1 = 0
                 if namesplitSP[0].lower() in prefix_list:
                     s1 = 1
-                for x in range(len(namesplitSP)-s1):
+                for x in xrange(len(namesplitSP)-s1):
                     # check if any subsurname is not cap
                     notcap = False
                     if namesplitSP[s1+x] != namesplitSP[s1+x].capitalize():
@@ -127,7 +122,7 @@ class ChangeNames(Tool.BatchTool, ManagedWindow.ManagedWindow):
                 # check if first string is in prefix_list, if so test for cap
                 if namesplitSP[0].lower() in prefix_list:
                     namesplitHY[0] = namesplitHY[0].replace(namesplitSP[0],'').strip()
-                for x in range(len(namesplitHY)):
+                for x in xrange(len(namesplitHY)):
                     # check if any subsurname is not cap
                     notcap = False
                     if namesplitHY[x] != namesplitHY[x].capitalize():
@@ -146,7 +141,8 @@ class ChangeNames(Tool.BatchTool, ManagedWindow.ManagedWindow):
             self.progress.close()
             self.close()
             OkDialog(_('No modifications made'),
-                     _("No capitalization changes were detected."))
+                     _("No capitalization changes were detected."),
+                     parent=uistate.window)
 
     def name_cap(self, name):
         name.strip()            
@@ -178,19 +174,17 @@ class ChangeNames(Tool.BatchTool, ManagedWindow.ManagedWindow):
 
     def display(self):
 
-        base = os.path.dirname(__file__)
-        glade_file = os.path.join(base,"changenames.glade")
-        
-        self.top = glade.XML(glade_file,"top","gramps")
-        window = self.top.get_widget('top')
-        self.top.signal_autoconnect({
+        self.top = Glade()
+        window = self.top.toplevel
+        self.top.connect_signals({
             "destroy_passed_object" : self.close,
             "on_ok_clicked" : self.on_ok_clicked,
             "on_help_clicked" : self.on_help_clicked,
+            "on_delete_event"   : self.close,
             })
         
-        self.list = self.top.get_widget("list")
-        self.set_window(window,self.top.get_widget('title'),self.label)
+        self.list = self.top.get_object("list")
+        self.set_window(window,self.top.get_object('title'),self.label)
 
         self.model = gtk.ListStore(gobject.TYPE_BOOLEAN, gobject.TYPE_STRING, 
                                    gobject.TYPE_STRING)
@@ -225,7 +219,7 @@ class ChangeNames(Tool.BatchTool, ManagedWindow.ManagedWindow):
         self.show()
 
     def toggled(self,cell,path_string):
-        path = tuple([int (i) for i in path_string.split(':')])
+        path = tuple(map(int, path_string.split(':')))
         row = self.model[path]
         row[0] = not row[0]
 
@@ -237,25 +231,30 @@ class ChangeNames(Tool.BatchTool, ManagedWindow.ManagedWindow):
         GrampsDisplay.help(WIKI_HELP_PAGE , WIKI_HELP_SEC)
 
     def on_ok_clicked(self, obj):
-        self.trans = self.db.transaction_begin("",batch=True)
-        self.db.disable_signals()
-        changelist = [self.model.get_value(node,1)
-                      for node in self.iter_list
-                      if self.model.get_value(node,0)]
+        with DbTxn(_("Capitalization changes"), self.db, batch=True
+                   ) as self.trans:
+            self.db.disable_signals()
+            changelist = set(self.model.get_value(node,1)
+                            for node in self.iter_list
+                                if self.model.get_value(node,0))
 
-        for handle in self.db.get_person_handles():
-            change = False
-            person = self.db.get_person_from_handle(handle)
-            for name in [person.get_primary_name()] + person.get_alternate_names():
-                sname = name.get_surname()
-                if sname in changelist:
-                    change = True
-                    sname = self.name_cap(sname)
-                    name.set_surname(sname)
-            if change:
-                self.db.commit_person(person,self.trans)
+            #with self.db.get_person_cursor(update=True, commit=True) as cursor:
+            #  for handle, data in cursor:
+            for handle in self.db.get_person_handles(False):
+                person = self.db.get_person_from_handle(handle)
+                #person = Person(data)
+                change = False
+                for name in [person.get_primary_name()] + person.get_alternate_names():
+                    sname = find_surname_name(handle, name.serialize())
+                    if sname in changelist:
+                        change = True
+                        for surn in name.get_surname_list():
+                            sname = self.name_cap(surn.get_surname())
+                            surn.set_surname(sname)
+                if change:
+                    #cursor.update(handle, person.serialize())
+                    self.db.commit_person(person, transaction=self.trans)
 
-        self.db.transaction_commit(self.trans,_("Capitalization changes"))
         self.db.enable_signals()
         self.db.request_rebuild()
         # FIXME: this probably needs to be removed, and bookmarks
@@ -269,30 +268,10 @@ class ChangeNames(Tool.BatchTool, ManagedWindow.ManagedWindow):
 # 
 #
 #------------------------------------------------------------------------
-class ChangeNamesOptions(Tool.ToolOptions):
+class ChangeNamesOptions(tool.ToolOptions):
     """
     Defines options and provides handling interface.
     """
 
     def __init__(self, name,person_id=None):
-        Tool.ToolOptions.__init__(self, name,person_id)
-
-#------------------------------------------------------------------------
-#
-# 
-#
-#------------------------------------------------------------------------
-pmgr = PluginManager.get_instance()
-pmgr.register_tool(
-    name = 'chname',
-    category = Tool.TOOL_DBPROC,
-    tool_class = ChangeNames,
-    options_class = ChangeNamesOptions,
-    modes = PluginManager.TOOL_MODE_GUI,
-    translated_name = _("Fix Capitalization of Family Names"),
-    status = _("Stable"),
-    author_name = "Donald N. Allingham",
-    author_email = "don@gramps-project.org",
-    description = _("Searches the entire database and attempts to "
-                    "fix capitalization of the names.")
-    )
+        tool.ToolOptions.__init__(self, name,person_id)
