@@ -3,6 +3,7 @@
 #
 # Copyright (C) 2000-2007  Donald N. Allingham
 # Copyright (C) 2008       Brian G. Matherly
+# Copyright (C) 2010       Jakim Friant
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,7 +20,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: EventNames.py 14091 2010-01-18 04:42:17Z pez4brian $
+# $Id: EventNames.py 16979 2011-03-30 01:50:08Z pez4brian $
 
 """Tools/Database Processing/Extract Event Descriptions from Event Data"""
 
@@ -44,9 +45,10 @@ from gen.ggettext import ngettext
 #-------------------------------------------------------------------------
 import ManagedWindow
 import gen.lib
+from gen.db import DbTxn
 import Utils
 
-from PluginUtils import Tool
+from gui.plug import tool
 from gen.display.name import displayer as name_displayer
 from QuestionDialog import OkDialog
 
@@ -55,7 +57,7 @@ from QuestionDialog import OkDialog
 # EventNames
 #
 #-------------------------------------------------------------------------
-class EventNames(Tool.BatchTool, ManagedWindow.ManagedWindow):
+class EventNames(tool.BatchTool, ManagedWindow.ManagedWindow):
     """
     Look for events that do not have a description, and build the description 
     from the item that contains it. 
@@ -66,7 +68,7 @@ class EventNames(Tool.BatchTool, ManagedWindow.ManagedWindow):
     """
 
     def __init__(self, dbstate, uistate, options_class, name, callback=None):
-        Tool.BatchTool.__init__(self, dbstate, options_class, name)
+        tool.BatchTool.__init__(self, dbstate, options_class, name)
 
         if not self.fail:
             uistate.set_busy_cursor(True)
@@ -77,34 +79,33 @@ class EventNames(Tool.BatchTool, ManagedWindow.ManagedWindow):
         """
         Perform the actual extraction of information.
         """
-        trans = self.db.transaction_begin("", batch=True)
-        self.db.disable_signals()
-        self.change = False
-        counter = 0
+        with DbTxn(_("Event name changes"), self.db, batch=True) as trans:
+            self.db.disable_signals()
+            self.change = False
+            counter = 0
         
-        for person in self.db.iter_people():
-            for event_ref in person.get_event_ref_list():
-                if event_ref.get_role() == gen.lib.EventRoleType.PRIMARY:
-                    event_handle = event_ref.ref
-                    event = self.db.get_event_from_handle(event_handle)
-                    if event.get_description() == "":
-                        person_event_name(event, person)
-                        self.db.commit_event(event, trans)
-                        self.change = True
-                        counter += 1
+            for person in self.db.iter_people():
+                for event_ref in person.get_event_ref_list():
+                    if event_ref.get_role() == gen.lib.EventRoleType.PRIMARY:
+                        event_handle = event_ref.ref
+                        event = self.db.get_event_from_handle(event_handle)
+                        if event.get_description() == "":
+                            person_event_name(event, person)
+                            self.db.commit_event(event, trans)
+                            self.change = True
+                            counter += 1
 
-        for family in self.db.iter_families():
-            for event_ref in family.get_event_ref_list():
-                if event_ref.get_role() == gen.lib.EventRoleType.FAMILY:
-                    event_handle = event_ref.ref
-                    event = self.db.get_event_from_handle(event_handle)
-                    if event.get_description() == "":
-                        family_event_name(event, family, self.db)
-                        self.db.commit_event(event, trans)
-                        self.change = True
-                        counter += 1
+            for family in self.db.iter_families():
+                for event_ref in family.get_event_ref_list():
+                    if event_ref.get_role() == gen.lib.EventRoleType.FAMILY:
+                        event_handle = event_ref.ref
+                        event = self.db.get_event_from_handle(event_handle)
+                        if event.get_description() == "":
+                            family_event_name(event, family, self.db)
+                            self.db.commit_event(event, trans)
+                            self.change = True
+                            counter += 1
 
-        self.db.transaction_commit(trans, _("Event name changes"))
         self.db.enable_signals()
         self.db.request_rebuild()
 
@@ -152,10 +153,10 @@ def family_event_name(event, family, dbase):
 # 
 #
 #------------------------------------------------------------------------
-class EventNamesOptions(Tool.ToolOptions):
+class EventNamesOptions(tool.ToolOptions):
     """
     Define options and provides handling interface.
     """
 
     def __init__(self, name, person_id=None):
-        Tool.ToolOptions.__init__(self, name, person_id)
+        tool.ToolOptions.__init__(self, name, person_id)
