@@ -4,6 +4,7 @@
 #
 # Copyright (C) 2000-2007  Donald N. Allingham
 # Copyright (C) 2008       Brian G. Matherly
+# Copyright (C) 2010       Jakim Friant
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,7 +21,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: ExtractCity.py 14104 2010-01-21 18:42:53Z gbritton $
+# $Id: ExtractCity.py 16979 2011-03-30 01:50:08Z pez4brian $
 
 """Tools/Database Processing/Extract Place Data from a Place Title"""
 
@@ -45,10 +46,11 @@ import gobject
 # gramps modules
 #
 #-------------------------------------------------------------------------
+from gen.db import DbTxn
 import ManagedWindow
 import GrampsDisplay
 
-from PluginUtils import Tool
+from gui.plug import tool
 from gui.utils import ProgressMeter
 from glade import Glade
 
@@ -392,7 +394,7 @@ COLS = [
 # ExtractCity
 #
 #-------------------------------------------------------------------------
-class ExtractCity(Tool.BatchTool, ManagedWindow.ManagedWindow):
+class ExtractCity(tool.BatchTool, ManagedWindow.ManagedWindow):
     """
     Extracts city, state, and zip code information from an place description
     if the title is empty and the description falls into the category of:
@@ -415,7 +417,7 @@ class ExtractCity(Tool.BatchTool, ManagedWindow.ManagedWindow):
         ManagedWindow.ManagedWindow.__init__(self, uistate, [], self.__class__)
         self.set_window(gtk.Window(), gtk.Label(), '')
 
-        Tool.BatchTool.__init__(self, dbstate, options_class, name)
+        tool.BatchTool.__init__(self, dbstate, options_class, name)
 
         if not self.fail:
             uistate.set_busy_cursor(True)
@@ -590,27 +592,26 @@ class ExtractCity(Tool.BatchTool, ManagedWindow.ManagedWindow):
         GrampsDisplay.help()
 
     def on_ok_clicked(self, obj):
-        self.trans = self.db.transaction_begin("", batch=True)
-        self.db.disable_signals()
-        changelist = [node for node in self.iter_list
-                      if self.model.get_value(node, 0)]
+        with DbTxn(_("Extract Place data"), self.db, batch=True) as self.trans:
+            self.db.disable_signals()
+            changelist = [node for node in self.iter_list
+                          if self.model.get_value(node, 0)]
 
-        for change in changelist:
-            row = self.model[change]
-            place = self.db.get_place_from_handle(row[6])
-            (city, state, postal, country) = (row[2], row[3], row[4], row[5])
+            for change in changelist:
+                row = self.model[change]
+                place = self.db.get_place_from_handle(row[6])
+                (city, state, postal, country) = (row[2], row[3], row[4], row[5])
 
-            if city:
-                place.get_main_location().set_city(city)
-            if state:
-                place.get_main_location().set_state(state)
-            if postal:
-                place.get_main_location().set_postal_code(postal)
-            if country:
-                place.get_main_location().set_country(country)
-            self.db.commit_place(place, self.trans)
+                if city:
+                    place.get_main_location().set_city(city)
+                if state:
+                    place.get_main_location().set_state(state)
+                if postal:
+                    place.get_main_location().set_postal_code(postal)
+                if country:
+                    place.get_main_location().set_country(country)
+                self.db.commit_place(place, self.trans)
 
-        self.db.transaction_commit(self.trans, _("Extract Place data"))
         self.db.enable_signals()
         self.db.request_rebuild()
         self.close()
@@ -620,9 +621,9 @@ class ExtractCity(Tool.BatchTool, ManagedWindow.ManagedWindow):
 # 
 #
 #------------------------------------------------------------------------
-class ExtractCityOptions(Tool.ToolOptions):
+class ExtractCityOptions(tool.ToolOptions):
     """
     Defines options and provides handling interface.
     """
     def __init__(self, name, person_id=None):
-        Tool.ToolOptions.__init__(self, name, person_id)
+        tool.ToolOptions.__init__(self, name, person_id)
