@@ -3,6 +3,7 @@
 #
 # Copyright (C) 2000-2007  Donald N. Allingham
 # Copyright (C) 2008       Brian G. Matherly
+# Copyright (C) 2010       Jakim Friant
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,7 +20,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: FindDupes.py 14387 2010-02-15 12:45:33Z bmcage $
+# $Id: FindDupes.py 16979 2011-03-30 01:50:08Z pez4brian $
 
 """Tools/Database Processing/Find Possible Duplicate People"""
 
@@ -38,15 +39,15 @@ import gtk
 import const
 import gen.lib
 from gui.utils import ProgressMeter
+from gui.plug import tool
 import soundex
 from gen.display.name import displayer as name_displayer
 from QuestionDialog import OkDialog
 import ListModel
 import Errors
-from Merge import PersonCompare
+from Merge import MergePeople
 import GrampsDisplay
 import ManagedWindow
-from PluginUtils import Tool
 from QuestionDialog import RunDatabaseRepair
 from gen.ggettext import sgettext as _
 from glade import Glade
@@ -84,11 +85,11 @@ def is_initial(name):
 # The Actual tool.
 #
 #-------------------------------------------------------------------------
-class Merge(Tool.Tool,ManagedWindow.ManagedWindow):
+class Merge(tool.Tool,ManagedWindow.ManagedWindow):
     
     def __init__(self, dbstate, uistate, options_class, name, callback=None):
         
-        Tool.Tool.__init__(self, dbstate, options_class, name)
+        tool.Tool.__init__(self, dbstate, options_class, name)
         ManagedWindow.ManagedWindow.__init__(self, uistate, [],
                                              self.__class__)
         self.dbstate = dbstate
@@ -143,7 +144,7 @@ class Merge(Tool.Tool,ManagedWindow.ManagedWindow):
         
         GrampsDisplay.help(WIKI_HELP_PAGE , WIKI_HELP_SEC)
 
-    def ancestors_of(self,p1_id,id_list):
+    def ancestors_of(self, p1_id, id_list):
         if (not p1_id) or (p1_id in id_list):
             return
         id_list.append(p1_id)
@@ -180,9 +181,10 @@ class Merge(Tool.Tool,ManagedWindow.ManagedWindow):
             except Errors.WindowActiveError:
                 pass
     
-    def find_potentials(self,thresh):
+    def find_potentials(self, thresh):
         self.progress = ProgressMeter(_('Find Duplicates'),
-                                            _('Looking for duplicate people'))
+                                      _('Looking for duplicate people')
+                                     )
 
         index = 0
         males = {}
@@ -196,7 +198,7 @@ class Merge(Tool.Tool,ManagedWindow.ManagedWindow):
         for p1_id in self.db.iter_person_handles():
             self.progress.step()
             p1 = self.db.get_person_from_handle(p1_id)
-            key = self.gen_key(p1.get_primary_name().get_surname())
+            key = self.gen_key(get_surnames(p1.get_primary_name()))
             if p1.get_gender() == gen.lib.Person.MALE:
                 if key in males:
                     males[key].append(p1_id)
@@ -215,7 +217,7 @@ class Merge(Tool.Tool,ManagedWindow.ManagedWindow):
             self.progress.step()
             p1 = self.db.get_person_from_handle(p1key)
 
-            key = self.gen_key(p1.get_primary_name().get_surname())
+            key = self.gen_key(get_surnames(p1.get_primary_name()))
             if p1.get_gender() == gen.lib.Person.MALE:
                 remaining = males[key]
             else:
@@ -245,7 +247,7 @@ class Merge(Tool.Tool,ManagedWindow.ManagedWindow):
         self.length = len(self.list)
         self.progress.close()
 
-    def gen_key(self,val):
+    def gen_key(self, val):
         if self.use_soundex:
             try:
                 return soundex.soundex(val)
@@ -254,7 +256,7 @@ class Merge(Tool.Tool,ManagedWindow.ManagedWindow):
         else:
             return val
 
-    def compare_people(self,p1,p2):
+    def compare_people(self, p1, p2):
 
         name1 = p1.get_primary_name()
         name2 = p2.get_primary_name()
@@ -291,25 +293,25 @@ class Merge(Tool.Tool,ManagedWindow.ManagedWindow):
                                 birth2.get_date_object())
         if value == -1 :
             return -1
-        chance =+ value
+        chance += value
 
         value = self.date_match(death1.get_date_object(),
                                 death2.get_date_object()) 
         if value == -1 :
             return -1
-        chance =+ value
+        chance += value
 
         value = self.place_match(birth1.get_place_handle(),
                                  birth2.get_place_handle()) 
         if value == -1 :
             return -1
-        chance =+ value
+        chance += value
 
         value = self.place_match(death1.get_place_handle(),
                                  death2.get_place_handle()) 
         if value == -1 :
             return -1
-        chance =+ value
+        chance += value
 
         ancestors = []
         self.ancestors_of(p1.get_handle(),ancestors)
@@ -396,7 +398,7 @@ class Merge(Tool.Tool,ManagedWindow.ManagedWindow):
                                 chance += value
         return chance
 
-    def name_compare(self,s1,s2):
+    def name_compare(self, s1, s2):
         if self.use_soundex:
             try:
                 return soundex.compare(s1,s2)
@@ -405,7 +407,7 @@ class Merge(Tool.Tool,ManagedWindow.ManagedWindow):
         else:
             return s1 == s2
 
-    def date_match(self,date1,date2):
+    def date_match(self, date1, date2):
         if date1.is_empty() or date2.is_empty():
             return 0
         if date1.is_equal(date2):
@@ -424,16 +426,16 @@ class Merge(Tool.Tool,ManagedWindow.ManagedWindow):
         else:
             return -1
 
-    def range_compare(self,date1,date2):
+    def range_compare(self, date1, date2):
         start_date_1 = date1.get_start_date()[0:3]
         start_date_2 = date2.get_start_date()[0:3]
         stop_date_1 = date1.get_stop_date()[0:3]
         stop_date_2 = date2.get_stop_date()[0:3]
         if date1.is_compound() and date2.is_compound():
-            if start_date_2 <= start_date_1 <= stop_date_2 or \
-               start_date_1 <= start_date_2 <= stop_date_1 or \
-               start_date_2 <= stop_date_1 <= stop_date_2 or \
-               start_date_1 <= stop_date_2 <= stop_date_1:
+            if (start_date_2 <= start_date_1 <= stop_date_2 or
+                start_date_1 <= start_date_2 <= stop_date_1 or
+                start_date_2 <= stop_date_1 <= stop_date_2 or
+                start_date_1 <= stop_date_2 <= stop_date_1):
                 return 0.5
             else:
                 return -1
@@ -453,9 +455,9 @@ class Merge(Tool.Tool,ManagedWindow.ManagedWindow):
         if not name1 or not name:
             return 0
     
-        srn1 = name.get_surname()
+        srn1 = get_surnames(name)
         sfx1 = name.get_suffix()
-        srn2 = name1.get_surname()
+        srn2 = get_surnames(name1)
         sfx2 = name1.get_suffix()
 
         if not self.name_compare(srn1,srn2):
@@ -475,7 +477,7 @@ class Merge(Tool.Tool,ManagedWindow.ManagedWindow):
             else:
                 return self.list_reduce(list2,list1)
             
-    def place_match(self,p1_id,p2_id):
+    def place_match(self, p1_id, p2_id):
         if p1_id == p2_id:
             return 1
         
@@ -508,7 +510,7 @@ class Merge(Tool.Tool,ManagedWindow.ManagedWindow):
                     value += 0.25
         return min(value,1) if value else -1
         
-    def list_reduce(self,list1,list2):
+    def list_reduce(self, list1, list2):
         value = 0
         for name in list1:
             for name2 in list2:
@@ -525,7 +527,7 @@ class Merge(Tool.Tool,ManagedWindow.ManagedWindow):
 
 class ShowMatches(ManagedWindow.ManagedWindow):
 
-    def __init__(self,dbstate,uistate,track,the_list,the_map,callback):
+    def __init__(self, dbstate, uistate, track, the_list, the_map, callback):
         ManagedWindow.ManagedWindow.__init__(self,uistate,track,self.__class__)
 
         self.dellist = {}
@@ -598,16 +600,21 @@ class ShowMatches(ManagedWindow.ManagedWindow):
             return
 
         (self.p1,self.p2) = self.list.get_object(iter)
-        pn1 = self.db.get_person_from_handle(self.p1)
-        pn2 = self.db.get_person_from_handle(self.p2)
-
-        PersonCompare(self.dbstate,self.uistate,pn1,pn2,self.on_update)
+        MergePeople(self.dbstate, self.uistate, self.p1, self.p2,
+                    self.on_update, True)
 
     def on_update(self):
-        self.dellist[self.p2] = self.p1
+        if self.db.has_person_handle(self.p1):
+            phoenix = self.p1
+            titanic = self.p2
+        else:
+            phoenix = self.p2
+            titanic = self.p1
+
+        self.dellist[titanic] = phoenix
         for key, data in self.dellist.iteritems():
-            if data == self.p2:
-                self.dellist[key] = self.p1
+            if data == titanic:
+                self.dellist[key] = phoenix
         self.update()
         self.redraw()
         
@@ -631,7 +638,11 @@ def get_name_obj(person):
         return person.get_primary_name()
     else:
         return None
-    
+
+def get_surnames(name):
+    """Construct a full surname of the surnames"""
+    return ' '.join([surn.get_surname() for surn in name.get_surname_list()])
+
 #-------------------------------------------------------------------------
 #
 #
@@ -646,13 +657,13 @@ def by_id(p1,p2):
 # 
 #
 #------------------------------------------------------------------------
-class MergeOptions(Tool.ToolOptions):
+class MergeOptions(tool.ToolOptions):
     """
     Defines options and provides handling interface.
     """
 
     def __init__(self, name,person_id=None):
-        Tool.ToolOptions.__init__(self, name,person_id)
+        tool.ToolOptions.__init__(self, name,person_id)
 
         # Options specific for this report
         self.options_dict = {
