@@ -4,6 +4,7 @@
 # Copyright (C) 2000-2006  Donald N. Allingham
 # Copyright (C) 2008       B. Malengier
 # Copyright (C) 2008       Brian G. Matherly
+# Copyright (C) 2010       Jakim Friant
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -22,7 +23,7 @@
 
 # Written by Alex Roitman
 
-# $Id: MediaManager.py 14608 2010-03-04 13:25:30Z romjerome $
+# $Id: MediaManager.py 16979 2011-03-30 01:50:08Z pez4brian $
 
 """Tools/Utilities/Media Manager"""
 
@@ -52,10 +53,12 @@ import GrampsDisplay
 import Assistant
 import Errors
 from gen.lib import MediaObject
+from gen.db import DbTxn
 from gen.updatecallback import UpdateCallback
-from PluginUtils import Tool
+from gui.plug import tool
 from Utils import media_path_full, relative_path, media_path
 from gen.ggettext import sgettext as _
+import gen.mime
 
 #-------------------------------------------------------------------------
 #
@@ -70,11 +73,11 @@ WIKI_HELP_SEC = _('manual|Media_Manager...')
 # This is an Assistant implementation to guide the user
 #
 #-------------------------------------------------------------------------
-class MediaMan(Tool.Tool):
+class MediaMan(tool.Tool):
 
     def __init__(self, dbstate, uistate, options_class, name, callback=None):
 
-        Tool.Tool.__init__(self, dbstate, options_class, name)
+        tool.Tool.__init__(self, dbstate, options_class, name)
         self.uistate = uistate
         self.callback = uistate.pulse_progressbar
 
@@ -169,6 +172,7 @@ class MediaMan(Tool.Tool):
             PathChange,
             Convert2Abs,
             Convert2Rel,
+            ImagesNotIncluded,
             ]
 
         for batch_class in batches_to_use:
@@ -292,7 +296,7 @@ class MediaMan(Tool.Tool):
 
     def build_conclusion(self,success):
         if success:
-            conclusion_title =  _('Operation succesfully finished.')
+            conclusion_title =  _('Operation successfully finished.')
             conclusion_text = _(
                 'The operation you requested has finished successfully. '
                 'You may press OK button now to continue.')
@@ -355,27 +359,12 @@ class BatchOp(UpdateCallback):
         and transactions before and after the running.
         Should not be overridden without good reasons.
         """
-        self._pre_run()
-        success = self._run()
-        self._post_run()
-        return success
-
-    def _pre_run(self):
-        """
-        Low-level method for starting transaction and disabling signals.
-        Should not be overridden without good reasons.
-        """
-        self.trans = self.db.transaction_begin("",batch=True)
         self.db.disable_signals()
-
-    def _post_run(self):
-        """
-        Low-level method for committing transaction and enabling signals.
-        Should not be overridden without good reasons.
-        """
-        self.db.transaction_commit(self.trans,self.title)
+        with DbTxn(self.title, self.db, batch=True) as self.trans:
+            success = self._run()
         self.db.enable_signals()
         self.db.request_rebuild()
+        return success
 
     def _run(self):
         """
@@ -555,15 +544,81 @@ class Convert2Rel(BatchOp):
             self.update()
         return True
 
+#------------------------------------------------------------------------
+#An op to look for images that may have been forgotten.
+#------------------------------------------------------------------------
+class ImagesNotIncluded(BatchOp):
+    title       = _('Add images not included in database')
+    description = _("Check directories for images not included in database")
+    description = _("This tool adds images in directories that are "
+                    "referenced by existing images in the database.")
+
+    def _prepare(self):
+        """
+        Get all of the fullpaths, and the directories of media
+        objects in the database.
+        """
+        self.dir_list = set()
+        self.set_total(self.db.get_number_of_media_objects())
+        with self.db.get_media_cursor() as cursor:
+            for handle, data in cursor:
+                obj = MediaObject()
+                obj.unserialize(data)
+                self.handle_list.append(handle)
+                full_path = media_path_full(self.db, obj.path)
+                self.path_list.append(full_path)
+                directory, filename = os.path.split(full_path)
+                if directory not in self.dir_list:
+                    self.dir_list.add(directory)
+                self.update()
+        self.reset()
+
+    def build_path_list(self):
+        """
+        This method returns a list of the path names that would be
+        affected by the batch op. Typically it would rely on prepare()
+        to do the actual job, but it does not have to be that way.
+        """
+        self.prepare()
+        return self.dir_list
+
+    def _run(self):
+        """
+        Go through directories that are mentioned in the database via
+        media files, and include all images that are not all ready
+        included.
+        """
+        if not self.prepared:
+            self.prepare()
+        self.set_total(len(self.dir_list))
+        for directory in self.dir_list:
+            for (dirpath, dirnames, filenames) in os.walk(directory):
+                if ".svn" in dirnames: 
+                    dirnames.remove('.svn')  # don't visit .svn directories
+                for filename in filenames:
+                    media_full_path = os.path.join(dirpath, filename)
+                    if media_full_path not in self.path_list:
+                        self.path_list.append(media_full_path)
+                        mime_type = gen.mime.get_type(media_full_path)
+                        if gen.mime.is_image_type(mime_type):
+                            obj = MediaObject()
+                            obj.set_path(media_full_path)
+                            obj.set_mime_type(mime_type)
+                            (root, ext) = os.path.splitext(filename)
+                            obj.set_description(root)
+                            self.db.add_object(obj, self.trans)
+            self.update()
+        return True
+
 #------------------------------------------------------------------------
 #
 # 
 #
 #------------------------------------------------------------------------
-class MediaManOptions(Tool.ToolOptions):
+class MediaManOptions(tool.ToolOptions):
     """
     Defines options and provides handling interface.
     """
 
     def __init__(self, name,person_id=None):
-        Tool.ToolOptions.__init__(self, name,person_id)
+        tool.ToolOptions.__init__(self, name,person_id)
