@@ -1,8 +1,10 @@
 #
 # Gramps - a GTK+/GNOME based genealogy program
 #
-# Copyright (C) 2007-2009 Stephane Charette
-# Copyright (C) 2008 Brian Matherly
+# Copyright (C) 2007-2009   Stephane Charette
+# Copyright (C) 2008        Brian Matherly
+# Copyright (C) 2010        Jakim Friant
+# Copyright (C) 2010        Nick Hall
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,16 +21,16 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: NotRelated.py 12362 2009-03-19 04:25:37Z pez4brian $
+# $Id: NotRelated.py 16979 2011-03-30 01:50:08Z pez4brian $
 
 "Find people who are not related to the selected person"
 
 #------------------------------------------------------------------------
 #
-# standard python modules
+# Python modules
 #
 #------------------------------------------------------------------------
-import os
+import locale
 
 #------------------------------------------------------------------------
 #
@@ -44,15 +46,17 @@ import gobject
 #
 #------------------------------------------------------------------------
 import const
-from PluginUtils import Tool
-from gen.plug import PluginManager
-from ReportBase import ReportUtils
-from PluginUtils import Tool
-from Editors import EditPerson, EditFamily
+from gen.ggettext import ngettext
+from gui.plug import tool
+from gen.plug.report import utils as ReportUtils
+from gui.editors import EditPerson, EditFamily
 import ManagedWindow
-import Utils
+from gui.utils import ProgressMeter
 import GrampsDisplay
-from TransUtils import sgettext as _
+from gen.ggettext import sgettext as _
+from glade import Glade
+from gen.lib import Tag
+from gen.db import DbTxn
 
 #-------------------------------------------------------------------------
 #
@@ -61,62 +65,72 @@ from TransUtils import sgettext as _
 #-------------------------------------------------------------------------
 WIKI_HELP_PAGE = '%s_-_Tools' % const.URL_MANUAL_PAGE
 WIKI_HELP_SEC = _('manual|Not_Related...')
+
 #------------------------------------------------------------------------
 #
-# 
+# NotRelated class
 #
 #------------------------------------------------------------------------
-class NotRelated(Tool.ActivePersonTool, ManagedWindow.ManagedWindow) :
+class NotRelated(tool.ActivePersonTool, ManagedWindow.ManagedWindow) :
 
     def __init__(self, dbstate, uistate, options_class, name, callback=None):
-        Tool.ActivePersonTool.__init__(self, dbstate, options_class, name)
+        tool.ActivePersonTool.__init__(self, dbstate, uistate, options_class,
+                                       name)
 
         if self.fail:   # bug #2709 -- fail if we have no active person
             return
 
-        person = dbstate.get_active_person()
+        person_handle = uistate.get_active('Person')
+        person = dbstate.db.get_person_from_handle(person_handle)
         self.name = person.get_primary_name().get_regular_name()
         self.title = _('Not related to "%s"') % self.name
         ManagedWindow.ManagedWindow.__init__(self, uistate, [], self.__class__)
         self.dbstate = dbstate
         self.uistate = uistate
         self.db = dbstate.db
-        glade_file = "%s/NotRelated.glade" % os.path.dirname(__file__)
-        topDialog = gtk.glade.XML(glade_file, "top", "gramps")
-        topDialog.signal_autoconnect({
+        
+        topDialog = Glade()
+        
+        topDialog.connect_signals({
             "destroy_passed_object" : self.close,
             "on_help_clicked"       : self.on_help_clicked,
+            "on_delete_event"       : self.close,
         })
 
-        window = topDialog.get_widget("top")
-        title = topDialog.get_widget("title")
+        window = topDialog.toplevel
+        title = topDialog.get_object("title")
         self.set_window(window, title, self.title)
 
-        self.markercombo = topDialog.get_widget("markercombo")
-        self.markerapply = topDialog.get_widget("markerapply")
-        self.markercombo.set_sensitive(False)
-        self.markerapply.set_sensitive(False)
-        self.markerapply.connect('clicked', self.applyMarkerClicked)
+        self.tagcombo = topDialog.get_object("tagcombo")
+        tagmodel = gtk.ListStore(str)
+        self.tagcombo.set_model(tagmodel)
+        self.tagcombo.set_text_column(0)
+        tagmodel.append((_('ToDo'),))
+        tagmodel.append((_('NotRelated'),))
+        self.tagcombo.set_sensitive(False)
+
+        self.tagapply = topDialog.get_object("tagapply")
+        self.tagapply.set_sensitive(False)
+        self.tagapply.connect('clicked', self.applyTagClicked)
 
-    
         # start the progress indicator
-        self.progress = Utils.ProgressMeter(self.title,_('Starting'))
+        self.progress = ProgressMeter(self.title,_('Starting'))
 
         # setup the columns
         self.model = gtk.TreeStore(
             gobject.TYPE_STRING,    # 0==name
             gobject.TYPE_STRING,    # 1==person gid
             gobject.TYPE_STRING,    # 2==parents
-            gobject.TYPE_STRING,    # 3==marker
+            gobject.TYPE_STRING,    # 3==tags
             gobject.TYPE_STRING)    # 4==family gid (not shown to user)
 
         # note -- don't assign the model to the tree until it has been populated,
         # otherwise the screen updates are terribly slow while names are appended
-        self.treeView = topDialog.get_widget("treeview")
+        self.treeView = topDialog.get_object("treeview")
         col1 = gtk.TreeViewColumn(_('Name'),    gtk.CellRendererText(), text=0)
         col2 = gtk.TreeViewColumn(_('ID'),      gtk.CellRendererText(), text=1)
         col3 = gtk.TreeViewColumn(_('Parents'), gtk.CellRendererText(), text=2)
-        col4 = gtk.TreeViewColumn(_('Marker'),  gtk.CellRendererText(), text=3)
+        col4 = gtk.TreeViewColumn(_('Tags'),    gtk.CellRendererText(), text=3)
         col1.set_resizable(True)
         col2.set_resizable(True)
         col3.set_resizable(True)
@@ -190,8 +204,8 @@ class NotRelated(Tool.ActivePersonTool, ManagedWindow.ManagedWindow) :
 
     def rowSelectionChanged(self, selection) :
         state = selection.count_selected_rows() > 0
-        self.markercombo.set_sensitive(state)
-        self.markerapply.set_sensitive(state)
+        self.tagcombo.set_sensitive(state)
+        self.tagapply.set_sensitive(state)
 
 
     def rowActivated(self, treeView, path, column) :
@@ -223,47 +237,74 @@ class NotRelated(Tool.ActivePersonTool, ManagedWindow.ManagedWindow) :
         GrampsDisplay.help(WIKI_HELP_PAGE , WIKI_HELP_SEC)    
 
 
-    def applyMarkerClicked(self, button) :
+    def applyTagClicked(self, button) :
         progress    = None
         rows        = self.treeSelection.count_selected_rows()
-        marker      = self.markercombo.get_active_text()
-
-        # if more than 1 person is selected, use a progress indicator
-        if rows > 1:
-            progress = Utils.ProgressMeter(self.title,_('Starting'))
-            progress.set_pass(_('Setting marker for %d people') % rows, rows)
+        tag_name    = self.tagcombo.get_active_text()
 
         # start the db transaction
-        transaction = self.db.transaction_begin()
+        with DbTxn("Tag not related", self.db) as transaction:
 
-        # iterate through all of the selected rows
-        (model, paths) = self.treeSelection.get_selected_rows()
+            tag = self.db.get_tag_from_name(tag_name)
+            if not tag:
+                # create the tag if it doesn't already exist
+                tag = Tag()
+                tag.set_name(tag_name)
+                tag.set_priority(self.db.get_number_of_tags())
+                tag_handle = self.db.add_tag(tag, transaction)
+            else:
+                tag_handle = tag.get_handle()
+
+            # if more than 1 person is selected, use a progress indicator
+            if rows > 1:
+                progress = ProgressMeter(self.title,_('Starting'))
+                #TRANS: no singular form needed, as rows is always > 1
+                progress.set_pass(ngettext("Setting tag for %d person",
+                                           "Setting tag for %d people", 
+                                            rows) % rows, rows)
+
+    
+            # iterate through all of the selected rows
+            (model, paths) = self.treeSelection.get_selected_rows()
+
+            for path in paths:
+                if progress:
+                    progress.step()
+    
+                # for the current row, get the GID and the person from the database
+                iter        = self.model.get_iter(path)
+                personGid   = self.model.get_value(iter, 1)
+                person      = self.db.get_person_from_gramps_id(personGid)
+
+                # add the tag to the person
+                person.add_tag(tag_handle)
+
+                # save this change
+                self.db.commit_person(person, transaction)
+
+
+        # refresh the tags column
+        self.treeView.set_model(None)
         for path in paths:
-            if progress:
-                progress.step()
-
-            # for the current row, get the GID and the person from the database
             iter        = self.model.get_iter(path)
             personGid   = self.model.get_value(iter, 1)
             person      = self.db.get_person_from_gramps_id(personGid)
-
-            # change the marker
-            person.set_marker(marker)
-            self.model.set_value(iter, 3, marker)
-
-            # save this change
-            self.db.commit_person(person, transaction)
-
-        # commit the entire transaction
-        self.db.transaction_commit(transaction, "mark not related")
+            self.model.set_value(iter, 3, self.get_tag_list(person))
+        self.treeView.set_model(self.model)
+        self.treeView.expand_all()
 
         if progress:
             progress.close()
 
-
     def findRelatedPeople(self) :
 
-        self.progress.set_pass(_('Finding relationships between %d people') % self.numberOfPeopleInDatabase, self.numberOfPeopleInDatabase)
+        #TRANS: No singular form is needed.
+        self.progress.set_pass(
+            ngettext("Finding relationships between %d person",
+                     "Finding relationships between %d people",
+                     self.numberOfPeopleInDatabase) %
+                        self.numberOfPeopleInDatabase,
+            self.numberOfPeopleInDatabase)
 
         # as long as we have people we haven't processed yet, keep looping
         while len(self.handlesOfPeopleToBeProcessed) > 0:
@@ -292,48 +333,50 @@ class NotRelated(Tool.ActivePersonTool, ManagedWindow.ManagedWindow) :
             for familyHandle in person.get_family_handle_list():
                 family = self.db.get_family_from_handle(familyHandle)
                 spouseHandle = ReportUtils.find_spouse(person, family)
-                if spouseHandle:
-                    if spouseHandle not in self.handlesOfPeopleAlreadyProcessed or spouseHandle not in self.handlesOfPeopleToBeProcessed:
-                        self.handlesOfPeopleToBeProcessed.add(spouseHandle)
+                if spouseHandle and \
+                  spouseHandle not in self.handlesOfPeopleAlreadyProcessed:
+                    self.handlesOfPeopleToBeProcessed.add(spouseHandle)
 
             # step 2 -- parents
             for familyHandle in person.get_parent_family_handle_list():
                 family = self.db.get_family_from_handle(familyHandle)
                 fatherHandle = family.get_father_handle()
                 motherHandle = family.get_mother_handle()
-                if fatherHandle:
-                    if fatherHandle not in self.handlesOfPeopleAlreadyProcessed or fatherHandle not in self.handlesOfPeopleToBeProcessed:
-                        self.handlesOfPeopleToBeProcessed.add(fatherHandle)
-                if motherHandle:
-                    if motherHandle not in self.handlesOfPeopleAlreadyProcessed or motherHandle not in self.handlesOfPeopleToBeProcessed:
-                        self.handlesOfPeopleToBeProcessed.add(motherHandle)
+                if fatherHandle and \
+                  fatherHandle not in self.handlesOfPeopleAlreadyProcessed:
+                    self.handlesOfPeopleToBeProcessed.add(fatherHandle)
+                if motherHandle and \
+                  motherHandle not in self.handlesOfPeopleAlreadyProcessed:
+                    self.handlesOfPeopleToBeProcessed.add(motherHandle)
 
             # step 3 -- children
             for familyHandle in person.get_family_handle_list():
                 family = self.db.get_family_from_handle(familyHandle)
                 for childRef in family.get_child_ref_list():
                     childHandle = childRef.ref
-                    if childHandle:
-                        if childHandle not in self.handlesOfPeopleAlreadyProcessed or childHandle not in self.handlesOfPeopleToBeProcessed:
-                            self.handlesOfPeopleToBeProcessed.add(childHandle)
+                    if childHandle and \
+                      childHandle not in self.handlesOfPeopleAlreadyProcessed:
+                        self.handlesOfPeopleToBeProcessed.add(childHandle)
 
 
     def findUnrelatedPeople(self) :
 
         # update our numbers
-        self.numberOfRelatedPeople   = len(self.handlesOfPeopleAlreadyProcessed)
-        self.numberOfUnrelatedPeople = self.numberOfPeopleInDatabase - self.numberOfRelatedPeople
+        self.numberOfRelatedPeople = len(self.handlesOfPeopleAlreadyProcessed)
+        self.numberOfUnrelatedPeople = (self.numberOfPeopleInDatabase -
+                                        self.numberOfRelatedPeople)
 
         if self.numberOfUnrelatedPeople > 0:
             # we have at least 1 "unrelated" person to find
 
-            if self.numberOfUnrelatedPeople == 1:
-                self.progress.set_pass(_('Looking for 1 person'), self.numberOfPeopleInDatabase)
-            else:
-                self.progress.set_pass(_('Looking for %d people') % self.numberOfUnrelatedPeople, self.numberOfPeopleInDatabase)
+            self.progress.set_pass(
+                    ngettext("Looking for %d person", "Looking for %d people",
+                    self.numberOfUnrelatedPeople) %
+                        self.numberOfUnrelatedPeople,
+                    self.numberOfPeopleInDatabase) 
 
             # loop through everyone in the database
-            for handle in self.db.get_person_handles(False):
+            for handle in self.db.iter_person_handles():
 
                 self.progress.step()
 
@@ -352,10 +395,12 @@ class NotRelated(Tool.ActivePersonTool, ManagedWindow.ManagedWindow) :
 
     def populateModel(self) :
 
-        if self.numberOfUnrelatedPeople == 1:
-            self.progress.set_pass(_('Looking up the name for 1 person'), 1)
-        else:
-            self.progress.set_pass(_('Looking up the names for %d people') % self.numberOfUnrelatedPeople, self.numberOfUnrelatedPeople)
+        self.progress.set_pass(
+                ngettext("Looking up the name of %d person",
+                         "Looking up the names of %d people",
+                         self.numberOfUnrelatedPeople) %
+                            self.numberOfUnrelatedPeople,
+                self.numberOfUnrelatedPeople)
 
         # loop through the entire list of unrelated people
         for handle in self.handlesOfPeopleNotRelated:
@@ -365,7 +410,9 @@ class NotRelated(Tool.ActivePersonTool, ManagedWindow.ManagedWindow) :
             surname     = primaryname.get_surname()
             name        = primaryname.get_name()
             gid         = person.get_gramps_id()
-            marker      = person.get_marker()
+
+            # Retrieve the sorted tag list
+            tag_list = self.get_tag_list(person)
 
             # find the names of the parents
             familygid   = ''
@@ -408,42 +455,32 @@ class NotRelated(Tool.ActivePersonTool, ManagedWindow.ManagedWindow) :
                 iter = self.model.append(None, [surname, '', '', '', ''])
 
             # finally, we now get to add this person to the model
-            self.model.append(iter, [name, gid, parentNames, marker, familygid])
-
+            self.model.append(iter, [name, gid, parentNames, tag_list,
+                                     familygid])
 
     def build_menu_names(self, obj):
         return (self.title, None)
-    
+
+    def get_tag_list(self, person):
+        """
+        Return a sorted list of tag names for the given person. 
+        """
+        tags = []
+        for handle in person.get_tag_list():
+            tag = self.db.get_tag_from_handle(handle)
+            tags.append(tag.get_name())
+        tags.sort(key=locale.strxfrm)
+        return ', '.join(tags)
+
 #------------------------------------------------------------------------
 #
 # NotRelatedOptions
 #
 #------------------------------------------------------------------------
-class NotRelatedOptions(Tool.ToolOptions):
+class NotRelatedOptions(tool.ToolOptions):
     """
     Defines options and provides handling interface.
     """
     def __init__(self, name, person_id=None):
         """ Initialize the options class """
-        Tool.ToolOptions.__init__(self, name, person_id)
-
-#-------------------------------------------------------------------------
-#
-#
-#
-#-------------------------------------------------------------------------
-pmgr = PluginManager.get_instance()
-pmgr.register_tool(
-    name = 'not_related',
-    category = Tool.TOOL_UTILS,
-    tool_class = NotRelated,
-    options_class = NotRelatedOptions,
-    modes = PluginManager.TOOL_MODE_GUI,
-    translated_name = _("Not Related"),
-    status = _("Stable"),
-    author_name = "Stephane Charette",
-    author_email = "stephanecharette@gmail.com",
-    description = _("Find people who are not in any way related to the "
-                    "selected person")
-    )
-
+        tool.ToolOptions.__init__(self, name, person_id)
