@@ -3,6 +3,8 @@
 #
 # Copyright (C) 2000-2007  Donald N. Allingham
 # Copyright (C) 2008       Brian G. Matherly
+# Copyright (C) 2010       Jakim Friant
+# Copyright (C) 2010       Nick Hall
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,7 +21,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: OwnerEditor.py 14091 2010-01-18 04:42:17Z pez4brian $
+# $Id: OwnerEditor.py 16979 2011-03-30 01:50:08Z pez4brian $
 
 """Tools/Database Processing/Edit Database Owner Information"""
 
@@ -41,7 +43,7 @@ from Utils import get_researcher
 import GrampsDisplay
 from gui.widgets import MonitoredEntry
 import ManagedWindow
-from PluginUtils import Tool
+from gui.plug import tool
 from gen.ggettext import sgettext as _
 from glade import Glade
 
@@ -61,6 +63,7 @@ WIKI_HELP_SEC = _('manual|Edit_Database_Owner_Information...')
 config_keys = (
     'researcher.researcher-name',
     'researcher.researcher-addr',
+    'researcher.researcher-locality',
     'researcher.researcher-city',
     'researcher.researcher-state',
     'researcher.researcher-country',
@@ -74,7 +77,7 @@ config_keys = (
 # OwnerEditor
 #
 #-------------------------------------------------------------------------
-class OwnerEditor(Tool.Tool, ManagedWindow.ManagedWindow):
+class OwnerEditor(tool.Tool, ManagedWindow.ManagedWindow):
     """
     Allow editing database owner information.
 
@@ -83,7 +86,7 @@ class OwnerEditor(Tool.Tool, ManagedWindow.ManagedWindow):
     """
     def __init__(self, dbstate, uistate, options_class, name, callback=None):
         ManagedWindow.ManagedWindow.__init__(self, uistate, [], self.__class__)
-        Tool.Tool.__init__(self, dbstate, options_class, name)
+        tool.Tool.__init__(self, dbstate, options_class, name)
        
         self.display()
 
@@ -121,14 +124,15 @@ class OwnerEditor(Tool.Tool, ManagedWindow.ManagedWindow):
         
         self.entries = []
         entry = [
-            ("entry1", self.owner.set_name, self.owner.get_name),
-            ("entry2", self.owner.set_address, self.owner.get_address),
-            ("entry3", self.owner.set_city, self.owner.get_city),
-            ("entry4", self.owner.set_state, self.owner.get_state),
-            ("entry5", self.owner.set_country, self.owner.get_country),
-            ("entry6", self.owner.set_postal_code, self.owner.get_postal_code),
-            ("entry7", self.owner.set_phone, self.owner.get_phone),
-            ("entry8", self.owner.set_email, self.owner.get_email),
+            ("name", self.owner.set_name, self.owner.get_name),
+            ("address", self.owner.set_address, self.owner.get_address),
+            ("locality", self.owner.set_locality, self.owner.get_locality),
+            ("city", self.owner.set_city, self.owner.get_city),
+            ("state", self.owner.set_state, self.owner.get_state),
+            ("country", self.owner.set_country, self.owner.get_country),
+            ("zip", self.owner.set_postal_code, self.owner.get_postal_code),
+            ("phone", self.owner.set_phone, self.owner.get_phone),
+            ("email", self.owner.set_email, self.owner.get_email),
         ]
 
         for (name,set_fn,get_fn) in entry:
@@ -172,7 +176,7 @@ class OwnerEditor(Tool.Tool, ManagedWindow.ManagedWindow):
 # OwnerEditorOptions (None at the moment)
 #
 #-------------------------------------------------------------------------
-class OwnerEditorOptions(Tool.ToolOptions):
+class OwnerEditorOptions(tool.ToolOptions):
     """Defines options and provides handling interface."""
     def __init__(self, name,person_id=None):
-        Tool.ToolOptions.__init__(self, name,person_id)
+        tool.ToolOptions.__init__(self, name,person_id)
