@@ -4,6 +4,7 @@
 # Copyright (C) 2000-2007  Donald N. Allingham
 # Copyright (C) 2008       Brian G. Matherly
 # Copyright (C) 2008       Stephane Charette
+# Copyright (C) 2010       Jakim Friant
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,17 +21,17 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: RemoveUnused.py 12362 2009-03-19 04:25:37Z pez4brian $
+# $Id: RemoveUnused.py 16979 2011-03-30 01:50:08Z pez4brian $
 
-"Find unused objects and remove with the user's permission"
+"Find unused objects and remove with the user's permission."
 
 #-------------------------------------------------------------------------
 #
 # python modules
 #
 #-------------------------------------------------------------------------
-import os
-from gettext import gettext as _
+from __future__ import with_statement
+from gen.ggettext import gettext as _
 
 #------------------------------------------------------------------------
 #
@@ -46,28 +47,27 @@ log = logging.getLogger(".RemoveUnused")
 #
 #-------------------------------------------------------------------------
 import gtk
-from gtk import glade
 import gobject
-from gen.lib import StyledText
 
 #-------------------------------------------------------------------------
 #
 # GRAMPS modules
 #
 #-------------------------------------------------------------------------
+from gen.db import DbTxn
 import Errors
 import ManagedWindow
 from DateHandler import displayer as _dd
-from BasicUtils import UpdateCallback
-from PluginUtils import Tool
-from gen.plug import PluginManager
+from gen.updatecallback import UpdateCallback
+from gui.plug import tool
+from glade import Glade
 
 #-------------------------------------------------------------------------
 #
 # runTool
 #
 #-------------------------------------------------------------------------
-class RemoveUnused(Tool.Tool,ManagedWindow.ManagedWindow,UpdateCallback):
+class RemoveUnused(tool.Tool, ManagedWindow.ManagedWindow, UpdateCallback):
     MARK_COL       = 0
     OBJ_ID_COL     = 1
     OBJ_NAME_COL   = 2
@@ -77,13 +77,13 @@ class RemoveUnused(Tool.Tool,ManagedWindow.ManagedWindow,UpdateCallback):
     def __init__(self, dbstate, uistate, options_class, name, callback=None):
         self.title = _('Unused Objects')
 
-        Tool.Tool.__init__(self, dbstate, options_class, name)
+        tool.Tool.__init__(self, dbstate, options_class, name)
 
         if self.db.readonly:
             return
 
-        ManagedWindow.ManagedWindow.__init__(self, uistate,[],self.__class__)
-        UpdateCallback.__init__(self,self.uistate.pulse_progressbar)
+        ManagedWindow.ManagedWindow.__init__(self, uistate,[], self.__class__)
+        UpdateCallback.__init__(self, self.uistate.pulse_progressbar)
 
         self.dbstate = dbstate
         self.uistate = uistate
@@ -130,21 +130,18 @@ class RemoveUnused(Tool.Tool,ManagedWindow.ManagedWindow,UpdateCallback):
         self.init_gui()
 
     def init_gui(self):
-        base = os.path.dirname(__file__)
-        self.glade_file = base + os.sep + "unused.glade"
+        self.top = Glade()
+        window = self.top.toplevel
+        self.set_window(window, self.top.get_object('title'), self.title)
 
-        self.top = glade.XML(self.glade_file,"unused","gramps")
-        window = self.top.get_widget("unused")
-        self.set_window(window,self.top.get_widget('title'),self.title)
-
-        self.events_box = self.top.get_widget('events_box')
-        self.sources_box = self.top.get_widget('sources_box')
-        self.places_box = self.top.get_widget('places_box')
-        self.media_box = self.top.get_widget('media_box')
-        self.repos_box = self.top.get_widget('repos_box')
-        self.notes_box = self.top.get_widget('notes_box')
-        self.find_button = self.top.get_widget('find_button')
-        self.remove_button = self.top.get_widget('remove_button')
+        self.events_box = self.top.get_object('events_box')
+        self.sources_box = self.top.get_object('sources_box')
+        self.places_box = self.top.get_object('places_box')
+        self.media_box = self.top.get_object('media_box')
+        self.repos_box = self.top.get_object('repos_box')
+        self.notes_box = self.top.get_object('notes_box')
+        self.find_button = self.top.get_object('find_button')
+        self.remove_button = self.top.get_object('remove_button')
 
         self.events_box.set_active(self.options.handler.options_dict['events'])
         self.sources_box.set_active(
@@ -155,19 +152,19 @@ class RemoveUnused(Tool.Tool,ManagedWindow.ManagedWindow,UpdateCallback):
         self.repos_box.set_active(self.options.handler.options_dict['repos'])
         self.notes_box.set_active(self.options.handler.options_dict['notes'])
 
-        self.warn_tree = self.top.get_widget('warn_tree')
+        self.warn_tree = self.top.get_object('warn_tree')
         self.warn_tree.connect('button_press_event', self.double_click)
 
         self.selection = self.warn_tree.get_selection()
         
-        self.mark_button = self.top.get_widget('mark_button')
-        self.mark_button.connect('clicked',self.mark_clicked)
+        self.mark_button = self.top.get_object('mark_button')
+        self.mark_button.connect('clicked', self.mark_clicked)
 
-        self.unmark_button = self.top.get_widget('unmark_button')
-        self.unmark_button.connect('clicked',self.unmark_clicked)
+        self.unmark_button = self.top.get_object('unmark_button')
+        self.unmark_button.connect('clicked', self.unmark_clicked)
 
-        self.invert_button = self.top.get_widget('invert_button')
-        self.invert_button.connect('clicked',self.invert_clicked)
+        self.invert_button = self.top.get_object('invert_button')
+        self.invert_button.connect('clicked', self.invert_clicked)
 
         self.real_model = gtk.ListStore(gobject.TYPE_BOOLEAN, 
                                         gobject.TYPE_STRING, 
@@ -180,17 +177,17 @@ class RemoveUnused(Tool.Tool,ManagedWindow.ManagedWindow,UpdateCallback):
         self.renderer = gtk.CellRendererText()
         self.img_renderer = gtk.CellRendererPixbuf()
         self.bool_renderer = gtk.CellRendererToggle()
-        self.bool_renderer.connect('toggled',self.selection_toggled)
+        self.bool_renderer.connect('toggled', self.selection_toggled)
 
         # Add mark column
-        mark_column = gtk.TreeViewColumn(_('Mark'),self.bool_renderer,
+        mark_column = gtk.TreeViewColumn(_('Mark'), self.bool_renderer,
                                            active=RemoveUnused.MARK_COL)
         mark_column.set_sort_column_id(RemoveUnused.MARK_COL)
         self.warn_tree.append_column(mark_column)
         
         # Add image column
         img_column = gtk.TreeViewColumn(None, self.img_renderer )
-        img_column.set_cell_data_func(self.img_renderer,self.get_image)
+        img_column.set_cell_data_func(self.img_renderer, self.get_image)
         self.warn_tree.append_column(img_column)        
 
         # Add column with object gramps_id
@@ -205,17 +202,18 @@ class RemoveUnused(Tool.Tool,ManagedWindow.ManagedWindow,UpdateCallback):
         name_column.set_sort_column_id(RemoveUnused.OBJ_NAME_COL)
         self.warn_tree.append_column(name_column)
 
-        self.top.signal_autoconnect({
+        self.top.connect_signals({
             "destroy_passed_object"   : self.close,
             "on_remove_button_clicked": self.do_remove,
             "on_find_button_clicked"  : self.find,
+            "on_delete_event"         : self.close,
             })
 
-        self.dc_label = self.top.get_widget('dc_label')
+        self.dc_label = self.top.get_object('dc_label')
 
-        self.sensitive_list = [self.warn_tree,self.mark_button,
-                               self.unmark_button,self.invert_button,
-                               self.dc_label,self.remove_button]
+        self.sensitive_list = [self.warn_tree, self.mark_button,
+                               self.unmark_button, self.invert_button,
+                               self.dc_label, self.remove_button]
 
         for item in self.sensitive_list:
             item.set_sensitive(False)
@@ -223,21 +221,17 @@ class RemoveUnused(Tool.Tool,ManagedWindow.ManagedWindow,UpdateCallback):
         self.show()
 
     def build_menu_names(self, obj):
-        return (self.title,None)
+        return (self.title, None)
 
     def find(self, obj):
-        self.options.handler.options_dict['events'] = \
-                                        int(self.events_box.get_active())
-        self.options.handler.options_dict['sources'] = \
-                                        int(self.sources_box.get_active())
-        self.options.handler.options_dict['places'] = \
-                                        int(self.places_box.get_active())
-        self.options.handler.options_dict['media'] = \
-                                        int(self.media_box.get_active())
-        self.options.handler.options_dict['repos'] = \
-                                        int(self.repos_box.get_active())
-        self.options.handler.options_dict['notes'] = \
-                                        int(self.notes_box.get_active())
+        self.options.handler.options_dict.update(
+            events  = self.events_box.get_active(),
+            sources = self.sources_box.get_active(),
+            places  = self.places_box.get_active(),
+            media   = self.media_box.get_active(),
+            repos   = self.repos_box.get_active(),
+            notes   = self.notes_box.get_active(),
+            )
 
         for item in self.sensitive_list:
             item.set_sensitive(True)
@@ -260,89 +254,77 @@ class RemoveUnused(Tool.Tool,ManagedWindow.ManagedWindow,UpdateCallback):
     def collect_unused(self):
         # Run through all requested tables and check all objects
         # for being referenced some place. If not, add_results on them.
-        
-        tables = {
-            'events'  : {'cursor_func': self.db.get_event_cursor,
-                         'total_func' : self.db.get_number_of_events},
-            'sources' : {'cursor_func': self.db.get_source_cursor,
-                         'total_func' : self.db.get_number_of_sources},
-            'places'  : {'cursor_func': self.db.get_place_cursor,
-                         'total_func' : self.db.get_number_of_places},
-            'media'   : {'cursor_func': self.db.get_media_cursor,
-                         'total_func' : self.db.get_number_of_media_objects},
-            'repos'   : {'cursor_func': self.db.get_repository_cursor,
-                         'total_func' : self.db.get_number_of_repositories},
-            'notes'   : {'cursor_func': self.db.get_note_cursor,
-                         'total_func' : self.db.get_number_of_notes},
-            }
 
-        for the_type in tables.keys():
+        db = self.db
+        tables = (
+            ('events', db.get_event_cursor, db.get_number_of_events),
+            ('sources', db.get_source_cursor, db.get_number_of_sources),
+            ('places', db.get_place_cursor, db.get_number_of_places),
+            ('media', db.get_media_cursor, db.get_number_of_media_objects),
+            ('repos', db.get_repository_cursor, db.get_number_of_repositories),
+            ('notes', db.get_note_cursor, db.get_number_of_notes),
+            )
+
+        for (the_type, cursor_func, total_func) in tables:
             if not self.options.handler.options_dict[the_type]:
                 # This table was not requested. Skip it.
                 continue
 
-            cursor = tables[the_type]['cursor_func']()
-            total = tables[the_type]['total_func']()
-            self.set_total(total)
-            item = cursor.first()
-            while item:
-                (handle,data) = item
-
-                hlist = [x for x in self.db.find_backlink_handles(handle)]
-                if len(hlist) == 0:
-                    self.add_results((the_type, handle, data))
-                item = cursor.next()
-                self.update()
-            cursor.close()
+            with cursor_func() as cursor:
+                self.set_total(total_func())
+                fbh = db.find_backlink_handles
+                for handle, data in cursor:
+                    if not any(h for h in fbh(handle)):
+                        self.add_results((the_type, handle, data))
+                    self.update()
             self.reset()
 
     def do_remove(self, obj):
-        trans = self.db.transaction_begin("",batch=False)
-        self.db.disable_signals()
+        with DbTxn(_("Remove unused objects"), self.db, batch=False) as trans:
+            self.db.disable_signals()
 
-        for row_num in range(len(self.real_model)-1,-1,-1):
-            path = (row_num,)
-            row = self.real_model[path]
-            if not row[RemoveUnused.MARK_COL]:
-                continue
+            for row_num in range(len(self.real_model)-1, -1, -1):
+                path = (row_num,)
+                row = self.real_model[path]
+                if not row[RemoveUnused.MARK_COL]:
+                    continue
 
-            the_type = row[RemoveUnused.OBJ_TYPE_COL]
-            handle = row[RemoveUnused.OBJ_HANDLE_COL]
-            remove_func = self.tables[the_type]['remove']
-            remove_func(handle, trans)
+                the_type = row[RemoveUnused.OBJ_TYPE_COL]
+                handle = row[RemoveUnused.OBJ_HANDLE_COL]
+                remove_func = self.tables[the_type]['remove']
+                remove_func(handle, trans)
 
-            self.real_model.remove(row.iter)
+                self.real_model.remove(row.iter)
 
-        self.db.transaction_commit(trans, _("Remove unused objects"))
         self.db.enable_signals()
         self.db.request_rebuild()
 
-    def selection_toggled(self,cell,path_string):
-        sort_path = tuple([int (i) for i in path_string.split(':')])
+    def selection_toggled(self, cell, path_string):
+        sort_path = tuple(map(int, path_string.split(':')))
         real_path = self.sort_model.convert_path_to_child_path(sort_path)
         row = self.real_model[real_path]
         row[RemoveUnused.MARK_COL] = not row[RemoveUnused.MARK_COL]
-        self.real_model.row_changed(real_path,row.iter)
+        self.real_model.row_changed(real_path, row.iter)
 
-    def mark_clicked(self,mark_button):
+    def mark_clicked(self, mark_button):
         for row_num in range(len(self.real_model)):
             path = (row_num,)
             row = self.real_model[path]
             row[RemoveUnused.MARK_COL] = True
 
-    def unmark_clicked(self,unmark_button):
+    def unmark_clicked(self, unmark_button):
         for row_num in range(len(self.real_model)):
             path = (row_num,)
             row = self.real_model[path]
             row[RemoveUnused.MARK_COL] = False
 
-    def invert_clicked(self,invert_button):
+    def invert_clicked(self, invert_button):
         for row_num in range(len(self.real_model)):
             path = (row_num,)
             row = self.real_model[path]
             row[RemoveUnused.MARK_COL] = not row[RemoveUnused.MARK_COL]
 
-    def double_click(self, obj,event):
+    def double_click(self, obj, event):
         if event.type == gtk.gdk._2BUTTON_PRESS and event.button == 1:
             (model, node) = self.selection.get_selected()
             if not node:
@@ -354,11 +336,12 @@ class RemoveUnused(Tool.Tool,ManagedWindow.ManagedWindow,UpdateCallback):
             handle = row[RemoveUnused.OBJ_HANDLE_COL]
             self.call_editor(the_type, handle)
 
-    def call_editor(self,the_type, handle):
+    def call_editor(self, the_type, handle):
         try:
             obj = self.tables[the_type]['get_func'](handle)
-            editor_str = 'from Editors import %s as editor' \
-                         % self.tables[the_type]['editor']
+            editor_str = 'from gui.editors import %s as editor' % (
+                            self.tables[the_type]['editor']
+                            )
             exec(editor_str)            
             editor(self.dbstate, self.uistate, [], obj)
         except Errors.WindowActiveError:
@@ -430,7 +413,7 @@ class RemoveUnused(Tool.Tool,ManagedWindow.ManagedWindow,UpdateCallback):
         text = note.get()
 
         # convert whitespace to a single space
-        text = " ".join(text.split())
+        text = u" ".join(text.split())
 
         # if the note is too long, truncate it
         if len(text) > 80:
@@ -443,13 +426,13 @@ class RemoveUnused(Tool.Tool,ManagedWindow.ManagedWindow,UpdateCallback):
 # 
 #
 #------------------------------------------------------------------------
-class CheckOptions(Tool.ToolOptions):
+class CheckOptions(tool.ToolOptions):
     """
     Defines options and provides handling interface.
     """
 
-    def __init__(self, name,person_id=None):
-        Tool.ToolOptions.__init__(self, name,person_id)
+    def __init__(self, name, person_id=None):
+        tool.ToolOptions.__init__(self, name, person_id)
 
         # Options specific for this report
         self.options_dict = {
@@ -480,22 +463,3 @@ class CheckOptions(Tool.ToolOptions):
                           ["Do not check notes","Check notes"],
                           True),
             }
-
-#------------------------------------------------------------------------
-#
-# 
-#
-#------------------------------------------------------------------------
-pmgr = PluginManager.get_instance()
-pmgr.register_tool(
-    name = 'remove_unused',
-    category = Tool.TOOL_DBFIX,
-    tool_class = RemoveUnused,
-    options_class = CheckOptions,
-    modes = PluginManager.TOOL_MODE_GUI,
-    translated_name = _("Remove Unused Objects"),
-    status = _("Stable"),
-    author_name = "Donald N. Allingham",
-    author_email = "don@gramps-project.org",
-    description = _("Removes unused objects from the database")
-    )
