@@ -2,6 +2,7 @@
 # Gramps - a GTK+/GNOME based genealogy program
 #
 # Copyright (C) 2009       Gary Burton
+# Copyright (C) 2010       Jakim Friant
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,7 +19,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: SortEvents.py 14091 2010-01-18 04:42:17Z pez4brian $
+# $Id: SortEvents.py 16979 2011-03-30 01:50:08Z pez4brian $
 
 """Tools/Database Processing/Sort Events"""
 
@@ -36,8 +37,9 @@ from gen.ggettext import gettext as _
 #-------------------------------------------------------------------------
 import Sort
 
-from PluginUtils import Tool, MenuToolOptions, PluginWindows
-from ReportBase import ReportUtils
+from gen.db import DbTxn
+from gui.plug import MenuToolOptions, PluginWindows
+from gen.plug.report import utils as ReportUtils
 from gen.plug.menu import FilterOption, PersonOption, \
                           EnumeratedListOption, BooleanOption
 
@@ -94,12 +96,11 @@ class SortEvents(PluginWindows.ToolManagedWindowBatch):
         self.sort_name = sort_functions[sort_func_num][0]
         self.sort_func = sort_functions[sort_func_num][1]
         self.sort = Sort.Sort(self.db)
-        trans = self.db.transaction_begin("", batch=True)
-        self.db.disable_signals()
-        family_handles = self.sort_person_events(trans)
-        if len(family_handles) > 0:
-            self.sort_family_events(family_handles, trans)
-        self.db.transaction_commit(trans, _("Sort event changes"))
+        with DbTxn(_("Sort event changes"), self.db, batch=True) as trans:
+            self.db.disable_signals()
+            family_handles = self.sort_person_events(trans)
+            if len(family_handles) > 0:
+                self.sort_family_events(family_handles, trans)
         self.db.enable_signals()
         self.db.request_rebuild()
 
