@@ -3,6 +3,8 @@
 #
 # Copyright (C) 2000-2007  Donald N. Allingham
 # Copyright (C) 2008       Brian G. Matherly
+# Copyright (C) 2010       Jakim Friant
+# Copyright (C) 2011       PaulFranklin
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,7 +21,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: Verify.py 14628 2010-03-05 15:43:47Z bmcage $
+# $Id: Verify.py 17470 2011-05-11 02:42:06Z pez4brian $
 
 """
 A plugin to verify the data against user-adjusted tests.
@@ -59,7 +61,7 @@ import Utils
 import GrampsDisplay
 from ManagedWindow import ManagedWindow
 from gen.updatecallback import UpdateCallback
-from PluginUtils import Tool
+from gui.plug import tool
 from gen.ggettext import sgettext as _
 from glade import Glade
 
@@ -114,32 +116,45 @@ def clear_cache():
 # helper functions
 #
 #-------------------------------------------------------------------------
-def get_date_from_event_handle(db,event_handle):
+def get_date_from_event_handle(db,event_handle,estimate=False):
     if not event_handle:
         return 0
     event =  find_event(db,event_handle)
-    date_obj = event.get_date_object()
-    if date_obj.get_day() == 0 or date_obj.get_month() == 0:
+    if event:
+        date_obj = event.get_date_object()
+        if not estimate and \
+           (date_obj.get_day() == 0 or date_obj.get_month() == 0):
+            return 0
+        return date_obj.get_sort_value()
+    else:
         return 0
-    return date_obj.get_sort_value()
 
-def get_date_from_event_type(db,person,event_type):
+def get_date_from_event_type(db,person,event_type,estimate=False):
     if not person:
         return 0
     for event_ref in person.get_event_ref_list():
         event = find_event(db,event_ref.ref)
-        if event.get_type() == event_type:
-            date_obj = event.get_date_object()
-            if date_obj.get_day() == 0 or date_obj.get_month() == 0:
-                return 0
-            return date_obj.get_sort_value()
+        if event:
+            if event.get_type() == event_type:
+                date_obj = event.get_date_object()
+                if not estimate and \
+                   (date_obj.get_day() == 0 or date_obj.get_month() == 0):
+                    return 0
+                return date_obj.get_sort_value()
     return 0
 
-def get_bapt_date(db,person):
-    return get_date_from_event_type(db,person,gen.lib.EventType.BAPTISM)
+def get_bapt_date(db,person,estimate=False):
+    return get_date_from_event_type(db, person, 
+                                    gen.lib.EventType.BAPTISM, estimate)
 
-def get_bury_date(db,person):
-    return get_date_from_event_type(db,person,gen.lib.EventType.BURIAL)
+def get_bury_date(db,person,estimate=False):
+    # check role on burial event
+    for event_ref in person.get_event_ref_list():
+        event = find_event(db, event_ref.ref)
+        if event and event.get_type() == gen.lib.EventType.BURIAL and \
+        event_ref.get_role() == gen.lib.EventRoleType.PRIMARY:
+            return get_date_from_event_type(db, person, 
+                                            gen.lib.EventType.BURIAL, estimate)
 
 def get_birth_date(db,person,estimate=False):
     if not person:
@@ -148,9 +163,9 @@ def get_birth_date(db,person,estimate=False):
     if not birth_ref:
         ret = 0
     else:
-        ret = get_date_from_event_handle(db,birth_ref.ref)
+        ret = get_date_from_event_handle(db,birth_ref.ref,estimate)
     if estimate and (ret == 0):
-        ret = get_bapt_date(db,person)
+        ret = get_bapt_date(db,person,estimate)
     return ret
 
 def get_death_date(db,person,estimate=False):
@@ -160,9 +175,9 @@ def get_death_date(db,person,estimate=False):
     if not death_ref:
         ret = 0
     else:
-        ret = get_date_from_event_handle(db,death_ref.ref)
+        ret = get_date_from_event_handle(db,death_ref.ref,estimate)
     if estimate and (ret == 0):
-        ret = get_bury_date(db,person)
+        ret = get_bury_date(db,person,estimate)
     return ret
 
 def get_age_at_death(db,person,estimate):
@@ -222,13 +237,15 @@ def get_marriage_date(db,family):
 # Actual tool
 #
 #-------------------------------------------------------------------------
-class Verify(Tool.Tool, ManagedWindow, UpdateCallback):
+class Verify(tool.Tool, ManagedWindow, UpdateCallback):
 
     def __init__(self, dbstate, uistate, options_class, name, callback=None):
         self.label = _('Database Verify tool')
-        Tool.Tool.__init__(self, dbstate, options_class, name)
+        self.vr = None
+        tool.Tool.__init__(self, dbstate, options_class, name)
         ManagedWindow.__init__(self, uistate,[], self.__class__)
-        UpdateCallback.__init__(self, self.uistate.pulse_progressbar)
+        if uistate:
+            UpdateCallback.__init__(self, self.uistate.pulse_progressbar)
 
         self.dbstate = dbstate
         if uistate:
@@ -325,7 +342,8 @@ class Verify(Tool.Tool, ManagedWindow, UpdateCallback):
           self.options.handler.options_dict.iteritems():
             exec '%s = %s' % (option, value)
 
-        self.vr.real_model.clear()
+        if self.vr:
+            self.vr.real_model.clear()
 
         self.set_total(self.db.get_number_of_people() +
                        self.db.get_number_of_families())
@@ -659,13 +677,13 @@ class VerifyResults(ManagedWindow):
 # 
 #
 #------------------------------------------------------------------------
-class VerifyOptions(Tool.ToolOptions):
+class VerifyOptions(tool.ToolOptions):
     """
     Defines options and provides handling interface.
     """
 
     def __init__(self, name,person_id=None):
-        Tool.ToolOptions.__init__(self, name,person_id)
+        tool.ToolOptions.__init__(self, name,person_id)
 
         # Options specific for this report
         self.options_dict = {
@@ -981,12 +999,12 @@ class TooManyChildren(PersonRule):
     def broken(self):
         n_child = get_n_children(self.db,self.obj)
 
-        if (self.obj.get_gender == gen.lib.Person.MALE) \
-               and (n_child > self.mx_child_dad):
+        if (self.obj.get_gender == gen.lib.Person.MALE
+               and n_child > self.mx_child_dad):
             return True
 
-        if (self.obj.get_gender == gen.lib.Person.FEMALE) \
-               and (n_child > self.mx_child_mom):
+        if (self.obj.get_gender == gen.lib.Person.FEMALE
+               and n_child > self.mx_child_mom):
             return True
 
         return False
