@@ -18,7 +18,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: eventview.py 14950 2010-03-27 22:59:41Z nick-h $
+# $Id: eventview.py 17446 2011-05-07 18:41:14Z nick-h $
 
 """
 Provide the event view.
@@ -53,6 +53,7 @@ import Errors
 import Bookmarks
 import config
 from DdTargets import DdTargets
+from QuestionDialog import ErrorDialog
 from gui.editors import EditEvent, DeleteEventQuery
 from Filters.SideBar import EventSidebarFilter
 from gen.plug import CATEGORY_QR_EVENT
@@ -96,10 +97,11 @@ class EventView(ListView):
     ADD_MSG     = _("Add a new event")
     EDIT_MSG    = _("Edit the selected event")
     DEL_MSG     = _("Delete the selected event")
+    MERGE_MSG   = _("Merge the selected events")
     FILTER_TYPE = "Event"
     QR_CATEGORY = CATEGORY_QR_EVENT
 
-    def __init__(self, dbstate, uistate, nav_group=0):
+    def __init__(self, pdata, dbstate, uistate, nav_group=0):
         """
         Create the Event View
         """
@@ -111,7 +113,7 @@ class EventView(ListView):
             }
 
         ListView.__init__(
-            self, _('Events'), dbstate, uistate,
+            self, _('Events'), pdata, dbstate, uistate,
             EventView.COLUMN_NAMES, len(EventView.COLUMN_NAMES), 
             EventModel,
             signal_map, dbstate.db.get_event_bookmarks(),
@@ -120,14 +122,15 @@ class EventView(ListView):
             filter_class=EventSidebarFilter,
             markup = EventView.MARKUP_COLS)
             
-        self.func_list = {
+        self.func_list.update({
             '<CONTROL>J' : self.jump,
             '<CONTROL>BackSpace' : self.key_delete,
-            }
+            })
 
-        config.connect("interface.filter", self.filter_toggle)
         uistate.connect('nameformat-changed', self.build_tree)
 
+        self.additional_uis.append(self.additional_ui())
+
     def navigation_type(self):
         return 'Event'
 
@@ -149,7 +152,7 @@ class EventView(ListView):
         """
         return 'gramps-event'
 
-    def ui_definition(self):
+    def additional_ui(self):
         """
         Defines the UI string for UIManager
         """
@@ -178,6 +181,7 @@ class EventView(ListView):
                 <menuitem action="Add"/>
                 <menuitem action="Edit"/>
                 <menuitem action="Remove"/>
+                <menuitem action="Merge"/>
               </placeholder>
               <menuitem action="FilterEdit"/>
             </menu>
@@ -191,6 +195,7 @@ class EventView(ListView):
               <toolitem action="Add"/>
               <toolitem action="Edit"/>
               <toolitem action="Remove"/>
+              <toolitem action="Merge"/>
             </placeholder>
           </toolbar>
           <popup name="Popup">
@@ -200,6 +205,7 @@ class EventView(ListView):
             <menuitem action="Add"/>
             <menuitem action="Edit"/>
             <menuitem action="Remove"/>
+            <menuitem action="Merge"/>
             <separator/>
             <menu name="QuickReport" action="QuickReport">
               <menuitem action="Dummy"/>
@@ -256,6 +262,22 @@ class EventView(ListView):
             except Errors.WindowActiveError:
                 pass
 
+    def merge(self, obj):
+        """
+        Merge the selected events.
+        """
+        mlist = self.selected_handles()
+
+        if len(mlist) != 2:
+            msg = _("Cannot merge event objects.")
+            msg2 = _("Exactly two events must be selected to perform a merge. "
+                     "A second object can be selected by holding down the "
+                     "control key while clicking on the desired event.")
+            ErrorDialog(msg, msg2)
+        else:
+            import Merge
+            Merge.MergeEvents(self.dbstate, self.uistate, mlist[0], mlist[1])
+
     def dummy_report(self, obj):
         """ For the xml UI definition of popup to work, the submenu 
             Quick Report must have an entry in the xml
@@ -263,3 +285,13 @@ class EventView(ListView):
         """
         pass
 
+    def get_default_gramplets(self):
+        """
+        Define the default gramplets for the sidebar and bottombar.
+        """
+        return (("Event Filter",),
+                ("Event Gallery",
+                 "Event Sources",
+                 "Event Notes",
+                 "Event Attributes",
+                 "Event Backlinks"))
