@@ -1,6 +1,7 @@
 # Gramps - a GTK+/GNOME based genealogy program
 #
 # Copyright (C) 2001-2006  Donald N. Allingham
+# Copyright (C) 2010       Nick Hall
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,7 +18,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: familyview.py 14950 2010-03-27 22:59:41Z nick-h $
+# $Id: familyview.py 17446 2011-05-07 18:41:14Z nick-h $
 
 """
 Family View.
@@ -50,8 +51,10 @@ from gui.editors import EditFamily
 import Bookmarks
 import Errors
 import config
+from QuestionDialog import ErrorDialog
 from Filters.SideBar import FamilySidebarFilter
 from gen.plug import CATEGORY_QR_FAMILY
+from DdTargets import DdTargets
 
 #-------------------------------------------------------------------------
 #
@@ -67,14 +70,17 @@ class FamilyView(ListView):
     COL_MOTHER = 2
     COL_REL = 3
     COL_MARDATE = 4
-    COL_CHAN = 5
+    COL_TAGS = 5
+    COL_CHAN = 6
     # name of the columns
+    MARKUP_COLS = [COL_MARDATE]
     COLUMN_NAMES = [
         _('ID'),
         _('Father'),
         _('Mother'),
         _('Relationship'),
         _('Marriage Date'),
+        _('Tags'),
         _('Last Changed'),
         ]
     #default setting with visible columns, order of the col, and their size
@@ -82,48 +88,53 @@ class FamilyView(ListView):
         ('columns.visible', [COL_ID, COL_FATHER, COL_MOTHER, COL_REL, 
                              COL_MARDATE]),
         ('columns.rank', [COL_ID, COL_FATHER, COL_MOTHER, COL_REL, 
-                           COL_MARDATE, COL_CHAN]),
-        ('columns.size', [75, 200, 200, 100, 100, 100])
+                           COL_MARDATE, COL_TAGS, COL_CHAN]),
+        ('columns.size', [75, 200, 200, 100, 100, 100, 100])
         )    
 
     ADD_MSG     = _("Add a new family")
     EDIT_MSG    = _("Edit the selected family")
     DEL_MSG     = _("Delete the selected family")
+    MERGE_MSG   = _("Merge the selected families")
     FILTER_TYPE = "Family"
     QR_CATEGORY = CATEGORY_QR_FAMILY
 
-    def __init__(self, dbstate, uistate, nav_group=0):
+    def __init__(self, pdata, dbstate, uistate, nav_group=0):
 
         signal_map = {
             'family-add'     : self.row_add,
             'family-update'  : self.row_update,
             'family-delete'  : self.row_delete,
             'family-rebuild' : self.object_build,
+            'tag-update'     : self.tag_updated
             }
 
         ListView.__init__(
-            self, _('Families'), dbstate, uistate,
+            self, _('Families'), pdata, dbstate, uistate,
             FamilyView.COLUMN_NAMES, len(FamilyView.COLUMN_NAMES), 
             FamilyModel,
             signal_map, dbstate.db.get_family_bookmarks(),
             Bookmarks.FamilyBookmarks, nav_group,
-            filter_class=FamilySidebarFilter)
+            multiple=True,
+            filter_class=FamilySidebarFilter,
+            markup=FamilyView.MARKUP_COLS)
 
-        self.func_list = {
+        self.func_list.update({
             '<CONTROL>J' : self.jump,
             '<CONTROL>BackSpace' : self.key_delete,
-            }
+            })
 
-        config.connect("interface.filter", self.filter_toggle)
         uistate.connect('nameformat-changed', self.build_tree)
 
+        self.additional_uis.append(self.additional_ui())
+
     def navigation_type(self):
         return 'Family'
 
     def get_stock(self):
         return 'gramps-family'
 
-    def ui_definition(self):
+    def additional_ui(self):
         return '''<ui>
           <menubar name="MenuBar">
             <menu action="FileMenu">
@@ -143,6 +154,7 @@ class FamilyView(ListView):
                 <menuitem action="Add"/>
                 <menuitem action="Edit"/>
                 <menuitem action="Remove"/>
+                <menuitem action="Merge"/>
               </placeholder>
               <menuitem action="FilterEdit"/>
             </menu>
@@ -162,6 +174,7 @@ class FamilyView(ListView):
               <toolitem action="Add"/>
               <toolitem action="Edit"/>
               <toolitem action="Remove"/>
+              <toolitem action="Merge"/>
             </placeholder>
           </toolbar>
           <popup name="Popup">
@@ -171,6 +184,10 @@ class FamilyView(ListView):
             <menuitem action="Add"/>
             <menuitem action="Edit"/>
             <menuitem action="Remove"/>
+            <menuitem action="Merge"/>
+            <separator/>
+            <menuitem action="MakeFatherActive"/>
+            <menuitem action="MakeMotherActive"/>
             <separator/>
             <menu name="QuickReport" action="QuickReport">
               <menuitem action="Dummy"/>
@@ -188,11 +205,29 @@ class FamilyView(ListView):
                         
         self.all_action = gtk.ActionGroup(self.title + "/FamilyAll")
         self.all_action.add_actions([
+                ('MakeFatherActive', gtk.STOCK_APPLY, _("Make Father Active Person"), 
+                 None, None, self._make_father_active),
+                ('MakeMotherActive', gtk.STOCK_APPLY, _("Make Mother Active Person"), 
+                 None, None, self._make_mother_active),
                 ('QuickReport', None, _("Quick View"), None, None, None),
                 ('Dummy', None, '  ', None, None, self.dummy_report),
                 ])
         self._add_action_group(self.all_action)
 
+    def set_active(self):
+        """
+        Called when the page is displayed.
+        """
+        ListView.set_active(self)
+        self.uistate.viewmanager.tags.tag_enable()
+
+    def set_inactive(self):
+        """
+        Called when the page is no longer displayed.
+        """
+        ListView.set_inactive(self)
+        self.uistate.viewmanager.tags.tag_disable()
+
     def get_bookmarks(self):
         return self.dbstate.db.get_family_bookmarks()
 
@@ -236,9 +271,84 @@ class FamilyView(ListView):
             except Errors.WindowActiveError:
                 pass
                 
+    def merge(self, obj):
+        """
+        Merge the selected families.
+        """
+        mlist = self.selected_handles()
+
+        if len(mlist) != 2:
+            msg = _("Cannot merge families.")
+            msg2 = _("Exactly two families must be selected to perform a merge."
+                     " A second family can be selected by holding down the "
+                     "control key while clicking on the desired family.")
+            ErrorDialog(msg, msg2)
+        else:
+            import Merge
+            Merge.MergeFamilies(self.dbstate, self.uistate, mlist[0], mlist[1])
+
+    def _make_father_active(self, obj):
+        """
+        Make the father of the family the active person.
+        """
+        fhandle = self.first_selected()
+        if fhandle:
+            family = self.dbstate.db.get_family_from_handle(fhandle)
+            if family:
+                self.uistate.set_active(family.father_handle, 'Person')
+
+    def _make_mother_active(self, obj):
+        """
+        Make the mother of the family the active person.
+        """
+        fhandle = self.first_selected()
+        if fhandle:
+            family = self.dbstate.db.get_family_from_handle(fhandle)
+            if family:
+                self.uistate.set_active(family.mother_handle, 'Person')
+            
     def dummy_report(self, obj):
         """ For the xml UI definition of popup to work, the submenu 
             Quick Report must have an entry in the xml
             As this submenu will be dynamically built, we offer a dummy action
         """
         pass
+
+    def drag_info(self):
+        """
+        Indicate that the drag type is a FAMILY_LINK
+        """
+        return DdTargets.FAMILY_LINK
+
+    def tag_updated(self, handle_list):
+        """
+        Update tagged rows when a tag color changes.
+        """
+        all_links = set([])
+        for tag_handle in handle_list:
+            links = set([link[1] for link in
+                         self.dbstate.db.find_backlink_handles(tag_handle,
+                                                    include_classes='Family')])
+            all_links = all_links.union(links)
+        self.row_update(list(all_links))
+
+    def add_tag(self, transaction, family_handle, tag_handle):
+        """
+        Add the given tag to the given family.
+        """
+        family = self.dbstate.db.get_family_from_handle(family_handle)
+        family.add_tag(tag_handle)
+        self.dbstate.db.commit_family(family, transaction)
+
+    def get_default_gramplets(self):
+        """
+        Define the default gramplets for the sidebar and bottombar.
+        """
+        return (("Family Filter",),
+                ("Family Gallery",
+                 "Family Events",
+                 "Family Children",
+                 "Family Sources",
+                 "Family Notes",
+                 "Family Attributes",
+                 "Family Backlinks"))
