@@ -17,7 +17,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-# $Id: fanchartview.py 14811 2010-03-14 13:54:24Z dsblank $
+# $Id: fanchartview.py 16319 2010-12-28 17:04:41Z nick-h $
 
 ## Based on the paper:
 ##   http://www.cs.utah.edu/~draperg/research/fanchart/draperg_FHT08.pdf
@@ -566,9 +566,9 @@ class FanChartView(NavigationView):
     """
     The Gramplet code that realizes the FanChartWidget. 
     """
-    def __init__(self, dbstate, uistate, nav_group=0):
+    def __init__(self, pdata, dbstate, uistate, nav_group=0):
         NavigationView.__init__(self, _('Fan Chart'),
-                                      dbstate, uistate, 
+                                      pdata, dbstate, uistate, 
                                       dbstate.db.get_bookmarks(), 
                                       Bookmarks.PersonBookmarks,
                                       nav_group)        
@@ -579,6 +579,8 @@ class FanChartView(NavigationView):
         self.generations = 9
         self.format_helper = FormattingHelper(self.dbstate)
 
+        self.additional_uis.append(self.additional_ui())
+
     def navigation_type(self):
         return 'Person'
 
@@ -598,7 +600,7 @@ class FanChartView(NavigationView):
         """
         return 'gramps-fanchart'
 
-    def ui_definition(self):
+    def additional_ui(self):
         return '''<ui>
           <menubar name="MenuBar">
             <menu action="GoMenu">
@@ -622,7 +624,7 @@ class FanChartView(NavigationView):
         '''
 
     def build_tree(self):
-        pass # will build when active_change is called
+        pass # will build when active_changes
 
     def active_changed(self, handle):
         """
