@@ -36,7 +36,6 @@ from libpersonview import BasePersonView
 from gui.views.treemodels.peoplemodel import PersonListModel
 import gen.lib
 import Errors
-from gui.editors import EditPerson
 
 #-------------------------------------------------------------------------
 #
@@ -54,7 +53,7 @@ class PersonListView(BasePersonView):
     """
     A hierarchical view of the top three levels of places.
     """
-    def __init__(self, dbstate, uistate, nav_group=0):
-        BasePersonView.__init__(self, dbstate, uistate,
+    def __init__(self, pdata, dbstate, uistate, nav_group=0):
+        BasePersonView.__init__(self, pdata, dbstate, uistate,
                                _('Person View'), PersonListModel,
                                nav_group=nav_group)
