@@ -18,7 +18,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: relview.py 16680 2011-02-20 10:52:06Z bmcage $
+# $Id: relview.py 18088 2011-08-30 18:35:29Z nick-h $
 
 """
 Relationship View
@@ -520,9 +520,6 @@ class RelationshipView(NavigationView):
             for family_handle in family_handle_list:
                 if family_handle:
                     self.write_family(family_handle, person)
-        else:
-            self.write_label("%s:" % _('Family'), None, False, person)
-            self.row += 1
 
         self.row = 0
 
@@ -1341,7 +1338,8 @@ class RelationshipView(NavigationView):
         else:
             # show "V Family: ..." and the rest
             self.write_label("%s:" % _('Family'), family, False, person)
-            if handle:
+            if (handle or
+                    family.get_relationship() != gen.lib.FamilyRelType.UNKNOWN):
                 box = self.write_person(_('Spouse'), handle)
 
                 if not self.write_relationship_events(box, family):
