@@ -23,7 +23,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: WebCal.py 17475 2011-05-11 09:32:10Z bmcage $
+# $Id: WebCal.py 18130 2011-09-09 19:09:44Z m_d_n $
 
 """
 Web Calendar generator.
@@ -401,7 +401,7 @@ class WebCalReport(Report):
         """
 
         # number of subdirectories up to reach root
-        subdirs = '../'*nr_up
+        subdirs = ['..'] * nr_up
 
         # Header contants
         xmllang = xml_lang()
@@ -416,10 +416,10 @@ class WebCalReport(Report):
             body.attr = "id = '%(idtag)s'" % { 'idtag' : body_id }
 
         # GRAMPS favicon
-        fname1 = os.path.join(subdirs, "images", "favicon2.ico")
+        fname1 = "/".join(subdirs + ["images", "favicon2.ico"])
 
         # _CALENDARSCREEN stylesheet
-        fname2 = os.path.join(subdirs, "styles", _CALENDARSCREEN)
+        fname2 = "/".join(subdirs + ["styles", _CALENDARSCREEN])
 
         # create additional meta tags
         meta = Html("meta", attr = _META1) + (
@@ -433,14 +433,14 @@ class WebCalReport(Report):
 
         # add printer stylesheet to webcalendar() and one_day() only
         if add_print:
-            fname = os.path.join(subdirs, "styles", _CALENDARPRINT)
+            fname = "/".join(subdirs + ["styles", _CALENDARPRINT])
             links += Html("link",rel = "stylesheet", href = fname,type = "text/css", media = "print", indent = False)
 
         # add horizontal menu if css == Blue or Visually because there is no menus
         if CSS[self.css]["navigation"]:
 
             # Link to Navigation Menus stylesheet
-            fname = os.path.join(subdirs, "styles", "Web_Navigation-Menus.css")
+            fname = "/".join(subdirs + ["styles", "Web_Navigation-Menus.css"])
             links.extend( 
                 Html("link", href = fname, type = "text/css", media = "screen", rel = "stylesheet")
             )
@@ -719,7 +719,7 @@ class WebCalReport(Report):
                         specday = __get_previous_month_day(year, month, day_col) if week_row == 0 \
                             else __get_next_month_day(year, month, day_col) 
 
-                        specclass = "previous " if week_row == 0 else "Next "
+                        specclass = "previous " if week_row == 0 else "next "
                         specclass += dayclass
 
                         # continue table cell, <td>, without id tag
@@ -867,8 +867,10 @@ class WebCalReport(Report):
             body += self.month_navigation(nr_up, year, currentsection, True)
 
             # build the calendar
+            content = Html("div", class_="content")
+            body += content
             monthly_calendar = self.calendar_build("wc", year, month)
-            body += monthly_calendar
+            content += monthly_calendar
 
             # create note section for webcalendar()
             note = self.month_notes[month-1].strip()
@@ -934,17 +936,16 @@ class WebCalReport(Report):
                        'that date, if there are any.\n'))
 
         # page description 
-        body += Html("div", class_ = "content") + (
-
-            # message line
+        content = Html("div", class_ = "content") + (
             Html("p", msg, id='description')
             )
+        body += content
 
         for month in range(1, 13):
 
             # build the calendar
             monthly_calendar = self.calendar_build("yg", year, month)
-            body += monthly_calendar  
+            content += monthly_calendar  
 
             # increase progress bar
             self.progress.step()
@@ -998,11 +999,13 @@ class WebCalReport(Report):
         body += self.month_navigation(nr_up, year, currentsection, True)
 
         # set date display as in user prevferences 
-        body += Html("h3", _dd.display(event_date), inline = True)
+        content = Html("div", class_="content")
+        body += content
+        content += Html("h3", _dd.display(event_date), inline = True)
 
         # list the events
         ordered = Html("ol")
-        body += ordered  
+        content += ordered  
         for nyears, date, text, event in day_list:
             ordered += Html("li", text, inline = False if event == 'Anniversary' else True)
 
@@ -1399,11 +1402,12 @@ class WebCalOptions(MenuReportOptions):
         cright.set_help( _("The copyright to be used for the web files"))
         menu.add_option(category_name, "cright", cright)
 
-        css = EnumeratedListOption(_('StyleSheet'), CSS["default"]["id"])
-        for (fname, id) in sorted([(CSS[key]["translation"], CSS[key]["id"]) 
-                                  for key in CSS.keys()]):                                      
-            if CSS[id]["user"]:
-                css.add_item(CSS[id]["id"], CSS[id]["translation"])
+        css_list = sorted([(CSS[key]["translation"], CSS[key]["id"]) 
+                            for key in CSS.keys()
+                            if CSS[key]["user"]])
+        css = EnumeratedListOption(_('StyleSheet'), css_list[0][1])
+        for css_item in css_list:                              
+            css.add_item(css_item[1], css_item[0])
         css.set_help( _('The stylesheet to be used for the web pages'))
         menu.add_option(category_name, "css", css)
 
