@@ -46,11 +46,12 @@ TABLE  = string.maketrans('ABCDEFGIJKLMNOPQRSTUVXYZ',
 #-------------------------------------------------------------------------
 def soundex(strval):
     "Return the soundex value to a string argument."
-    
+
     strval = unicodedata.normalize('NFKD', 
-                strval.upper().strip()).encode('ASCII', 'ignore')
+                    unicode(strval.upper().strip())).encode('ASCII', 'ignore')
     if not strval:
         return "Z000"
+    strval = strval.encode('iso-8859-1')
     str2 = strval[0]
     strval = strval.translate(TABLE, IGNORE)
     if not strval:
